-- /*  
-- * Copyright (c) 2009 Ant Kutschera, maxant
-- * 
-- * This file is part of the maxant GWT Demo.
-- * 
-- * maxant GWT Demo is free software: you can redistribute it and/or modify
-- * it under the terms of the Lesser GNU General Public License as published by
-- * the Free Software Foundation, either version 3 of the License, or
-- * (at your option) any later version.
-- * 
-- * maxant GWT Demo is distributed in the hope that it will be useful,
-- * but WITHOUT ANY WARRANTY; without even the implied warranty of
-- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- * Lesser GNU General Public License for more details.
-- * You should have received a copy of the Lesser GNU General Public License
-- * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
-- */

use gwtdemo;
begin;

-- drop tables, in reverse order not to break fk constraints

drop table if exists front_page_design;
drop table if exists design_comment;
drop table if exists sales_order_item;
drop table if exists sales_order;
drop table if exists design_tag;
drop table if exists design_rating;
drop table if exists design;
drop table if exists artist_view;
drop table if exists role;
drop table if exists party;
drop table if exists config;


-- ------------------------------------------------------------
create table config (
	uid integer not null auto_increment, -- for hibernate only
	name varchar(255) unique not null, -- the key
	val varchar(512) not null, -- the value
	description varchar(255) not null, -- descriptive text for storing in the db or displaying to the user
	update_time datetime not null, -- when was this last changed?
	primary key (uid),
    index (name)
) type = innodb;

-- ------------------------------------------------------------
-- the table of people who can log into the site (artists, customers and staff)
create table party (
	uid integer not null auto_increment, -- used for assisting hibernate generation only
	login varchar(255) not null, -- the username entered into the site
	password varchar(255) not null, -- the password
    screenname varchar(50),
    firstname varchar(50) not null,
    lastname  varchar(50) not null,
    title     varchar(10),
    dob       date,
    registration_date date not null,
    phone  varchar(50),
    mobile varchar(50),
    joined_mailing_list bool not null,
    joined_mobile_list bool not null,
    where_heard_about varchar(255),
    is_staff bool not null,
    staff_number varchar(255) unique,
    is_artist bool not null,
    is_blocked bool not null, -- are they banned from using the site?
    artist_description varchar(1000),
    reg_code varchar(10), -- if they need to confirm registration then its not null
    preferred_locale varchar(20),
    customer_notes text, -- notes by merchant about their shopping
    staff_notes text, -- notes by merchant if they are staff
    artist_notes text, -- notes by merchant if they are an artist
    primary key (uid),
    index (login)
) type = innodb;
alter table party auto_increment = 7516;

-- ------------------------------------------------------------
-- create a table for holding the roles of artists/customers/staff. currently valid roles are artist,
-- picking, backoffice.
-- NO REFERENTIAL INTEGRITY, because hibernate is not able to cope!!!
create table role (
	uid integer not null auto_increment, -- used for assisting hibernate generation only
	login varchar(50) not null,
	role varchar(50) not null,
	primary key (uid),
    index (login)
) type = innodb;

-- ------------------------------------------------------------
-- create the table for the designs;
create table design (
	uid integer not null auto_increment, -- used for assisting hibernate generation only
    in_catalogue bool not null, -- is this item in the catalogue? set by designer
    approved bool not null, -- is this item approved? set by admin
    name varchar(50) not null,
    description varchar(500) not null,
    artist_uid integer not null, -- must always belong to an artist!
    number_of_ratings integer not null,
    sum_of_ratings bigint not null,
    base_price decimal(10,2) not null, -- artist can add a "zuschlag"
    created datetime not null, -- when was it added?
    primary key (uid),
    foreign key (artist_uid) references party(uid)
) type = innodb;
alter table design auto_increment = 5543;

-- ------------------------------------------------------------
create table artist_view (
    uid integer auto_increment not null,
    artist_uid integer not null,
    day date not null,
    number_of_views integer not null,
    foreign key (artist_uid) references party(uid),
    index (day),
    primary key (uid)
) type = innodb;

-- ------------------------------------------------------------
-- create the table for the designs;
create table design_rating (
	uid integer not null auto_increment, -- used for assisting hibernate generation only
    design_uid integer not null, 
    user_uid integer not null,
    val integer not null,
    primary key (uid),
    foreign key (design_uid) references design(uid),
    foreign key (user_uid) references party(uid),
    index(design_uid)
) type = innodb;

-- ------------------------------------------------------------
-- tags for each design
create table design_tag (
	uid integer not null auto_increment, -- used for assisting hibernate generation only
    design_uid integer not null, 
    tag varchar(50) not null, 
    primary key (uid),
    foreign key (design_uid) references design(uid),
    index (uid)
) type = innodb;

-- ------------------------------------------------------------
-- create the table for the orders;
create table sales_order (
	uid integer not null auto_increment, -- used for assisting hibernate generation only
	customer_uid integer, -- if not logged in, create if we cannot find their email address. no excuse to ever be null. if they are not registered, we simply dont have them in the user table.
	email varchar(50) not null, -- email address of orderer
    payment_type varchar(20), 	-- 
			-- a) paypal 		paypal/credit/debit cards		-> redirect to paypal and use IPN and PDT
			-- b) giftCertOnly	gift certificate
    delivery_total decimal(10,2), -- what did the delivery cost?
	delivery_description varchar(255), -- what type of delivery was it?
	payment_details text,
    status_order_created datetime,      -- the order has been created, meaning that the customer has gone 
										-- to the checkout page. they have not paid or even attempted to 
										-- pay yet, but we need an order number!
	status_order_placed datetime, 		-- the order has been placed. if (trustedCustomer) || 
										-- cashOnDelivery || cashOnCollection || invoice then set status to 
										-- awaiting_picking
    status_payment_successful datetime,	-- payment has been received. if not already awaiting_picking, set to awaiting_picking
	status_payment_failed datetime, 	-- contact customer and get them to pay some other way. DO NOT DISPATCH! Unless trusted, do not pick?
	status_awaiting_picking datetime, 	-- when payment_successful, set this state. it means that the order needs to be picked
	status_started_picking datetime, 	-- order is being picked
	status_finished_picking datetime,	-- set when finished picking. order may be ready for dispatch, but not necessarily. 
										-- wait until system updates the ready for dispatch status
	status_ready_for_dispatch datetime,	-- order is ready for dispatch
	status_dispatched datetime, 		-- order has been dispatched
	status_delivered datetime, 			-- when notification of delivery has been received
 	status_cancelled datetime, 			-- when if at all was the order cancelled?
 	status_discarded datetime, 			-- if it was never created, because the user let the session die - this is a special case allowing the merchant to track unordered orders.
	needs_attention bool not null,
    txn_id varchar(50), 		-- transaction ID from paypal or other gateway
	pp_mc_currency varchar(10), 	-- See PayPal table of supported currencies. This is the currency of the payment. 
	tax_cost decimal(8, 2), 	-- amount of VAT paid 
    customer_notes text, -- notes added by a customer at the time of purchase
	shop_notes text, -- notes added by the user when processing the order. hidden to the customer.
	is_gift bool not null default false, -- is this to be wrapped as a gift?
    picked_by integer,
	primary key (uid),
	foreign key (customer_uid) references party(uid),
	foreign key (picked_by) references party(uid)
) type = innodb;
alter table sales_order auto_increment = 397950;
-- ------------------------------------------------------------
-- create the table of items in each sales order;
create table sales_order_item (
	uid integer not null auto_increment, -- used for assisting hibernate generation only
    sales_order_uid integer not null,
    design_uid integer not null,
    quantity integer not null,
    base_price decimal(10,2) not null, -- at time of purchase, see design
    additional_delivery_charge decimal(10,2) not null, -- copied from the media
    special_offer_discount decimal (10,3) not null, -- see media. copied from the media
    primary key (uid),
    foreign key (sales_order_uid) references sales_order(uid),
    foreign key (design_uid) references design(uid),
    index (uid),
    index (design_uid)
) type = innodb;

-- ------------------------------------------------------------
-- best selling designs or newest ones
create table front_page_design (
    uid integer auto_increment not null,
    design_uid integer not null,
    foreign key (design_uid) references design(uid),
    primary key (uid)
) type = innodb;

-- ------------------------------------------------------------
-- comments for designs. if approved is null, then its shown in a list to be moderated.
-- if rejected is null, it can be shown. if its rejected, it is neither shown nor is it on the list to be moderated.
-- the config can of course override this, if no moderation is needed.
create table design_comment (
    uid integer auto_increment not null,
    added datetime not null,
    approved datetime, -- if not null, AND rejected is null, then it can be shown
    rejected datetime, -- when rejected if at all. if null, AND approved not null, then it can be shown
    comment varchar(4096) not null,
    design_uid integer not null,
    user_uid integer not null,
    moderated_by_uid integer, -- can be null if not approved. last person to moderate it
    foreign key (design_uid) references design(uid),
    foreign key (user_uid) references party(uid),
    foreign key (moderated_by_uid) references party(uid),
    index (design_uid), -- since we almost always load by design ID
    primary key (uid)
) type = innodb;


commit;
