package uk.co.maxant.gwtdemo.client;

import java.io.Serializable;

//TODO instead of using a cause, this exception should just contain a stack trace of the cause, since its handed
//		back to the client, who may not have server exceptions on its classpath.

/**
 * High Level Exception Class.
 * 
 * this class appears to have to be in the client package, otherwise GWT
 * cannot use it? A sub-package doesnt suffice...
 */
public class ApplicationException extends Exception implements Serializable {

	private static final long serialVersionUID = 1L;

	/**
	 * Technical details of the exception, not shown to user, but added to log.
	 */
	private String technicalDetails;

	/**
	 * error code.
	 */
	private String code;

	/**
	 * details for the user to see
	 */
	private String userDetails;
	
	private long timestamp = System.currentTimeMillis();

	/**
	 * used PURELY for GWT serialisation!!
	 */
	public ApplicationException() {
	}

	/**
	 * @param code The code, that is internationalisable
	 * @param technicalDetails Technical details that should be logged
	 */
	public ApplicationException(String code, String technicalDetails, String userDetails){
		this.code = code;
		this.technicalDetails = technicalDetails;
		this.userDetails = userDetails;
	}

	/**
	 * @return the error code, should be given to user, for referring to the manual/helpdesk
	 */
	public String getCode(){
		return code;
	}

	/**
	 * @return tech details that can be logged. do not show to user.
	 */
	public String getTechnicalDetails(){
		return technicalDetails;
	}

	/**
	 * @return the message designed for the user to see
	 */
	public String getUserDetails(){
		return userDetails;
	}

	/**
	 * @return User details.
	 */
	@Override
	public String getMessage() {
		return getUserDetails();
	}

	public long getTimestamp() {
		return timestamp;
	}

}
