/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client;

import java.util.List;

import com.google.gwt.user.client.rpc.AsyncCallback;

import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.BasketItem;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.DesignComment;
import uk.co.maxant.gwtdemo.client.to.OrderState;
import uk.co.maxant.gwtdemo.client.to.PaypalConfig;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.to.SearchResults;
import uk.co.maxant.gwtdemo.client.to.SessionInfo;
import uk.co.maxant.gwtdemo.client.to.User;


/**
 * @see  ApplicationLayerService
 */
public interface ApplicationLayerServiceAsync {

    /**
     * @see  ApplicationLayerService#getFeaturedDesigns()
     */
    void getFeaturedDesigns(AsyncCallback<List<Design>> callback);

    /**
     * @see  ApplicationLayerService#getNewestDesigns()
     */
    void getNewestDesigns(AsyncCallback<List<Design>> callback);
    
    /**
     * @see  ApplicationLayerService#addToBasket(BasketItem)
     */
    void addToBasket(BasketItem basketItem, AsyncCallback<List<BasketItem>> callback);

    /**
     * @see  ApplicationLayerService#getBasket()
     */
    void getBasket(AsyncCallback<List<BasketItem>> callback);

    /**
     * @see  ApplicationLayerService#emptyBasket()
     */
    void emptyBasket(AsyncCallback<List<BasketItem>> callback);

    /**
     * @see  ApplicationLayerService#createOrder(Address)
     */
    void createOrder(String email, String name, AsyncCallback<Integer> callback);

    /**
     * @see  ApplicationLayerService#getSessionInfo()
     */
    void getSessionInfo(AsyncCallback<SessionInfo> callback);

    /**
     * @see  ApplicationLayerService#getPaypalConfig(Integer)
     */
    void getPaypalConfig(Integer soUid, AsyncCallback<PaypalConfig> callback);

    /**
     * @see  ApplicationLayerService#getDesigns(Integer, Integer, boolean)
     */
    void getDesigns(boolean getAllContent, AsyncCallback<List<Design>> callback);
    
    /**
     * @see  ApplicationLayerService#getDesignsForArtist(Integer, Integer)
     */
    void getDesignsForArtist(Integer artistUid, AsyncCallback<List<Design>> callback);

    /**
     * @see  ApplicationLayerService#getDesignsAdmin(String)
     */
    void getDesignsAdmin(String filter, AsyncCallback<List<Design>> callback);
    
    /**
     * @see  ApplicationLayerService#getDesignsAdminLight()
     */
    void getDesignsAdminLight(AsyncCallback<List<Design>> callback);

    /**
     * @see ApplicationLayerService#getDesign(Integer)
     */
    void getDesign(Integer designID, AsyncCallback<Design> callback);
    
    /**
     * @see  ApplicationLayerService#getArtistsAdmin(String)
     */
    void getArtistsAdmin(String filter, AsyncCallback<List<Artist>> callback);
    
    /**
     * @see  ApplicationLayerService#search(String, Integer)
     */
    void search(String searchExpression, AsyncCallback<SearchResults> callback);

    /**
     * @see  ApplicationLayerService#saveOrUpdateDesign(Design, List, List)
     */
    void saveOrUpdateDesign(Design design, AsyncCallback<Integer> callback);

    /**
     * @see  ApplicationLayerService#saveOrUpdateArtist(Artist)
     */
    void saveOrUpdateArtist(Artist artist, AsyncCallback<Integer> callback);
    
    /**
     * @see  ApplicationLayerService#getArtistViewCloud()
     */
    void getArtistViewCloud(AsyncCallback<List<Artist>> callback);

    /**
     * @see  ApplicationLayerService#getPendingSalesOrders()
     */
    void getPendingSalesOrders(AsyncCallback<List<SalesOrder>> callback);
    
    /**
     * @see  ApplicationLayerService#getDiscardedSalesOrders()
     */
    void getDiscardedSalesOrders(AsyncCallback<List<SalesOrder>> callback);
    
    /**
     * @see  ApplicationLayerService#getProblemSalesOrders()
     */
    void getProblemSalesOrders(AsyncCallback<List<SalesOrder>> callback);
    
    /**
     * @see  ApplicationLayerService#searchSalesOrders(String)
     */
    void searchSalesOrders(String search, AsyncCallback<List<SalesOrder>> callback);
    
    /**
     * @see  ApplicationLayerService#getCustomerSalesOrders()
     */
    void getCustomerSalesOrders(AsyncCallback<List<SalesOrder>> callback);
    
    /**
     * @see  ApplicationLayerService#getSalesOrder(String)
     */
	void getSalesOrder(String uid, AsyncCallback<SalesOrder> callback);
	
	/**
	 * @see  ApplicationLayerService#registerCustomerOrArtist(User, String, String, boolean, String)
	 */
	void registerCustomerOrArtist(User user, String password, String captcha, boolean alsoArtist, String artistsDescription, AsyncCallback<Integer> callback);
	
	/**
	 * @see  ApplicationLayerService#rateDesign(Design, int)
	 */
	void rateDesign(Design design, Integer ratingGiven, AsyncCallback<Void> callback);

	/**
	 * @see ApplicationLayerService#updateOrderStatus(OrderState, int, String)
	 */
	void updateOrderStatus(OrderState requiredState, int soUid, String notes, AsyncCallback<Void> callback);
	
	/**
	 * @see ApplicationLayerService#updatePaymentStatus(OrderState, int, String, String)
	 */
	void updatePaymentStatus(OrderState requiredState, int soUid, String extraNotes, String txnId, AsyncCallback<Void> callback);

	/**
	 * @see ApplicationLayerService#addShopNotes(int, String)
	 */
	void addShopNotes(int soUid, String notes, AsyncCallback<Void> callback);

	/**
	 * @see ApplicationLayerService#markOrderAsNotNeedingAttention(int, String)
	 */
	void markOrderAsNotNeedingAttention(int soUid, String msg, AsyncCallback<Void> callback);

	/**
	 * @see ApplicationLayerService#markOrderAsNeedingAttention(Integer, String)
	 */
	void markOrderAsNeedingAttention(Integer orderNum, String msg, AsyncCallback<Void> callback);

	/**
	 * @see ApplicationLayerService#getAllComments(Integer)
	 */
	void getAllComments(Integer designUid, AsyncCallback<List<DesignComment>> callback);

	/**
	 * @see ApplicationLayerService#getComments(Integer)
	 */
	void getComments(Integer designUid, AsyncCallback<List<DesignComment>> callback);

	/**
	 * @see ApplicationLayerService#addDesignComment(DesignComment)
	 */
	void addDesignComment(DesignComment dc, AsyncCallback<Integer> callback);
	
	/**
	 * @see ApplicationLayerService#approveDesignComment(DesignComment)
	 */
	void approveDesignComment(DesignComment dc, AsyncCallback<Void> callback);
	
	/**
	 * @see ApplicationLayerService#rejectDesignComment(DesignComment)
	 */
    void rejectDesignComment(DesignComment dc, AsyncCallback<Void> callback);

    /**
     * @see ApplicationLayerService#getDesignsPendingModeration()
     */
    void getDesignsPendingModeration(AsyncCallback<List<Design>> callback);

    /**
     * @see ApplicationLayerService#getLoggedInArtist()
     */
    void getLoggedInArtist(AsyncCallback<Artist> callback);

    /**
     * @see ApplicationLayerService#updateCustomer(User)
     */
    void updateCustomer(User user, String newPassword, AsyncCallback<Integer> callback);

    /**
     * @see ApplicationLayerService#updateArtistProfile(Artist)
     */
    void updateArtistProfile(Artist artist, AsyncCallback<Integer> callback);

}
