/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.artist_admin.core;


import java.util.List;

import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.util.AsyncCallbackImpl;
import uk.co.maxant.gwtdemo.client.widgets.core.WizardServerCaller;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * delegated to by controller
 */
public class ArtistAdminServerCaller extends WizardServerCaller {

	private IArtistAdmin artistAdmin;
	private ArtistAdminModel model;
	
	public ArtistAdminServerCaller(IArtistAdmin artistAdmin, ArtistAdminModel model, AppWizard wizard){
		super(wizard, artistAdmin);
		this.artistAdmin = artistAdmin;
		this.model = model;
	}
	
	/**
	 * called to set the Artists
	 * 
	 * @param filter the filter to use
	 */
	public void getArtists(String filter) {
		
		AsyncCallback<List<Artist>> callback = new AsyncCallbackImpl<List<Artist>>(this) {
			
			public void onSuccess_(List<Artist> artists) {
				model.setArtists(artists);
				artistAdmin.setArtists();
			}
		};
		callServer();
		getServiceClient().getArtistsAdmin(filter, callback);
	}

	public void updateArtist(final Artist a, final String filter) {

		AsyncCallback<Integer> callback = new AsyncCallbackImpl<Integer>(this) {
			
			public void onSuccess_(Integer uid) {
				//update master data
				wizard.updateArtistMasterData(a);

				//now go reload using the original filter.
				getArtists(String.valueOf(uid));
			}
		};
		callServer();
		getServiceClient().saveOrUpdateArtist(a, callback);
		
	}

}
