/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.artist_admin.ui;


import java.util.HashMap;
import java.util.Map;

import uk.co.maxant.gwtdemo.client.Roles;
import uk.co.maxant.gwtdemo.client.artist_admin.core.ArtistAdminController;
import uk.co.maxant.gwtdemo.client.artist_admin.core.IArtistAdmin;
import uk.co.maxant.gwtdemo.client.generic.widgets.AsyncFilterBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.Button;
import uk.co.maxant.gwtdemo.client.generic.widgets.CheckBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.ListBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.TextBoxWithValidation;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.util.ClientFormats;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;
import uk.co.maxant.gwtdemo.client.widgets.ui.richtextarea.SmallRichTextToolbar;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.FileUpload;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.Hidden;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.RichTextArea;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteEvent;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteHandler;
import com.google.gwt.user.client.ui.FormPanel.SubmitEvent;
import com.google.gwt.user.client.ui.FormPanel.SubmitHandler;

/**
 * the artist admin screen, which lets admin manage designs.
 * 
 */
public class ArtistAdmin extends WizardPage implements IArtistAdmin {

	private static final int MAX_ARTISTS_TO_SHOW = 5;

	private ArtistAdminController controller;

	private FlexTable artistArea;
	
	private AsyncFilterBox fb;

	private Map<Artist, ListBox> artistTitles = new HashMap<Artist, ListBox>();

	private Map<Artist, TextBoxWithValidation> artistEmails = new HashMap<Artist, TextBoxWithValidation>();

	private Map<Artist, TextBoxWithValidation> artistScreennames = new HashMap<Artist, TextBoxWithValidation>();

	private Map<Artist, TextBoxWithValidation> artistFirstNames = new HashMap<Artist, TextBoxWithValidation>();

	private Map<Artist, TextBoxWithValidation> artistLastNames = new HashMap<Artist, TextBoxWithValidation>();

	private Map<Artist, RichTextArea> artistDescriptions = new HashMap<Artist, RichTextArea>();

	private Map<Artist, RichTextArea> artistNotes = new HashMap<Artist, RichTextArea>();
	
	private HTML tooManyArtistsWarning;

	/**
	 */
	public ArtistAdmin(final AppWizard wizard){
		super(wizard);

		controller = new ArtistAdminController(this, wizard);

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(1);
		
		int r = 0;
		int c = 0;
		
		/***************************************************
		 * title
		 */
		String s = "<h2>Artist Administration</h2>";
		s += "Administrators can manipulate artists using this page.";
		s += "<br><br>Artists will have a similar page for managing specifically their own profiles.";
		s += "<br><br>Use the filter to search for specific artists. This screen will display up to " + MAX_ARTISTS_TO_SHOW + " artists. Each artist has its own form and 'update' button.";

		HTML l = UIFactory.createHTML(s);
		table.setWidget(r, c, l);
		r++; c = 0;
		
		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		r++; c = 0;

		/***************************************************
		 * filter input area
		 */
		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		fb = new AsyncFilterBox(
				"Filter",
				"Enter a regular expression to filter artists",
				"matches: ",
				"clear",
				"info",
				'f',
				200,
				1000
			){
			
			@Override
			protected void doServerCall(String filter) {
				controller.setArtists(filter);
			}
		};
		hp.add(fb);
		tooManyArtistsWarning = UIFactory.createHTML("Too many artists to display. Please use filter!");
		tooManyArtistsWarning.setStyleName(UIFactory.GENERAL_ERROR);
		tooManyArtistsWarning.setVisible(false); //default is off
		hp.add(tooManyArtistsWarning);
		table.setWidget(r, c, hp);
		r++; c = 0;

		/***************************************************
		 * artist area
		 */
		artistArea = UIFactory.createFlexTable();
		table.setWidget(r, c, artistArea);
		r++; c = 0;
		
		//tell the composite (this) which widget its wrapping
		initWidget(table);
	}
	
	@Override
	public void activated(){
	}

	/**
	 * @see IArtistAdmin#setArtists()
	 */
	public void setArtists(){
		fb.serverCallComplete(controller.getModel().getArtists().size());
		
		artistTitles.clear();
		artistEmails.clear();
		artistFirstNames.clear();
		artistLastNames.clear();
		artistDescriptions.clear();
		artistNotes.clear();

		artistArea.clear();
		int r = 0;
		int c = 0;
		if(controller.getModel().getArtists().isEmpty()){
			HTML html = UIFactory.createHTML("No artists found.");
			artistArea.setWidget(r, c, html);
		}else{
			//limit how much we show on screen, because of browser resources.
			if(controller.getModel().getArtists().size() > MAX_ARTISTS_TO_SHOW){
				tooManyArtistsWarning.setVisible(true);
			}else{
				tooManyArtistsWarning.setVisible(false);
			}

			int i = 0;
			for(Artist a : controller.getModel().getArtists()){
				if(i++ >= MAX_ARTISTS_TO_SHOW){
					break;
				}

				final Artist fa = a;
				FlexTable outerTable = UIFactory.createFlexTable();
				
				/*********************************************
				 * artist image
				 */
				VerticalPanel vp = UIFactory.createVerticalPanel();
				final Image img = UIFactory.createImage(a.getUrl());
				vp.add(img);

				VerticalPanel holder = UIFactory.createVerticalPanel();
				final FormPanel form = new FormPanel();
				form.setEncoding(FormPanel.ENCODING_MULTIPART);
				form.setMethod(FormPanel.METHOD_POST);
				holder.add(new Hidden("uid", String.valueOf(a.getUid())));
				final FileUpload updateImage = UIFactory.createFileUpload();
				updateImage.setName("data");
				holder.add(updateImage);
				Button go = UIFactory.createButton("Update Image");
				go.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						 form.submit();
					}
				});
				holder.add(go);
				form.add(holder);

				String url = Window.Location.getProtocol() + "//";
				url += Window.Location.getHost() + "/updateArtistImage.jsp";
				form.setAction(url);

				form.addSubmitHandler(new SubmitHandler() {
					public void onSubmit(SubmitEvent event) {
						//called just before the request is sent

						if(!updateImage.getFilename().toLowerCase().endsWith(".jpg")){
							UIFactory.createWarningDialog("Validation", 
									"File needs to be of type JPEG (with a '.jpg' file extension)."
									).center();
							event.cancel();
						}else{
							setWait(true);
							img.setUrl("/images/wait.gif");
						}
					}
				});
				form.addSubmitCompleteHandler(new SubmitCompleteHandler() {
					public void onSubmitComplete(SubmitCompleteEvent event) {
						//stop waiting, and reload, regardless - to show what we currently have in the system
						setWait(false);
						img.setUrl(fa.getUrl() + "?t=" + System.currentTimeMillis()); //to force reloading and avoiding the cache :-) TODO actually needed?
						if(!event.getResults().trim().equals("OK")){
							UIFactory.createErrorDialog("Error", event.getResults()).center();
						}
					}
				});
				if(getWizard().getModel().getSessionInfo().isInRole(Roles.BACKOFFICE)){
					vp.add(form);
				}
				
				/*********************************************
				 * help text
				 */
				HTML l = UIFactory.createHTML(
						"Manage artists information<br>" +
						"to the left.<br>" +
						"<br>" +
						"Use the 'update' button above<br>" +
						"to update the artists image.<br>" +
						"Regardless of the original<br>" +
						"size, a 150 pixel wide image<br>" +
						"will be generated.<br>" +
						"<br>" +
						"Note: if you update this image<br>" +
						"you may need to refresh the<br>" +
						"browser before you see the new<br>" +
						"image in the shop." +
						"<br><br>" +
						"The description is viewable to<br>" +
						"all users.<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"Notes on the other hand are<br>" +
						"viewable only to administrators.<br>" +
						"Use this field to make notes about<br>" +
						"your artists.<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"The designs table to the right<br>" +
						"shows this artists designs. Use<br>" +
						"its links to quickly edit specific<br>" +
						"designs." +
						"<br>" +
						"<br>" +
						"<br>" +
						"<br>" +
						"Use the 'update' button to the<br>" +
						"right to update the artists<br>" +
						"details.<br>" +
						"");
				l.setWordWrap(true);
				vp.add(l);
				
				outerTable.setWidget(0, 0, vp);
				outerTable.getFlexCellFormatter().setVerticalAlignment(0, 0, HasVerticalAlignment.ALIGN_TOP);
				
				/*********************************************
				 * form
				 */
				FlexTable table = UIFactory.createFlexTable();
				table.setBorderWidth(1);

				int r2 = 0;
				int c2 = 0;

				HTML html = UIFactory.createHTML();
				html.setStyleName(UIFactory.BIG_BOLD_TEXT);
				html.setHTML("ID: " + a.getUid());
				table.setWidget(r2, c2, html);
				r2++;

				/*********************************************
				 * title
				 */
				HorizontalPanel hp = UIFactory.createHorizontalPanel();
				l = UIFactory.createHTML("Title:");
				hp.add(l);
				
				ListBox title = UIFactory.createListBox();
				artistTitles.put(a, title);
				title.addItem("Mr");
				title.addItem("Ms");
				title.addItem("Mrs");
				if(a.getTitle() != null && a.getTitle().equalsIgnoreCase("Mr")) title.setSelectedIndex(0);
				if(a.getTitle() != null && a.getTitle().equalsIgnoreCase("Ms")) title.setSelectedIndex(1);
				if(a.getTitle() != null && a.getTitle().equalsIgnoreCase("Mrs")) title.setSelectedIndex(2);
				hp.add(title);

				table.setWidget(r2, c2, hp);
				r2++;

				/*********************************************
				 * email
				 */
				hp = UIFactory.createHorizontalPanel();
				l = UIFactory.createHTML("Email:");
				hp.add(l);
				
				TextBoxWithValidation email = UIFactory.createTextBoxWithValidation("Please enter a valid email address", true);
				email.setAssertEmail(true);
				artistEmails.put(a, email);
				email.setText(a.getEmail());
				hp.add(email);
				
				table.setWidget(r2, c2, hp);
				r2++;
				
				/*********************************************
				 * screenname
				 */
				hp = UIFactory.createHorizontalPanel();
				l = UIFactory.createHTML("Screen name:");
				hp.add(l);
				
				TextBoxWithValidation screenname = UIFactory.createTextBoxWithValidation("Please enter a screen name", true);
				screenname.setAssertTrimNotEmpty(true);
				artistScreennames.put(a, screenname);
				screenname.setText(a.getScreenname());
				hp.add(screenname);
				
				table.setWidget(r2, c2, hp);
				r2++;
				
				/*********************************************
				 * firstname
				 */
				hp = UIFactory.createHorizontalPanel();
				l = UIFactory.createHTML("Firstname:");
				hp.add(l);
				
				TextBoxWithValidation firstName = UIFactory.createTextBoxWithValidation("Please enter a first name", true);
				firstName.setAssertTrimNotEmpty(true);

				artistFirstNames.put(a, firstName);
				firstName.setText(a.getFirstname());
				hp.add(firstName);
				
				table.setWidget(r2, c2, hp);
				r2++;
				
				/*********************************************
				 * lastname
				 */
				hp = UIFactory.createHorizontalPanel();
				l = UIFactory.createHTML("Lastname:");
				hp.add(l);
				
				TextBoxWithValidation lastName = UIFactory.createTextBoxWithValidation("Please enter a last name", true);
				lastName.setAssertTrimNotEmpty(true);
				artistLastNames.put(a, lastName);
				lastName.setText(a.getLastname());
				hp.add(lastName);
				
				table.setWidget(r2, c2, hp);
				r2++;
				
				/*********************************************
				 * regDate
				 */
				l = UIFactory.createHTML("Registration date: " + ClientFormats.DTF_ddMMMyyyy.format(a.getRegistrationDate()));
				table.setWidget(r2, c2, l);
				r2++;
				
				/*********************************************
				 * description
				 */
				l = UIFactory.createHTML("Description:<br><small>" +
											"(the artists<br>" +
											"description of<br>" +
											"themselves)<small>");
				table.setWidget(r2, c2, l);
				r2++;

				RichTextArea artistDescription = UIFactory.createRichTextArea();
				artistDescription.setWidth("400px"); //important because otherwise validation warnings cause it to resize!
				artistDescriptions.put(a, artistDescription);

				SmallRichTextToolbar toolbar = UIFactory.createSmallRichTextToolbar(artistDescription);
				toolbar.setWidth("400px"); //important because otherwise validation warnings cause it to resize!
				table.setWidget(r2, c2, toolbar);
			    r2++;
			    
			    artistDescription.setHTML(a.getDescription());
				table.setWidget(r2, c2, artistDescription);
				r2++;

				/*********************************************
				 * notes
				 */
				l = UIFactory.createHTML("Notes:<br><small>" +
										"(your notes<br>" +
										"about this<br>" +
										"artist)</small>");
				table.setWidget(r2, c2, l);
				r2++;
				
				RichTextArea artistNote = UIFactory.createRichTextArea();
				artistNotes.put(a, artistNote);
				
				toolbar = UIFactory.createSmallRichTextToolbar(artistNote);
				table.setWidget(r2, c2, toolbar);
				r2++;
				
				artistNote.setHTML(a.getNotes());
				table.setWidget(r2, c2, artistNote);
				r2++;
				
				/*********************************************
				 * designs
				 */
				vp = new VerticalPanel();
				l = UIFactory.createHTML("Designs:");
				vp.add(l);

				//create table and add headers
				int r3 = 0;
				int c3 = 0;
				FlexTable designTable = UIFactory.createFlexTable();
				designTable.setWidth("100%");
				l = UIFactory.createHTML("&nbsp;"); //image
				designTable.getFlexCellFormatter().setStyleName(r3, c3, UIFactory.BIG_BOLD_TEXT);
				designTable.setWidget(r3, c3, l); c3++;
				l = UIFactory.createHTML("ID");
				designTable.getFlexCellFormatter().setStyleName(r3, c3, UIFactory.BIG_BOLD_TEXT);
				designTable.setWidget(r3, c3, l); c3++;
				l = UIFactory.createHTML("Name");
				designTable.getFlexCellFormatter().setStyleName(r3, c3, UIFactory.BIG_BOLD_TEXT);
				designTable.setWidget(r3, c3, l); c3++;
				l = UIFactory.createHTML("In catalogue?");
				designTable.getFlexCellFormatter().setStyleName(r3, c3, UIFactory.BIG_BOLD_TEXT);
				designTable.setWidget(r3, c3, l); c3++;
				l = UIFactory.createHTML("Approved?");
				designTable.getFlexCellFormatter().setStyleName(r3, c3, UIFactory.BIG_BOLD_TEXT);
				designTable.setWidget(r3, c3, l); c3++;
				l = UIFactory.createHTML("&nbsp;"); //view column
				designTable.getFlexCellFormatter().setStyleName(r3, c3, UIFactory.BIG_BOLD_TEXT);
				designTable.setWidget(r3, c3, l); c3++;

				designTable.getRowFormatter().setStyleName(r3, UIFactory.LIST_HEADER);

				r3++; c3 = 0;
				
				for(Design d : a.getDesigns()){

					/*****************
					 * image
					 */
					Image img2 = UIFactory.createImage(d.getUrl());
					img2.setSize("20px", "20px");
					designTable.setWidget(r3, c3, img2);
					c3++;
					
					/*****************
					 * id
					 */
					l = UIFactory.createHTML(String.valueOf(d.getUid()));
					designTable.setWidget(r3, c3, l);
					c3++;
					
					/*****************
					 * name
					 */
					l = UIFactory.createHTML(d.getName());
					designTable.setWidget(r3, c3, l);
					c3++;
					
					/*****************
					 * in catalogue
					 */
					CheckBox cb = UIFactory.createCheckBox("");
					cb.setValue(d.isInCatalogue());
					cb.setEnabled(false);
					designTable.setWidget(r3, c3, cb);
					c3++;
					
					/*****************
					 * approved
					 */
					cb = UIFactory.createCheckBox("");
					cb.setValue(d.isApproved());
					cb.setEnabled(false);
					designTable.setWidget(r3, c3, cb);
					c3++;
					
					/*****************
					 * edit link
					 */
					final HTML designLink = UIFactory.createHTMLLikeHyperlink("edit");
					designLink.setData(d);
					designLink.addClickHandler(new ClickHandler(){
						public void onClick(ClickEvent event) {
							Design d = (Design)designLink.getData();
							getWizard().showDesignAdminPage(d);
						}
					});
					designTable.setWidget(r3, c3, designLink);

					String style = UIFactory.LIST_ROW_EVEN_SELECTABLE;
					if(r3 % 2 != 0){
						style = UIFactory.LIST_ROW_ODD_SELECTABLE;
					}
					designTable.getRowFormatter().setStyleName(r3, style);

					r3++; c3 = 0;
				}
				ScrollPanel sp = new ScrollPanel();
				sp.setHeight("100px");
				sp.setWidget(designTable);
				vp.add(sp);
				table.setWidget(r2, c2, vp);
				r2++;

				/*********************************************
				 * update button
				 */
				if(getWizard().getModel().getSessionInfo().isInRole(Roles.BACKOFFICE)){
					Button b = UIFactory.createButton("Update");
					table.setWidget(r2, c2, b);
					b.addClickHandler(new ClickHandler(){
						public void onClick(ClickEvent event) {
							validateArtistForUpdate(fa);
						}
					});
					r2++;
				}
				
				outerTable.setWidget(0, 1, table);

				artistArea.setWidget(r, c, outerTable);
				r++; c = 0;

				l = UIFactory.createHTML("<hr>");
				artistArea.setWidget(r, c, l);
				artistArea.getFlexCellFormatter().setColSpan(r, c, 2);
				r++; c = 0;
			}
		}
	}

	protected void validateArtistForUpdate(Artist a ) {
		//TODO put this into a validator which throws validation exceptions

		boolean valid = true;

		ListBox title = artistTitles.get(a);
		String newTitle = title.getItemText(title.getSelectedIndex());

		TextBoxWithValidation firstName = artistFirstNames.get(a);
		TextBoxWithValidation lastName = artistLastNames.get(a);
		TextBoxWithValidation screenName = artistScreennames.get(a);
		TextBoxWithValidation email = artistEmails.get(a);
		String description = null;
		String notes = null;
		
		if(firstName.validate() &&
				lastName.validate() &&
				screenName.validate() &&
				email.validate()){

			RichTextArea rta = artistDescriptions.get(a);
			description = rta.getHTML().trim();
			if(description.length() == 0){
				UIFactory.createWarningDialog("Validation", "Description may not be empty.").center();
				rta.setFocus(true);
				valid = false;
			}else{
				rta = artistNotes.get(a);
				notes = rta.getHTML().trim();
				//notes may be empty, so no validation required.
				
			}
		}else{
			valid = false;
		}

		if(valid){

			controller.updateArtist(
					a, 
					newTitle, 
					email.getTrimmedText(), 
					screenName.getTrimmedText(), 
					firstName.getTrimmedText(), 
					lastName.getTrimmedText(), 
					description, 
					notes, 
					fb.getInputText());
		}else{
			UIFactory.createWarningDialog("Validation", "Something is still not valid. Please correct, and try again.").center();
		}
	}

	public void showArist(Artist a) {
		
		//clear the area since it may show irrelevant artists.
		artistArea.clear();
		
		String s = "";
		if(a != null){
			s = String.valueOf(a.getUid());
		}
		fb.setInputText(s, true);
	}
	
}
