/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.artistprofile.ui;

import java.util.List;

import uk.co.maxant.gwtdemo.client.artistprofile.core.ArtistProfileController;
import uk.co.maxant.gwtdemo.client.artistprofile.core.IArtistProfile;
import uk.co.maxant.gwtdemo.client.generic.widgets.Button;
import uk.co.maxant.gwtdemo.client.generic.widgets.Dialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.generic.widgets.WarningDialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.AbstractInputDialog.Callback;
import uk.co.maxant.gwtdemo.client.images.Images;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;
import uk.co.maxant.gwtdemo.client.widgets.ui.richtextarea.SmallRichTextToolbar;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.FileUpload;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.RichTextArea;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteEvent;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteHandler;
import com.google.gwt.user.client.ui.FormPanel.SubmitEvent;
import com.google.gwt.user.client.ui.FormPanel.SubmitHandler;

public class ArtistProfile extends WizardPage implements IArtistProfile, SubmitCompleteHandler {

	private ArtistProfileController controller;

	private NewDesignDialog ndd;
	
	private FlexTable infoTable;
	
	private RichTextArea artistsDescription;

	private FlexTable bottom;

	public ArtistProfile(final AppWizard wizard){
		super(wizard);
		
		this.controller = new ArtistProfileController(this, wizard);
		
		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(0);

		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		hp.setVerticalAlignment(HasVerticalAlignment.ALIGN_TOP);

		HTML header = UIFactory.createHTML("Your profile as an artist");
		header.setStyleName(UIFactory.BIG_BOLD_TEXT);
		hp.add(header);

		int r = 0;
		int c = 0;
		table.setWidget(r, c, hp);
		r++; c = 0;

		HTML lbl = UIFactory.createHTML("To edit your profile, please edit this form and submit it:");
		table.setWidget(r, c, lbl);
		r++; c = 0;

		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		r++; c = 0;

		/***************************************************
		 * main table
		 */
		infoTable = UIFactory.createFlexTable();
		infoTable.setCellSpacing(3);
		infoTable.setCellPadding(0);
		infoTable.setBorderWidth(0);
		table.setWidget(r, c, infoTable);
		c++;

		initWidget(table);
	}
	
	public void setArtist(final Artist a) {
		
		FlexTable left = UIFactory.createFlexTable();
		left.setCellSpacing(3);
		left.setCellPadding(0);
		left.setBorderWidth(0);
		infoTable.setWidget(0, 0, left);
		
		int r = 0;
		int c = 0;

		HTML lbl = UIFactory.createHTML("ID");
		left.setWidget(r, c, lbl);
		left.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		lbl = UIFactory.createHTML(String.valueOf(a.getUid()));
		left.setWidget(r, c, lbl);
		r++; c = 0;

		/////////////////
		//ARTIST DESC
		/////////////////
		final HTML artistDescLabel = UIFactory.createHTML(
				"A description of<br>" +
				"you as an artist");
		left.setWidget(r, c, artistDescLabel);
		left.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		
		artistsDescription = UIFactory.createRichTextArea();
		artistsDescription.setWidth("400px"); 
		artistsDescription.setHTML(a.getDescription());

		final SmallRichTextToolbar toolbar = UIFactory.createSmallRichTextToolbar(artistsDescription);
		toolbar.setWidth("127px");
		left.setWidget(r, c, toolbar);
		r++; c = 0;

		lbl = UIFactory.createHTML("&nbsp;");
		left.setWidget(r, c, lbl);
		left.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		left.setWidget(r, c, artistsDescription);
		r++; c = 0;


		Button update = UIFactory.createButton("Update Profile");
		update.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				update();
			}
		});
		left.setWidget(r, c, update);
		left.getFlexCellFormatter().setColSpan(r, c, 2);
		left.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_CENTER);
		r++; c = 0;

		/*********************************************
		 * artist image
		 */
		final VerticalPanel artistImageVp = UIFactory.createVerticalPanel();
		final Image img = UIFactory.createImage(a.getUrl());
		artistImageVp.add(img);

		VerticalPanel holder = UIFactory.createVerticalPanel();
		final FormPanel form = new FormPanel();
		form.setEncoding(FormPanel.ENCODING_MULTIPART);
		form.setMethod(FormPanel.METHOD_POST);
		final FileUpload updateImage = UIFactory.createFileUpload();
		updateImage.setName("data");
		holder.add(updateImage);
		Button go = UIFactory.createButton("Add/Update Image");
		go.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				 form.submit();
			}
		});
		holder.add(go);
		form.add(holder);

		String url = Window.Location.getProtocol() + "//";
		url += Window.Location.getHost() + "/updateArtistImage.jsp";
		form.setAction(url);

		form.addSubmitHandler(new SubmitHandler() {
			public void onSubmit(SubmitEvent event) {
				//called just before the request is sent

				if(!updateImage.getFilename().toLowerCase().endsWith(".jpg")){
					UIFactory.createWarningDialog("Validation", 
							"File needs to be of type JPEG (with a '.jpg' file extension)."
							).center();
					event.cancel();
				}else{
					setWait(true);
					img.setUrl("/images/wait.gif");
				}
			}
		});
		form.addSubmitCompleteHandler(new SubmitCompleteHandler() {
			public void onSubmitComplete(SubmitCompleteEvent event) {
				//stop waiting, and reload, regardless - to show what we currently have in the system
				setWait(false);
				img.setUrl(a.getUrl() + "?t=" + System.currentTimeMillis()); //to force reloading and avoiding the cache :-) TODO actually needed?
				if(!event.getResults().trim().equals("OK")){
					UIFactory.createErrorDialog("Error", event.getResults()).center();
				}
			}
		});
		artistImageVp.add(form);
		infoTable.setWidget(0, 1, artistImageVp);

		/*********************************************
		 * middle
		 */
		VerticalPanel vp = UIFactory.createVerticalPanel();
		HTML title = UIFactory.createHTML("Your Designs");
		title.setStyleName(UIFactory.BIG_BOLD_TEXT);
		vp.add(title);
		HTML createNewLink = UIFactory.createHTMLLikeHyperlink("New design...");
		createNewLink.addClickHandler(new ClickHandler() {

			public void onClick(ClickEvent event) {
				ndd = new NewDesignDialog(
						ArtistProfile.this,
						new Callback<Design>() {
					public void onOK(Design d) {
						DOM.setStyleAttribute(RootPanel.get().getElement(), "cursor", "wait");
						
						//great - lets call the service and when we have the uid we can upload the form
						controller.createNewDesign(d);
					}
					public void onCancel() {
					}
				});
				ndd.center();
			}
		});
		vp.add(createNewLink);
		infoTable.setWidget(1, 0, vp);
		infoTable.getFlexCellFormatter().setColSpan(1, 0, 2);

		/*********************************************
		 * design Table
		 */
		bottom = UIFactory.createBorderedFlexTable();
		infoTable.setWidget(2, 0, bottom);
		infoTable.getFlexCellFormatter().setColSpan(2, 0, 2);
		bottom.setCellSpacing(3);
		bottom.setCellPadding(0);
		bottom.setBorderWidth(0);

		updateDesignList(a.getDesigns());
	}

	/**
	 * {@inheritDoc}
	 */
	public void updateDesignList(List<Design> designs) {
		bottom.clear();
		int r = 0; int c = 0;
		
		if(designs.isEmpty()){
			HTML l = UIFactory.createHTML("You have no designs. Please add one.");
			bottom.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			bottom.setWidget(r, c, l); c++;
		}else{
			HTML l = UIFactory.createHTML("&nbsp"/*image*/);
			bottom.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			bottom.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("ID");
			bottom.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			bottom.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("Name");
			bottom.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			bottom.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("Description");
			bottom.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			bottom.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("In catalogue?");
			bottom.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			bottom.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("Approved?");
			bottom.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			bottom.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("&nbsp;");
			bottom.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			bottom.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("&nbsp;");
			bottom.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			bottom.setWidget(r, c, l); c++;
			
			r++; c = 0;
			
			for(Design d : designs){
				final Design fd = d;
				String url = d.getUrl();
				if(!d.isImageExists()){
					url = "/images/designs/design_really_small_unknown.jpg";
				}
				Image img = UIFactory.createImage(url);
				img.setSize("40px", "40px");
				bottom.setWidget(r, c, img);
				c++;
				
				l = UIFactory.createHTML(String.valueOf(d.getUid()));
				bottom.setWidget(r, c, l);
				c++;
				
				l = UIFactory.createHTML(d.getName());
				bottom.setWidget(r, c, l);
				c++;
				
				l = UIFactory.createHTML(d.getDescription());
				bottom.getFlexCellFormatter().setWordWrap(r, c, true);
				bottom.setWidget(r, c, l);
				c++;
				
				com.google.gwt.user.client.ui.Image inCat = null;
				if(d.isInCatalogue()){
					inCat = UIFactory.createImage(Images.images.payment_ok().createImage());
				}else{
					inCat = UIFactory.createImage(Images.images.payment_nok().createImage());
				}
				bottom.setWidget(r, c, inCat);
				bottom.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_CENTER);
				c++;
				
				com.google.gwt.user.client.ui.Image approved = null;
				if(d.isApproved()){
					approved = UIFactory.createImage(Images.images.ok().createImage());
				}else{
					approved = UIFactory.createImage(Images.images.nok().createImage());
				}
				bottom.setWidget(r, c, approved);
				bottom.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_CENTER);
				c++;
				
				l = UIFactory.createHTMLLikeHyperlink("details");
				l.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						getWizard().showDesignDetailPage(fd);
					}
				});
				bottom.setWidget(r, c, l);
				c++;
				
				l = UIFactory.createHTMLLikeHyperlink("shop");
				l.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						controller.showDesignInShop(fd);
					}
				});
				bottom.setWidget(r, c, l);
				c++;
				
				/*********************************************
				 * row styling - set colour, then add mouseOver/mouseOut styles.
				 * also react to clicks.
				 */
				String style = UIFactory.LIST_ROW_EVEN;
				if(r % 2 != 0){
					style = UIFactory.LIST_ROW_ODD;
				}
				bottom.getRowFormatter().setStyleName(r, style);
				
				r++; c = 0;
			}
		}
	}

	/**
	 * user clicked button to udpate
	 */
	private void update(){
		if(artistsDescription.getHTML().trim().length() > 0){
			Artist art = new Artist();
			art.setDescription(artistsDescription.getHTML());
			controller.update(art);
		}else{
			UIFactory.createWarningDialog("Validation", "Please provide a description of you as an artist.").center();
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void userUpdated() {
		UIFactory.createInformationDialog(
				"Thanks", 
				"Thank you, you have successfully updated your artist profile.", 
			new Dialog.Callback() {
				public void onClose() {
					getWizard().showArtistProfilePage();
				}
			}
		).center();
	}

	/**
	 * called by say the screen wizard to activate this page. its a hook whereby we have a chance to
	 * set contents of this page when its activated.
	 */
	@Override
	public void activated() {
		controller.getProfile();
	}

	/**
	 * once the design is added, this method is called.
	 */
	public void designAdded(Integer newDesignUid) {
		//the design has been created on the server
		//now upload the file...
		controller.setNewDesignId(newDesignUid);
		ndd.submitForm(newDesignUid);
	}

	/**
	 * called right at the end of submitting a new design, when the file upload form completes.
	 */
	public void onSubmitComplete(SubmitCompleteEvent event) {
		ndd.hide();
		DOM.setStyleAttribute(RootPanel.get().getElement(), "cursor", null); 
		
		if(!event.getResults().contains("OK")){
			//error!
			new WarningDialog("Warning", 
						"Failed to upload your design. The design has been created with ID " + 
						controller.getModel().getNewDesignId() + " but uploading of your design file has failed. " +
						"Please get in touch with us and send it via email."
				).center();
		}else{
			//the form was submitted
			UIFactory.createInformationDialog(
					"Design " + controller.getModel().getNewDesignId() + " has been submitted", 
					"The new design has been submitted for checks. Once these checks have been carried out " +
					"and the design has been tested, it will become available for sale in the shop.",
					new Dialog.Callback() {
						public void onClose() {
							getWizard().showArtistProfilePage();
						}
					}
			).center();
		}

		//refresh the page, now that we have a new design in it
		controller.getProfile();
	}	
	
}
