/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.artistprofile.ui;

import uk.co.maxant.gwtdemo.client.generic.widgets.AbstractInputDialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.TagListBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.widgets.ui.richtextarea.SmallRichTextToolbar;

import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.FileUpload;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Hidden;
import com.google.gwt.user.client.ui.RichTextArea;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteHandler;

/**
 * used for getting the name and description of a new design 
 * or potentially artist (altho not used for that today).
 */
public class NewDesignDialog extends AbstractInputDialog<Design> {
	
	private TextBox name;
	private RichTextArea description;
	private TagListBox tags;
	private FormPanel form;
	private Hidden uid;
	private FileUpload upload;
	
	public NewDesignDialog(SubmitCompleteHandler submitCompleteHandler, Callback<Design> callback) {
		super(
				"New Design", 
				"Please enter details of the new design:", 
				null /*validation done explicitly below*/, 
				callback);

		form.addSubmitCompleteHandler(submitCompleteHandler);

		//client needs to hide it, once they submit the 
		//form, otherwise the form is disposed and the file aint uploaded!
		setHideOnComplete(false); 
	}
	
	@Override
	protected Widget createContents() {
		int r = 0;
		int c = 0;

		FlexTable ft = UIFactory.createFlexTable();

		////////////////////////////////
		//name
		////////////////////////////////
		HTML l = UIFactory.createHTML("Name");
		ft.setWidget(r, c, l);
		ft.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		
		name = UIFactory.createTextBox();
		ft.setWidget(r, c, name);
		ft.getFlexCellFormatter().setColSpan(r, c, 3);
		r++; c = 0;

		////////////////////////////////
		//description
		////////////////////////////////
		l = UIFactory.createHTML("Description");
		ft.setWidget(r, c, l);
		ft.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;

		description = UIFactory.createRichTextArea();
		description.setWidth("500px");

		final SmallRichTextToolbar toolbar = UIFactory.createSmallRichTextToolbar(description);
		toolbar.setWidth("127px");
		ft.setWidget(r, c, toolbar);
		ft.getFlexCellFormatter().setColSpan(r, c, 3);
		r++; c = 0;
		
		l = UIFactory.createHTML("&nbsp;");
		ft.setWidget(r, c, l);
		c++;
		ft.setWidget(r, c, description);
		ft.getFlexCellFormatter().setColSpan(r, c, 3);

		r++; c = 0;

		////////////////////////////////
		//tags
		////////////////////////////////
		l = UIFactory.createHTML(
				"Design Tags<br>" +
				"<small>(used for searches)</small>"); //TODO make this a disclosure panel
		ft.setWidget(r, c, l);
		ft.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		
		tags = UIFactory.createTagListBox(
				"add", 
				"remove",
				"", //no title needed since we have the description to the left
				"New Tag",
				"Please add a tag to help customers search for this design", 
				"Tag may not be empty");
		ft.setWidget(r, c, tags);
		c++;

		////////////////////////////////
		//upload
		////////////////////////////////
		l = UIFactory.createHTML("Design Upload"); //TODO make this a disclosure panel
		ft.setWidget(r, c, l);
		ft.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		
		VerticalPanel holder = UIFactory.createVerticalPanel();
		form = new FormPanel();
		form.setEncoding(FormPanel.ENCODING_MULTIPART);
		form.setMethod(FormPanel.METHOD_POST);
		upload = UIFactory.createFileUpload();
		upload.setName("data");
		holder.add(upload);
		uid = new Hidden("uid");
		holder.add(uid);
		form.add(holder);
		
		String url = Window.Location.getProtocol() + "//";
		url += Window.Location.getHost() + "/artist/uploadNewDesign.jsp";
		form.setAction(url);

		ft.setWidget(r, c, form);
		r++; c = 0;

		return ft;
	}

	/**
	 * {@inheritDoc}
	 */
	protected boolean validate(){
		if(name.getText().trim().length() == 0){
			validationError.setHTML("Name may not be empty!");
			name.setFocus(true);
			return false;
		}else if(description.getHTML().trim().length() == 0){
			validationError.setHTML("Description may not be empty!");
			description.setFocus(true);
			return false;
		}else if(upload.getFilename().trim().length() == 0){
			validationError.setHTML("Filename may not be empty!");
			return false;
		}else{
			data = new Design();
			data.setName(name.getText().trim());
			data.setDescription(description.getHTML().trim());
			data.setTags(tags.getContents());
			return true;
		}
	}

	/**
	 * gives access to send the form containing the file 
	 * upload, so that the file upload can be carried out.
	 */
	public void submitForm(Integer newDesignId) {
		uid.setValue(String.valueOf(newDesignId));
		form.submit();
	}
	
	@Override
	public void setFocus(boolean focused) {
		name.setFocus(focused);
	}

}
