/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.checkout.ui;

import java.util.List;

import org.gwt.advanced.client.ui.widget.border.RoundCornerBorder;

import uk.co.maxant.gwtdemo.client.checkout.core.CheckoutController;
import uk.co.maxant.gwtdemo.client.checkout.core.ICheckout;
import uk.co.maxant.gwtdemo.client.generic.widgets.Button;
import uk.co.maxant.gwtdemo.client.generic.widgets.Dialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.TextBoxWithValidation;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.to.BasketItem;
import uk.co.maxant.gwtdemo.client.util.ClientFormats;
import uk.co.maxant.gwtdemo.client.widgets.core.MasterDataListener;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

public class Checkout extends WizardPage implements ICheckout, MasterDataListener {

	private CheckoutController controller;
	
	private FlexTable basketContents;

	private FlexTable addressTable;
	
	private TextBoxWithValidation email;
	private TextBoxWithValidation name;
	private TextBoxWithValidation street1;
	private TextBoxWithValidation street2;
	private TextBoxWithValidation city;
	private TextBoxWithValidation state;
	private TextBoxWithValidation zip;
	private TextBoxWithValidation country;

	private HTML totalLabel;

	private HorizontalPanel loginLink;

	public Checkout(final AppWizard wizard){
		super(wizard);
		
		this.controller = new CheckoutController(this, wizard);
		
		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(5);
		table.setCellPadding(0);
		table.setBorderWidth(0);

		Image headerImage = UIFactory.createImage("/images/stage2banner.jpg");
		
		int r = 0;
		int c = 0;
		table.setWidget(r, c, headerImage);
		r++; c = 0;

		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		r++; c = 0;

		/***************************************************
		 * address table
		 */
		addressTable = UIFactory.createFlexTable();
		addressTable.setCellSpacing(3);
		addressTable.setCellPadding(0);
		addressTable.setBorderWidth(0);
		RoundCornerBorder border = UIFactory.createRoundCornerBorder();
		border.setWidget(addressTable);
		table.setWidget(r, c, border);
		c++;

		/***************************************************
		 * basket table
		 */
		VerticalPanel vp = UIFactory.createVerticalPanel();
		totalLabel = UIFactory.createHTML();
		vp.add(totalLabel);
		ScrollPanel sp = UIFactory.createScrollPanel();
		vp.add(sp);
		sp.setHeight("200px");
		sp.setWidth("350px");
		basketContents = UIFactory.createFlexTable();
		sp.add(basketContents);
		table.setWidget(r, c, vp);
		r++; c = 0;
		
		buildAddressTable();

		/***************************************************
		 * back button
		 */
		Button back = UIFactory.createButton("back");
		back.addClickHandler(new ClickHandler(){
			public void onClick(ClickEvent event) {
				wizard.showPreviousPage();
			}
		});
		table.setWidget(r, c, back);
		c++;
		
		/***************************************************
		 * next button
		 */
		Image button = UIFactory.createImageButton("/images/ui/nextUp.jpg", "/images/ui/nextDown.jpg");
		button.addClickHandler(new ClickHandler(){
			public void onClick(ClickEvent event) {
				completeOrder();
			}
		});
		table.setWidget(r, c, button);
		r++; c = 0;
		
		initWidget(table);
		
		wizard.addMasterDataListener(this);
	}
	
	private void buildAddressTable() {
		int r = 0;
		int c = 0;

		loginLink = UIFactory.createHorizontalPanel();
		HTML lbl = UIFactory.createHTML("Already registered? Then");
		loginLink.add(lbl);
		lbl = UIFactory.createHTMLLikeHyperlink("login");
		lbl.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				Window.Location.assign("/secure/index.jsp?screen=2"); //goto checkout screen after logging in
			}
		});
		loginLink.add(lbl);
		lbl = UIFactory.createHTML("to use old delivery addresses.");
		loginLink.add(lbl);
		addressTable.setWidget(r, c, loginLink);
		addressTable.getFlexCellFormatter().setColSpan(r, c, 3);
		r++; c = 0;

		lbl = UIFactory.createHTML("Email");
		addressTable.setWidget(r, c, lbl);
		addressTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		email = UIFactory.createTextBoxWithValidation("Invalid email address", true);
		email.setAssertEmail(true);
		addressTable.setWidget(r, c, email);
		r++; c = 0;

		lbl = UIFactory.createHTML("Name");
		addressTable.setWidget(r, c, lbl);
		addressTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		name = UIFactory.createTextBoxWithValidation("Please enter your full name", true);
		name.setAssertTrimNotEmpty(true);
		addressTable.setWidget(r, c, name);
		r++; c = 0;

		lbl = UIFactory.createHTML("Street");
		addressTable.setWidget(r, c, lbl);
		addressTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		street1 = UIFactory.createTextBoxWithValidation("Please enter the first line of your address", true);
		street1.setAssertTrimNotEmpty(true);
		addressTable.setWidget(r, c, street1);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("&nbsp;");
		addressTable.setWidget(r, c, lbl);
		addressTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		street2 = UIFactory.createTextBoxWithValidation("Please enter the second line of your address", true);
		//street2 has no validation at the mo!
		addressTable.setWidget(r, c, street2);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("City");
		addressTable.setWidget(r, c, lbl);
		addressTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		city = UIFactory.createTextBoxWithValidation("Please enter your city", true);
		city.setAssertTrimNotEmpty(true);
		addressTable.setWidget(r, c, city);
		r++; c = 0;

		lbl = UIFactory.createHTML("State");
		addressTable.setWidget(r, c, lbl);
		addressTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		state = UIFactory.createTextBoxWithValidation("Please enter your state or county", true);
		state.setAssertTrimNotEmpty(true);
		addressTable.setWidget(r, c, state);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Postcode");
		addressTable.setWidget(r, c, lbl);
		addressTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		zip = UIFactory.createTextBoxWithValidation("Please enter your ZIP or post code", true);
		zip.setAssertTrimNotEmpty(true);
		addressTable.setWidget(r, c, zip);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Country");
		addressTable.setWidget(r, c, lbl);
		addressTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		country = UIFactory.createTextBoxWithValidation("Please enter your country", true);
		country.setAssertTrimNotEmpty(true);
		addressTable.setWidget(r, c, country);
		r++; c = 0;
	}

	/**
	 * user clicked button to go to checkout
	 */
	private void completeOrder(){

		if(email.validate() &&
				name.validate() &&
				street1.validate() &&
				street2.validate() &&
				city.validate() &&
				state.validate() &&
				zip.validate() &&
				country.validate()){

			//thats all, lets do it!
			
			/*
			 * this GWT demo doesnt include addresses, but it could!
			 */
			
			controller.createOrder(email.getText(), name.getText());
		}else{
			UIFactory.createWarningDialog("Validation", "Something is still not valid. Please correct, and try again.").center();
		}
	}

	/**
	 * once the order is created, this method is called to let us know what orderUid it has.
	 * with it, the wizard can move to the next page.
	 */
	public void orderCreated(Integer orderUid) {
		getWizard().getModel().setOrderUid(orderUid);
		getWizard().showNextPage();
	}

	/**
	 * called by say the screen wizard to activate this page. its a hook whereby we have a chance to
	 * set contents of this page when its activated.
	 */
	@Override
	public void activated() {
		
		//display login link if required
		loginLink.setVisible(false);
		if(getWizard().getModel().getSessionInfo() != null){
			if(!getWizard().getModel().getSessionInfo().isLoggedIn()){
				loginLink.setVisible(true);
			}
		}
		
		//might be able to use cached content, if the user is already logged in. if they were not logged
		//in, then the basket will be null. so we need to go get it from the server.
		if(getWizard().getModel().getBasket() == null){
			controller.getBasket();
		}else{
			updateBasket(getWizard().getModel().getBasket());
		}
	}
	
	/**
	 * caller wants to set the basket contents as given
	 */
	@Override
	public void updateBasket(List<BasketItem> basketItems) {
		
		if(basketItems.isEmpty() && isActive()){
			//nothing the basket, so lets go back to the shop
			UIFactory.createWarningDialog("Basket empty", "Your basket is empty, please place something in it before coming to the checkout.", new Dialog.Callback(){
				public void onClose() {
					getWizard().showPreviousPage();
				}
			}).center();
		}
		
		basketContents.clear();
		basketContents.setStyleName("listTable");
		int r = 0;
		int c = 0;
		HTML l = UIFactory.createHTML(String.valueOf("Design ID"));
		l.setStyleName("listHeader");
		basketContents.setWidget(r, c++, l);
		l = UIFactory.createHTML(String.valueOf("Name"));
		l.setStyleName("listHeader");
		basketContents.setWidget(r, c++, l);
		l = UIFactory.createHTML(String.valueOf("Price"));
		l.setStyleName("listHeader");
		basketContents.setWidget(r, c++, l);
		r++; c = 0;
		double total = 0.0;
		for(BasketItem bi : basketItems){
			String s = "listRowOdd";
			if(r % 2 == 0){
				s = "listRowEven";
			}
			l = UIFactory.createHTML(String.valueOf(bi.getDesign().getUid()));
			l.setStyleName(s);
			basketContents.setWidget(r, c++, l);
			l = UIFactory.createHTML(String.valueOf(bi.getDesign().getName()));
			l.setStyleName(s);
			basketContents.setWidget(r, c++, l);
			l = UIFactory.createHTML(ClientFormats.NF_0_00.format(bi.getDesign().getBasePrice()));
			l.setStyleName(s);
			basketContents.setWidget(r, c++, l);
			r++; c = 0;
			total += bi.getDesign().getBasePrice();
		}
		
		totalLabel.setHTML("Basket: " + ClientFormats.NF_0_00.format(total) + " UKP:"); //TODO get currency from config
	}

	/**
	 * @see MasterDataListener#notifyMasterDataIsReady()
	 */
	public void notifyMasterDataIsReady() {
		//prepopulate based on logged in user
		if(getWizard().getModel().getSessionInfo() != null){
			//email
			if(getWizard().getModel().getSessionInfo().getLogin() != null){
				email.setText(getWizard().getModel().getSessionInfo().getLogin());
			}
			
			//name
			name.setText(getWizard().getModel().getSessionInfo().getFullname());
		}

		//focus on first field needing info
		if(email.getText().trim().length() == 0){
			email.setFocus(true);
		}else if(name.getText().trim().length() == 0){
			name.setFocus(true);
		}else{
			street1.setFocus(true);
		}
	}
}
