/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.design_admin.core;

import java.util.List;

import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

public class DesignAdminController {

	protected IDesignAdmin view;
	protected DesignAdminServerCaller serverCaller;
	protected DesignAdminModel model;
	
	public DesignAdminController(IDesignAdmin view, AppWizard wizard){
		this.view = view;
		this.model = new DesignAdminModel();
		this.serverCaller = new DesignAdminServerCaller(view, model, wizard);
	}
	
	/**
	 * causes the relevant designs to be loaded.
	 * @param filter filter to be used in selecting relevant designs
	 */
	public void setDesigns(String filter) {
		serverCaller.getDesigns(filter);
	}

	public DesignAdminModel getModel() {
		return model;
	}

	/**
	 * updates/adds the given design and refreshes the page contents with the given filter.
	 */
	public void updateDesign(boolean createCopy, Design d, String newName, double newBasePrice, String newDescription,
			boolean inCatalogue, boolean approved, List<String> tags, String filter) {

		d.setName(newName);
		d.setBasePrice(newBasePrice);
		d.setDescription(newDescription);
		d.setTags(tags);
		d.setInCatalogue(inCatalogue);
		d.setApproved(approved);
		
		if(createCopy){
			d.setUid(null);
		}

		serverCaller.updateDesign(d, filter);
	}

}
