/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.design_admin.core;


import java.util.List;

import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.util.AsyncCallbackImpl;
import uk.co.maxant.gwtdemo.client.widgets.core.WizardServerCaller;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * delegated to by controller
 */
public class DesignAdminServerCaller extends WizardServerCaller {

	private IDesignAdmin designAdmin;
	private DesignAdminModel model;
	
	public DesignAdminServerCaller(IDesignAdmin designAdmin, DesignAdminModel model, AppWizard wizard){
		super(wizard, designAdmin);
		this.designAdmin = designAdmin;
		this.model = model;
	}
	
	/**
	 * called to set the designs
	 * 
	 * @param filter the filter to use
	 */
	public void getDesigns(String filter) {
		
		AsyncCallback<List<Design>> callback = new AsyncCallbackImpl<List<Design>>(this) {
			
			@Override
			protected void onFailurePost(Throwable caught) {
				designAdmin.setDesignsFailed();
			}

			public void onSuccess_(List<Design> designs) {
				model.setDesigns(designs);
				designAdmin.setDesigns();
			}
		};
		callServer();
		getServiceClient().getDesignsAdmin(filter, callback);
	}

	public void updateDesign(final Design d, final String filter) {

		//clear any old data that is being cached
		wizard.getModel().clear();
		
		AsyncCallback<Integer> callback = new AsyncCallbackImpl<Integer>(this) {
			
			public void onSuccess_(Integer uid) {
				//now go reload based on original filter.
				getDesigns(String.valueOf(uid));
			}
		};
		callServer();
		getServiceClient().saveOrUpdateDesign(d, callback);
		
	}

}

