/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.design_admin.ui;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import uk.co.maxant.gwtdemo.client.Roles;
import uk.co.maxant.gwtdemo.client.design_admin.core.DesignAdminController;
import uk.co.maxant.gwtdemo.client.design_admin.core.IDesignAdmin;
import uk.co.maxant.gwtdemo.client.generic.widgets.AsyncFilterBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.Button;
import uk.co.maxant.gwtdemo.client.generic.widgets.CheckBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.ListBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.TagListBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.TextBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.util.ClientFormats;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;
import uk.co.maxant.gwtdemo.client.widgets.ui.richtextarea.SmallRichTextToolbar;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.FileUpload;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.Hidden;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.RichTextArea;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteEvent;
import com.google.gwt.user.client.ui.FormPanel.SubmitCompleteHandler;
import com.google.gwt.user.client.ui.FormPanel.SubmitEvent;
import com.google.gwt.user.client.ui.FormPanel.SubmitHandler;

/**
 * the design admin screen, which lets admin manage designs.
 * 
 */
public class DesignAdmin extends WizardPage implements IDesignAdmin {

	private static final int MAX_DESIGNS_TO_SHOW = 5;

	private DesignAdminController controller;

	private FlexTable designArea;
	
	private AsyncFilterBox fb;

	private Map<Design, TextBox> designNames = new HashMap<Design, TextBox>();

	private Map<Design, TextBox> designBasePrices = new HashMap<Design, TextBox>();

	private Map<Design, ListBox> designArtists = new HashMap<Design, ListBox>();

	private Map<Design, CheckBox> designInCatalogues = new HashMap<Design, CheckBox>();

	private Map<Design, CheckBox> designApproveds = new HashMap<Design, CheckBox>();

	private Map<Design, RichTextArea> designDescriptions = new HashMap<Design, RichTextArea>();

	private Map<Design, TagListBox> tags = new HashMap<Design, TagListBox>();

	private HTML tooManyDesignsWarning;

	/**
	 */
	public DesignAdmin(final AppWizard wizard){
		super(wizard);

		controller = new DesignAdminController(this, wizard);

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(1);
		
		int r = 0;
		int c = 0;
		
		/***************************************************
		 * title
		 */
		String s = "<h2>Design Administration</h2>";
		s += "Administrators can manipulate designs using this page.";
		s += "<br><br>Artists will have a similar page for managing specifically their own designs.";
		s += "<br><br>Use the filter to search for specific designs. This screen will display up to " + MAX_DESIGNS_TO_SHOW + " designs. Each design has its own form and 'update' button.";
		s += "<br><br>There are three criteria which decide if a design is actually displayed in the " +
				"shop. The first criteria is whether the designs is in the 'catalogue'. That " +
				"checkbox can be used to quickly make designs unavailable if for example stock runs out. " +
				"The artist may also remove designs from the catalogue if they no longer want them " +
				"to be sellable. The second criteria is whether the design has been approved. " +
				"Initially, designs are not approved. Only administrators may approve designs.";

		HTML l = UIFactory.createHTML(s);
		table.setWidget(r, c, l);
		r++; c = 0;
		
		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		r++; c = 0;

		/***************************************************
		 * filter input area
		 */
		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		fb = new AsyncFilterBox(
				"Filter",
				"Enter a regular expression to filter designs",
				"matches: ",
				"clear",
				"info",
				'f',
				200,
				500
			){
			
			@Override
			protected void doServerCall(String filter) {
				controller.setDesigns(filter);
			}
		};
		hp.add(fb);
		tooManyDesignsWarning = UIFactory.createHTML("Too many designs to display. Please use filter!");
		tooManyDesignsWarning.setStyleName(UIFactory.GENERAL_ERROR);
		tooManyDesignsWarning.setVisible(false); //default is off
		hp.add(tooManyDesignsWarning);
		table.setWidget(r, c, hp);
		r++; c = 0;

		/***************************************************
		 * design area
		 */
		designArea = UIFactory.createFlexTable();
		table.setWidget(r, c, designArea);
		r++; c = 0;
		
		//tell the composite (this) which widget its wrapping
		initWidget(table);
	}
	
	@Override
	public void activated(){
	}

	/**
	 * @see IDesignAdmin#setDesignsFailed()
	 */
	public void setDesignsFailed() {
		fb.serverCallComplete(0);
	}
	
	/**
	 * @see IDesignAdmin#setDesigns()
	 */
	public void setDesigns(){
		fb.serverCallComplete(controller.getModel().getDesigns().size());
		
		designNames.clear();
		designBasePrices.clear();
		designArtists.clear();
		designInCatalogues.clear();
		designApproveds.clear();
		designDescriptions.clear();
		tags.clear();
		
		designArea.clear();
		int r = 0;
		int c = 0;
		if(controller.getModel().getDesigns().isEmpty()){
			HTML html = UIFactory.createHTML("No designs found.");
			designArea.setWidget(r, c, html);
		}else{
			//limit how much we show on screen, because of browser resources.
			if(controller.getModel().getDesigns().size() > MAX_DESIGNS_TO_SHOW){
				tooManyDesignsWarning.setVisible(true);
			}else{
				tooManyDesignsWarning.setVisible(false);
			}

			int i = 0;
			for(Design d : controller.getModel().getDesigns()){
				if(i++ >= MAX_DESIGNS_TO_SHOW){
					break;
				}

				final Design fd = d;
				FlexTable outerTable = UIFactory.createFlexTable();
				FlexTable table = UIFactory.createFlexTable();
				table.setBorderWidth(1);

				VerticalPanel vp = UIFactory.createVerticalPanel();
				final Image img = UIFactory.createImage(d.getUrl());
				vp.add(img);

				VerticalPanel holder = UIFactory.createVerticalPanel();
				final FormPanel form = new FormPanel();
				form.setEncoding(FormPanel.ENCODING_MULTIPART);
				form.setMethod(FormPanel.METHOD_POST);
				holder.add(new Hidden("designUid", String.valueOf(d.getUid())));
				final FileUpload updateImage = UIFactory.createFileUpload();
				updateImage.setName("data");
				holder.add(updateImage);
				Button go = UIFactory.createButton("Update Image");
				go.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						 form.submit();
					}
				});
				holder.add(go);
				form.add(holder);

				String url = Window.Location.getProtocol() + "//";
				url += Window.Location.getHost() + "/updateDesignImage.jsp";
				form.setAction(url);

				form.addSubmitHandler(new SubmitHandler() {
					public void onSubmit(SubmitEvent event) {
						//called just before the request is sent

						if(!updateImage.getFilename().toLowerCase().endsWith(".jpg")){
							UIFactory.createWarningDialog("Validation", 
									"File needs to be of type JPEG (with a '.jpg' file extension)."
									).center();
							event.cancel();
						}else{
							setWait(true);
							img.setUrl("/images/wait.gif");
						}
					}
				});
				form.addSubmitCompleteHandler(new SubmitCompleteHandler() {
					public void onSubmitComplete(SubmitCompleteEvent event) {
						//stop waiting, and reload, regardless - to show what we currently have in the system
						setWait(false);
						img.setUrl(fd.getUrl() + "?t=" + System.currentTimeMillis()); //to force reloading and avoiding the cache :-) TODO actually needed?
						if(!event.getResults().trim().equals("OK")){
							UIFactory.createErrorDialog("Error", event.getResults()).center();
						}
					}
				});
				if(getWizard().getModel().getSessionInfo().isInRole(Roles.BACKOFFICE)){
					vp.add(form);
				}

				HTML l = UIFactory.createHTML(
						"Designs have a number of images<br>" +
						"attributed to them. firstly, they<br>" +
						"have a default, which is that shown<br>" +
						"above.<br>" +
						"<br>" +
						"Update the default image above<br>" +
						"using a high resolution<br>" +
						"image. When you upload the image,<br>" +
						"a low resolution smaller image<br>" +
						"(100 pixels wide, same aspect<br>" +
						"ratio) will be generated, for<br>" +
						"displaying in the shop. This<br>" +
						"prevents design theft.<br>" +
						"<br>" +
						"Note: if you update this image<br>" +
						"you may need to refresh the<br>" +
						"browser before you see the new<br>" +
						"image in the shop.");
				vp.add(l);

				HorizontalPanel hp = UIFactory.createHorizontalPanel();
				l = UIFactory.createHTMLLikeHyperlink("open hi res");
				l.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						//http://msdn.microsoft.com/en-us/library/ms536651(VS.85).aspx
						String features = "menubar=no, status=no, titlebar=no, toolbar=no";
						Window.open(fd.getHiresUrl(), "_blank", features);
					}
				});
				hp.add(l);

				l = UIFactory.createHTMLLikeHyperlink("save hi res");
				l.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						//http://msdn.microsoft.com/en-us/library/ms536651(VS.85).aspx
						String features = "menubar=no, status=no, titlebar=no, toolbar=no";
						Window.open("/getHiResDesign?uid=" + fd.getUid(), "_blank", features); //open in a blank window since thats all that seems to work...
					}
				});
				hp.add(l);

				vp.add(hp);
				
				outerTable.setWidget(0, 0, vp);
				outerTable.getFlexCellFormatter().setVerticalAlignment(0, 0, HasVerticalAlignment.ALIGN_TOP);

				int r2 = 0;
				int c2 = 0;

				HTML html = UIFactory.createHTML();
				html.setStyleName(UIFactory.BIG_BOLD_TEXT);
				html.setHTML("ID: " + d.getUid());
				table.setWidget(r2, c2, html);
				r2++;

				/*********************************************
				 * name
				 */
				hp = UIFactory.createHorizontalPanel();
				hp.setSpacing(3);
				l = UIFactory.createHTML("Name:");
				hp.add(l);
				
				TextBox designName = UIFactory.createTextBox();
				designNames.put(d, designName);
				designName.setText(d.getName());
				hp.add(designName);

				table.setWidget(r2, c2, hp);
				r2++;
				
				/*********************************************
				 * additional price
				 */
				hp = UIFactory.createHorizontalPanel();
				hp.setSpacing(3);
				l = UIFactory.createHTML("Additional Price:");
				hp.add(l);
				
				TextBox designBasePrice = UIFactory.createTextBox();
				designBasePrices.put(d, designBasePrice);
				designBasePrice.setText(ClientFormats.NF_0_00.format(d.getBasePrice()));
				hp.add(designBasePrice);
				
				table.setWidget(r2, c2, hp);
				r2++;
				
				/*********************************************
				 * artist
				 */
				hp = UIFactory.createHorizontalPanel();
				l = UIFactory.createHTML("Artist:");
				hp.add(l);
				ListBox artistCombo = UIFactory.createListBox();
				artistCombo.setVisibleItemCount(1);
				designArtists.put(d, artistCombo);
				int idToSelect = 0;
				int j = 0;
				for(Artist a : getWizard().getModel().getAllArtists()){
					artistCombo.addItem(a.getUid() + ") " + a.getScreenname() + " - " + a.getFirstname() + " " + a.getLastname(), String.valueOf(a.getUid()));
					if(d.getArtist().getUid().equals(a.getUid())){
						idToSelect = j;
					}
					j++;
				}
				artistCombo.setSelectedIndex(idToSelect);
				hp.add(artistCombo);
				
				HTML artistLink = UIFactory.createHTMLLikeHyperlink("go to artist");
				hp.add(artistLink);
				artistLink.addClickHandler(new ClickHandler(){
					public void onClick(ClickEvent event) {
						int artistId = Integer.parseInt(designArtists.get(fd).getValue(designArtists.get(fd).getSelectedIndex()));
						Artist a = new Artist();
						a.setUid(artistId);
						getWizard().showArtistAdminPage(a);
					}
				});
				
				table.setWidget(r2, c2, hp);
				r2++;

				/*********************************************
				 * inCatalogue
				 */
				CheckBox designInCatalogue = UIFactory.createCheckBox("In catalogue?");
				designInCatalogues.put(d, designInCatalogue);
				designInCatalogue.setValue(d.isInCatalogue());
				table.setWidget(r2, c2, designInCatalogue);
				r2++;

				/*********************************************
				 * approved
				 */
				CheckBox designApproved = UIFactory.createCheckBox("Approved?");
				designApproveds.put(d, designApproved);
				designApproved.setValue(d.isApproved());
				table.setWidget(r2, c2, designApproved);
				r2++;
				
				/*********************************************
				 * description
				 */
				l = UIFactory.createHTML("Description:");
				table.setWidget(r2, c2, l);

				RichTextArea designDescription = UIFactory.createRichTextArea();
				designDescriptions.put(d, designDescription);

				SmallRichTextToolbar toolbar = UIFactory.createSmallRichTextToolbar(designDescription);
				table.setWidget(r2, c2, toolbar);
			    r2++;
			    
			    designDescription.setHTML(d.getDescription());
				table.setWidget(r2, c2, designDescription);
				r2++;
				
				/****************************************
				 * tags, remove and add
				 */
				TagListBox tlb = UIFactory.createTagListBox(
						"add", 
						"remove", 
						"Tags (for searching):",
						"Add a design tag", 
						"Please add a tag to describe the design:",
						"Tag may not be empty!");
				tlb.setContents(d.getTags());
				tags.put(d, tlb);

				table.setWidget(r2, c2, tlb);
				r2++;
				
				/*********************************************
				 * update button - only if in BACKOFFICE role
				 */
				if(getWizard().getModel().getSessionInfo().isInRole(Roles.BACKOFFICE)){
					Button b = UIFactory.createButton("Update");
					table.setWidget(r2, c2, b);
					b.addClickHandler(new ClickHandler(){
						public void onClick(ClickEvent event) {
							validateDesignForUpdate(fd, false);
						}
					});
					r2++;

					b = UIFactory.createButton("Add/Copy");
					table.setWidget(r2, c2, b);
					b.addClickHandler(new ClickHandler(){
						public void onClick(ClickEvent event) {
							validateDesignForUpdate(fd, true);
						}
					});
					r2++;
				}

				outerTable.setWidget(0, 1, table);

				designArea.setWidget(r, c, outerTable);
				r++; c = 0;

				l = UIFactory.createHTML("<hr>");
				designArea.setWidget(r, c, l);
				designArea.getFlexCellFormatter().setColSpan(r, c, 2);
				r++; c = 0;
			}
		}
	}

	protected void validateDesignForUpdate(Design d, boolean createCopy) {
		//TODO put this into a validator which throws validation exceptions

		boolean valid = true;

		String newName = designNames.get(d).getText().trim();
		if(newName.length() == 0){
			UIFactory.createWarningDialog("Validation", "Name may not be empty.").center();
			designNames.get(d).setFocus(true);
			valid = false;
		}
		
		String sNewBasePrice = designBasePrices.get(d).getText().trim();
		if(sNewBasePrice.length() == 0){
			UIFactory.createWarningDialog("Validation", "Price may not be empty.").center();
			designBasePrices.get(d).setFocus(true);
			valid = false;
		}
		double newBasePrice = 0.0;
		try{
			newBasePrice = ClientFormats.NF_0_00.parse(sNewBasePrice);
		}catch(NumberFormatException e){
			UIFactory.createWarningDialog("Validation", "Price must be a decimal number.").center();
			designBasePrices.get(d).setFocus(true);
			valid = false;
		}
		
		String newDescription = designDescriptions.get(d).getHTML().trim();
		if(newDescription.length() == 0){
			UIFactory.createWarningDialog("Validation", "Description may not be empty.").center();
			designDescriptions.get(d).setFocus(true);
			valid = false;
		}

		if(valid){

			boolean inCatalogue = designInCatalogues.get(d).getValue();

			boolean approved = designApproveds.get(d).getValue();

			List<String> designTags = tags.get(d).getContents();

			int artistId = Integer.parseInt(designArtists.get(d).getValue(designArtists.get(d).getSelectedIndex()));
			Artist a = new Artist();
			a.setUid(artistId);
			d.setArtist(a);
			
			controller.updateDesign(createCopy, d, newName, newBasePrice, newDescription, inCatalogue, 
					approved, designTags, fb.getInputText());
		}
	}

	public void showDesign(Design d) {
		
		//clear the area since it may show irrelevant designs.
		designArea.clear();
		
		String s = "";
		if(d != null){
			s = String.valueOf(d.getUid());
		}
		fb.setInputText(s, true);
	}
	
}
