/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.design_artist_list.ui;


import uk.co.maxant.gwtdemo.client.design_admin_list.core.IDesignAdminList;
import uk.co.maxant.gwtdemo.client.design_artist_list.core.DesignArtistListController;
import uk.co.maxant.gwtdemo.client.design_artist_list.core.IDesignArtistList;
import uk.co.maxant.gwtdemo.client.generic.widgets.CheckBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.HorizontalPanel;

/**
 * the design admin screen, which lets admin manage designs.
 */
public class DesignArtistList extends WizardPage implements IDesignArtistList {

	private DesignArtistListController controller;

	private FlexTable designArea;
	
	/**
	 */
	public DesignArtistList(final AppWizard wizard){
		super(wizard);

		controller = new DesignArtistListController(this, wizard);

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(0);
		
		int r = 0;
		int c = 0;
		
		/***************************************************
		 * title
		 */
		String s = "<h2>Design List</h2>";
		s += "All of your designs are listed here.";
		s += "<br><br>Use the table below to click on the design image, to see details.";
		HTML l = UIFactory.createHTML(s);
		table.setWidget(r, c, l);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;
		
		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;

		/***************************************************
		 * design area
		 */
		designArea = UIFactory.createFlexTable();
		table.setWidget(r, c, designArea);
		r++; c = 0;

		//tell the composite (this) which widget its wrapping
		initWidget(table);
	}
	
	@Override
	public void activated(){
		controller.setDesigns();
	}

	/**
	 * @see IDesignAdminList#setDesigns()
	 */
	public void setDesigns(){
		
		designArea.clear();
		
		int r = 0;
		int c = 0;
		if(controller.getModel().getDesigns().isEmpty()){
			HTML html = UIFactory.createHTML("No designs found.");
			designArea.setWidget(r, c, html);
		}else{

			HTML l = UIFactory.createHTML("&nbsp;"); //image
			designArea.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			designArea.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("ID");
			designArea.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			designArea.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("Name");
			designArea.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			designArea.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("In catalogue");
			designArea.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			designArea.setWidget(r, c, l); c++;
			l = UIFactory.createHTML("Approved");
			designArea.getFlexCellFormatter().setStyleName(r, c, UIFactory.BIG_BOLD_TEXT);
			designArea.setWidget(r, c, l); c++;

			designArea.getRowFormatter().setStyleName(r, UIFactory.LIST_HEADER);

			r++; c = 0;

			for(Design d : controller.getModel().getDesigns()){

				final Design fd = d;
				
				/*********************************************
				 * image
				 */
				Image img = UIFactory.createImage(d.getUrl());
				img.setSize("40px", "40px");
				img.addClickHandler(new ClickHandler(){
					public void onClick(ClickEvent event) {
						getWizard().showDesignDetailPage(fd);
					}
				});
				designArea.setWidget(r, c, img);
				c++;
				
				/*********************************************
				 * id
				 */
				l = UIFactory.createHTML();
				l.setHTML(String.valueOf(d.getUid()));
				designArea.setWidget(r, c, l);
				c++;

				/*********************************************
				 * name
				 */
				l = UIFactory.createHTML();
				l.setHTML(d.getName());
				designArea.setWidget(r, c, l);
				c++;
				
				/*********************************************
				 * inCatalogue
				 */
				CheckBox designInCatalogue = UIFactory.createCheckBox("");
				designInCatalogue.setValue(d.isInCatalogue());
				designInCatalogue.setEnabled(false);
				designArea.setWidget(r, c, designInCatalogue);
				c++;

				/*********************************************
				 * approved
				 */
				CheckBox designApproved = UIFactory.createCheckBox("");
				designApproved.setValue(d.isApproved());
				designApproved.setEnabled(false);
				designArea.setWidget(r, c, designApproved);
				c++;
				
				String style = UIFactory.LIST_ROW_EVEN_SELECTABLE;
				if(r % 2 != 0){
					style = UIFactory.LIST_ROW_ODD_SELECTABLE;
				}
				designArea.getRowFormatter().setStyleName(r, style);

				r++; c = 0;
			}
		}
	}

}
