/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.event.dom.client.KeyUpEvent;

/**
 * a filter component. contains a list of {@link Object}s
 * which it goes thru on each text change to see which objects match.
 * non-matching objects can be hidden or whatever, depending upon the impl.
 */
public abstract class AbstractFilterBox extends VeryAbstractFilterBox {

	protected List<AbstractFilterableObject> filterableObjects = new ArrayList<AbstractFilterableObject>();

	protected int divisor;

	public AbstractFilterBox(
					String labelText, 
					final String infoText, 
					String resultsText, 
					String clearTooltip,
					String infoTooltip,
					char accessKey,
					int filterTextboxWidth,
					int divisor
					){
		super(labelText, 
				infoText, 
				resultsText, 
				clearTooltip,
				infoTooltip,
				accessKey,
				filterTextboxWidth);
		
		this.divisor = divisor;
	}
	
	/**
	 * @return an array of lists of filterable objects. the first list is those which match.
	 * 		the second is a list of which do not match. 
	 */
	public List<? extends AbstractFilterableObject>[] getFilterables(){
		String search = input.getText().trim().toLowerCase();
		search = ".*" + search + ".*";
		int count = 0;
		List<AbstractFilterableObject> matching = new ArrayList<AbstractFilterableObject>();
		List<AbstractFilterableObject> noMatching = new ArrayList<AbstractFilterableObject>();
		for(AbstractFilterableObject fo : filterableObjects){
			boolean matches = false;
			for(String s : fo.getTagsToCompare()){
				if(s.toLowerCase().matches(search)){
					matches = true;
					count++;
					break;
				}
			}
			if(matches){
				matching.add(fo);
			}else{
				noMatching.add(fo);
			}
		}
		
		@SuppressWarnings("unchecked")
		List<AbstractFilterableObject>[] array = new List[2];
		array[0] = matching;
		array[1] = noMatching;
		
		return array;
	}
	
	/**
	 * add an object which is subject to being filtered.
	 */
	public void addFilterableObject(AbstractFilterableObject object){
		this.filterableObjects.add(object);
		results.setText(resultsText + (filterableObjects.size() / divisor));
	}
	
	/**
	 * clears all object out of the filterable list.
	 */
	public void clearFilterableObjects(){
		this.filterableObjects.clear();
	}
	
	@Override
	protected final void doFilter(KeyUpEvent event, boolean fireImmediately) {
		doFilter();
	}

	/**
	 * subclasses MUST implement this, and not {@link #doFilter(boolean)}
	 */
	protected abstract void doFilter();
	
	/**
	 * a container which is used to query what tags should be examined when filtering.
	 */
	public static abstract class AbstractFilterableObject {
		protected Object object;

		/**
		 * @param object the contained object which is filtered if no tags match the search input.
		 */
		public AbstractFilterableObject(Object object) {
			this.object = object;
		}
		
		/**
		 * get the contained object, for example to hide it.
		 */
		public Object getObject() {
			return object;
		}

		/**
		 * @return a list of tags to inspect when trying to match the search filter.
		 */
		public abstract String[] getTagsToCompare();
	}
}

