/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;


import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.Widget;

public abstract class AbstractInputDialog<T> extends DialogBox implements ClickHandler {
	
	protected Callback<T> callback;
	protected HTML validationError;
	protected T data;
	protected String validationMessage;
	private boolean hideOnComplete = true;
	
	public AbstractInputDialog(String title, String message, String validationMessage, final Callback<T> callback) {

		this.validationMessage = validationMessage;
		this.callback = callback;
		
		setModal(true);
		
		setText(title);

		FlexTable table = UIFactory.createFlexTable();
		
		int r = 0;
		int c = 0;
		Label label = UIFactory.createHTML(message);
		table.setWidget(r, c, label);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;
		
		table.setWidget(r, c, createContents());
		r++; c = 0;
		
		table.setWidget(r, c, createValidationArea());
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;
		
		Button button = UIFactory.createButton("OK");
		button.addClickHandler(this);
		table.setWidget(r, c, button);
		table.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		
		Button cancel = UIFactory.createButton("Cancel");
		cancel.addClickHandler(new ClickHandler(){
			public void onClick(ClickEvent event) {
	            hide();
	            callback.onCancel();
			}
		});
		table.setWidget(r, c, cancel);
		table.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_LEFT);
		r++; c = 0;
		
		setWidget(table);
	}

	protected Widget createValidationArea() {
		validationError = UIFactory.createHTML(validationMessage);
		validationError.setVisible(false);
		validationError.setStyleName(UIFactory.GENERAL_ERROR);
		return validationError;
	}

	public void onClick(ClickEvent event) {
		if(!validate()){
			validationError.setVisible(true);
			doInvalid();
		}else{
			if(hideOnComplete){
				hide();
			}
			callback.onOK(data);
		}
	}
	
	protected void doInvalid() {
	}

	protected abstract Widget createContents();

	/**
	 * @return false if not valid
	 */
	protected abstract boolean validate();

	public static interface Callback<T> {
		void onOK(T t);
		void onCancel();
	}
	
	@Override
	public void center() {
		super.center();
		setFocus(true);
	}

	/**
	 * default does nowt, clients may override this.
	 */
	public void setFocus(boolean focused){
	}

	public void setHideOnComplete(boolean hideOnComplete) {
		this.hideOnComplete = hideOnComplete;
	}
	
}
