/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;


import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.Image;

public class Dialog extends DialogBox {
	
	protected Button defaultButton;

	protected FlexTable table;
	
	public Dialog(String title, String message) {
		this(title, message, null);
	}

	public Dialog(String title, String message, final Callback callback) {
		
		setModal(true);
		
		setAnimationEnabled(true);
		
		setHTML(title);

		int r = 0;
		int c = 1;

		table = UIFactory.createFlexTable();
		table.setCellPadding(5);
		table.setCellSpacing(5);
		table.setBorderWidth(0);
		
		HTML label = UIFactory.createHTML(message);
		table.setWidget(r, c, label);
		r++; c = 0;

		defaultButton = UIFactory.createButton("OK");
		defaultButton.addClickHandler(new ClickHandler(){
			public void onClick(ClickEvent event) {
	            hide();
	            if(callback != null){
	            	callback.onClose();
	            }
			}
		});
		table.setWidget(r, c, defaultButton);
		table.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_CENTER);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;
		
		setWidget(table);
	}

	/**
	 * centres and shows, setting focus on the default button.
	 */
	@Override
	public void center() {
		super.center();
		defaultButton.setFocus(true);
	}
	
	protected void setDefaultButton(Button button){
		this.defaultButton = button;
	}
	
	protected void setImage(Image img){
		table.setWidget(0, 0, img);
	}
	
	public static interface Callback {
		void onClose();
	}
	

}
