/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;

import java.util.List;

import com.google.gwt.user.client.ui.UIObject;

/**
 * a filter component. contains a list of {@link AbstractFilterableObject}s
 * which it goes thru on each text change to see which objects match.
 * non-matching objects are hidden.
 */
public class FilterBox extends AbstractFilterBox {

	/**
	 * @param labelText text to show, eg "Filter"
	 * @param infoText info text to show when info button is clicked
	 * @param resultsText beginning of result to show, eg "matches: "
	 * @param clearTooltip tool tip for button which clears filter input
	 * @param infoTooltip tool tip for button which shows info
	 * @param accessKey mnemonic
	 * @param filterTextboxWidth eg "200px" - width of input text box
	 * @param divisor number with which to divide number of matches, normally 1. but if several UI items are added to say each row then its the number of columns.
	 */
	public FilterBox(
					String labelText, 
					final String infoText, 
					String resultsText, 
					String clearTooltip,
					String infoTooltip,
					char accessKey,
					int filterTextboxWidth,
					int divisor
					){
		
		super(labelText, infoText, resultsText, clearTooltip, infoTooltip, accessKey, filterTextboxWidth, divisor);
	}
	
	/**
	 * depending upon input, filter objects
	 */
	@Override
	protected void doFilter(){
		List<FilterableUIObject>[] filterables = getFilterables();
		for(FilterableUIObject fuio : filterables[0]){
			fuio.getUIObject().setVisible(true);
		}
		for(FilterableUIObject fuio : filterables[1]){
			fuio.getUIObject().setVisible(false);
		}
		results.setText(resultsText + (filterables[0].size() / divisor));
	}
	
	
	public void addFilterableObject(FilterableUIObject object) {
		super.addFilterableObject(object);
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<FilterableUIObject>[] getFilterables() {
		return (List<FilterableUIObject>[])super.getFilterables();
	}
	
	/**
	 * a container which is used to query what tags should be examined when filtering.
	 */
	public static abstract class FilterableUIObject extends AbstractFilterBox.AbstractFilterableObject {

		/**
		 * @param object the contained object which is filtered if no tags match the search input.
		 */
		public FilterableUIObject(UIObject object) {
			super(object);
		}
		
		/**
		 * get the contained object, for example to hide it.
		 */
		public UIObject getUIObject() {
			return (UIObject)object;
		}

	}

}

