/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.MouseOutEvent;
import com.google.gwt.event.dom.client.MouseOutHandler;
import com.google.gwt.event.dom.client.MouseOverEvent;
import com.google.gwt.event.dom.client.MouseOverHandler;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;

/**
 * "rate it" box with five stars. rollover and everything.
 */
public class Rater extends Composite {

	private int rating;
	
	private boolean readonly;

	private Image[] imgs = new Image[5];

	private int defaultRating;
	
	/**
	 * @param defaultRating 
	 */
	public Rater(final int defaultRating, boolean readonly, final Callback callback){

		this.readonly = readonly;
		this.defaultRating = defaultRating;
		
		addHandler(new MouseOutHandler() {
			public void onMouseOut(MouseOutEvent event) {
				selectDefault();
			}
		}, MouseOutEvent.getType());
		sinkEvents(Event.ONMOUSEOUT); //register for mouse out events
		
		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		hp.setSpacing(0);
		
		for(int i = 0; i < 5; i++){
			Image img = UIFactory.createImage("/images/ui/norate.jpg");
			imgs[i] = img;
			hp.add(img);

			if(!readonly){
				final int fi = i;
				
				img.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						if(!Rater.this.readonly){
							UIFactory.createInformationDialog("Rated", "Thanks for rating this design.").center();
							selectDefault(); //return it to its normal state
							Rater.this.readonly = true; //disable from now on
							callback.rated(fi);
						}
					}
				});
				
				img.addMouseOverHandler(new MouseOverHandler() {
					public void onMouseOver(MouseOverEvent event) {
						select(fi);
					}
				});
			}
		}
		
		selectDefault(); //default select first one

		//tell the composite (this) which widget its wrapping
		initWidget(hp);
	}

	private void select(int fi){
		if(readonly) return;
		for(int i = 0; i < 5; i++){
			if(i <= fi){
				imgs[i].setUrl("/images/ui/rate.jpg");
			}else{
				imgs[i].setUrl("/images/ui/norate.jpg");
			}
		}
	}
	
	private void selectDefault(){
		for(int i = 0; i < 5; i++){
			if(i <= defaultRating){
				imgs[i].setUrl("/images/ui/rate.jpg");
			}else{
				imgs[i].setUrl("/images/ui/norate.jpg");
			}
		}
	}
	
	public int getRating(){
		return rating;
	}
	
	public static interface Callback{
		void rated(int ratingGiven);
	}
	
}
