/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;

import java.util.ArrayList;
import java.util.List;

import uk.co.maxant.gwtdemo.client.widgets.ui.TagInputDialog;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * a component which lets you add "tags" to a listbox.
 */
public class TagListBox extends Composite {

	private ListBox tags;
	
	public TagListBox(
			String addText,
			String removeText,
			String title,
			final String inputDialogTitle,
			final String inputDialogMessage,
			final String inputDialogValidation
					){

		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		hp.setSpacing(3);
		HTML l = UIFactory.createHTML(title);
		hp.add(l);
		
		VerticalPanel vp = UIFactory.createVerticalPanel();
		tags = UIFactory.createListBox(true);
		tags.setVisibleItemCount(5);
		tags.setWidth("200px");
		vp.add(tags);
		HorizontalPanel hp2 = UIFactory.createHorizontalPanel();
		hp2.setSpacing(3);
		HTML remove = UIFactory.createHTMLLikeHyperlink(removeText);
		hp2.add(remove);
		remove.addClickHandler(new ClickHandler(){
			public void onClick(ClickEvent event) {
				/*
				 * 0) selected
				 * 1) not
				 * 2) selected
				 * 
				 * 0) not
				 * 1) selected
				 * 
				 * 0) not
				 */
				int i = 0;
				while(true){
					if(tags.isItemSelected(i)){
						tags.removeItem(i);
					}else{
						i++;
					}
					if(i >= tags.getItemCount()){
						break;
					}
				}
			}
		});
		HTML add = UIFactory.createHTMLLikeHyperlink(addText);
		add.setStyleName(UIFactory.STANDARD_TEXT_POINTER_CURSOR_UNDERLINE);
		hp2.add(add);
		add.addClickHandler(new ClickHandler(){

			public void onClick(ClickEvent event) {
				TagInputDialog tid = new TagInputDialog(
						inputDialogTitle,
						inputDialogMessage,
						inputDialogValidation,
						new AbstractInputDialog.Callback<String>(){
							public void onOK(String s) {
								tags.addItem(s);
							}
							public void onCancel() {
							}
						});
	            tid.center();
			}
		});
		vp.add(hp2);
		hp.add(vp);
		
		initWidget(hp);
	}
	
	public void setContents(List<String> content){
		for(String tag : content){
			tags.addItem(tag);
		}
	}
	
	public List<String> getContents(){
		List<String> contents = new ArrayList<String>();
		for(int i = 0; i < tags.getItemCount(); i++){
			contents.add(tags.getItemText(i));
		}
		return contents;
	}
}

