/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;

import uk.co.maxant.gwtdemo.client.images.Images;

import com.google.gwt.event.dom.client.BlurEvent;
import com.google.gwt.event.dom.client.BlurHandler;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;

/**
 * a text box which contains an error message to the right
 * 
 * if validation fails, the message is shown
 */
public class TextBoxWithValidation extends Composite {

	public static final String EMAIL_REGEXP = ".*@.*\\..*";

	public static final String PASSWORD_6_CHARS_ANY_KIND = "......*";
	
	protected boolean assertNotEmpty = false;

	protected boolean assertTrimNotEmpty = false;

	protected boolean assertInteger = false;

	protected boolean assertEmail = false;
	
	protected boolean isPassword = false;
	
	protected String errorMessage;

	protected com.google.gwt.user.client.ui.Image errorIcon;

	protected HTML errorLabel;

	protected com.google.gwt.user.client.ui.TextBox input;
	
	/**
	 * @see #TextBoxWithValidation(String, boolean, String, boolean)
	 */
	public TextBoxWithValidation(String errorMessage, final boolean validateOnBlur){
		this(errorMessage, validateOnBlur, null, false);
	}

	/**
	 * @see #TextBoxWithValidation(String, boolean, String, boolean)
	 */
	public TextBoxWithValidation(String errorMessage, final boolean validateOnBlur, String width){
		this(errorMessage, validateOnBlur, width, false);
	}

	/**
	 * @see #TextBoxWithValidation(String, boolean, String, boolean)
	 */
	public TextBoxWithValidation(String errorMessage, final boolean validateOnBlur, boolean usePasswordTextBox){
		this(errorMessage, validateOnBlur, null, usePasswordTextBox);
	}

	/**
	 * @param errorMessage the message to show the user if theres a problem
	 * @param validateOnBlur vlaidate automatically, when the text box loses focus?
	 * @param width how wide to make the input?
	 * @param usePasswordTextBox if true, then a password text box is created.
	 */
	public TextBoxWithValidation(String errorMessage, final boolean validateOnBlur, String width, boolean usePasswordTextBox){
		this.errorMessage = errorMessage;

		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		
		if(usePasswordTextBox){
			input = UIFactory.createPasswordTextBox();
			isPassword = true;
		}else{
			input = UIFactory.createTextBox();
			isPassword = false;
		}
		hp.add(input);

		if(width != null){
			input.setWidth(width);
		}
		
		input.addBlurHandler(new BlurHandler() {
			public void onBlur(BlurEvent event) {
				if(validateOnBlur){
					validate();
				}
			}
		});
		
		input.addChangeHandler(new ChangeHandler() {
			public void onChange(ChangeEvent event) {
				errorIcon.setVisible(false);
				errorLabel.setVisible(false);
			}
		});
		
		errorIcon = Images.images.validation_error().createImage();
		hp.add(errorIcon);
		errorIcon.setVisible(false);
		
		errorLabel = UIFactory.createHTML(errorMessage);
		hp.add(errorLabel);
		errorLabel.setStyleName(UIFactory.VALIDATION_ERROR);
		errorLabel.setVisible(false);

		//tell the composite (this) which widget its wrapping
		initWidget(hp);
	}

	public boolean validate(){
		boolean valid = true;
		String s = input.getText();
		if(s.length() == 0 && assertNotEmpty){
			valid = false;
		}else{
			if(s.length() > 0){
				if(isPassword){
					if(!s.matches(PASSWORD_6_CHARS_ANY_KIND)){ //TODO 000 not working on profile update!?
						valid = false;
					}
				}
			}
		}
		
		if(valid){
			s = s.trim();
			if(s.length() == 0 && assertTrimNotEmpty){
				valid = false;
			}else{
				if(s.length() > 0){
					if(isPassword){
						if(!s.matches(PASSWORD_6_CHARS_ANY_KIND)){
							valid = false;
						}
					}
				}
			}

			if(valid && assertInteger){
				try{
					Integer.parseInt(s);
				}catch(Exception e){
					valid = false;
				}
			}

			if(valid && assertEmail){
				if(!s.trim().matches(EMAIL_REGEXP)){
					valid = false;
				}
			}
		}

		errorIcon.setVisible(!valid);
		errorLabel.setVisible(!valid);
		
		if(!valid){
			//input.setFocus(true); TODO cant call this since it causes an infinite loop - how to improve?
		}
		
		return valid;
	}
	
	/** should validation also check that its not empty? */
	public void setAssertNotEmpty(boolean assertNotEmpty) {
		this.assertNotEmpty = assertNotEmpty;
	}

	/** should validation also check if its an integer, if not empty? */
	public void setAssertInteger(boolean assertInteger) {
		this.assertInteger = assertInteger;
	}

	/** should validation also check that its not full of white space? */
	public void setAssertTrimNotEmpty(boolean assertTrimNotEmpty) {
		this.assertTrimNotEmpty = assertTrimNotEmpty;
	}

	/** should validation also check that its an email address? */
	public void setAssertEmail(boolean assertEmail) {
		this.assertEmail = assertEmail;
	}

	/** sets focus on the input */
	public void setFocus(boolean f) {
		input.setFocus(f);
	}
	
	/** sets the text showin in the input */
	public void setText(String text) {
		input.setText(text);
	}

	/**
	 * sets the width on the input textbox.
	 */
	public void setInputWidth(String width){
		input.setWidth(width);
	}

	/**
	 * sets the length on the input textbox.
	 */
	public void setInputMaxLength(int length){
		input.setMaxLength(length);
	}
	
	/**
	 * @return the input
	 */
	public String getText(){
		return input.getText();
	}

	/**
	 * @return trimmed input
	 */
	public String getTrimmedText(){
		return input.getText().trim();
	}
	
	/**
	 * @return a parsed integer of the trimmed input, or null, if not valid.
	 */
	public Integer getInteger(){
		try{
			return Integer.parseInt(getTrimmedText());
		}catch(Exception e){
			return null;
		}
	}
	
	public void addHandler(Handler handler){
		input.sinkEvents(Event.KEYEVENTS); //for copy/paste TODO 000 get this working!
		input.addBlurHandler(handler);
		input.addChangeHandler(handler);
		input.addClickHandler(handler);
		input.addFocusHandler(handler);
		input.addKeyUpHandler(handler);
		input.addKeyDownHandler(handler);
		input.addKeyPressHandler(handler);
		input.addValueChangeHandler(handler);
	}
}
