/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;


import org.gwt.advanced.client.ui.widget.border.RoundCornerBorder;

import uk.co.maxant.gwtdemo.client.widgets.ui.richtextarea.RichTextToolbar;
import uk.co.maxant.gwtdemo.client.widgets.ui.richtextarea.SmallRichTextToolbar;

import com.google.gwt.user.client.ui.DeckPanel;
import com.google.gwt.user.client.ui.DisclosurePanel;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.FileUpload;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.RichTextArea;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.StackPanel;
import com.google.gwt.user.client.ui.TabPanel;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.Tree;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * factory for UI components
 */
public class UIFactory {

	public static final String RATER = "rater";
	public static final String CLOUD_1 = "cloud1";
	public static final String CLOUD_2 = "cloud2";
	public static final String CLOUD_3 = "cloud3";
	public static final String CLOUD_4 = "cloud4";
	public static final String CLOUD_5 = "cloud5";
	public static final String SMALL_TEXT_POINTER_CURSOR_UNDERLINE = "smallTextPointerCursorUnderline";
	public static final String STANDARD_TEXT_POINTER_CURSOR_UNDERLINE = "standardTextPointerCursorUnderline";
	public static final String POINTER_CURSOR = "pointerCursor";
	public static final String STANDARD_TEXT = "standardText";
	public static final String STANDARD_TEXT_PADDED = "standardTextPadded";
	public static final String STANDARD_TEXT_BOLD = "standardTextBold";
	public static final String GENERAL_ERROR = "generalError";
	public static final String GENERAL_INFO = "generalInfo";
	public static final String VALIDATION_ERROR = "validationError";
	public static final String BIG_BOLD_TEXT = "bigBoldText";
	public static final String LIST_TABLE = "listTable";
	public static final String LIST_HEADER = "listHeader";
	public static final String LIST_ROW_EVEN = "listRowEven";
	public static final String LIST_ROW_ODD = "listRowOdd";
	public static final String LIST_ROW_EVEN_SELECTABLE = "listRowEvenSelectable";
	public static final String LIST_ROW_ODD_SELECTABLE = "listRowOddSelectable";
	public static final String LIST_ROW_SELECTED = "listRowSelected";
	public static final String SHOP_CUSTOMER_REVIEWS = "shopCustomerReviews";
	public static final String BORDERED_BOX = "borderedBox";

	private UIFactory(){}
	
	public static HTML createHTML(){
		HTML html = new HTML();
		html.setStyleName(STANDARD_TEXT);
		return html;
	}
	
	public static HTML createHTML(String html){
		HTML html_ = createHTML();
		html_.setHTML(html);
		return html_;
	}
	
	public static ImageButton createImageButton(String upUrl, String downUrl){
		ImageButton ib = new ImageButton(upUrl, downUrl);
		ib.setStyleName(POINTER_CURSOR);
		return ib;
	}
	
	public static TextBox createTextBox(){
		TextBox tb = new TextBox();
		tb.setStyleName(STANDARD_TEXT);
		return tb;
	}
	
	public static FlexTable createFlexTable(){
		FlexTable ft = new FlexTable();
		return ft;
	}
	
	public static FlexTable createBorderedFlexTable(){
		FlexTable ft = createFlexTable();
		ft.setStyleName(UIFactory.LIST_TABLE);
		return ft;
	}
	
	public static Button createButton(String html){
		Button b = new Button(html);
		b.setStyleName(STANDARD_TEXT);
		return b;
	}
	
	public static Image createImage(String url){
		Image i = new Image(url);
		return i;
	}
	
	/**
	 * use with image bundles
	 */
	public static com.google.gwt.user.client.ui.Image createImage(com.google.gwt.user.client.ui.Image image){
		//set style if required... currently no need!
		//just providing a standard interface in the UIFactory for creating images
		return image;
	}
	
	public static Image createImage(){
		Image i = new Image();
		return i;
	}
	
	/**
	 * creates an image with a pointer cursor for style.
	 */
	public static Image createImageButton(String url){
		Image i = new Image(url);
		i.setStyleName(POINTER_CURSOR);
		return i;
	}
	
	/**
	 * used with image bundles!
	 */
	public static com.google.gwt.user.client.ui.Image createImageButton(com.google.gwt.user.client.ui.Image image){
		image.setStyleName(POINTER_CURSOR);
		return image;
	}
	
	public static VerticalPanel createVerticalPanel(){
		VerticalPanel vp = new VerticalPanel();
		vp.setSpacing(3);
		return vp;
	}
	
	public static HorizontalPanel createHorizontalPanel(){
		HorizontalPanel hp = new HorizontalPanel();
		hp.setSpacing(3);
		return hp;
	}
	
	public static ScrollPanel createScrollPanel(){
		ScrollPanel sp = new ScrollPanel();
		return sp;
	}
	
	public static FlowPanel createFlowPanel(){
		FlowPanel fp = new FlowPanel();
		return fp;
	}
	
	public static Hyperlink createHyperlink(String html){
		Hyperlink hl = new Hyperlink(html);
		hl.setStyleName(STANDARD_TEXT_POINTER_CURSOR_UNDERLINE);
		return hl;
	}

	public static Hyperlink createHyperlinkSmall(String html){
		Hyperlink hl = createHyperlink(html);
		hl.setStyleName(SMALL_TEXT_POINTER_CURSOR_UNDERLINE);
		return hl;
	}
	
	public static CheckBox createCheckBox(String html){
		CheckBox cb = new CheckBox();
		cb.setHTML(html);
		cb.setStyleName(STANDARD_TEXT);
		return cb;
	}

	public static CheckBox createCheckBox(){
		return createCheckBox("");
	}
	
	/**
	 * @return a single selection listbox
	 */
	public static ListBox createListBox(){
		return createListBox(false, null);
	}

	public static ListBox createListBox(String width){
		return createListBox(false, width);
	}
	
	public static ListBox createListBox(boolean isMultipleSelect){
		return createListBox(isMultipleSelect, null);
	}

	public static ListBox createListBox(boolean isMultipleSelect, String width){
		ListBox lb = new ListBox(isMultipleSelect);
		if(width != null){
			lb.setWidth(width);
		}
		lb.setStyleName(STANDARD_TEXT);
		return lb;
	}
	
	public static TagListBox createTagListBox(String addText, String removeText, String title, String inputDialogTitle, String inputDialogMessage, String inputDialogValidation){
		TagListBox tlb = new TagListBox(addText, removeText, title, inputDialogTitle, inputDialogMessage, inputDialogValidation);
		//TODO styles? i guess it sets its own. make generic for a library!
		return tlb;
	}
	
	/**
	 * @return a vertical menu bar
	 */
	public static MenuBar createMenuBar(boolean vertical){
		MenuBar mb = new MenuBar(vertical);
		mb.setAutoOpen(true);
		mb.setAnimationEnabled(false);
		return mb;
	}
	
	public static RichTextArea createRichTextArea(){
		RichTextArea rta = new RichTextArea();
	    rta.setSize("100%", "14em");
	    rta.setStyleName(STANDARD_TEXT);
	    rta.addStyleName(BORDERED_BOX);
	    return rta;
	}
	
	public static RichTextToolbar createRichTextToolbar(RichTextArea rta){
		RichTextToolbar toolbar = new RichTextToolbar(rta);
		toolbar.setWidth("100%");
		toolbar.setStyleName(STANDARD_TEXT);
		return toolbar;
	}
	
	public static SmallRichTextToolbar createSmallRichTextToolbar(RichTextArea rta){
		SmallRichTextToolbar toolbar = new SmallRichTextToolbar(rta);
		toolbar.setWidth("100%");
		toolbar.setStyleName(STANDARD_TEXT);
		return toolbar;
	}
	
	public static FilterBox createFilterBox(
					String labelText, 
					final String infoText, 
					String resultsText, 
					String clearTooltip,
					String infoTooltip,
					char accessKey,
					int filterTextboxWidth){
		
		return createFilterBox(labelText, infoText, resultsText, clearTooltip, infoTooltip, accessKey, filterTextboxWidth, 1);
	}
	
	public static FilterBox createFilterBox(
			String labelText, 
			final String infoText, 
			String resultsText, 
			String clearTooltip,
			String infoTooltip,
			char accessKey,
			int filterTextboxWidth,
					int divisor
			){
		FilterBox fb = new FilterBox(
								labelText, 
								infoText, 
								resultsText, 
								clearTooltip, 
								infoTooltip, 
								accessKey, 
								filterTextboxWidth,
								divisor);
		//filterbox has its own style definitions!
		return fb;
	}

	public static DeckPanel createDeckPanel() {
		DeckPanel dp = new DeckPanel();
		dp.setAnimationEnabled(false);
		return dp;
	}

	/**
	 * creates something that looks like a hyperlink, but it isnt. because hyperlinks
	 * intrinsically work with history, and we dont want to always do that.
	 * 
	 * TODO hover, visited, etc would be nice here.
	 */
	public static HTML createHTMLLikeHyperlink(String html) {
		HTML h = createHTML(html);
		h.setStyleName(STANDARD_TEXT_POINTER_CURSOR_UNDERLINE);
		return h;
	}
	
	/**
	 * @see #createHTMLLikeHyperlink(String)
	 * TODO hover, visited, etc would be nice here.
	 */
	public static HTML createSmallHTMLLikeHyperlink(String html) {
		HTML h = createHTMLLikeHyperlink(html);
		h.setStyleName(SMALL_TEXT_POINTER_CURSOR_UNDERLINE);
		return h;
	}

	public static TextArea createTextArea() {
		TextArea ta = new TextArea();
		ta.setStyleName(STANDARD_TEXT);
		return ta;
	}

	public static TabPanel createTabPanel() {
		TabPanel tp = new TabPanel();
		tp.setAnimationEnabled(false);
		tp.setStyleName(STANDARD_TEXT);
		return tp;
	}
	
	public static FileUpload createFileUpload() {
		FileUpload fu = new FileUpload();
		fu.setStyleName(STANDARD_TEXT);
		return fu;
	}

	public static DisclosurePanel createDisclosurePanel(String text) {
		DisclosurePanel dp = new DisclosurePanel(text, false);
		dp.setStyleName(STANDARD_TEXT);
		return dp;
	}

	public static DockPanel createDockPanel() {
		DockPanel dp = new DockPanel();
		dp.setSpacing(3);
		return dp;
	}
	
	public static InformationDialog createInformationDialog(String title, String message){
		InformationDialog d = new InformationDialog(title, message);
		return d;
	}
	
	public static InformationDialog createInformationDialog(
			String title, 
			String message, 
			Dialog.Callback callback){
		InformationDialog d = new InformationDialog(title, message, callback);
		return d;
	}
	
	public static WarningDialog createWarningDialog(String title, String message){
		WarningDialog d = new WarningDialog(title, message);
		return d;
	}
	
	public static WarningDialog createWarningDialog(String title, String message, Dialog.Callback callback){
		WarningDialog d = new WarningDialog(title, message, callback);
		return d;
	}
	
	public static ErrorDialog createErrorDialog(String title, String message){
		ErrorDialog d = new ErrorDialog(title, message);
		return d;
	}

	public static ErrorDialog createErrorDialog(String title, String message, Dialog.Callback callback){
		ErrorDialog d = new ErrorDialog(title, message, callback);
		return d;
	}
	
	public static QuestionDialog createQuestionDialog(String title, String message, QuestionDialog.Callback callback) {
		QuestionDialog d = new QuestionDialog(title, message, callback);
		return d;
	}

	public static PasswordTextBox createPasswordTextBox() {
		PasswordTextBox ptb = new PasswordTextBox();
		ptb.setStyleName(STANDARD_TEXT);
		return ptb;
	}
	
	public static TextBoxWithValidation createTextBoxWithValidation(String errorMessage, boolean validateOnBlur){
		TextBoxWithValidation tb = new TextBoxWithValidation(errorMessage, validateOnBlur);
		//it sets styles itself
		return tb;
	}

	public static TextBoxWithValidation createTextBoxWithValidation(String errorMessage, boolean validateOnBlur, boolean usePasswordTextBox){
		TextBoxWithValidation tb = new TextBoxWithValidation(errorMessage, validateOnBlur, usePasswordTextBox);
		//it sets styles itself
		return tb;
	}

	public static TextBoxWithValidation createTextBoxWithValidation(String errorMessage, boolean validateOnBlur, String width){
		TextBoxWithValidation tb = new TextBoxWithValidation(errorMessage, validateOnBlur, width);
		//it sets styles itself
		return tb;
	}

	public static TextBoxWithValidation createTextBoxWithValidation(String errorMessage, boolean validateOnBlur, String width, boolean usePasswordTextBox){
		TextBoxWithValidation tb = new TextBoxWithValidation(errorMessage, validateOnBlur, width, usePasswordTextBox);
		//it sets styles itself
		return tb;
	}
	
	public static Rater createRater(int defaultRating, boolean readonly, Rater.Callback callback){
		Rater rater = new Rater(defaultRating, readonly, callback);
		rater.setStyleName(RATER);
		return rater;
	}

	public static PopupPanel createPopupPanel(boolean autoHide) {
		PopupPanel pp = new PopupPanel(autoHide);
		pp.setStyleName(STANDARD_TEXT);
		return pp;
	}

	public static Tree createTree() {
		Tree tree = new Tree();
		tree.setStyleName(STANDARD_TEXT);
		tree.setAnimationEnabled(true);
		return tree;
	}

	/**
	 * define the background colour using:
	 * 
	 * .advanced-RoundCornerBorder div {
	 *     background-color: #FFD6C7;
	 * }
	 * 
	 * in the GWTDemo.css file, which is specified in header.jsp after the advanced-components CSS file from
	 * Advanced GWT.
	 */
	public static RoundCornerBorder createRoundCornerBorder(){
		RoundCornerBorder border = new RoundCornerBorder();
		border.setShadowVisibile(true);
		return border;
	}

	public static StackPanel createStackPanel() {
		StackPanel stp = new StackPanel();
		return stp;
	}
}
