/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;


import uk.co.maxant.gwtdemo.client.images.Images;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.event.dom.client.KeyUpEvent;
import com.google.gwt.event.dom.client.KeyUpHandler;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.TextBox;

/**
 * the basis for a filter component. basically, just the UI part.
 */
public abstract class VeryAbstractFilterBox extends Composite {

	protected TextBox input;
	protected HTML results;
	protected String resultsText;

	public VeryAbstractFilterBox(
					String labelText, 
					final String infoText, 
					String resultsText, 
					String clearTooltip,
					String infoTooltip,
					char accessKey,
					int filterTextboxWidth
					){
		this.resultsText = resultsText;
		
		HorizontalPanel hp = new HorizontalPanel();
		hp.setSpacing(3);

		// ///////////////////////////////
		// info
		// ///////////////////////////////
		com.google.gwt.user.client.ui.Image info = Images.images.info().createImage();
		info.setStyleName("maxantFilterBoxImage");
		info.addClickHandler(new ClickHandler(){
			public void onClick(ClickEvent event) {
				UIFactory.createInformationDialog("Information", infoText).center();
			}
		});
		info.setTitle(infoTooltip);
		hp.add(info);

		// ///////////////////////////////
		// label
		// ///////////////////////////////
		HTML label = new HTML(labelText);
		label.setStyleName("maxantFilterBoxText");
		hp.add(label);

		// ///////////////////////////////
		// results
		// ///////////////////////////////
		results = new HTML();
		results.setWordWrap(false);
		results.setStyleName("maxantFilterBoxResult");

		// ///////////////////////////////
		// input
		// ///////////////////////////////
		input = new TextBox();
		input.setStyleName("maxantFilterBoxText");
		input.setAccessKey(accessKey);
		input.setWidth(String.valueOf(filterTextboxWidth) + "px");
		input.addKeyUpHandler(new KeyUpHandler(){
			public void onKeyUp(KeyUpEvent event) {
				if(event.getNativeKeyCode() == KeyCodes.KEY_ENTER){
					//return was hit, so fire immediately!
					doFilter(null, true);
				}else{
					doFilter(event, false);
				}
			}
		});
		hp.add(input);
		
		// ///////////////////////////////
		// clear
		// ///////////////////////////////
		com.google.gwt.user.client.ui.Image clear = Images.images.clear_filter().createImage();
		clear.setStyleName("maxantFilterBoxImage");
		clear.addClickHandler(new ClickHandler(){
			public void onClick(ClickEvent event) {
				setInput("", true);
			}
		});
		clear.setTitle(clearTooltip);
		hp.add(clear);
		
		//now add results to the end
		hp.add(results);
		
		
		//tell the composite (this) which widget its wrapping
		initWidget(hp);
	}

	protected void setInput(String s, boolean fireImmediately) {
		input.setText(s);
		doFilter(null, fireImmediately);
	}
	
	/**
	 * depending upon input, filter objects
	 * 
	 * @param event can be null, in which case its ignored.
	 */
	protected abstract void doFilter(KeyUpEvent event, boolean fireImmediately);
	
	/**
	 * sets the filter input to be the given string, and then calls the {@link #doFilter()} method.
	 */
	public void setInputText(String s, boolean fireImmediately) {
		setInput(s, fireImmediately);
	}

	public String getInputText() {
		return input.getText();
	}


}

