/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.generic.widgets;

import uk.co.maxant.gwtdemo.client.generic.widgets.WizardPage;
import uk.co.maxant.gwtdemo.client.util.JSUtil;

import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.user.client.History;
import com.google.gwt.user.client.ui.DeckPanel;

/**
 * this class is a wizard.
 * 
 * the pages are contained in a deckpanel which is used to show/hide the current page.
 */
public abstract class Wizard implements IWizard {

	protected static final String HISTORY_PAGE = "HP_";

	/**
	 * 0 is the first page
	 */
	protected int screenIndex = 0;
	
	protected DeckPanel deck;
	
	public Wizard() {
		deck = UIFactory.createDeckPanel();
		
		History.addValueChangeHandler(new ValueChangeHandler<String>(){
			public void onValueChange(ValueChangeEvent<String> event) {
				String historyToken = event.getValue();

				// Parse the history token
				try {
					if (historyToken.startsWith(HISTORY_PAGE)) {
						String pageNumber = historyToken.substring(HISTORY_PAGE.length());
						int pageIndex = Integer.parseInt(pageNumber);
						// Select the specified tab panel
						screenIndex = pageIndex;
						activateCurrentPage(false);
					} else {
						//unknown, so ignore it. potentially we could add other history types here...
					}

				} catch (Exception e) {
					//do nothing... we dont know what the history token contained!
				}
			}
		});

		//add history for the index page
		History.newItem(HISTORY_PAGE + screenIndex, false);
	}
	
	protected void addPage(WizardPage page) {
		deck.add(page);
	}

	/**
	 * shows the given page and calls its activate method.
	 */
	protected void activateCurrentPage(){
		activateCurrentPage(true);
	}

	protected void activateCurrentPage(boolean addHistoryToken){

		for(int i = 0; i < deck.getWidgetCount(); i++){
			if(i != screenIndex){
				((WizardPage)deck.getWidget(i)).deactivated_();
			}
		}
		if(addHistoryToken){
			History.newItem(HISTORY_PAGE + screenIndex, false);
			
			//and now add some google analytics
			JSUtil.trackPageView("page" + screenIndex);
		}
		deck.showWidget(screenIndex);
		((WizardPage)deck.getWidget(screenIndex)).activated_();
	}
	
	/**
	 * @see Wizard#showFirstPage()
	 */
	public void showFirstPage() {
		screenIndex = 0;
		activateCurrentPage();
	}
	
	/**
	 * @see Wizard#showNextPage()
	 */
	public void showNextPage() {
		if(screenIndex < deck.getWidgetCount()){
			screenIndex++;
		}
		activateCurrentPage();
	}
	
	/**
	 * @see Wizard#showPreviousPage()
	 */
	public void showPreviousPage() {
		screenIndex--;
		if(screenIndex < 0){
			screenIndex = 0;
		}
		activateCurrentPage();
	}
	
	/**
	 * @see Wizard#showPage(WizardPage)
	 */
	public void showPage(WizardPage page) {
		if(getCurrentPage().equals(page)){
			//already showing, nothing to do...
		}else{
			for(int i = 0; i < deck.getWidgetCount(); i++){
				if(deck.getWidget(i).equals(page)){
					screenIndex = i;
					activateCurrentPage();
					break;
				}
			}
		}
	}

	/**
	 * @return the current active page.
	 */
	public WizardPage getCurrentPage(){
		return ((WizardPage)deck.getWidget(screenIndex));
	}
	
	/**
	 * @return the current active page number. 0 is the first one.
	 */
	protected int getCurrentPageNumber() {
		return screenIndex;
	}

	protected int getPageCount() {
		return deck.getWidgetCount();
	}

}
