/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.home.core;


import java.util.List;

import uk.co.maxant.gwtdemo.client.ApplicationLayerService;
import uk.co.maxant.gwtdemo.client.home.core.IHome;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.util.AsyncCallbackImpl;
import uk.co.maxant.gwtdemo.client.widgets.core.WizardServerCaller;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * delegated to by controller
 */
public class HomeServerCaller extends WizardServerCaller {

	private IHome home;
	private HomeModel model;
	
	public HomeServerCaller(IHome home, HomeModel model, AppWizard wizard){
		super(wizard, home);
		this.home = home;
		this.model = model;
	}
	
	/**
	 * @see ApplicationLayerService#getFeaturedDesigns()
	 */
	public void getFeaturedDesigns() {
		AsyncCallback<List<Design>> callback = new AsyncCallbackImpl<List<Design>>(this) {

			@Override
			protected boolean onFailurePre(Throwable caught) {
				handleFailedReload();
				return false;
			}

			public void onSuccess_(List<Design> designs) {
				wizard.getModel().setFeaturedDesigns(designs);
				home.setFeaturedDesigns(designs);
			}
		};
		callServer();
		getServiceClient().getFeaturedDesigns(callback);
	}

	/**
	 * @see ApplicationLayerService#getArtistViewCloud()
	 */
	public void getCloud() {
		AsyncCallback<List<Artist>> callback = new AsyncCallbackImpl<List<Artist>>(this) {

			public void onSuccess_(List<Artist> artists) {
				wizard.getModel().setCloud(artists);
				home.setCloud(artists);
			}
		};
		callServer();
		getServiceClient().getArtistViewCloud(callback);
	}

	/**
	 * @see ApplicationLayerService#getNewestDesigns()
	 */
	public void getNewestDesigns() {
		AsyncCallback<List<Design>> callback = new AsyncCallbackImpl<List<Design>>(this) {

			public void onSuccess_(List<Design> designs) {
				wizard.getModel().setNewestDesigns(designs);
				home.setNewestDesigns(designs);
			}
		};
		callServer();
		getServiceClient().getNewestDesigns(callback);
	}

	private void handleFailedReload() {
		//we dont really care about timeouts or errors.
		//so simply fail silently.
		//also need to cancel timers, since we arent able to continue.
		home.cancelTimers();
	}
}

