/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.home.ui;


import java.util.ArrayList;
import java.util.List;

import org.gwt.advanced.client.ui.widget.border.RoundCornerBorder;

import uk.co.maxant.gwtdemo.client.generic.widgets.Button;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.TextBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.home.core.HomeController;
import uk.co.maxant.gwtdemo.client.home.core.IHome;
import uk.co.maxant.gwtdemo.client.images.Images;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.OrderState;
import uk.co.maxant.gwtdemo.client.to.SessionInfo;
import uk.co.maxant.gwtdemo.client.widgets.core.MasterDataListener;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.dom.client.KeyPressHandler;
import com.google.gwt.user.client.Random;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * the home screen, which shows top selling designs and links.
 */
public class Home extends WizardPage implements IHome, MasterDataListener {

	private static final int ROWS_FEATURED = 2;

	private static final int COLS_FEATURED = 3;

	private static final int ROWS_NEWEST = 3;

	private static final int COLS_NEWEST = 2;

	private List<uk.co.maxant.gwtdemo.client.generic.widgets.Image> featuredImages = new ArrayList<uk.co.maxant.gwtdemo.client.generic.widgets.Image>();

	private List<uk.co.maxant.gwtdemo.client.generic.widgets.Image> newestImages = new ArrayList<uk.co.maxant.gwtdemo.client.generic.widgets.Image>();

	private FlexTable featuredInnerTable;

	private FlexTable newestInnerTable;
	
	private HomeController controller;

	private FlowPanel cloud; //TODO 000 this needs to be a different component since flowPanel dont work with HTMLs :-(

	private FlexTable searchArea;

	private Timer refreshFeaturedDesignsTimer;

	private Timer refreshSpotlightTimer;

	private uk.co.maxant.gwtdemo.client.generic.widgets.Image imgSpotlight;
	
	/**
	 */
	public Home(final AppWizard wizard){
		super(wizard);

		controller = new HomeController(this, wizard);

		/*
		 *   -------------------------------------------
		 *   | logo | menu...                          |
		 *   -------------------------------------------
		 *   | about  |                     | tag      |
		 *   |    us  |                     | cloud    |
		 *   |        |                     +----------|
		 *   |--------+  featured designs   |          |
		 *   |search  |                     |  new     |
		 *   |--------+                     |  designs |
		 *   |        +---------------------+          |
		 *   | news   |   spotlight         |          |
		 *   |        |                     |          |
		 *   -------------------------------------------
		 */

		HorizontalPanel main = UIFactory.createHorizontalPanel();
		VerticalPanel left = UIFactory.createVerticalPanel();
		VerticalPanel centre = UIFactory.createVerticalPanel();
		VerticalPanel right = UIFactory.createVerticalPanel();
		main.add(left);
		main.add(centre);
		main.add(right);

		/***************************************************
		 * about us
		 */
		//TODO i18n - demo:
		//demo of how to use i18n. based upon the locale request parameter
		//HTML lbl = UIFactory.createHTML(I18NHelper.getMessages().welcome("!"));

		//TODO 000 load from DB and allow admin to exit
		VerticalPanel vp = UIFactory.createVerticalPanel();
		HTML lbl = UIFactory.createHTML("<h1>Welcome to our shop!</h1>" + OrderState.AWAITING_PICKING.getName());
		vp.add(lbl);
		lbl = UIFactory.createHTML(
				"Here the business will specify some text like " +
				"the mission statement. If I had some latin, I would put it here " +
				"to, the idea being that this should be a fairly long paragraph. " +
				"Eum eros consequat lorem aliquam et ad luptatum illum sit qui ut, " +
				"tation enim dolore euismod et iusto nulla autem wisi odio quis nisl. ");
		lbl.setWidth("200px");
		vp.add(lbl);
		
		RoundCornerBorder border = UIFactory.createRoundCornerBorder();
		border.setWidget(vp);
		left.add(border);

		/***************************************************
		 * search
		 */
		buildSearchArea();
		border = UIFactory.createRoundCornerBorder();
		border.setWidget(searchArea);
		left.add(border);

		/***************************************************
		 * news
		 */
		//TODO 000 load from DB and allow admin to add / edit / delete
		vp = UIFactory.createVerticalPanel();
		lbl = UIFactory.createHTML("<h1>News</h1>");
		vp.add(lbl);
		ScrollPanel sp = UIFactory.createScrollPanel();
		sp.setPixelSize(220/*20 more than aboutUs, because of vScrollBar*/, 200);
		String s = 
				"<h2>Release 0.75</h2>" +
				"Slowly but surely getting there!" +
				"<h2>Release 0.67</h2>" +
				"This release went live a few nights ago but immediately needed an update because " +
				"of problems with the search and how it was shown on the front page. Ut diam commodo ad eu in consequat esse in ut facilisis aliquip dolore feugiat facilisis facilisi qui esse vel nostrud, at veniam eum euismod, diam. Eum eros consequat lorem aliquam et ad luptatum illum sit qui ut, tation enim dolore euismod et iusto nulla autem wisi odio quis nisl. Enim adipiscing luptatum minim nisl, quis eu vel qui quis sit dolor eros hendrerit dignissim ex. Aliquip in, duis et delenit ullamcorper at feugiat ullamcorper ullamcorper dolor vulputate iusto esse luptatum te autem esse dolore qui, te. Dolore et at nisl luptatum in exerci blandit illum augue ut dolor illum facilisis zzril commodo, augue iriure, autem augue augue molestie lobortis velit laoreet. Wisi ad, adipiscing aliquip, et consequat vel aliquip in dolor te, feugait suscipit. In vel erat duis et diam commodo ad eu in consequat esse in ut facilisis aliquip dolore feugiat facilisis facilisi qui." +
				"<h2>Release 0.66</h2>" +
				"Sed accumsan tincidunt, feugait ullamcorper, consequat lobortis veniam wisi augue, nisl diam esse. Ea consequat delenit eum iriure dolore ut ut nulla ex iusto et. Magna ex tincidunt illum luptatum, eu te in in feugiat eros augue sit. Delenit, blandit, odio duis duis illum aliquip duis at ipsum quis esse at et feugait, elit lorem velit hendrerit eum, ut feugiat commodo eu ut vel." +
				"<h2>Release 0.65</h2>" +
				"Euismod exerci feugait enim in vero commodo quis ex tincidunt illum elit ad, sed ut suscipit euismod autem, ullamcorper aliquip nulla. Dolor blandit nulla dignissim in tation consequat dolore, autem in iriure, iriure zzril, consectetuer odio suscipit veniam, wisi. Laoreet facilisi ut vero laoreet, dignissim vel nulla duis eu minim ea, nulla iriure feugiat, ut, enim lobortis. Luptatum delenit hendrerit, volutpat consectetuer nonummy esse sed ea esse, suscipit duis lobortis, iusto, dolore accumsan, et quis facilisis, molestie ea vel dolor. Iriure blandit dolor, et nostrud accumsan vel dignissim dolore magna nulla nibh et velit aliquam ut consequat dolor ut consequat autem. Dolore delenit volutpat exerci vulputate praesent eum magna ipsum velit hendrerit amet dolore hendrerit praesent zzril tation veniam." +
				"<h2>Release 0.64</h2>" +
				"At eum, ut feugiat commodo eu ut vel ullamcorper euismod autem qui facilisis in amet, vel esse ad ad commodo, vel facilisis. Ut in praesent dolore dolor, ullamcorper eros molestie dolor feugiat illum quis. Tation autem sit et exerci velit augue erat in nisl iusto, lobortis vulputate aliquam laoreet, dignissim qui dolor adipiscing ad qui tation feugait at diam. Nulla nostrud aliquip, exerci, wisi, erat luptatum luptatum adipiscing, nonummy nostrud zzril nisl. Sed accumsan tincidunt, feugait ullamcorper, consequat lobortis veniam wisi augue, nisl diam esse. Ea consequat delenit eum iriure dolore ut ut nulla ex iusto et." +
				"<h2>Release 0.63</h2>" +
				"Ex nulla nibh et velit aliquam ut consequat dolor ut consequat. Sit te delenit volutpat exerci vulputate praesent eum magna ipsum velit hendrerit amet dolore. Et praesent zzril tation veniam molestie at vero enim dolore vulputate, nisl eros feugait. Euismod exerci feugait enim in vero commodo quis ex tincidunt illum elit ad, sed ut suscipit euismod autem, ullamcorper aliquip nulla. " +
				"";
		HTML news = UIFactory.createHTML(s);
		sp.add(news);
		vp.add(sp);
		border = UIFactory.createRoundCornerBorder();
		border.setWidget(vp);
		left.add(border);
		
		
		/***************************************************
		 * featured designs
		 */
		vp = UIFactory.createVerticalPanel();
		lbl = UIFactory.createHTML("Featured Designs:");
		lbl.setStyleName(UIFactory.BIG_BOLD_TEXT);
		vp.add(lbl);
		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		createFeaturedInnerTable();
		featuredInnerTable.setWidth("350px");
		hp.add(featuredInnerTable);

		Image more = UIFactory.createImageButton("/images/ui/nextUp.jpg", "/images/ui/nextDown.jpg");
		more.setTitle("See more featured designs...");
		hp.add(more);
		more.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				//cancel the timer, since the user is now in control! otherwise it might fire in a half a second and thats not nice
				refreshFeaturedDesignsTimer.cancel();
				controller.getFeaturedDesigns();
			}
		});
		hp.add(more);
		vp.add(hp);
		border = UIFactory.createRoundCornerBorder();
		border.setWidget(vp);
		centre.add(border);

		/***************************************************
		 * spot light
		 */
		//TODO add admin ability to upload spotlights
		vp = UIFactory.createVerticalPanel();
		lbl = UIFactory.createHTML("In the Spotlight");
		lbl.setStyleName(UIFactory.BIG_BOLD_TEXT);
		vp.add(lbl);
		imgSpotlight = UIFactory.createImage();
		setSpotlight();
		vp.add(imgSpotlight);
		border = UIFactory.createRoundCornerBorder();
		border.setWidget(vp);
		centre.add(border);

		/***************************************************
		 * create cloud for most viewed artists
		 */
		//TODO turn this into a widget of its own
		vp = UIFactory.createVerticalPanel();
		vp.add(UIFactory.createHTML("Other people have been viewing these artists:"));
		cloud = UIFactory.createFlowPanel();
		cloud.setWidth("220px");
		vp.add(cloud);
		border = UIFactory.createRoundCornerBorder();
		border.setWidget(vp);
		right.add(border);

		/***************************************************
		 * newest designs
		 */
		vp = UIFactory.createVerticalPanel();
		lbl = UIFactory.createHTML("Newest Designs:");
		lbl.setStyleName(UIFactory.BIG_BOLD_TEXT);
		vp.add(lbl);
		hp = UIFactory.createHorizontalPanel();
		createNewestInnerTable();
		hp.add(newestInnerTable);
		vp.add(hp);
		border = UIFactory.createRoundCornerBorder();
		border.setWidget(vp);
		right.add(border);
		
		
		//tell the composite (this) which widget its wrapping
		initWidget(main);
		
		//now populate the cloud and featured designs
		controller.getCloud();
		controller.getNewestDesigns();
		controller.getFeaturedDesigns();
		
		refreshFeaturedDesignsTimer = new Timer() {
			@Override
			public void run() {
				controller.getFeaturedDesigns();
			}
		};

		refreshSpotlightTimer = new Timer() {
			@Override
			public void run() {
				setSpotlight();
			}
		};

		wizard.addMasterDataListener(this);
	}

	/**
	 * {@inheritDoc}
	 */
	public void cancelTimers() {
		refreshFeaturedDesignsTimer.cancel();
	}	
	
	private void setSpotlight() {
		SessionInfo sessionInfo = getWizard().getModel().getSessionInfo();
		if(sessionInfo != null){
			String[] spotlights = sessionInfo.getSpotlights();
			int n = Random.nextInt(spotlights.length);
			imgSpotlight.setUrl("/images/spotlight/" + spotlights[n]);
		}
	}

	private void buildSearchArea() {
		searchArea = UIFactory.createFlexTable();

		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		HTML l = UIFactory.createHTML("Search:");
		hp.add(l);
		Image img = Images.images.help().createImage();
		img.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				UIFactory.createInformationDialog("Information", 
						"Enter any text or numbers (e.g. design ID) to search for.<br>" +
						"Enter an artists name to search for all of their designs.<br>" +
						"Hit enter/return to do the search, and use the links below to view the search " +
						"results."
						).center();
			}
		});
		hp.add(img);

		final TextBox searchField = UIFactory.createTextBox();
		searchField.addKeyPressHandler(new KeyPressHandler(){
			public void onKeyPress(KeyPressEvent event) {
				if(event.getCharCode() == '\r' ||
						event.getCharCode() == '\n'){
					
					//return was hit - do search!
					getWizard().doSearch(searchField.getText().trim());
				}
			}
		});
		hp.add(searchField);

		img = Images.images.searchicon().createImage();
		img.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				getWizard().doSearch(searchField.getText().trim());
			}
		});
		hp.add(img);
		hp.setVerticalAlignment(HasVerticalAlignment.ALIGN_MIDDLE);

		searchArea.setWidget(1, 0, hp);
	}

	public void activated(){
		refreshFeaturedDesignsTimer.scheduleRepeating(10000);
		refreshSpotlightTimer.scheduleRepeating(7000);
	}

	public void deactivated() {
		refreshFeaturedDesignsTimer.cancel();
		refreshSpotlightTimer.cancel();
	}
	
	/**
	 * the inner table contains the designs. this method builds that table.
	 */
	private void createFeaturedInnerTable(){
		featuredInnerTable = UIFactory.createFlexTable();
		for(int i = 0; i < ROWS_FEATURED; i++){
			for(int j = 0; j < COLS_FEATURED; j++){
				uk.co.maxant.gwtdemo.client.generic.widgets.Image img = UIFactory.createImage();
				featuredImages.add(img);
				img.setStyleName(UIFactory.POINTER_CURSOR);
				img.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						uk.co.maxant.gwtdemo.client.generic.widgets.Image i = (uk.co.maxant.gwtdemo.client.generic.widgets.Image)event.getSource();
						Design d = (Design)i.getData();
						controller.showDesignInShop(d);
					}
				});

				//put image in an HP to enable DND
				HorizontalPanel hp = UIFactory.createHorizontalPanel();
				hp.setSize("105 px", "105 px"); //TODO necessary? better solution?
				hp.add(img);
				featuredInnerTable.setWidget(i, j, hp);
			}
		}
	}

	/**
	 * the inner table contains the designs. this method builds that table.
	 */
	private void createNewestInnerTable(){
		newestInnerTable = UIFactory.createFlexTable();
		for(int i = 0; i < ROWS_NEWEST; i++){
			for(int j = 0; j < COLS_NEWEST; j++){
				uk.co.maxant.gwtdemo.client.generic.widgets.Image img = UIFactory.createImage();
				newestImages.add(img);
				img.setStyleName(UIFactory.POINTER_CURSOR);
				img.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						uk.co.maxant.gwtdemo.client.generic.widgets.Image i = (uk.co.maxant.gwtdemo.client.generic.widgets.Image)event.getSource();
						Design d = (Design)i.getData();
						controller.showDesignInShop(d);
					}
				});

				//put image in an HP to enable DND
				HorizontalPanel hp = UIFactory.createHorizontalPanel();
				hp.setSize("105 px", "105 px"); //TODO necessary? better solution?
				hp.add(img);
				newestInnerTable.setWidget(i, j, hp);
			}
		}
	}

	/**
	 * @see IHome#setFeaturedDesigns(List)
	 */
	public void setFeaturedDesigns(List<Design> designs){

		int maxImages = ROWS_FEATURED * COLS_FEATURED;
		if(maxImages > featuredImages.size()){
			maxImages = featuredImages.size();
		}

		for(int i = 0; i < maxImages; i++){
			uk.co.maxant.gwtdemo.client.generic.widgets.Image img = featuredImages.get(i);
			if(i >= designs.size()){
				img.setVisible(false);
			}else{
				img.setVisible(true); //in case it was hidden previously? not sure, but cant hurt...
				Design d = designs.get(i);
				img.setData(d);
				img.setUrl(d.getUrl());
				img.setTitle(d.getToolTip());
			}
		}

	}

	/**
	 * @see IHome#setNewestDesigns(List)
	 */
	public void setNewestDesigns(List<Design> designs) {
		
		int maxImages = ROWS_NEWEST * COLS_NEWEST;
		if(maxImages > newestImages.size()){
			maxImages = newestImages.size();
		}
		for(int i = 0; i < maxImages; i++){
			uk.co.maxant.gwtdemo.client.generic.widgets.Image img = newestImages.get(i);
			if(i >= designs.size()){
				img.setVisible(false);
			}else{
				img.setVisible(true); //in case it was hidden previously? not sure, but cant hurt...
				Design d = designs.get(i);
				img.setData(d);
				img.setUrl(d.getUrl());
				img.setTitle(d.getToolTip());
			}
		}
		
	}
	
	/**
	 * @see IHome#setCloud(List)
	 */
	public void setCloud(List<Artist> artists){
		//build a list containing the names as links. build like this: 9 7 5 3 1 2 4 6 8
		//that way, when its wrapped it will look cool and the most important ones will
		//be in the middle
		boolean addAtFront = true;
		List<Button> arrangedList = new ArrayList<Button>();
		int i = 0;
		for(Artist a : artists){
			final uk.co.maxant.gwtdemo.client.generic.widgets.Button link = UIFactory.createButton(a.getName());
			link.setStyleName(getCloudStyle(i++));
			link.setData(a);
			link.addClickHandler(new ClickHandler(){
				public void onClick(ClickEvent event) {
					Artist a = (Artist)link.getData();
					controller.showArtistInShop(a);
				}
			});
			if(addAtFront){
				arrangedList.add(0, link);
			}else{
				arrangedList.add(link);
			}
			addAtFront = !addAtFront;
		}

		//now build graphics
		cloud.clear();
		for(Button h : arrangedList){
			cloud.add(h);
		}
		
	}

	private String getCloudStyle(int i) {
		switch (i) {
		case 0:
			return UIFactory.CLOUD_1;
		case 1:
		case 2:
			return UIFactory.CLOUD_2;
		case 3:
		case 4:
			return UIFactory.CLOUD_3;
		case 5:
		case 6:
			return UIFactory.CLOUD_4;
			
		default:
			return UIFactory.CLOUD_5;
		}
	}

	public void notifyMasterDataIsReady() {
		setSpotlight();
	}
}
