/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.order_admin_list.core;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.user.client.ui.UIObject;

import uk.co.maxant.gwtdemo.client.generic.widgets.FilterBox.FilterableUIObject;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;

public class OrderFilterableUIObject extends FilterableUIObject {
	
	public static final String STARTED_PICKING = "Started Picking";

	public static final String READY_FOR_DISPATCH = "Ready for Dispatch";

	public static final String PAYMENT_SUCCESSFUL = "Payment Successful";

	public static final String PLACED = "Placed";

	public static final String FINISHED_PICKING = "Finished Picking";

	public static final String AWAITING_PICKING = "Awaiting Picking";

	public static final String PAYMENT_FAILED = "Payment Failed";

	public static final String CANCELLED = "Cancelled";

	public static final String DELIVERED = "Delivered";

	public static final String DISPATCHED = "Dispatched";

	public static final String CREATED = "Created";

	public static final String DISCARDED = "Discarded";

	private SalesOrder salesOrder;
	
	public OrderFilterableUIObject(UIObject uiObject, SalesOrder salesOrder){
		super(uiObject);
		this.salesOrder = salesOrder;
	}
	
	@Override
	public String[] getTagsToCompare() {
		List<String> selectables = new ArrayList<String>();
		selectables.add(String.valueOf(salesOrder.getUid()));
		selectables.add(String.valueOf(salesOrder.getCustomer().getUid()));
		selectables.add(salesOrder.getEmail());
		selectables.add(salesOrder.getShopNotes());
		selectables.add(String.valueOf(salesOrder.getPickedBy()));
		if(salesOrder.getStatusAwaitingPicking() != null){
			selectables.add(AWAITING_PICKING);
		}
		if(salesOrder.getStatusFinishedPicking() != null){
			selectables.add(FINISHED_PICKING);
		}
		if(salesOrder.getStatusOrderPlaced() != null){
			selectables.add(PLACED);
		}
		if(salesOrder.getStatusPaymentSuccessful() != null){
			selectables.add(PAYMENT_SUCCESSFUL);
		}
		if(salesOrder.getStatusPaymentFailed() != null){
			selectables.add(PAYMENT_FAILED);
		}
		if(salesOrder.getStatusReadyForDispatch() != null){
			selectables.add(READY_FOR_DISPATCH);
		}
		if(salesOrder.getStatusStartedPicking() != null){
			selectables.add(STARTED_PICKING);
		}
		if(salesOrder.getStatusDiscarded() != null){
			selectables.add(DISCARDED);
		}
		
		return selectables.toArray(new String[]{});
	}
}
