/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.order_admin_list.core;

import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

public class OrdersListController {

	protected IOrdersList view;
	protected OrdersListServerCaller serverCaller;
	protected OrdersListModel model;
	
	public OrdersListController(IOrdersList view, AppWizard wizard){
		this.view = view;
		this.model = new OrdersListModel();
		this.serverCaller = new OrdersListServerCaller(view, model, wizard);
	}
	
	/**
	 * causes the pending sales orders to be loaded.
	 */
	public void setSalesOrders(SEARCH_TYPE type) {
		
		if(type.equals(SEARCH_TYPE.ADMIN_SEARCH)){
			throw new IllegalArgumentException("to search, call the #setSalesOrders(String) method - code change required");
		}
		
		serverCaller.getSalesOrders(type);
	}

	/**
	 * causes the sales orders to be loaded.
	 */
	public void setSalesOrders(String search) {
		serverCaller.getSalesOrders(search);
	}
	
	public OrdersListModel getModel() {
		return model;
	}

	/**
	 * type of orders to load
	 */
	public static enum SEARCH_TYPE {
		
		/** search for all matching the given string */
		ADMIN_SEARCH,

		/** search for all pending orders */
		ADMIN_PENDING,

		/** search for all problem orders */
		ADMIN_PROBLEMS,
		
		/** search for all orders from the given customer */
		CUSTOMER, 
		
		/** search for all discarded orders */
		ADMIN_DISCARDED;

	}

}
