/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.order_admin_list.core;


import java.util.List;

import uk.co.maxant.gwtdemo.client.order_admin_list.core.OrdersListController.SEARCH_TYPE;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.util.AsyncCallbackImpl;
import uk.co.maxant.gwtdemo.client.widgets.core.WizardServerCaller;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * delegated to by controller
 */
public class OrdersListServerCaller extends WizardServerCaller {

	private IOrdersList designList;
	private OrdersListModel model;
	
	public OrdersListServerCaller(IOrdersList designList, OrdersListModel model, AppWizard wizard){
		super(wizard, designList);
		this.designList = designList;
		this.model = model;
	}
	
	/**
	 * called to set the sales orders
	 */
	public void getSalesOrders(SEARCH_TYPE type) {
		getSalesOrders(type, null);
	}

	/**
	 * called to search the sales orders
	 */
	public void getSalesOrders(String search) {
		getSalesOrders(SEARCH_TYPE.ADMIN_SEARCH, search);
	}

	private void getSalesOrders(SEARCH_TYPE type, String search) {
		
		AsyncCallback<List<SalesOrder>> callback = new AsyncCallbackImpl<List<SalesOrder>>(this) {
			
			@Override
			protected void onFailurePost(Throwable caught) {
				designList.setSalesOrdersFailed();
			}

			public void onSuccess_(List<SalesOrder> designs) {
				model.setSalesOrders(designs);
				designList.setSalesOrders();
			}
		};
		callServer();
		
		if(type.equals(SEARCH_TYPE.ADMIN_PENDING)){
			getServiceClient().getPendingSalesOrders(callback);
		}else if(type.equals(SEARCH_TYPE.ADMIN_PROBLEMS)){
			getServiceClient().getProblemSalesOrders(callback);
		}else if(type.equals(SEARCH_TYPE.ADMIN_DISCARDED)){
			getServiceClient().getDiscardedSalesOrders(callback);
		}else if(type.equals(SEARCH_TYPE.ADMIN_SEARCH)){
			getServiceClient().searchSalesOrders(search, callback);
		}else if(type.equals(SEARCH_TYPE.CUSTOMER)){
			getServiceClient().getCustomerSalesOrders(callback);
		}else{
			throw new RuntimeException("unknown type " + type + " - code change required");
		}
	}

}

