/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.order_admin_list.ui;


import uk.co.maxant.gwtdemo.client.Roles;
import uk.co.maxant.gwtdemo.client.generic.widgets.AsyncFilterBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.FilterBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.images.Images;
import uk.co.maxant.gwtdemo.client.order_admin_list.core.IOrdersList;
import uk.co.maxant.gwtdemo.client.order_admin_list.core.OrderFilterableUIObject;
import uk.co.maxant.gwtdemo.client.order_admin_list.core.OrdersListController;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.util.ClientFormats;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPageWithListClickHandling;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * the order admin screen, which lets admin manage orders.
 */
public abstract class OrdersAdminList extends WizardPageWithListClickHandling<SalesOrder> implements IOrdersList {

	private static final int MAX_ORDERS_TO_SHOW = 20;

	protected OrdersListController controller;

	protected FlexTable orderArea;
	
	private final boolean useAsyncFilter;
	
	protected AsyncFilterBox afb;

	protected HTML tooManyDesignsWarning;
	
	/**
	 */
	public OrdersAdminList(final AppWizard wizard, boolean useAsyncFilter){
		super(wizard);
		
		this.useAsyncFilter = useAsyncFilter;

		controller = new OrdersListController(this, wizard);

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(0);
		
		int r = 0;
		int c = 0;
		
		/***************************************************
		 * title
		 */
		String s = "<h2>Sales Order List</h2>" + getDescription();
		HTML l = UIFactory.createHTML(s);
		table.setWidget(r, c, l);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;
		
		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;

			
		/***************************************************
		 * filter input area
		 */
		tooManyDesignsWarning = UIFactory.createHTML("Too many sales orders to display. Please use filter!");
		tooManyDesignsWarning.setStyleName(UIFactory.GENERAL_ERROR);
		tooManyDesignsWarning.setVisible(false); //default is off
		if(useAsyncFilter){
			HorizontalPanel hp = UIFactory.createHorizontalPanel();
			afb = new AsyncFilterBox(
					"Filter",
					"Enter a regular expression to filter orders",
					"matches: ",
					"clear",
					"info",
					'f',
					200,
					500
				){
				
				@Override
				protected void doServerCall(String filter) {
					controller.setSalesOrders(filter);
				}
			};
			hp.add(afb);
			hp.add(tooManyDesignsWarning);
			table.setWidget(r, c, hp);
			r++; c = 0;
		}
		
		/***************************************************
		 * order area
		 */
		orderArea = UIFactory.createFlexTable();
		table.setWidget(r, c, orderArea);
		r++; c = 0;

		//tell the composite (this) which widget its wrapping
		initWidget(table);
	}
	
	protected abstract String getDescription();

	/**
	 * @see IOrdersList#setSalesOrdersFailed()
	 */
	public void setSalesOrdersFailed() {
		if(useAsyncFilter){
			afb.serverCallComplete(0);
		}
	}
	
	/**
	 * @see IOrdersList#setSalesOrders()
	 */
	public void setSalesOrders(){

		if(useAsyncFilter){
			afb.serverCallComplete(controller.getModel().getSalesOrders().size());
		}

		orderArea.clear();
		
		int r = 0;
		int c = 0;
		if(controller.getModel().getSalesOrders().isEmpty()){
			HTML html = UIFactory.createHTML("No orders found.");
			orderArea.setWidget(r, c, html);
		}else{

			//limit how much we show on screen, because of browser resources.
			if(useAsyncFilter && controller.getModel().getSalesOrders().size() > MAX_ORDERS_TO_SHOW){
				tooManyDesignsWarning.setVisible(true);
			}else{
				tooManyDesignsWarning.setVisible(false);
			}


			int numCols = 7;

			/***************
			 * filter
			 */
			FilterBox sfb = null;
			if(!useAsyncFilter){
				HorizontalPanel hp = UIFactory.createHorizontalPanel();
				sfb = UIFactory.createFilterBox(
						"Filter",
						"Enter a regular expression to orders",
						"matches: ",
						"clear",
						"info",
						'f',
						200,
						numCols
				);
				hp.add(sfb);
				orderArea.setWidget(r, c, hp);
				orderArea.getFlexCellFormatter().setColSpan(r, c, numCols);
				r++; c = 0;
			}

			final FlexTable table = UIFactory.createBorderedFlexTable();
			orderArea.setWidget(r, c, table);
			orderArea.getFlexCellFormatter().setColSpan(r, c, numCols);
			r++; c = 0;

			int r2 = 0;
			int c2 = 0;
			HTML l = UIFactory.createHTML("UID");
			table.setWidget(r2, c2, l); c2++;
			l = UIFactory.createHTML("Customer");
			table.setWidget(r2, c2, l); c2++;
			l = UIFactory.createHTML("Placed");
			table.setWidget(r2, c2, l); c2++;
			l = UIFactory.createHTML("Status");
			table.setWidget(r2, c2, l); c2++;
			l = UIFactory.createHTML("Payment");
			table.setWidget(r2, c2, l); c2++;
			l = UIFactory.createHTML("Attention");
			table.setWidget(r2, c2, l); c2++;
			l = UIFactory.createHTML("Total");
			table.setWidget(r2, c2, l); c2++;

			table.getRowFormatter().setStyleName(r2, UIFactory.LIST_HEADER);
			
			r2++; c2 = 0;
			
			int i2 = 0;
			for(SalesOrder so : controller.getModel().getSalesOrders()){

				if(useAsyncFilter && i2++ >= MAX_ORDERS_TO_SHOW){
					break;
				}
				
				/*********************************************
				 * id
				 */
				l = UIFactory.createHTML();
				l.setHTML(String.valueOf(so.getUid()));
				table.setWidget(r2, c2, l);
				if(!useAsyncFilter) sfb.addFilterableObject(new OrderFilterableUIObject(l, so));
				c2++;

				/*********************************************
				 * customer
				 */
				l = UIFactory.createHTML(so.getCustomer().getEmail());
				table.setWidget(r2, c2, l);
				if(!useAsyncFilter) sfb.addFilterableObject(new OrderFilterableUIObject(l, so));
				c2++;
				
				/*********************************************
				 * placed / created / discarded / unknown
				 */
				String s = null;
				if(so.getStatusOrderPlaced() == null){
					if(so.getStatusOrderCreated() == null){
						if(so.getStatusDiscarded() == null){
							s = "Unknown";
						}else{
							s = ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusDiscarded());
						}
					}else{
						s = ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusOrderCreated());
					}
				}else{
					s = ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusOrderPlaced());
				}
				l = UIFactory.createHTML(s);
				table.setWidget(r2, c2, l);
				if(!useAsyncFilter) sfb.addFilterableObject(new OrderFilterableUIObject(l, so));
				c2++;
				
				/*********************************************
				 * status
				 */
				String status = "Unknown";
				if(so.getStatusCancelled() != null){
					status = OrderFilterableUIObject.CANCELLED;
				}else if(so.getStatusDelivered() != null){
					status = OrderFilterableUIObject.DELIVERED;
				}else if(so.getStatusDispatched() != null){
					status = OrderFilterableUIObject.DISPATCHED;
				}else if(so.getStatusReadyForDispatch() != null){
					status = OrderFilterableUIObject.READY_FOR_DISPATCH;
				}else if(so.getStatusFinishedPicking() != null){
					status = OrderFilterableUIObject.FINISHED_PICKING;
				}else if(so.getStatusStartedPicking() != null){
					status = OrderFilterableUIObject.STARTED_PICKING;
				}else if(so.getStatusAwaitingPicking() != null){
					status = OrderFilterableUIObject.AWAITING_PICKING;
				}else if(so.getStatusOrderPlaced() != null){
					status = OrderFilterableUIObject.PLACED;
				}else if(so.getStatusDiscarded() != null){
					status = OrderFilterableUIObject.DISCARDED;
				}else if(so.getStatusOrderCreated() != null){
					status = OrderFilterableUIObject.CREATED;
				}
				l = UIFactory.createHTML(status);
				table.setWidget(r2, c2, l);
				if(!useAsyncFilter) sfb.addFilterableObject(new OrderFilterableUIObject(l, so));
				c2++;
				
				/*********************************************
				 * payment status
				 */
				s = so.getPaymentStatusHTML();
				l = UIFactory.createHTML(s);
				table.setWidget(r2, c2, l);
				if(!useAsyncFilter) sfb.addFilterableObject(new OrderFilterableUIObject(l, so));
				c2++;
				
				/*********************************************
				 * attention
				 */
				Widget w = null;
				if(so.isNeedsAttention()){
					w = Images.images.needsAttention().createImage();
				}else{
					w = UIFactory.createHTML("&nbsp;");
				}
				table.setWidget(r2, c2, w);
				if(!useAsyncFilter) sfb.addFilterableObject(new OrderFilterableUIObject(w, so));
				c2++;
				
				/*********************************************
				 * total
				 */
				l = UIFactory.createHTML(ClientFormats.NF_0_00.format(so.getTotal()));
				table.setWidget(r2, c2, l);
				table.getFlexCellFormatter().setHorizontalAlignment(r2, c2, HasHorizontalAlignment.ALIGN_RIGHT);
				if(!useAsyncFilter) sfb.addFilterableObject(new OrderFilterableUIObject(l, so));

				
				/*********************************************
				 * row styling - set colour, then add mouseOver/mouseOut styles.
				 * also react to clicks.
				 */
				String style = UIFactory.LIST_ROW_EVEN_SELECTABLE;
				if(r2 % 2 != 0){
					style = UIFactory.LIST_ROW_ODD_SELECTABLE;
				}
				table.getRowFormatter().setStyleName(r2, style);
				for(int i = 0; i < table.getCellCount(r2); i++){
					addHandlers(table.getWidget(r2, i), so, table, r2, style);
				}
				
				r2++; c2 = 0;
			}
		}
	}

	@Override
	public void handleClickOnTable(ClickEvent event, SalesOrder so, Object extra) {
		if(getWizard().getModel().getSessionInfo().isInRole(Roles.PICKING)){
			getWizard().showOrderAdminDetailPage(so);
		}else{
			getWizard().showOrderCustomerDetailPage(so);
		}
	}
}
