/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.order_complete.ui;

import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.order_complete.core.IOrderComplete;
import uk.co.maxant.gwtdemo.client.order_complete.core.OrderCompleteController;
import uk.co.maxant.gwtdemo.client.to.PaypalConfig;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;

import com.google.gwt.http.client.URL;
import com.google.gwt.user.client.Timer;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HTML;

/**
 * the "order complete" page in the wizard. ie, just before redirecting to paypal.
 */
public class OrderComplete extends WizardPage implements IOrderComplete {

	private HTML result;
	
	private OrderCompleteController controller;
	
	public OrderComplete(AppWizard wizard){
		super(wizard);

		controller = new OrderCompleteController(this, wizard);

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(0);

		int r = 0;
		int c = 0;
		
		result = UIFactory.createHTML();
		table.setWidget(r, c, result);
		r++; c = 0;

		
		//tell the composite (this) which widget its wrapping
		initWidget(table);
	}
	
	@Override
	public void activated() {
		result.setHTML("Order " + getWizard().getModel().getOrderUid() + " is being created..." +
				"<br><br>" +
				"<img src='/images/wait.gif' />Redirecting to Paypal for payment...");
		
		controller.redirectToPaypal(getWizard().getModel().getOrderUid());
	}
	
	/**
	 * @see IOrderComplete#configLoaded(PaypalConfig)
	 */
	public void configLoaded(PaypalConfig config) {
		//from Table A.2 in PP_WebsitePaymentsStandard_IntegrationGuide.pdf
		String paypalUrl = config.getPaypalURL();
		paypalUrl += "?business=" + URL.encode(config.getPaypalMerchantEmail());
		paypalUrl += "&cmd=" + URL.encode("_ext-enter");
		paypalUrl += "&redirect_cmd=" + URL.encode("_xclick");
		paypalUrl += "&amount=" + URL.encode(config.getTotal());
		paypalUrl += "&item_name=" + URL.encode(config.getItemName());
		paypalUrl += "&item_number=" + URL.encode(config.getOrderUid());
		paypalUrl += "&cancel_return=" + URL.encode(config.getCancelReturnURL());
		paypalUrl += "&email=" + URL.encode(config.getEmail());

		paypalUrl += "&cbt=" + URL.encode(config.getContinueButtonText());
		paypalUrl += "&cpp_header_image=" + URL.encode(config.getLogo());
		paypalUrl += "&cpp_headerback_color=ffffff";
		paypalUrl += "&cpp_headerborder_color=ffffff";
		paypalUrl += "&cpp_payflow_color=ffffff";

		paypalUrl += "&no_note=1";
		paypalUrl += "&return=" + URL.encode(config.getReturnURL());
		paypalUrl += "&notify_url=" + URL.encode(config.getIpnURL());
		paypalUrl += "&rm=2"; //return using POST with all tx data
		paypalUrl += "&currency_code=" + URL.encode(config.getPaypalCurrency());
		paypalUrl += "&invoice=" + URL.encode(config.getOrderUid());
		paypalUrl += "&lc=" + URL.encode(config.getLanguage());
		paypalUrl += "&no_shipping=" + URL.encode(config.getAllowShippingAddress());
		paypalUrl += "&shipping=" + URL.encode(config.getShipping());
		
		final String paypalUrlf = paypalUrl;

		//so you even get to see the redirect message, pause for a second...
		Timer t = new Timer(){
			@Override
			public void run() {
				Window.Location.assign(paypalUrlf);
			}
		};
		t.schedule(1000);
	}
	
}
