/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.order_detail.core;

import uk.co.maxant.gwtdemo.client.to.OrderState;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.util.AsyncCallbackImpl;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * delegated to by controller
 */
public class OrderAdminDetailServerCaller extends OrderCustomerDetailServerCaller {

	private IOrderAdminDetail orderAdminDetail;
	
	public OrderAdminDetailServerCaller(IOrderAdminDetail orderAdminDetail, OrderDetailModel model, AppWizard wizard){
		super(orderAdminDetail, model, wizard);
		this.orderAdminDetail = orderAdminDetail;
	}
	
	/**
	 * called to set the sales order
	 */
	public void getSalesOrder(String uid) {
		
		AsyncCallback<SalesOrder> callback = new AsyncCallbackImpl<SalesOrder>(this) {
			
			@Override
			protected void onFailurePost(Throwable caught) {
				orderAdminDetail.setSalesOrderFailed();
			}
			
			public void onSuccess_(SalesOrder so) {
				model.setSalesOrder(so);
				orderAdminDetail.setSalesOrder();
			}
		};
		callServer();
		getServiceClient().getSalesOrder(uid, callback);
	}

	/**
	 * called to update the sales order
	 */
	public void updatePaymentStatus(OrderState requiredState, final int uid, String extraNotes, String txnId) {
		
		AsyncCallback<Void> callback = new AsyncCallbackImpl<Void>(this) {
			
			public void onSuccess_(Void v) {
				//reload the order
				getSalesOrder(String.valueOf(uid));
			}
		};
		callServer();
		getServiceClient().updatePaymentStatus(requiredState, uid, extraNotes, txnId, callback);
	}
	
	/**
	 * called to update the sales order
	 */
	public void updateOrderStatus(OrderState requiredState, final int uid, String notes) {
		
		AsyncCallback<Void> callback = new AsyncCallbackImpl<Void>(this) {
			
			public void onSuccess_(Void v) {
				//reload the order
				getSalesOrder(String.valueOf(uid));
			}
		};
		callServer();
		getServiceClient().updateOrderStatus(requiredState, uid, notes, callback);
	}

	/**
	 * called to update the sales order
	 */
	public void addShopNotes(final int uid, String notes) {
		
		AsyncCallback<Void> callback = new AsyncCallbackImpl<Void>(this) {
			
			public void onSuccess_(Void v) {
				//reload the order
				getSalesOrder(String.valueOf(uid));
			}
		};
		callServer();
		getServiceClient().addShopNotes(uid, notes, callback);
	}
	
	/**
	 * called to update the sales order
	 */
	public void markOrderAsNeedingAttention(final int uid, String notes) {
		
		AsyncCallback<Void> callback = new AsyncCallbackImpl<Void>(this) {
			
			public void onSuccess_(Void v) {
				//reload the order
				getSalesOrder(String.valueOf(uid));
			}
		};
		callServer();
		getServiceClient().markOrderAsNeedingAttention(uid, notes, callback);
	}
	
	/**
	 * called to update the sales order
	 */
	public void markOrderAsNotNeedingAttention(final int uid, String notes) {
		
		AsyncCallback<Void> callback = new AsyncCallbackImpl<Void>(this) {
			
			public void onSuccess_(Void v) {
				//reload the order
				getSalesOrder(String.valueOf(uid));
			}
		};
		callServer();
		getServiceClient().markOrderAsNotNeedingAttention(uid, notes, callback);
	}
	
	
	
}

