/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.order_detail.ui;


import uk.co.maxant.gwtdemo.client.generic.widgets.AbstractInputDialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.AsyncFilterBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.images.Images;
import uk.co.maxant.gwtdemo.client.order_detail.core.IOrderAdminDetail;
import uk.co.maxant.gwtdemo.client.order_detail.core.OrderAdminDetailController;
import uk.co.maxant.gwtdemo.client.to.OrderState;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.to.SalesOrderItem;
import uk.co.maxant.gwtdemo.client.util.ClientFormats;
import uk.co.maxant.gwtdemo.client.util.StringUtil;
import uk.co.maxant.gwtdemo.client.widgets.ui.LongCommentInputDialog;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * the order admin screen, which lets admin manage orders.
 */
public class OrderAdminDetail extends WizardPage implements IOrderAdminDetail {

	private OrderAdminDetailController controller;

	private FlexTable orderArea;

	private AsyncFilterBox fb;

	/**
	 */
	public OrderAdminDetail(final AppWizard wizard){
		super(wizard);

		controller = new OrderAdminDetailController(this, wizard);

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(1);
		
		int r = 0;
		int c = 0;
		
		/***************************************************
		 * title
		 */
		String s = "<h2>Sales Order</h2>";
		s += "Administrators can view sales order details.";
		s += "<br><br>Update payment status or picking status as required.";
		HTML l = UIFactory.createHTML(s);
		table.setWidget(r, c, l);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;
		
		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;

		/***************************************************
		 * filter input area
		 */
		fb = new AsyncFilterBox(
				"Filter",
				"Enter the sales order to find",
				"matches: ",
				"clear",
				"info",
				'f',
				200,
				500
			){
			
			@Override
			protected void doServerCall(String filter) {
				controller.setSalesOrder(filter);
			}
		};
		table.setWidget(r, c, fb);
		r++; c = 0;

		/***************************************************
		 * order area
		 */
		orderArea = UIFactory.createFlexTable();
		orderArea.setBorderWidth(1);
		orderArea.setCellPadding(0);
		orderArea.setCellSpacing(5);
		table.setWidget(r, c, orderArea);
		r++; c = 0;

		//tell the composite (this) which widget its wrapping
		initWidget(table);
	}
	
	/**
	 * @see IOrderAdminDetail#setSalesOrderFailed()
	 */
	public void setSalesOrderFailed() {
		fb.serverCallComplete(0);
	}
	
	/**
	 * @see IOrderAdminDetail#setSalesOrder()
	 */
	public void setSalesOrder(){
		fb.serverCallComplete(1);
		
		orderArea.clear();
		
		int r = 0;
		int c = 0;
		final SalesOrder so = controller.getModel().getSalesOrder();
		if(so == null){
			HTML html = UIFactory.createHTML("No order found.");
			orderArea.setWidget(r, c, html);
		}else{
			// *************************
			// ID
			// *************************
			HTML html = UIFactory.createHTML("Order ID");
			html.setStyleName(UIFactory.BIG_BOLD_TEXT);
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;

			html = UIFactory.createHTML(String.valueOf(so.getUid()));
			html.setStyleName(UIFactory.BIG_BOLD_TEXT);
			orderArea.setWidget(r, c, html);
			r++; c = 0;

			// *************************
			// needs attention
			// *************************
			HorizontalPanel hp = UIFactory.createHorizontalPanel();
			Button change = UIFactory.createButton("");
			if(so.isNeedsAttention()){
				hp.add(Images.images.needsAttention().createImage());
				html = UIFactory.createHTML("Needs Special Attention - see history below");
				html.setStyleName(UIFactory.GENERAL_ERROR);
				hp.add(html);
				change.setText("Acknowledge attention given...");
			}else{
				change.setText("Mark as needing attention...");
			}
			hp.add(change);
			change.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					LongCommentInputDialog lcid = new LongCommentInputDialog(
							"Notes",
							"Please enter the details:",
							"Details may not be empty!",
							new AbstractInputDialog.Callback<String>(){
								public void onOK(String notes) {
									if(so.isNeedsAttention()){
										controller.markOrderAsNotNeedingAttention(notes);
									}else{
										controller.markOrderAsNeedingAttention(notes);
									}
								}
								public void onCancel() {
									//nothing to do
								}
							});
					lcid.center();
				}
			});
			orderArea.setWidget(r, c, hp);
			orderArea.getFlexCellFormatter().setColSpan(r, c, 2);
			r++; c = 0;
			
			// *************************
			// customer
			// *************************
			html = UIFactory.createHTML("Customer");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			html = UIFactory.createHTML(String.valueOf(so.getCustomer().getUid()) + "<br>" + so.getCustomer().getFullName() + "<br>" + so.getCustomer().getEmail());
			orderArea.setWidget(r, c, html);
			r++; c = 0;

			// *************************
			// email
			// *************************
			if(!so.getCustomer().getEmail().trim().equalsIgnoreCase(so.getEmail())){
				html = UIFactory.createHTML("Email for<br>this order");
				orderArea.setWidget(r, c, html);
				orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
				c++;
				
				html = UIFactory.createHTML(so.getEmail());
				orderArea.setWidget(r, c, html);
				r++; c = 0;
			}
			
			// *************************
			// payment type
			// *************************
			html = UIFactory.createHTML("Payment Type");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			html = UIFactory.createHTML(StringUtil.getHTMLSpaceIfNullOrEmpty(so.getPaymentType()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// delivery total
			// *************************
			html = UIFactory.createHTML("Delivery Total");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			double d = so.getDeliveryTotal();
			html = UIFactory.createHTML(ClientFormats.NF_0_00.format(d));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// delivery description
			// *************************
			html = UIFactory.createHTML("Delivery Description");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			html = UIFactory.createHTML(StringUtil.getHTMLSpaceIfNullOrEmpty(so.getDeliveryDescription()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// payment details
			// *************************
			html = UIFactory.createHTML("Payment Details");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			html = UIFactory.createHTML(StringUtil.getHTMLSpaceIfNullOrEmpty(so.getPaymentDetails())); //TODO format?
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// payment status
			// *************************
			html = UIFactory.createHTML("Payment Status");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;

			String s = so.getPaymentStatusHTML();
			html = UIFactory.createHTML(s);
			if(so.getStatusPaymentFailed() != null){
				html.setStyleName(UIFactory.GENERAL_ERROR);
			}
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// update payment status
			// *************************
			html = UIFactory.createHTML("Update Payment Status");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;

			hp = UIFactory.createHorizontalPanel();
			OrderState[] oss = OrderState.getNext(OrderState.getPaymentState(so));
			for(OrderState os : oss){
				final OrderState osf = os;
				Button button = UIFactory.createButton(os.getName());
				button.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						PaymentStatusChangedInputDialog pscid = new PaymentStatusChangedInputDialog(
								new AbstractInputDialog.Callback<PaymentStatusChangedInputDialog.Result>(){
									public void onOK(PaymentStatusChangedInputDialog.Result result) {
										controller.updatePaymentStatus(osf, result.comment, result.txnId);
									}
									public void onCancel() {
										//nothing to do
									}
								});
						pscid.center();
					}
				});
				hp.add(button);
			}
			orderArea.setWidget(r, c, hp);
			r++; c = 0;

			// *************************
			// order status
			// *************************
			html = UIFactory.createHTML("Order Status");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			s = "";
			if(so.getStatusOrderCreated() != null){
				s += "Created: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusOrderCreated()) + "<br>";
			}
			if(so.getStatusOrderPlaced() != null){
				s += "Placed: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusOrderPlaced()) + "<br>";
			}
			if(so.getStatusAwaitingPicking() != null){
				s += "Awaiting Picking: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusAwaitingPicking()) + "<br>";
			}
			if(so.getStatusStartedPicking() != null){
				s += "Started Picking: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusStartedPicking()) + "<br>";
			}
			if(so.getStatusFinishedPicking() != null){
				s += "Finished Picking: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusFinishedPicking()) + "<br>";
			}
			if(so.getStatusReadyForDispatch() != null){
				s += "Ready for Dispatch: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusReadyForDispatch()) + "<br>";
			}
			if(so.getStatusDispatched() != null){
				s += "Dispatched: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusDispatched()) + "<br>";
			}
			if(so.getStatusDelivered() != null){
				s += "Delivered: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusDelivered()) + "<br>";
			}
			if(so.getStatusCancelled() != null){
				s += "<b>Cancelled: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusCancelled()) + "</b><br>";
			}
			html = UIFactory.createHTML(s);
			orderArea.setWidget(r, c, html);
			r++; c = 0;

			// *************************
			// update order status
			// *************************
			html = UIFactory.createHTML("Update Order Status");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;

			hp = UIFactory.createHorizontalPanel();
			oss = OrderState.getNext(OrderState.getOrderState(so));
			for(OrderState os : oss){
				final OrderState osf = os;
				Button button = UIFactory.createButton(os.getName());
				button.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						if(osf.equals(OrderState.CANCELLED)){
							//special case - we need to collect information!
							LongCommentInputDialog lcid = new LongCommentInputDialog(
									"Cancellation Details",
									"Please enter the details of the cancellation:",
									"Cancellation details may not be empty!",
									new AbstractInputDialog.Callback<String>(){
										public void onOK(String notes) {
											controller.updateOrderStatus(osf, notes);
										}
										public void onCancel() {
											//nothing to do
										}
									});
							lcid.center();
						}else{
							controller.updateOrderStatus(osf, null);
						}
					}
				});
				hp.add(button);
			}
			orderArea.setWidget(r, c, hp);
			r++; c = 0;

			// *************************
			// TX ID
			// *************************
			html = UIFactory.createHTML("Transaction ID<br><small>(from payment system)</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			html = UIFactory.createHTML(StringUtil.getHTMLSpaceIfNullOrEmpty(so.getTxnId()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// pp mc currency
			// *************************
			html = UIFactory.createHTML("Currency<br><small>(from payment system)</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			html = UIFactory.createHTML(StringUtil.getHTMLSpaceIfNullOrEmpty(so.getPpMcCurrency()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// tax cost
			// *************************
			html = UIFactory.createHTML("Tax Cost<br><small>(from payment system)</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			html = UIFactory.createHTML(ClientFormats.NF_0_00.format(so.getTaxCost()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// grand total
			// *************************
			html = UIFactory.createHTML(
					"Grand Total<br><small>" +
					"incl. delivery,<br>" +
					"discounts, etc.</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			html = UIFactory.createHTML(ClientFormats.NF_0_00.format(so.getTotal()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// customers notes
			// *************************
			html = UIFactory.createHTML("Customers notes / wishes<br>" +
					"<small>" +
					"This is extra information which<br>" +
					"the customer supplied at the time<br>" +
					"that they placed the order.</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;

			TextArea ta = UIFactory.createTextArea();
			ta.setReadOnly(true);
			ta.setText(so.getCustomerNotes());
			ta.setHeight("100px");
			ta.setWidth("600px");
			orderArea.setWidget(r, c, ta);
			r++; c = 0;
			
			// *************************
			// is gift
			// *************************
			html = UIFactory.createHTML("Is gift?");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			html = UIFactory.createHTML(String.valueOf(so.isIsGift()));
			if(so.isIsGift()){
				html.setStyleName(UIFactory.GENERAL_ERROR);
			}
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// picked by
			// *************************
			if(so.getPickedBy() != null){
				html = UIFactory.createHTML("Picked by");
				orderArea.setWidget(r, c, html);
				orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
				c++;
				
				html = UIFactory.createHTML(so.getPickedBy().getDescription());
				orderArea.setWidget(r, c, html);
				r++; c = 0;
			}

			// *************************
			// shop notes
			// *************************
			VerticalPanel vp = UIFactory.createVerticalPanel();
			html = UIFactory.createHTML("History / Info</a><br><small>" +
					"This is information about the<br>" +
					"order, for example changes which<br>" +
					"have occurred.</small>");
			vp.add(html);
			Button addNotesButton = UIFactory.createButton("Add notes...");
			addNotesButton.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					LongCommentInputDialog lcid = new LongCommentInputDialog(
							"Notes",
							"Please enter the details:",
							"Details may not be empty!",
							new AbstractInputDialog.Callback<String>(){
								public void onOK(String notes) {
									controller.addShopNotes(notes);
								}
								public void onCancel() {
									//nothing to do
								}
							});
					lcid.center();
				}
			});
			vp.add(addNotesButton);
			orderArea.setWidget(r, c, vp);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			ta = UIFactory.createTextArea();
			ta.setReadOnly(true);
			ta.setText(so.getShopNotes());
			ta.setHeight("200px");
			ta.setWidth("600px");
			orderArea.setWidget(r, c, ta);
			r++; c = 0;
			
			// *************************
			// sales order items
			// *************************
			html = UIFactory.createHTML("Items Ordered<br><small>" +
					"Click the link in the table<br>" +
					"to view the design</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;
			
			FlexTable soiArea = UIFactory.createFlexTable();
			fillInOrderItems(so, soiArea);
			orderArea.setWidget(r, c, soiArea);
		}
	}
	
	private void fillInOrderItems(SalesOrder so, final FlexTable table) {
		int r2 = 0;
		int c2 = 0;
		HTML l = UIFactory.createHTML("Design");
		table.setWidget(r2, c2, l); c2++;
		l = UIFactory.createHTML("Quantity");
		table.setWidget(r2, c2, l); c2++;
		l = UIFactory.createHTML("Unit Price");
		table.setWidget(r2, c2, l); c2++;
		l = UIFactory.createHTML("Additional<br>Delivery<br>Charge");
		table.setWidget(r2, c2, l); c2++;
		l = UIFactory.createHTML("Special<br>Offer<br>Discount");
		table.setWidget(r2, c2, l); c2++;
		l = UIFactory.createHTML("HiRes Image");
		table.setWidget(r2, c2, l); c2++;

		table.getRowFormatter().setStyleName(r2, UIFactory.LIST_HEADER);
		
		r2++; c2 = 0;
		
		for(SalesOrderItem soi : so.getSalesOrderItems()){

			final SalesOrderItem soif = soi;
			
			/*********************************************
			 * design
			 */
			HorizontalPanel hp = UIFactory.createHorizontalPanel();
			Image img = UIFactory.createImage(soi.getDesign().getUrl());
			img.setSize("40px", "40px");
			img.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					getWizard().showDesignAdminPage(soif.getDesign());
				}
			});

			hp.add(img);
			l = UIFactory.createHTMLLikeHyperlink(String.valueOf(soi.getDesign().getUid()) + ") " + soi.getDesign().getName());
			l.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					getWizard().showDesignAdminPage(soif.getDesign());
				}
			});
			l.setTitle("Click to view this design");
			hp.add(l);
			table.setWidget(r2, c2, hp);
			c2++;

			/*********************************************
			 * quantity
			 */
			l = UIFactory.createHTML(String.valueOf(soi.getQuantity()));
			table.setWidget(r2, c2, l);
			c2++;
			
			/*********************************************
			 * unitPrice
			 */
			l = UIFactory.createHTML(ClientFormats.NF_0_00.format(soi.getBasePrice()));
			table.setWidget(r2, c2, l);
			c2++;
			
			/*********************************************
			 * additionalDeliveryCharge
			 */
			l = UIFactory.createHTML(ClientFormats.NF_0_00.format(soi.getAdditionalDeliveryCharge()));
			table.setWidget(r2, c2, l);
			c2++;
			
			/*********************************************
			 * specialOfferDiscount
			 */
			l = UIFactory.createHTML(ClientFormats.NF_0_00.format(soi.getSpecialOfferDiscount()) + "%");
			table.setWidget(r2, c2, l);
			table.getFlexCellFormatter().setHorizontalAlignment(r2, c2, HasHorizontalAlignment.ALIGN_RIGHT);
			c2++;
			
			/*********************************************
			 * hi res link
			 */
			hp = UIFactory.createHorizontalPanel();
			l = UIFactory.createHTMLLikeHyperlink("open");
			l.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					//http://msdn.microsoft.com/en-us/library/ms536651(VS.85).aspx
					String features = "menubar=no, status=no, titlebar=no, toolbar=no";
					Window.open(soif.getDesign().getHiresUrl(), "_blank", features);
				}
			});
			hp.add(l);

			l = UIFactory.createHTMLLikeHyperlink("save");
			l.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					//http://msdn.microsoft.com/en-us/library/ms536651(VS.85).aspx
					String features = "menubar=no, status=no, titlebar=no, toolbar=no";
					Window.open("/getHiResDesign?uid=" + soif.getDesign().getUid(), "_blank", features); //open in a blank window since thats all that seems to work...
				}
			});
			hp.add(l);

			table.setWidget(r2, c2, hp);
			table.getFlexCellFormatter().setHorizontalAlignment(r2, c2, HasHorizontalAlignment.ALIGN_RIGHT);

			
			/*********************************************
			 * row styling - set colour, then add mouseOver/mouseOut styles.
			 * also react to clicks.
			 */
			String style = UIFactory.LIST_ROW_EVEN_SELECTABLE;
			if(r2 % 2 != 0){
				style = UIFactory.LIST_ROW_ODD_SELECTABLE;
			}
			table.getRowFormatter().setStyleName(r2, style);
			
			r2++; c2 = 0;
		}
	}

	public void showOrder(SalesOrder so){
		if(so != null){
			fb.setInputText(String.valueOf(so.getUid()), true);
		}
	}

	@Override
	public void activated() {
	}

}

