/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.order_detail.ui;


import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.order_detail.core.IOrderAdminDetail;
import uk.co.maxant.gwtdemo.client.order_detail.core.IOrderCustomerDetail;
import uk.co.maxant.gwtdemo.client.order_detail.core.OrderCustomerDetailController;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.to.SalesOrderItem;
import uk.co.maxant.gwtdemo.client.util.ClientFormats;
import uk.co.maxant.gwtdemo.client.util.StringUtil;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPageWithListClickHandling;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.TextArea;

/**
 * the order customer screen, which lets customers manage orders.
 */
public class OrderCustomerDetail extends WizardPageWithListClickHandling<SalesOrderItem> implements IOrderCustomerDetail {

	private OrderCustomerDetailController controller;

	private FlexTable orderArea;

	/**
	 */
	public OrderCustomerDetail(final AppWizard wizard){
		super(wizard);

		this.controller = new OrderCustomerDetailController(this, wizard);

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(5);
		table.setCellPadding(0);
		table.setBorderWidth(0);
		
		int r = 0;
		int c = 0;
		
		/***************************************************
		 * title
		 */
		//TODO introduce user prefs in order to allow them to remove the details of the information below
		String s = "<h2>Sales Order</h2>";
		s += "You can view order details here.";
		HTML l = UIFactory.createHTML(s);
		table.setWidget(r, c, l);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;
		
		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;

		/***************************************************
		 * order area
		 */
		orderArea = UIFactory.createFlexTable();
		orderArea.setBorderWidth(0);
		orderArea.setCellPadding(0);
		orderArea.setCellSpacing(5);
		table.setWidget(r, c, orderArea);
		r++; c = 0;

		//tell the composite (this) which widget its wrapping
		initWidget(table);
	}
	
	/**
	 * @see IOrderAdminDetail#setSalesOrder()
	 */
	public void setSalesOrder(){
		
		orderArea.clear();
		
		int r = 0;
		int c = 0;
		SalesOrder so = controller.getModel().getSalesOrder();
		if(so == null){
			HTML html = UIFactory.createHTML("No order found.");
			orderArea.setWidget(r, c, html);
		}else{
			// *************************
			// ID
			// *************************
			HTML html = UIFactory.createHTML("Order ID");
			html.setStyleName(UIFactory.BIG_BOLD_TEXT);
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			c++;

			html = UIFactory.createHTML(String.valueOf(so.getUid()));
			html.setStyleName(UIFactory.BIG_BOLD_TEXT);
			orderArea.setWidget(r, c, html);
			r++; c = 0;

			// *************************
			// email
			// *************************
			if(!so.getCustomer().getEmail().trim().equalsIgnoreCase(so.getEmail())){
				html = UIFactory.createHTML("Email for<br>this order");
				orderArea.setWidget(r, c, html);
				orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
				orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
				c++;
				
				html = UIFactory.createHTML(so.getEmail());
				orderArea.setWidget(r, c, html);
				r++; c = 0;
			}
			
			// *************************
			// payment type
			// *************************
			html = UIFactory.createHTML("Payment Type");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;
			
			html = UIFactory.createHTML(StringUtil.getHTMLSpaceIfNullOrEmpty(so.getPaymentType()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// delivery total
			// *************************
			html = UIFactory.createHTML("Delivery Total");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;
			
			double d = so.getDeliveryTotal();
			html = UIFactory.createHTML(ClientFormats.NF_0_00.format(d));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// delivery description
			// *************************
			html = UIFactory.createHTML("Delivery Description");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;
			
			html = UIFactory.createHTML(StringUtil.getHTMLSpaceIfNullOrEmpty(so.getDeliveryDescription()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// payment details
			// *************************
			html = UIFactory.createHTML("Payment Details");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;
			
			html = UIFactory.createHTML(StringUtil.getHTMLSpaceIfNullOrEmpty(so.getPaymentDetails())); //TODO format?
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// payment status
			// *************************
			html = UIFactory.createHTML("Payment Status");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;

			String s = so.getPaymentStatusHTML();
			html = UIFactory.createHTML(s);
			if(so.getStatusPaymentFailed() != null){
				html.setStyleName(UIFactory.GENERAL_ERROR);
			}
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// order status
			// *************************
			html = UIFactory.createHTML("Order Status");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;
			
			s = "";
			if(so.getStatusOrderCreated() != null){
				s += "Created: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusOrderCreated()) + "<br>";
			}
			if(so.getStatusOrderPlaced() != null){
				s += "Placed: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusOrderPlaced()) + "<br>";
			}
			if(so.getStatusAwaitingPicking() != null){
				s += "Awaiting Picking: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusAwaitingPicking()) + "<br>";
			}
			if(so.getStatusStartedPicking() != null){
				s += "Started Picking: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusStartedPicking()) + "<br>";
			}
			if(so.getStatusFinishedPicking() != null){
				s += "Finished Picking: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusFinishedPicking()) + "<br>";
			}
			if(so.getStatusReadyForDispatch() != null){
				s += "Ready for Dispatch: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusReadyForDispatch()) + "<br>";
			}
			if(so.getStatusDispatched() != null){
				s += "Dispatched: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusDispatched()) + "<br>";
			}
			if(so.getStatusDelivered() != null){
				s += "Delivered: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusDelivered()) + "<br>";
			}
			if(so.getStatusCancelled() != null){
				s += "<b>Cancelled: " + ClientFormats.DTF_ddMMMyyyyHHmm.format(so.getStatusCancelled()) + "</b><br>";
			}
			html = UIFactory.createHTML(s);
			orderArea.setWidget(r, c, html);
			r++; c = 0;

			// *************************
			// TX ID
			// *************************
			html = UIFactory.createHTML("Transaction ID<br><small>(from payment system)</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;
			
			html = UIFactory.createHTML(StringUtil.getHTMLSpaceIfNullOrEmpty(so.getTxnId()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// grand total
			// *************************
			html = UIFactory.createHTML(
					"Grand Total<br><small>" +
					"incl. delivery,<br>" +
					"discounts, etc.</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;
			
			html = UIFactory.createHTML(ClientFormats.NF_0_00.format(so.getTotal()));
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// customers notes
			// *************************
			html = UIFactory.createHTML("Customers notes / wishes<br>" +
					"<small>" +
					"This is extra information which<br>" +
					"you may have supplied at the time<br>" +
					"that you placed the order.</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;

			TextArea ta = UIFactory.createTextArea();
			ta.setReadOnly(true);
			ta.setText(so.getCustomerNotes());
			ta.setHeight("100px");
			ta.setWidth("600px");
			orderArea.setWidget(r, c, ta);
			r++; c = 0;
			
			// *************************
			// is gift
			// *************************
			html = UIFactory.createHTML("Is gift?");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;
			
			html = UIFactory.createHTML(String.valueOf(so.isIsGift()));
			if(so.isIsGift()){
				html.setStyleName(UIFactory.GENERAL_ERROR);
			}
			orderArea.setWidget(r, c, html);
			r++; c = 0;
			
			// *************************
			// sales order items
			// *************************
			html = UIFactory.createHTML("Items Ordered<br><small>" +
					"Click the any row in the <br>" +
					"table to view the design</small>");
			orderArea.setWidget(r, c, html);
			orderArea.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
			orderArea.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
			c++;
			
			FlexTable soiArea = UIFactory.createFlexTable();
			fillInOrderItems(so, soiArea);
			orderArea.setWidget(r, c, soiArea);
		}
	}
	
	private void fillInOrderItems(SalesOrder so, final FlexTable table) {
		int r2 = 0;
		int c2 = 0;
		HTML l = UIFactory.createHTML("Design");
		table.setWidget(r2, c2, l); c2++;
		l = UIFactory.createHTML("Quantity");
		table.setWidget(r2, c2, l); c2++;
		l = UIFactory.createHTML("Unit Price");
		table.setWidget(r2, c2, l); c2++;
		l = UIFactory.createHTML("Additional<br>Delivery<br>Charge");
		table.setWidget(r2, c2, l); c2++;
		l = UIFactory.createHTML("Special<br>Offer<br>Discount");
		table.setWidget(r2, c2, l); c2++;

		table.getRowFormatter().setStyleName(r2, UIFactory.LIST_HEADER);
		
		r2++; c2 = 0;
		
		for(SalesOrderItem soi : so.getSalesOrderItems()){

			/*********************************************
			 * design
			 */
			HorizontalPanel hp = UIFactory.createHorizontalPanel();
			Image img = UIFactory.createImage(soi.getDesign().getUrl());
			img.setSize("40px", "40px");

			hp.add(img);
			l = UIFactory.createHTML(String.valueOf(soi.getDesign().getUid()) + ") " + soi.getDesign().getName());
			hp.add(l);
			table.setWidget(r2, c2, hp);
			c2++;

			/*********************************************
			 * quantity
			 */
			l = UIFactory.createHTML(String.valueOf(soi.getQuantity()));
			table.setWidget(r2, c2, l);
			c2++;
			
			/*********************************************
			 * unitPrice
			 */
			l = UIFactory.createHTML(ClientFormats.NF_0_00.format(soi.getBasePrice()));
			table.setWidget(r2, c2, l);
			c2++;
			
			/*********************************************
			 * additionalDeliveryCharge
			 */
			l = UIFactory.createHTML(ClientFormats.NF_0_00.format(soi.getAdditionalDeliveryCharge()));
			table.setWidget(r2, c2, l);
			c2++;
			
			/*********************************************
			 * specialOfferDiscount
			 */
			l = UIFactory.createHTML(ClientFormats.NF_0_00.format(soi.getSpecialOfferDiscount()) + "%");
			table.setWidget(r2, c2, l);
			table.getFlexCellFormatter().setHorizontalAlignment(r2, c2, HasHorizontalAlignment.ALIGN_RIGHT);

			
			/*********************************************
			 * row styling - set colour, then add mouseOver/mouseOut styles.
			 * also react to clicks.
			 */
			String style = UIFactory.LIST_ROW_EVEN_SELECTABLE;
			if(r2 % 2 != 0){
				style = UIFactory.LIST_ROW_ODD_SELECTABLE;
			}
			table.getRowFormatter().setStyleName(r2, style);
			for(int i = 0; i < table.getCellCount(r2); i++){
				addHandlers(table.getWidget(r2, i), soi, table, r2, style);
			}

			r2++; c2 = 0;
		}
	}

	public void showOrder(SalesOrder so){
		if(so != null){
			controller.setSalesOrder(String.valueOf(so.getUid()));
		}
	}

	@Override
	public void handleClickOnTable(ClickEvent event, SalesOrderItem soi, Object extra) {
		getWizard().showDesignDetailPage(soi.getDesign());
	}
	
	@Override
	public void activated() {
		//if the URL contained an ID, lets show that order now!
		Integer i = getWizard().getModel().getOptionalUid();
		if(i != null){
			controller.setSalesOrder(String.valueOf(i));
		}
	}

}

