/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.profile.ui;

import uk.co.maxant.gwtdemo.client.Roles;
import uk.co.maxant.gwtdemo.client.generic.widgets.Button;
import uk.co.maxant.gwtdemo.client.generic.widgets.Dialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Handler;
import uk.co.maxant.gwtdemo.client.generic.widgets.ListBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.TextBoxWithValidation;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.profile.core.IProfile;
import uk.co.maxant.gwtdemo.client.profile.core.ProfileController;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.User;
import uk.co.maxant.gwtdemo.client.util.StringUtil;
import uk.co.maxant.gwtdemo.client.util.Titles;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;

public class Profile extends WizardPage implements IProfile {

	private ProfileController controller;
	
	private FlexTable infoTable;
	
	private TextBoxWithValidation email;
	private TextBoxWithValidation emailConfirmation;
	private TextBoxWithValidation password;
	private TextBoxWithValidation passwordConfirmation;
	private ListBox title;
	private TextBoxWithValidation firstName;
	private TextBoxWithValidation lastName;
	private TextBoxWithValidation screenName;
	private TextBoxWithValidation phone;
	private CheckBox isJoinMailingList;
	private CheckBox isArtist;
	private ListBox preferredLocale;

	private boolean relogin;

	private HTML artistProfileLink;

	public Profile(final AppWizard wizard){
		super(wizard);
		
		this.controller = new ProfileController(this, wizard);
		
		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(0);

		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		hp.setVerticalAlignment(HasVerticalAlignment.ALIGN_TOP);

		HTML header = UIFactory.createHTML("Your profile");
		header.setStyleName(UIFactory.BIG_BOLD_TEXT);
		hp.add(header);

		int r = 0;
		int c = 0;
		table.setWidget(r, c, hp);
		r++; c = 0;

		HTML lbl = UIFactory.createHTML("To edit your profile, please edit this form and submit it.");
		table.setWidget(r, c, lbl);
		r++; c = 0;

		artistProfileLink = UIFactory.createHTMLLikeHyperlink("Alternatively edit your artist profile here...");
		artistProfileLink.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				getWizard().showArtistProfilePage();
			}
		});
		table.setWidget(r, c, artistProfileLink);
		r++; c = 0;
		
		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		r++; c = 0;

		/***************************************************
		 * address table
		 */
		infoTable = UIFactory.createFlexTable();
		infoTable.setCellSpacing(3);
		infoTable.setCellPadding(0);
		infoTable.setBorderWidth(0);
		table.setWidget(r, c, infoTable);
		c++;

		initWidget(table);
	}
	
	public void setArtist(final Artist a) {
		
		artistProfileLink.setVisible(getWizard().getModel().getSessionInfo().isInRole(Roles.ARTIST));
		
		int r = 0;
		int c = 0;

		HTML lbl = UIFactory.createHTML("ID");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		lbl = UIFactory.createHTML(String.valueOf(a.getUid()));
		infoTable.setWidget(r, c, lbl);
		r++; c = 0;

		lbl = UIFactory.createHTML("Email");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		email = UIFactory.createTextBoxWithValidation("Invalid email address", true);
		email.setText(StringUtil.getEmptyIfNullOrEmpty(a.getEmail()));
		email.setAssertEmail(true);
		email.setInputWidth("300px");
		email.setInputMaxLength(255);
		infoTable.setWidget(r, c, email);
		r++; c = 0;
		
		final HTML confirmEmailLabel = UIFactory.createHTML("Confirm email");
		confirmEmailLabel.setVisible(false);
		infoTable.setWidget(r, c, confirmEmailLabel);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		emailConfirmation = UIFactory.createTextBoxWithValidation("Invalid email address", true);
		emailConfirmation.setVisible(false);
		emailConfirmation.setAssertEmail(true);
		emailConfirmation.setInputWidth("300px");
		emailConfirmation.setInputMaxLength(255);
		infoTable.setWidget(r, c, emailConfirmation);

		email.addHandler(new Handler() {
			@Override
			public void onEvent(GwtEvent<? extends EventHandler> event) {
				boolean b = !controller.getModel().getArtist().getEmail().equals(email.getText());
				confirmEmailLabel.setVisible(b);
				emailConfirmation.setVisible(b);
				
			}
		});

		r++; c = 0;
		
		lbl = UIFactory.createHTML("Title");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		title = UIFactory.createListBox(false);
		title.setVisibleItemCount(1);

		String[] titles = Titles.TITLES;
		int idx = 0;
		for(String s : titles){
			title.addItem(s);
			if(a.getTitle().equals(s)){
				idx = title.getItemCount() - 1;
			}
		}
		title.setSelectedIndex(idx);

		infoTable.setWidget(r, c, title);
		r++; c = 0;

		lbl = UIFactory.createHTML("First name");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		firstName = UIFactory.createTextBoxWithValidation("Please enter your first name", true);
		firstName.setText(StringUtil.getEmptyIfNullOrEmpty(a.getFirstname()));
		firstName.setAssertTrimNotEmpty(true);
		firstName.setInputWidth("300px");
		firstName.setInputMaxLength(50);
		infoTable.setWidget(r, c, firstName);
		r++; c = 0;

		lbl = UIFactory.createHTML("Last name");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		lastName = UIFactory.createTextBoxWithValidation("Please enter your last name", true);
		lastName.setText(StringUtil.getEmptyIfNullOrEmpty(a.getLastname()));
		lastName.setAssertTrimNotEmpty(true);
		lastName.setInputWidth("300px");
		lastName.setInputMaxLength(50);
		infoTable.setWidget(r, c, lastName);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Change password");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		password = UIFactory.createTextBoxWithValidation("Invalid password - it must be at least 6 characters long", true, true);
		password.setAssertNotEmpty(false);
		password.setAssertTrimNotEmpty(false);
		password.setInputWidth("300px");
		password.setInputMaxLength(100);
		infoTable.setWidget(r, c, password);
		r++; c = 0;

		final HTML confirmPasswordLabel = UIFactory.createHTML("Confirm new password");
		confirmPasswordLabel.setVisible(false);
		infoTable.setWidget(r, c, confirmPasswordLabel);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		passwordConfirmation = UIFactory.createTextBoxWithValidation("Invalid password - it must be at least 6 characters long", true, true);
		passwordConfirmation.setVisible(false);
		passwordConfirmation.setAssertNotEmpty(true);
		passwordConfirmation.setInputWidth("300px");
		passwordConfirmation.setInputMaxLength(100);
		infoTable.setWidget(r, c, passwordConfirmation);

		password.addHandler(new Handler() {
			@Override
			public void onEvent(GwtEvent<? extends EventHandler> event) {
				boolean b = password.getTrimmedText().length() > 0;
				confirmPasswordLabel.setVisible(b);
				passwordConfirmation.setVisible(b);
			}
		});
		
		r++; c = 0;

		
		lbl = UIFactory.createHTML("Screen name<br><small>(used when you make comments)</small>");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		screenName = UIFactory.createTextBoxWithValidation("Please enter your screen name (nickname)", true);
		screenName.setText(StringUtil.getEmptyIfNullOrEmpty(a.getScreenname()));
		screenName.setAssertTrimNotEmpty(true);
		screenName.setInputWidth("300px");
		screenName.setInputMaxLength(50);
		infoTable.setWidget(r, c, screenName);
		r++; c = 0;

//		lbl = UIFactory.createHTML("Mobile Phone");
//		infoTable.setWidget(r, c, lbl);
//		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
//		c++;
//		mobile = UIFactory.createTextBoxWithValidation("Please enter your screen name (nickname)", true);
//		mobile.setText(StringUtil.getEmptyIfNullOrEmpty(a.getMobile()));
//		mobile.setAssertTrimNotEmpty(true);
//		mobile.setInputWidth("300px");
//		mobile.setInputMaxLength(50);
//		infoTable.setWidget(r, c, mobile);
//		r++; c = 0;
		
		lbl = UIFactory.createHTML("Phone");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		phone = UIFactory.createTextBoxWithValidation("Please enter your phone number", true);
		phone.setText(StringUtil.getEmptyIfNullOrEmpty(a.getPhone()));
		phone.setInputWidth("300px");
		phone.setInputMaxLength(50);
		infoTable.setWidget(r, c, phone);
		r++; c = 0;
		
		isJoinMailingList = UIFactory.createCheckBox("Join mailing list?");
		isJoinMailingList.setValue(a.isJoinedMailingList());
		c++;
		infoTable.setWidget(r, c, isJoinMailingList);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_LEFT);
		r++; c = 0;
		
		/////////////////
		//ARTIST DESC
		/////////////////
		//no artist...
		isArtist = UIFactory.createCheckBox(
				"Sign up as an artist?<br>" +
		"<small>(as an artist you can upload<br>" +
		"your own art and profit from sales)</small>"); 
		isArtist.setValue(a.isArtist());
		isArtist.setEnabled(!a.isArtist()); //enable if they are not an artist, as they may want to register as one
		c++;
		infoTable.setWidget(r, c, isArtist);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_LEFT);
		r++; c = 0;

		lbl = UIFactory.createHTML("Preferred language?");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		preferredLocale = UIFactory.createListBox(false);
		preferredLocale.setVisibleItemCount(1);
		infoTable.setWidget(r, c, preferredLocale);
		preferredLocale.addItem("English", "EN");
		r++; c = 0;

		Button update = UIFactory.createButton("Update Profile");
		update.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				update();
			}
		});
		infoTable.setWidget(r, c, update);
		infoTable.getFlexCellFormatter().setColSpan(r, c, 2);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_CENTER);
		r++; c = 0;
	}

	/**
	 * user clicked button to udpate
	 */
	private void update(){
		if(
			firstName.validate() &&
			lastName.validate() &&
			phone.validate() &&
			screenName.validate()
			){
			
			//if email address has changed, validate it
			boolean emailValid = false;
			if(email.getTrimmedText().equalsIgnoreCase(controller.getModel().getArtist().getEmail())){
				emailValid = true;
			}else{
				if(		email.validate() && 
						emailConfirmation.validate() &&
						email.getTrimmedText().equalsIgnoreCase(emailConfirmation.getTrimmedText())){
					emailValid = true;
				}
			}
			
			if(emailValid){

				//if password has changed, validate it
				boolean passwordValid = false;
				if(password.getTrimmedText().length() == 0){
					//dont want to change it
					passwordValid = true;
				}else{
					if(		password.validate() && 
							passwordConfirmation.validate() &&
							password.getTrimmedText().equalsIgnoreCase(passwordConfirmation.getTrimmedText())){
						passwordValid = true;
					}
				}

				if(passwordValid){
					User u = new User();
					
					if(!controller.getModel().getArtist().getEmail().equalsIgnoreCase(email.getTrimmedText())){
						u.setEmail(email.getTrimmedText());
					}else{
						u.setEmail(null); //no change required
					}
					u.setFirstname(firstName.getTrimmedText());
					u.setJoinedMailingList(isJoinMailingList.getValue());
					u.setLastname(lastName.getTrimmedText());
					u.setPhone(phone.getTrimmedText());
					u.setPreferredLocale(preferredLocale.getValue(preferredLocale.getSelectedIndex()));
					u.setScreenname(screenName.getTrimmedText());
					u.setTitle(title.getItemText(title.getSelectedIndex()));
					u.setArtist(isArtist.getValue());
					
					//if changed email, then force relogin, otherwise roles are not found later on
					relogin = u.getEmail() != null;
					
					String pwd = password.getText();
					if(pwd.length() == 0){
						pwd = null;
					}
					
					controller.update(u, pwd);
				}else{
					UIFactory.createWarningDialog("Validation", "Your passwords are not the same. Please try again").center();
				}
			}else{
				UIFactory.createWarningDialog("Validation", "Your email addresses are not the same. Please try again.").center();
			}
			
		}else{
			UIFactory.createWarningDialog("Validation", "Something is still not valid. Please correct, and try again.").center();
		}
	}

	/**
	 * once the order is created, this method is called to let us know what customer ID has been generated.
	 */
	public void userUpdated() {
		UIFactory.createInformationDialog(
				"Thanks", 
				"Thank you, you have successfully updated your profile.<br><br>" +
				"You will receive an email confirmation shortly." + (relogin?"<br><br>You MUST now log in again.":""), 
			new Dialog.Callback() {
				public void onClose() {
					if(!relogin){
						getWizard().showFirstPage();
					}else{
						Window.Location.assign("/relogin.jsp"); //force another login
					}
				}
			}
		).center();
	}

	/**
	 * called by say the screen wizard to activate this page. its a hook whereby we have a chance to
	 * set contents of this page when its activated.
	 */
	@Override
	public void activated() {
		controller.getProfile();
	}

}
