/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.register.ui;

import uk.co.maxant.gwtdemo.client.generic.widgets.Button;
import uk.co.maxant.gwtdemo.client.generic.widgets.Dialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.ListBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.TextBoxWithValidation;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.register.core.IRegister;
import uk.co.maxant.gwtdemo.client.register.core.RegisterController;
import uk.co.maxant.gwtdemo.client.to.User;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;
import uk.co.maxant.gwtdemo.client.widgets.ui.richtextarea.SmallRichTextToolbar;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.DisclosurePanel;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.RichTextArea;
import com.google.gwt.user.client.ui.VerticalPanel;

public class Register extends WizardPage implements IRegister {

	private RegisterController controller;
	
	private FlexTable infoTable;
	
	private TextBoxWithValidation email;
	private TextBoxWithValidation emailConfirmation;
	private TextBoxWithValidation password;
	private TextBoxWithValidation passwordConfirmation;
	private ListBox title;
	private TextBoxWithValidation firstName;
	private TextBoxWithValidation lastName;
	private TextBoxWithValidation screenName;
	private TextBoxWithValidation street1;
	private TextBoxWithValidation street2;
	private TextBoxWithValidation city;
	private TextBoxWithValidation state;
	private TextBoxWithValidation zip;
	private TextBoxWithValidation country;
	private TextBoxWithValidation phone;
	private TextBoxWithValidation captcha;
	private ListBox whereHeardAbout;
	private CheckBox isJoinMailingList;
	private CheckBox isArtist;
	private RichTextArea artistsDescription;
	private ListBox preferredLocale;

	private uk.co.maxant.gwtdemo.client.generic.widgets.Image cimg;

	public Register(final AppWizard wizard){
		super(wizard);
		
		this.controller = new RegisterController(this, wizard);
		
		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(0);

		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		hp.setVerticalAlignment(HasVerticalAlignment.ALIGN_TOP);

		HTML header = UIFactory.createHTML("Register");
		header.setStyleName(UIFactory.BIG_BOLD_TEXT);
		hp.add(header);

		DisclosurePanel dp = UIFactory.createDisclosurePanel("why register?");
		dp.setContent(UIFactory.createHTML(
				"Lots of benefits...<ul>" +
				"<li>You can rate and comment on designs</li>" +
				"<li>You can check out much faster by re-using addresses</li>" +
				"<li>You can subscribe to our news letter and get information about new designs and offers</li>" +
				"</ul>")); 
		hp.add(dp);
		
		int r = 0;
		int c = 0;
		table.setWidget(r, c, hp);
		r++; c = 0;

		HTML lbl = UIFactory.createHTML("Please enter all fields marked with an asterisk (*):");
		table.setWidget(r, c, lbl);
		r++; c = 0;

		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		r++; c = 0;

		/***************************************************
		 * address table
		 */
		infoTable = UIFactory.createFlexTable();
		infoTable.setCellSpacing(3);
		infoTable.setCellPadding(0);
		infoTable.setBorderWidth(0);
		table.setWidget(r, c, infoTable);
		c++;

		buildInfoTable();

		initWidget(table);
	}
	
	private void buildInfoTable() {
		int r = 0;
		int c = 0;

		HTML lbl = UIFactory.createHTML("Email*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		email = UIFactory.createTextBoxWithValidation("Invalid email address", true);
		email.setAssertEmail(true);
		email.setInputWidth("300px");
		email.setInputMaxLength(255);
		infoTable.setWidget(r, c, email);
		r++; c = 0;

		lbl = UIFactory.createHTML("Confirm email*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		emailConfirmation = UIFactory.createTextBoxWithValidation("Invalid email address", true);
		emailConfirmation.setAssertEmail(true);
		emailConfirmation.setInputWidth("300px");
		emailConfirmation.setInputMaxLength(255);
		infoTable.setWidget(r, c, emailConfirmation);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Title");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		title = UIFactory.createListBox(false);
		title.setVisibleItemCount(1);

		title.addItem("Mr");
		title.addItem("Mrs");
		title.addItem("Miss");
		title.addItem("Ms");
		title.addItem("Dr");

		infoTable.setWidget(r, c, title);
		r++; c = 0;

		lbl = UIFactory.createHTML("First name*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		firstName = UIFactory.createTextBoxWithValidation("Please enter your first name", true);
		firstName.setAssertTrimNotEmpty(true);
		firstName.setInputWidth("300px");
		firstName.setInputMaxLength(50);
		infoTable.setWidget(r, c, firstName);
		r++; c = 0;

		lbl = UIFactory.createHTML("Last name*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		lastName = UIFactory.createTextBoxWithValidation("Please enter your last name", true);
		lastName.setAssertTrimNotEmpty(true);
		lastName.setInputWidth("300px");
		lastName.setInputMaxLength(50);
		infoTable.setWidget(r, c, lastName);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Street*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		street1 = UIFactory.createTextBoxWithValidation("Please enter the first line of your address", true);
		street1.setAssertTrimNotEmpty(true);
		street1.setInputWidth("300px");
		street1.setInputMaxLength(100);
		infoTable.setWidget(r, c, street1);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("&nbsp;");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		street2 = UIFactory.createTextBoxWithValidation("Please enter the second line of your address", true);
		street2.setInputWidth("300px");
		street2.setInputMaxLength(100);
		//street2 has no validation
		infoTable.setWidget(r, c, street2);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("City*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		city = UIFactory.createTextBoxWithValidation("Please enter your city", true);
		city.setAssertTrimNotEmpty(true);
		city.setInputWidth("300px");
		city.setInputMaxLength(50);
		infoTable.setWidget(r, c, city);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("State*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		state = UIFactory.createTextBoxWithValidation("Please enter your state or county", true);
		state.setAssertTrimNotEmpty(true);
		state.setInputWidth("300px");
		state.setInputMaxLength(50);
		infoTable.setWidget(r, c, state);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Postcode*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		zip = UIFactory.createTextBoxWithValidation("Please enter your ZIP or post code", true);
		zip.setAssertTrimNotEmpty(true);
		zip.setInputWidth("300px");
		zip.setInputMaxLength(25);
		infoTable.setWidget(r, c, zip);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Country*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		country = UIFactory.createTextBoxWithValidation("Please enter your country", true);
		country.setAssertTrimNotEmpty(true);
		country.setInputWidth("300px");
		country.setInputMaxLength(100);
		infoTable.setWidget(r, c, country);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Password*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		password = UIFactory.createTextBoxWithValidation("Invalid password - it must be at least 6 characters long", true, true);
		password.setAssertNotEmpty(true);
		password.setInputWidth("300px");
		password.setInputMaxLength(100);
		infoTable.setWidget(r, c, password);
		r++; c = 0;

		lbl = UIFactory.createHTML("Confirm password*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		passwordConfirmation = UIFactory.createTextBoxWithValidation("Invalid password - it must be at least 6 characters long", true, true);
		passwordConfirmation.setAssertNotEmpty(true);
		passwordConfirmation.setInputWidth("300px");
		passwordConfirmation.setInputMaxLength(100);
		infoTable.setWidget(r, c, passwordConfirmation);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Screen name*<br><small>(used when you make comments)</small>");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		screenName = UIFactory.createTextBoxWithValidation("Please enter your screen name (nickname)", true);
		screenName.setAssertTrimNotEmpty(true);
		screenName.setInputWidth("300px");
		screenName.setInputMaxLength(50);
		infoTable.setWidget(r, c, screenName);
		r++; c = 0;

		lbl = UIFactory.createHTML(
				"Please enter the text you<br>" +
				"see in the image below*");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		infoTable.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
		c++;
		VerticalPanel vp = UIFactory.createVerticalPanel();
		captcha = UIFactory.createTextBoxWithValidation("Add the text from the image below", true);
		captcha.setAssertNotEmpty(true);
		captcha.setInputWidth("300px");
		captcha.setInputMaxLength(50);
		vp.add(captcha);
		cimg = UIFactory.createImage("jcaptcha.jpg?t=" + System.currentTimeMillis());
		vp.add(cimg);
		lbl = UIFactory.createHTMLLikeHyperlink("<small>click here to change the image</small>");
		lbl.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				cimg.setUrl("jcaptcha.jpg?t=" + System.currentTimeMillis());
			}
		});
		vp.add(lbl);
		infoTable.setWidget(r, c, vp);
		r++; c = 0;
		
//		lbl = UIFactory.createHTML("Mobile Phone");
//		infoTable.setWidget(r, c, lbl);
//		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
//		c++;
//		mobile = UIFactory.createTextBoxWithValidation("Please enter your screen name (nickname)", true);
//		mobile.setAssertTrimNotEmpty(true);
//		mobile.setInputWidth("300px");
//		mobile.setInputMaxLength(50);
//		infoTable.setWidget(r, c, mobile);
//		r++; c = 0;
		
		lbl = UIFactory.createHTML("Phone");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		phone = UIFactory.createTextBoxWithValidation("Please enter your phone number", true);
		phone.setInputWidth("300px");
		phone.setInputMaxLength(50);
		infoTable.setWidget(r, c, phone);
		r++; c = 0;
		
		isJoinMailingList = UIFactory.createCheckBox("Join mailing list?");
		isJoinMailingList.setValue(true);
		c++;
		infoTable.setWidget(r, c, isJoinMailingList);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_LEFT);
		r++; c = 0;
		
		isArtist = UIFactory.createCheckBox("Sign up as an artist?<br><small>(as an artist you can upload your own art)</small>"); 
		isArtist.setValue(false);
		c++;
		infoTable.setWidget(r, c, isArtist);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_LEFT);
		r++; c = 0;
		
		final HTML artistDescLabel = UIFactory.createHTML(
				"A description of<br>" +
				"you as an artist*");
		infoTable.setWidget(r, c, artistDescLabel);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		artistDescLabel.setVisible(false);
		c++;
		
		artistsDescription = UIFactory.createRichTextArea();
		artistsDescription.setWidth("400px"); 

		final SmallRichTextToolbar toolbar = UIFactory.createSmallRichTextToolbar(artistsDescription);
		toolbar.setWidth("127px");
		toolbar.setVisible(false);
		infoTable.setWidget(r, c, toolbar);
		r++; c = 0;

		lbl = UIFactory.createHTML("&nbsp;");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		infoTable.setWidget(r, c, artistsDescription);
		artistsDescription.setVisible(false);
		r++; c = 0;
		
		isArtist.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				boolean b = isArtist.getValue();
				artistsDescription.setVisible(b);
				toolbar.setVisible(b);
				artistDescLabel.setVisible(b);
			}
		});

		
		lbl = UIFactory.createHTML("Where did you hear about us?");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		whereHeardAbout = UIFactory.createListBox(false);
		whereHeardAbout.setVisibleItemCount(1);

		whereHeardAbout.addItem("Google", "GOOGLE");
		whereHeardAbout.addItem("Bing", "BING");
		whereHeardAbout.addItem("A friend", "FRIEND");
		whereHeardAbout.addItem("A \"powered by maxant\" link", "MAXANT");
		whereHeardAbout.addItem("Other", "OTHER");

		infoTable.setWidget(r, c, whereHeardAbout);
		r++; c = 0;
		
		lbl = UIFactory.createHTML("Preferred language?");
		infoTable.setWidget(r, c, lbl);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		c++;
		preferredLocale = UIFactory.createListBox(false);
		preferredLocale.setVisibleItemCount(1);
		infoTable.setWidget(r, c, preferredLocale);
		preferredLocale.addItem("English", "EN");
		r++; c = 0;

		
		Button register = UIFactory.createButton("Register");
		register.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				completeRegistration();
			}
		});
		infoTable.setWidget(r, c, register);
		infoTable.getFlexCellFormatter().setColSpan(r, c, 2);
		infoTable.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_CENTER);
		r++; c = 0;
	}

	/**
	 * user clicked button to go to checkout
	 */
	private void completeRegistration(){
		if(
			city.validate() &&
			country.validate() &&
			email.validate() &&
			emailConfirmation.validate() &&
			firstName.validate() &&
			lastName.validate() &&
			state.validate() &&
			street1.validate() &&
			street2.validate() &&
			zip.validate() &&
			phone.validate() &&
			screenName.validate() &&
			password.validate() &&
			passwordConfirmation.validate() &&
			captcha.validate()
			){
			
			if(email.getTrimmedText().equalsIgnoreCase(emailConfirmation.getTrimmedText())){
				
				if(password.getText().equalsIgnoreCase(passwordConfirmation.getText())){
					
					if(isArtist.getValue() && artistsDescription.getHTML().trim().length() == 0){
						UIFactory.createWarningDialog("Validation", "Please provide a description of you as an artist, or uncheck the box for registering as an artist.").center();
					}else{
						User user = new User();
						
						user.setEmail(email.getTrimmedText());
						user.setFirstname(firstName.getTrimmedText());
						user.setJoinedMailingList(isJoinMailingList.getValue());
						user.setLastname(lastName.getTrimmedText());
						user.setPhone(phone.getTrimmedText());
						user.setPreferredLocale(preferredLocale.getValue(preferredLocale.getSelectedIndex()));
						user.setScreenname(screenName.getTrimmedText());
						user.setTitle(title.getItemText(title.getSelectedIndex()));
						user.setWhereHeardAbout(whereHeardAbout.getValue(whereHeardAbout.getSelectedIndex()));
						
						controller.register(user, password.getText(), captcha.getText(), isArtist.getValue(), artistsDescription.getHTML());
					}
				}else{
					UIFactory.createWarningDialog("Validation", "Your passwords are not the same. Please try again").center();
				}
			}else{
				UIFactory.createWarningDialog("Validation", "Your email addresses are not the same. Please try again.").center();
			}
			
		}else{
			UIFactory.createWarningDialog("Validation", "Something is still not valid. Please correct, and try again.").center();
		}
	}

	/**
	 * once the order is created, this method is called to let us know what customer ID has been generated.
	 */
	public void userRegistered(Integer uid) {
		UIFactory.createInformationDialog(
				"Congratulations", 
				"Congratulations, you have successfully registered.<br><br>" +
				"You will receive an email shortly. In the mean time, please log in..." + (isArtist.getValue()?"<br><br>Once you have logged in, then please upload an image for your artists profile!":""), 
			new Dialog.Callback() {
				public void onClose() {
					Window.Location.assign("/secure");
				}
			}
		).center();
	}

	/**
	 * called by say the screen wizard to activate this page. its a hook whereby we have a chance to
	 * set contents of this page when its activated.
	 */
	@Override
	public void activated() {
		if(getWizard().getModel().getSessionInfo() != null){
			if(getWizard().getModel().getSessionInfo().isLoggedIn()){
				//how did they get here?!
				UIFactory.createWarningDialog("Already Logged In", 
						"You are already logged in, so you cannot register."
					).center();
				getWizard().showFirstPage();
				return;
			}
		}

		//focus on first field needing info
		if(email.getText().trim().length() == 0){
			email.setFocus(true);
		}
	}

	/**
	 * @see IRegister#failedToRegisterUser()
	 */
	public void failedToRegisterUser() {
		//need to reset image, since it was already validated once and will cause an error if used again.
		cimg.setUrl("jcaptcha.jpg?t=" + System.currentTimeMillis());
	}
}

