/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.shop.core;

import uk.co.maxant.gwtdemo.client.Roles;
import uk.co.maxant.gwtdemo.client.to.BasketItem;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.DesignComment;
import uk.co.maxant.gwtdemo.client.to.Displayable;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

public class DesignDetailController {

	protected IShop view;
	protected ShopServerCaller serverCaller;
	protected ShopModel model;
	protected AppWizard wizard;
	
	public DesignDetailController(IShop view, AppWizard wizard){
		this.view = view;
		this.wizard = wizard;
		model = new ShopModel();
		this.serverCaller = new ShopServerCaller(view, model, wizard);
	}
	
	/**
	 * calls the server to add a comment
	 * 
	 * @param comment the comment to add
	 * @param designToAddCommentTo the design to add the comment to
	 */
	public void addComment(String comment, Design designToAddCommentTo) {
		DesignComment dc = new DesignComment();
		dc.setDesign(designToAddCommentTo);
		dc.setComment(comment);
		serverCaller.addComment(dc, isModerator());
	}
	
	protected boolean isModerator(){
		boolean moderator = false;
		if(wizard.getModel().getSessionInfo() != null){
			if(wizard.getModel().getSessionInfo().isInRole(Roles.DESIGN_COMMENTS_MODERATOR)){
				moderator = true;
			}
		}
		return moderator;
	}

	public void getDesign(Design d) {
		serverCaller.getDesign(d);
	}

	public void showDisplayableInShop(Displayable displayable, Integer designToPreSelect) {
		serverCaller.showDisplayableInShop(displayable, designToPreSelect);
	}

	
	/**
	 * method to add to the basket.
	 * @param d the design to add.
	 */
	public void addToBasket(Design d) {
		serverCaller.addToBasket(new BasketItem(d));
	}

	
}
