/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.shop.core;

import java.util.List;

import uk.co.maxant.gwtdemo.client.shop.core.ShopController.ShopResultsType;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.DesignComment;
import uk.co.maxant.gwtdemo.client.to.Displayable;
import uk.co.maxant.gwtdemo.client.to.SearchResults;
import uk.co.maxant.gwtdemo.client.widgets.core.ModelListener;
import uk.co.maxant.gwtdemo.client.widgets.core.Waitable;

/**
 * technology independent interface for the shop screen. the controller only knows about this interface
 * so that the actual shop page could be swing, RCP or GWT.
 */
public interface IShop extends ModelListener, Waitable {

	/**
	 * selects the given artist in the shop, 
	 * plus optionally preselects the design.
	 */
	void updateShopView(Displayable toDisplay, List<Design> designs, Integer designToPreSelect);

	/**
	 * sets up the designs on display
	 * 
	 * @param results results to show
	 */
	void setWhatsOnDisplay(SearchResults results, ShopResultsType type, boolean selectFirstOne);

	/**
	 * callback for displaying the freshly loaded comments.
	 */
	void showComments(Design design, List<DesignComment> comments);

	/**
	 * shows the design in detail. meant for the DesignDetail page!
	 */
	void setDesign(Design design);

	/**
	 * updates the viewer with the contents of this design
	 */
	void showDesignDetail(Design preview);

	/**
	 * callback for when the comment is successfully added.
	 */
	void addedComment();

	/**
	 * callback for when the comment is successfully approved.
	 */
	void approvedComment();

	/**
	 * callback for when the comment is successfully rejected.
	 */
	void rejectedComment();

}
