/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.shop.core;

import uk.co.maxant.gwtdemo.client.ApplicationLayerService;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.DesignComment;
import uk.co.maxant.gwtdemo.client.to.Displayable;
import uk.co.maxant.gwtdemo.client.widgets.core.WizardServerCaller;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

public class ShopController extends DesignDetailController {

	public static enum ShopResultsType {

		/** like it says */
		ALL_ARTISTS,
		
		/** like it says */
		ALL_DESIGNS,
		
		/** featured designs, as in what we show on the home page */
		FEATURED_DESIGNS,
		
		/** artist cloud */
		MOST_VIEWED_ARTISTS,
		
		/** like it says */
		NEWEST_DESIGNS,
		
		/** ie needing one or more comments to be accepted or rejected, if moderation is active. */
		DESIGNS_PENDING_MODERATION

		;

	}

	public ShopController(IShop view, AppWizard wizard){
		super(view, wizard);
	}

	/**
	 * method to empty the basket.
	 */
	public void emptyBasket() {
		serverCaller.emptyBasket();
	}

	/**
	 * updates the basket
	 */
	public void getBasket() {
		serverCaller.getBasket();
	}

	/**
	 * sets all designs in the shop. 
	 */
	public void getAll() {
		serverCaller.getAll();
	}

	/**
	 * loads all the designs for the relevant artist.
	 * the artist is then displayed in the shop. the first 
	 * design in the artist is selected.
	 * 
	 * @see WizardServerCaller#showDisplayableInShop(Displayable, Integer)
	 */
	public void selectDisplayable(Displayable d) {
		if(d.getType().equals(Displayable.DESIGN)){
			view.showDesignDetail((Design)d);
		}else{
			serverCaller.showDisplayableInShop(d, null);
		}
	}
	
	/**
	 * takes the given design
	 * and then shows the given design in the preview.
	 * 
	 * @see WizardServerCaller#showDisplayableInShop(Displayable, Integer)
	 */
	public void selectDesign(Design d) {
		serverCaller.showDisplayableInShop(d.getArtist(), d.getUid());
	}

	/**
	 * searches design tags. matching tags get their artists loaded
	 * and set in the shop.
	 * 
	 * @param searchExpression the expression to search for.
	 * 
	 * @see ApplicationLayerService#search(String, Integer)
	 */
	public void search(String searchExpression, ShopResultsType type) {
		serverCaller.search(searchExpression, type);
	}

	/**
	 * does not set the selected tab in the search results after searching.
	 * uses standard logic to decide which one should be active.
	 * 
	 * @see #search(String, ShopResultsType)
	 */
	public void search(String searchExpression) {
		search(searchExpression, null);
	}

	/**
	 * adds a rating for the logged in user.
	 */
	public void addRating(Design design, int rating) {
		serverCaller.addRating(design, rating);
	}
	
	/**
	 * gets the comments for the given design
	 */
	public void getComments(Design design) {
		serverCaller.getComments(design, isModerator());
	}

	public void approveComment(String comment, DesignComment dc) {
		dc.setComment(comment);
		serverCaller.approveComment(dc);
	}
	
	public void rejectComment(DesignComment dc) {
		serverCaller.rejectComment(dc);
	}

	public void showDesignsPendingModeration(ShopResultsType resultType) {
		serverCaller.showDesignsPendingModeration(resultType);
	}

}
