/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.shop.core;


import java.util.List;

import uk.co.maxant.gwtdemo.client.ApplicationLayerService;
import uk.co.maxant.gwtdemo.client.shop.core.ShopController.ShopResultsType;
import uk.co.maxant.gwtdemo.client.to.BasketItem;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.DesignComment;
import uk.co.maxant.gwtdemo.client.to.SearchResults;
import uk.co.maxant.gwtdemo.client.util.AsyncCallbackImpl;
import uk.co.maxant.gwtdemo.client.widgets.core.WizardServerCaller;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * delegated to by controller
 */
public class ShopServerCaller extends WizardServerCaller {

	private IShop shop;
	private ShopModel model;
	
	public ShopServerCaller(IShop shop, ShopModel model, AppWizard wizard){
		super(wizard, shop);
		this.shop = shop;
		this.model = model;
	}

	/**
	 * @see ApplicationLayerService#addToBasket(BasketItem)
	 */
	public void addToBasket(BasketItem basketItem) {
		AsyncCallback<List<BasketItem>> callback = new AsyncCallbackImpl<List<BasketItem>>(this) {
			
			public void onSuccess_(List<BasketItem> basketItems) {
				wizard.getModel().setBasket(basketItems);
			}
		};
		callServer();
		getServiceClient().addToBasket(basketItem, callback);
	}
	

	/**
	 * @see ApplicationLayerService#emptyBasket()
	 */
	public void emptyBasket() {
		AsyncCallback<List<BasketItem>> callback = new AsyncCallbackImpl<List<BasketItem>>(this) {
			
			public void onSuccess_(List<BasketItem> basketItems) {
				wizard.getModel().setBasket(basketItems);
			}
		};
		callServer();
		getServiceClient().emptyBasket(callback);
	}
	
	/**
	 * searches for all artists and designs.
	 * 
	 * @see #search(String, ShopResultsType) with null and null.
	 */
	public void getAll() {
		search(null/*wildcard search*/, null);
	}

	/**
	 * does a search and sets the resulting artist on the shop.
	 * 
	 * @see ApplicationLayerService#search(String, Integer)
	 */
	public void search(String searchExpression, final ShopResultsType type) {
		AsyncCallback<SearchResults> callback = new AsyncCallbackImpl<SearchResults>(this) {
			
			public void onSuccess_(SearchResults results) {
				shop.setWhatsOnDisplay(results, type, true);
			}
		};
		callServer();
		getServiceClient().search(searchExpression, callback);
	}

	/**
	 * @see ApplicationLayerService#rateDesign(Design, Integer)
	 */
	public void addRating(Design design, int rating) {
		AsyncCallback<Void> callback = new AsyncCallbackImpl<Void>(this) {
			public void onSuccess_(Void v) {
			}
		};
		callServer();
		getServiceClient().rateDesign(design, rating, callback);
	}

	/**
	 * gets the designs comments and displays them.
	 * 
	 * @see ApplicationLayerService#getComments(Integer)
	 */
	public void getComments(final Design design, boolean moderator) {
		AsyncCallback<List<DesignComment>> callback = new AsyncCallbackImpl<List<DesignComment>>(this) {
			
			public void onSuccess_(List<DesignComment> comments) {
				shop.showComments(design, comments);
			}
		};
		callServer();
		
		if(moderator){
			getServiceClient().getAllComments(design.getUid(), callback);
		}else{
			getServiceClient().getComments(design.getUid(), callback);
		}
	}

	/**
	 * @see ApplicationLayerService#addDesignComment(DesignComment)
	 */
	public void addComment(final DesignComment dc, final boolean moderator) {
		AsyncCallback<Integer> callback = new AsyncCallbackImpl<Integer>(this) {
			
			public void onSuccess_(Integer uid) {
				shop.addedComment();
			}
		};
		callServer();
		getServiceClient().addDesignComment(dc, callback);
	}

	public void approveComment(DesignComment dc) {
		AsyncCallback<Void> callback = new AsyncCallbackImpl<Void>(this) {
			
			public void onSuccess_(Void v) {
				shop.approvedComment();
			}
		};
		callServer();
		getServiceClient().approveDesignComment(dc, callback);
	}
	
	public void rejectComment(DesignComment dc) {
		AsyncCallback<Void> callback = new AsyncCallbackImpl<Void>(this) {
			
			public void onSuccess_(Void v) {
				shop.rejectedComment();
			}
		};
		callServer();
		getServiceClient().rejectDesignComment(dc, callback);
	}

	/**
	 * shows all designs with pending comments. ie what a moderator still needs to look at.
	 */
	public void showDesignsPendingModeration(final ShopResultsType type) {
		AsyncCallback<List<Design>> callback = new AsyncCallbackImpl<List<Design>>(this) {
			
			public void onSuccess_(List<Design> designs) {
				shop.setWhatsOnDisplay(new SearchResults(designs, null), type, false);
			}
		};
		callServer();
		getServiceClient().getDesignsPendingModeration(callback);
	}

	/**
	 * @see ApplicationLayerService#getDesign(Integer)
	 */
	public void getDesign(Design d) {
		AsyncCallback<Design> callback = new AsyncCallbackImpl<Design>(this) {
			
			public void onSuccess_(Design d) {
				shop.setDesign(d);
			}
		};
		callServer();
		
		getServiceClient().getDesign(d.getUid(), callback);
	}

}


