/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.shop.ui;


import java.util.List;


import uk.co.maxant.gwtdemo.client.generic.widgets.CheckBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Handler;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.Rater;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.images.Images;
import uk.co.maxant.gwtdemo.client.shop.core.DesignDetailController;
import uk.co.maxant.gwtdemo.client.shop.core.IShop;
import uk.co.maxant.gwtdemo.client.shop.core.ShopController.ShopResultsType;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.DesignComment;
import uk.co.maxant.gwtdemo.client.to.Displayable;
import uk.co.maxant.gwtdemo.client.to.SearchResults;
import uk.co.maxant.gwtdemo.client.util.ClientFormats;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * a screen where the customer can view the design in detail.
 * this is required since the design may not be in the shop, if it has been removed.
 */
public class DesignDetail extends WizardPage implements IShop {

	private FlexTable designArea;
	
	private DesignDetailController controller;

	/**
	 */
	public DesignDetail(final AppWizard wizard){
		super(wizard);

		controller = new DesignDetailController(this, wizard);

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(0);
		table.setCellPadding(0);
		table.setBorderWidth(0);
		
		int r = 0;
		int c = 0;
		
		/***************************************************
		 * title
		 */
		String s = "<h2>Design Details</h2>";
		s += "Details of the selected design are shown below.";
		HTML l = UIFactory.createHTML(s);
		table.setWidget(r, c, l);
		table.getFlexCellFormatter().setColSpan(r, c, 2);
		r++; c = 0;
		
		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		table.setWidget(r, c, wait);
		r++; c = 0;

		/***************************************************
		 * order area
		 */
		designArea = UIFactory.createFlexTable();
		designArea.setBorderWidth(0);
		designArea.setCellPadding(0);
		designArea.setCellSpacing(5);
		table.setWidget(r, c, designArea);
		r++; c = 0;

		//tell the composite (this) which widget its wrapping
		initWidget(table);
	}
	
	/**
	 * @see IOrderCustomerDesignDetail#showDesign(Design)
	 */
	public void showDesign(Design d) {
		controller.getDesign(d);
	}
	
	/**
	 * @param d a fully loaded design
	 */
	public void setDesign(final Design d){
		designArea.clear();
		
		FlexTable outerTable = UIFactory.createFlexTable();
		final FlexTable table = UIFactory.createFlexTable();
		table.setBorderWidth(0);

		VerticalPanel vp = UIFactory.createVerticalPanel();
		final Image img = UIFactory.createImage(d.getViewerUrl());
		vp.add(img);

		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		SocialNetworkHelper.addLinks(d, hp);
		
		//add to cart
		com.google.gwt.user.client.ui.Image addToCart = UIFactory.createImageButton(Images.images.addToCart().createImage());
		hp.add(addToCart);
		addToCart.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				controller.addToBasket(d);
				getWizard().showShopPage(false); //go back to shop
			}
		});
		
		vp.add(hp);
		
		outerTable.setWidget(0, 0, vp);
		outerTable.getFlexCellFormatter().setVerticalAlignment(0, 0, HasVerticalAlignment.ALIGN_TOP);

		int r2 = 0;
		int c2 = 0;

		Rater rater = UIFactory.createRater(d.getRating(), true, null);
		table.setWidget(r2, c2, rater);
		r2++;

		HTML html = UIFactory.createHTML();
		html.setStyleName(UIFactory.BIG_BOLD_TEXT);
		html.setHTML("ID: " + d.getUid());
		table.setWidget(r2, c2, html);
		r2++;

		/*********************************************
		 * name
		 */
		hp = UIFactory.createHorizontalPanel();
		hp.setSpacing(3);
		HTML l = UIFactory.createHTML("Name:");
		hp.add(l);
		
		
		l = UIFactory.createHTML(d.getName());
		hp.add(l);

		table.setWidget(r2, c2, hp);
		r2++;
		
		/*********************************************
		 * artist
		 */
		hp = UIFactory.createHorizontalPanel();
		l = UIFactory.createHTML("Artist:");
		hp.add(l);
		l = UIFactory.createHTML(d.getArtist().getScreenname());
		hp.add(l);
		
		table.setWidget(r2, c2, hp);
		r2++;

		/*********************************************
		 * purchasable - customers need not know if its approved or not
		 */
		CheckBox designInCatalogue = UIFactory.createCheckBox("Purchaseable?");
		designInCatalogue.setEnabled(false); //read only in this view!
		designInCatalogue.setValue(d.isInCatalogue() && d.isApproved());
		table.setWidget(r2, c2, designInCatalogue);
		r2++;

		/*********************************************
		 * description
		 */
		hp = UIFactory.createHorizontalPanel();
		l = UIFactory.createHTML("Description:");
		hp.add(l);
		
		l = UIFactory.createHTML(d.getDescription());
		hp.add(l);
		table.setWidget(r2, c2, hp);
		r2++;
		
		/****************************************
		 * tags, remove and add
		 */
		hp = UIFactory.createHorizontalPanel();
		l = UIFactory.createHTML("Tags:");
		hp.add(l);
		
		String tags = "";
		for(String tag : d.getTags()){
			tags += tag + ", ";
		}
		if(tags.length() > 0) tags = tags.substring(0, tags.length() - 2); //trim trailing comma

		l = UIFactory.createHTML(tags);
		hp.add(l);
		table.setWidget(r2, c2, hp);
		r2++;

		//////////////////////////////////////
		/*********************************************
		 * comments
		 */
		vp = UIFactory.createVerticalPanel();

		l = UIFactory.createHTML("Reviews");
		l.setStyleName(UIFactory.BIG_BOLD_TEXT);
		vp.add(l);

		if(getWizard().getModel().getSessionInfo() != null &&
				getWizard().getModel().getSessionInfo().isLoggedIn()){
			//if logged in, then add space to add a comment!
			l = UIFactory.createHTML("Add a comment:");
			vp.add(l);

			final HTML remainingLengthLabel = UIFactory.createHTML("Remaining: 4096 characters");
			vp.add(remainingLengthLabel);

			final TextArea commentArea = UIFactory.createTextArea();
			commentArea.setPixelSize(400, 100);
			Handler handler = new Handler(){
				@Override
				public void onEvent(GwtEvent<? extends EventHandler> event) {
					int length = commentArea.getText().trim().length();
					if(length > 4096){
						commentArea.setText(commentArea.getText().trim().substring(0, 4096));
						length = 4096;
					}
					length = 4096 - length;
					remainingLengthLabel.setText("Remaining: " + length + " characters");
				}
			};
			commentArea.sinkEvents(Event.KEYEVENTS); //for copy/paste TODO 000 get this working!
			commentArea.addBlurHandler(handler);
			commentArea.addChangeHandler(handler);
			commentArea.addKeyPressHandler(handler);
			commentArea.addValueChangeHandler(handler);

			vp.add(commentArea);
					
			Button add = UIFactory.createButton("Add");
			add.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					String comment = commentArea.getText().trim();
					if(comment.length() > 0){
						controller.addComment(comment, d);
					}else{
						UIFactory.createWarningDialog("Warning", "Comment may not be empty.").center();
					}
				}
			});
			vp.add(add);
					
		}else {
			html = UIFactory.createHTMLLikeHyperlink("To add comments, please log in.");
			html.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					Window.Location.assign("/secure/");
				}
			});
			vp.add(html);
		}
			
		if(d.getComments() == null || d.getComments().isEmpty()){
			l = UIFactory.createHTML("None available.");
			vp.add(l);
		}else{
			VerticalPanel vp2 = UIFactory.createVerticalPanel();
			for(DesignComment dc : d.getComments()){
				l = UIFactory.createHTML("<hr>");
				vp2.add(l);

				l = UIFactory.createHTML("Added by " + dc.getCreator().getScreenname() + " at " + ClientFormats.DTF_ddMMMyyyyHHmm.format(dc.getAdded()));
				vp2.add(l);

				//simply display it as a label
				l = UIFactory.createHTML(dc.getComment());
				vp2.add(l);
			}
			vp.add(vp2);
		}
			
		//////////////////////////////////////
		
		table.setWidget(r2, c2, vp);
		r2++;
		
		outerTable.setWidget(0, 1, table);

		designArea.setWidget(1, 1, outerTable);
	}
	
	@Override
	public void activated() {
	}

	public void setDesigns(List<Design> designs) {
		throw new UnsupportedOperationException("this method is only to be called when in the shop page.");
	}
	
	public void setWhatsOnDisplay(SearchResults results, ShopResultsType type,
			boolean selectFirstOne) {
		throw new UnsupportedOperationException("this method is only to be called when in the shop page.");
	}
	
	public void showComments(Design design, List<DesignComment> comments) {
		throw new UnsupportedOperationException("this method is only to be called when in the shop page.");
	}

	public void updateShopView(Displayable toDisplay,
			List<Design> designs, Integer designToPreSelect) {
		throw new UnsupportedOperationException("this method is only to be called when in the shop page.");
	}

	public void showDesignDetail(Design preview) {
		throw new UnsupportedOperationException("this method is only to be called when in the shop page.");
	}
	
	public void addedComment() {
		String s = "Your comment has been added.";
		if(getWizard().getModel().isModerationActive()){
			s = "Your comment has been submitted. Once it has been reviewed, it will be visible.";
		}
		UIFactory.createInformationDialog("Thanks", s).center();
	}
	
	public void approvedComment() {
		throw new UnsupportedOperationException("this method is only to be called when in the shop page.");
	}
	
	public void rejectedComment() {
		throw new UnsupportedOperationException("this method is only to be called when in the shop page.");
	}
}
