/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.shop.ui;

import java.util.List;

import org.gwt.advanced.client.ui.widget.border.RoundCornerBorder;

import uk.co.maxant.gwtdemo.client.Roles;
import uk.co.maxant.gwtdemo.client.generic.widgets.HTML;
import uk.co.maxant.gwtdemo.client.generic.widgets.Handler;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.QuestionDialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.Rater;
import uk.co.maxant.gwtdemo.client.generic.widgets.TextBox;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.images.Images;
import uk.co.maxant.gwtdemo.client.shop.core.IShop;
import uk.co.maxant.gwtdemo.client.shop.core.ShopController;
import uk.co.maxant.gwtdemo.client.shop.core.ShopController.ShopResultsType;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.BasketItem;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.DesignComment;
import uk.co.maxant.gwtdemo.client.to.Displayable;
import uk.co.maxant.gwtdemo.client.to.SearchResults;
import uk.co.maxant.gwtdemo.client.util.ClientFormats;
import uk.co.maxant.gwtdemo.client.util.StringUtil;
import uk.co.maxant.gwtdemo.client.widgets.core.MasterDataListener;
import uk.co.maxant.gwtdemo.client.widgets.core.WizardModel;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;
import uk.co.maxant.gwtdemo.client.widgets.ui.WizardPage;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.dom.client.KeyPressHandler;
import com.google.gwt.event.dom.client.LoadEvent;
import com.google.gwt.event.dom.client.LoadHandler;
import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HasVerticalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.SimplePanel;
import com.google.gwt.user.client.ui.StackPanel;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * the shop screen, which shows designs you can buy plus the basket contents.
 */
public class Shop extends WizardPage implements IShop, MasterDataListener {

	public static final int DESIGN_IMAGE_SIZE_WIDTH = 100; //TODO 000 take from config!

	public static final int DESIGN_VIEWER_IMAGE_SIZE_WIDTH = 250; //TODO 000 take from config!

	public static final int DESIGN_ZOOM_IMAGE_SIZE_WIDTH = 400; //TODO 000 take from config!
	
	public static final int ARTIST_WIDTH = 150;

	private static final int BASKET_WIDTH = 200;

	private static final int SEARCH_WIDTH = 130;

	private static final int SEARCH_RESULTS_WIDTH = 165;
	
	private static final int SEARCH_RESULTS_HEIGHT = 250;

	private static final int IMAGES_FILLER = 5;

	/**
	 * how wide should the comments popup be?
	 */
	private static final int COMMENTS_WIDTH = 400;

	private ShopController controller;

	private TextBox searchField;

	private VerticalPanel search;
	private FlexTable searchArea;
	private FlexTable searchResults;
	private FlexTable resultsArea;
	private FlexTable detailArea;
	private FlexTable basketContents;

	private HTML emptyBasketLink;
	private Image checkout;

	private boolean isAwaitingMasterData = false;

	private boolean showingArtist;

	private boolean masterDataHasBeenLoaded = false;

	private SimplePanel basketBorder;

	/**
	 */
	public Shop(final AppWizard wizard){
		super(wizard);

		controller = new ShopController(this, wizard);

		FlexTable A = UIFactory.createFlexTable();
		A.setCellSpacing(0);
		A.setCellPadding(0);
		A.setBorderWidth(0);

		FlexTable B = UIFactory.createFlexTable();
		B.setCellSpacing(0);
		B.setCellPadding(0);
		B.setBorderWidth(0);

		FlexTable C = UIFactory.createFlexTable();
		C.setCellSpacing(5);
		C.setCellPadding(0);
		C.setBorderWidth(0);
		
		//           C   O   L   U   M   N   S
		//
		//          0            1             2
		//      --------------------------------------
		//    0 |          header          |         |
		// R    ---------------------------- detail  -
		//    1 |  search    |             |         |
		// O    -    +       -             -----------
		//    2 |            |             |         |
		// W    -  results   -   artist   -         -
		//    3 |            |             | basket  |
		//      --------------------------------------

		//      A:
		//      --------------------------------------
		//      |       B         |       C          |
		//      --------------------------------------
		//
		//      B:                      C:
		//      --------------------    -----------------
		//      |      header      |    |    detail     |
		//      --------------------    -----------------
		//      |  s&r   | artist |    |    basket     |
		//      --------------------    -----------------

		int r = 0;
		int c = 0;
		A.setWidget(r, c, B);
		A.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
		A.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_LEFT);
		
		r = 0; c = 1;
		A.setWidget(r, c, C);
		A.getFlexCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
		A.getFlexCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_RIGHT);
		
		/***************************************************
		 * header image
		 */
		final Image headerImage = UIFactory.createImage("/images/stage1banner.jpg");
		
		r = 0; c = 0;
		B.setWidget(r, c, headerImage);
		B.getFlexCellFormatter().setColSpan(r, c, 2);

		/***************************************************
		 * search
		 */
		r = 1; c = 0;
		search = UIFactory.createVerticalPanel();
		buildSearchArea();
		RoundCornerBorder border = UIFactory.createRoundCornerBorder();
		border.setWidget(searchArea);
		search.add(border);
		buildSearchResults();
		search.add(searchResults);
		B.setWidget(r, c, search);
		B.getCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
		
		/***************************************************
		 * artistArea
		 */
		r = 1; c = 1;
		buildArtistArea();
		B.setWidget(r, c, resultsArea);
		B.getCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
		B.getCellFormatter().setHorizontalAlignment(r, c, HasHorizontalAlignment.ALIGN_LEFT);
		
		/***************************************************
		 * detailArea
		 */
		r = 0; c = 0;
		buildDetailArea();
		basketBorder = UIFactory.createRoundCornerBorder();
		basketBorder.setWidget(detailArea);
		C.setWidget(r, c, basketBorder);
		C.getCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);

		/***************************************************
		 * basket
		 */
		r = 1; c = 0;
		buildBasket();
		border = UIFactory.createRoundCornerBorder();
		border.setWidget(basketContents);
		C.setWidget(r, c, border);
		C.getCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
		
		//tell the composite (this) which widget its wrapping
		initWidget(A);
		
		wizard.addMasterDataListener(this);
	}

	private void buildSearchArea() {
		searchArea = UIFactory.createFlexTable();

		/***************************************************
		 * wait image and text
		 */
		HorizontalPanel wait = createWait();
		searchArea.setWidget(0, 0, wait);
		
		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		HTML l = UIFactory.createHTML("Search:");
		hp.add(l);
		com.google.gwt.user.client.ui.Image img = UIFactory.createImageButton(Images.images.help().createImage());
		img.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				UIFactory.createInformationDialog("Information", 
						"Enter any text or numbers (e.g. design ID) to search for.<br>" +
						"Enter an artists name to search for all of their designs.<br>" +
						"Hit enter/return to do the search, and use the links below to view the search " +
						"results."
						).center();
			}
		});
		hp.add(img);
		searchArea.setWidget(1, 0, hp);

		hp = UIFactory.createHorizontalPanel();
		searchField = UIFactory.createTextBox();
		searchField.addKeyPressHandler(new KeyPressHandler(){
			public void onKeyPress(KeyPressEvent event) {
				if(event.getCharCode() == '\r' ||
						event.getCharCode() == '\n'){
					
					//return was hit - do search!
					controller.search(searchField.getText().trim());
				}
			}
		});
		hp.add(searchField);

		img = Images.images.searchicon().createImage();
		img.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				controller.search(searchField.getText().trim());
			}
		});
		hp.add(img);
		hp.setVerticalAlignment(HasVerticalAlignment.ALIGN_BOTTOM);
		searchArea.setWidget(2, 0, hp);

		HTML clearSearch = UIFactory.createHTMLLikeHyperlink("clear search");
		clearSearch.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				searchField.setText("");
				controller.getAll();
			}
		});
		searchArea.setWidget(4, 0, clearSearch);
	}
	
	private void buildSearchResults() {
		searchResults = UIFactory.createFlexTable();
		HTML l = UIFactory.createHTML();
		searchResults.setWidget(0, 0, l);
	}

	private void buildArtistArea() {
		resultsArea = UIFactory.createFlexTable();

		int r = 0;
		int c = 0;
		resultsArea.setWidget(r, c, UIFactory.createHTML());
		resultsArea.getCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
		
		r = 1;
		c = 0;
		resultsArea.setWidget(r, c, UIFactory.createHTML());
		resultsArea.getCellFormatter().setVerticalAlignment(r, c, HasVerticalAlignment.ALIGN_TOP);
	}
	
	private void buildDetailArea() {
		detailArea = UIFactory.createFlexTable();
		HTML l = UIFactory.createHTML();
		detailArea.setWidget(0, 0, l);
	}
	
	private void buildBasket() {
		basketContents = UIFactory.createFlexTable();
		//basketContents.setBorderWidth(1);

		emptyBasketLink = UIFactory.createSmallHTMLLikeHyperlink("empty basket");
		emptyBasketLink.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				emptyBasket();
			}
		});
		basketContents.setWidget(0, 0, emptyBasketLink);

		basketContents.setWidget(1, 0, UIFactory.createHTML());

		checkout = UIFactory.createImageButton("/images/ui/nextUp.jpg", "/images/ui/nextDown.jpg");
		checkout.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				gotoCheckout();
			}
		});
		
		basketContents.setWidget(0, 1, checkout);
		basketContents.getFlexCellFormatter().setRowSpan(0, 1, 2);
		basketContents.getFlexCellFormatter().setVerticalAlignment(0, 1, HasVerticalAlignment.ALIGN_TOP);
	}

	/**
	 * checks if the basket has something in it. if so, gets 
	 * confirmation, then empties the basket by calling the server.
	 */
	private void emptyBasket() {
		if(getWizard().getModel().getBasket().size() > 0){
			QuestionDialog qd = UIFactory.createQuestionDialog("&nbsp;", "Empty basket?", new QuestionDialog.Callback(){
				public void onClose(boolean yesClicked) {
					if(yesClicked){
						controller.emptyBasket();
					}
				}
			});
			qd.center();
		}
	}

	@Override
	public void activated(){
		WizardModel m = getWizard().getModel();
		if(m.getDisplayable() == null){
			if(masterDataHasBeenLoaded){
				//hmmm... what to do? well try reloading artists
				controller.getAll();
			}else{
				setWait(true);
				isAwaitingMasterData  = true;
			}
		}
		
		//get the updated basket.
		controller.getBasket();
	}

	/**
	 * callback for whenever the basket contents has changed. eg adding to it or emptying it.
	 * @param designs the contents of the basket.
	 */
	@Override
	public void updateBasket(List<BasketItem> basketItems) {
		
	    VerticalPanel vp = UIFactory.createVerticalPanel();
		String s = "Items chosen:<br><table>";
		double tot = 0.0;
		if(basketItems.size() == 0){
			s = "<tr><td class='standardText'>Your basket is empty.</td></tr>";
		}else{
			for(BasketItem bi : basketItems){
				double up = bi.getPrice();
				s += 
						"<tr>" +
						"	<td class='standardText'>" + bi.getDesign().getUid() + "</td>" +
						"	<td class='standardText'>\"" + bi.getDesign().getName() + "\"</td>" +
						"	<td class='standardText'>" + ClientFormats.NF_0_00.format(up) + "</td>" +
						"</tr>";
				tot += up;
			}
		}
		s += "</table>";
		
		HTML l = UIFactory.createHTML("Basket: " + ClientFormats.NF_0_00.format(tot) + " UKP"); //TODO 000 take from config
		vp.add(l);

		HTML html = UIFactory.createHTML(s);
		ScrollPanel sp = UIFactory.createScrollPanel();
		sp.setPixelSize(BASKET_WIDTH, 200);
		sp.add(html);
		vp.add(sp);

		basketContents.setWidget(1, 0, vp);
	}

	/**
	 * this method simply sets all the design images in the center section of the shop. 
	 * its called by {@link #updateShopView(Displayable, List, Integer)}
	 * 
	 * @see IShop#setDesigns(List)
	 * 
	 * @see #updateShopView(Displayable, List, Integer) for viewing a single artist and all its designs
	 * @see #updateShopView(List) for viewing all artists but not designs
	 * @see #setDesigns(List) which is called by updateShopView when given a single artist
	 */
	private void setDesigns(List<Design> designs){

		//would be nice to use a flowpanel which automatically lays out and such...
		//but that doesnt work. flowpanel can also only handle images when flowing. as soon as
		//you add say a spacer HTML, it just adds a new line, since flowPanels use HTML default layout.
		//so, lets go to the pain of working out how many columns of images we can create and fill a
		//table that way...

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(IMAGES_FILLER);
		table.setCellPadding(0);
		table.setBorderWidth(0);

		int widthPerImage = DESIGN_IMAGE_SIZE_WIDTH + IMAGES_FILLER;
		
		//total available height due to basket and viewer = 700 + viewer image height.
		//so height of scrollpanel is totalHeight - top - bottom - artistImageHeightIfShown
		int height = 
			700
			+ DESIGN_VIEWER_IMAGE_SIZE_WIDTH /* assume square :-( */
			- 210 /* top */
			- 70 /* bottom */
			- (showingArtist ?ARTIST_WIDTH :0/*works better during tests*/)  /* artist image */
			; //TODO 000 works on all monitor sizes?


		int width = Window.getClientWidth() 
							- Math.max(BASKET_WIDTH, DESIGN_VIEWER_IMAGE_SIZE_WIDTH) 
							- SEARCH_WIDTH 
							- 130/*spacing, padding, etc*/;
		int excess = width % widthPerImage; //show as many as we can, but dont use any more space!

		//TODO 000 width still aint quite right :-(
		
		int numCols = (width - excess);
		numCols /= widthPerImage;

		int row = 0;
		int col = 0;

		for(Design d : designs){
			
			final Design fd = d;
			
			ClickHandler ch = new ClickHandler(){
				public void onClick(ClickEvent event) {
					showDesignDetail(fd);
				}
			};

			VerticalPanel vp = UIFactory.createVerticalPanel();
			final Image img = UIFactory.createImage(d.getUrl());
			img.setStyleName(UIFactory.POINTER_CURSOR);
			img.setTitle(d.getLongToolTip());
			img.addClickHandler(ch);
			vp.add(img);
			HorizontalPanel hp = UIFactory.createHorizontalPanel();
			
			////////////////////
			// add to cart button
			////////////////////
			com.google.gwt.user.client.ui.Image addToCart = UIFactory.createImageButton(Images.images.addToCart().createImage());
			addToCart.setTitle("Add this design to the cart...");
			addToCart.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					controller.addToBasket(fd);
				}
			});
			hp.add(addToCart);

			////////////////////
			// view details button
			////////////////////
			com.google.gwt.user.client.ui.Image viewDetails = UIFactory.createImageButton(Images.images.viewDetails().createImage());
			viewDetails.setTitle("View details to the right...");
			viewDetails.addClickHandler(ch);
			hp.add(viewDetails);
			
			////////////////////
			// zoom button
			////////////////////
			com.google.gwt.user.client.ui.Image zoom = UIFactory.createImageButton(Images.images.zoom().createImage());
			zoom.setTitle("Zoom...");
			zoom.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					zoom(fd);
				}
			});
			hp.add(zoom);
			
			////////////////////
			// comments button
			////////////////////
			com.google.gwt.user.client.ui.Image comments = UIFactory.createImageButton(Images.images.comment().createImage());
			comments.setTitle("Reviews...");
			comments.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					controller.getComments(fd);
				}
			});
			hp.add(comments);
			
			////////////////////
			// details button
			////////////////////
			com.google.gwt.user.client.ui.Image details = UIFactory.createImageButton(Images.images.designDetails().createImage());
			details.setTitle("See details...");
			details.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					getWizard().showDesignDetailPage(fd);
				}
			});
			hp.add(details);

			vp.add(hp);

			////////////////////
			// name
			////////////////////
			HTML name = UIFactory.createHTML(StringUtil.trimToLengthIfNoWhitespace(d.getName(), 12));
			name.setWidth(DESIGN_IMAGE_SIZE_WIDTH + "px"); //ensure it doesnt screw the page up!
			name.setWordWrap(true);
			vp.add(name);
			
			table.setWidget(row, col, vp);
			table.getFlexCellFormatter().setVerticalAlignment(row, col, HasVerticalAlignment.ALIGN_BOTTOM);
			col++;
			if(col >= numCols){
				row++;
				col = 0;

				table.setWidget(row, col, UIFactory.createHTML("<hr>"));
				table.getFlexCellFormatter().setColSpan(row, col, numCols);
				row++;
			}
		}
		
		ScrollPanel sp = UIFactory.createScrollPanel();
		sp.setPixelSize(width, height);
		sp.add(table);
		resultsArea.setWidget(1, 0, sp);
	}

	/**
	 * @param d the fully loaded design which should be shown in the detail view
	 */
	public void showDesignDetail(final Design preview) {
		VerticalPanel vp = UIFactory.createVerticalPanel();
		if(preview != null){
			HTML html = UIFactory.createHTMLLikeHyperlink(preview.getName() + " (" + preview.getUid() + ")");
			html.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					getWizard().showDesignDetailPage(preview);
				}
			});
			vp.add(html);

			Image i2 = UIFactory.createImage(preview.getViewerUrl());
			i2.addLoadHandler(new LoadHandler() {
				public void onLoad(LoadEvent event) {
					GWT.log("loaded!", null);
				}
			});
			final Image detail = UIFactory.createImage(preview.getViewerUrl());

			detail.setTitle("Click me to enlarge me!");
			detail.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					zoom(preview);
				}
			});
			
			vp.add(detail);

			HorizontalPanel hp = UIFactory.createHorizontalPanel();
			Rater rater = UIFactory.createRater(preview.getRating(), !getWizard().getModel().getSessionInfo().isLoggedIn(), new Rater.Callback() {
				public void rated(int ratingGiven) {
					controller.addRating(preview, ratingGiven);
				}
			});
			hp.add(rater);

			com.google.gwt.user.client.ui.Image details = UIFactory.createImageButton(Images.images.designDetails().createImage());
			details.setTitle("See details...");
			details.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					getWizard().showDesignDetailPage(preview);
				}
			});
			hp.add(details);

			vp.add(hp);
			
			
			if(getWizard().getModel().getSessionInfo().isInRole(Roles.BACKOFFICE)){
				com.google.gwt.user.client.ui.Image edit = Images.images.edit().createImage();
				UIFactory.createImageButton("/images/ui/edit.gif");
				edit.setTitle("Edit this design...");
				edit.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						getWizard().showDesignAdminPage(preview);
					}
				});
				hp.add(edit);
			}

			hp = UIFactory.createHorizontalPanel();
			SocialNetworkHelper.addLinks(preview, hp);

			vp.add(hp);


			hp = UIFactory.createHorizontalPanel();
			vp.add(hp);
			final com.google.gwt.user.client.ui.Image comment = Images.images.comment().createImage();

			ClickHandler reviewHandler = new ClickHandler() {
				public void onClick(ClickEvent event) {
					controller.getComments(preview);
				}
			};

			comment.setTitle("Reviews (" + preview.getNumberOfComments() + ")");
			comment.addClickHandler(reviewHandler);
			hp.add(comment);
			HTML l = UIFactory.createHTMLLikeHyperlink("Reviews (" + preview.getNumberOfComments() + ")");
			l.addClickHandler(reviewHandler);
			hp.add(l);

			ScrollPanel sp = UIFactory.createScrollPanel();
			sp.setAlwaysShowScrollBars(false);
			html = UIFactory.createHTML(preview.getDescription());
			html.setWordWrap(true);
			html.setWidth(DESIGN_VIEWER_IMAGE_SIZE_WIDTH + "px");
			sp.setHeight("90px"); //TODO 000 doesnt work properly on mozilla :-(
			sp.add(html);
			vp.add(sp);

			//show filter link, to search for other stuff by this artist.
			boolean show = true;
			if(getWizard().getModel().getDisplayable() != null){
				if(getWizard().getModel().getDisplayable().getType().equals(Displayable.ARTIST)){
					if(getWizard().getModel().getDisplayable().getUid().equals(preview.getArtist().getUid())){
						show = false;
					}
				}
			}
			if(show){
				//no need to show if we are already filtering this artist
				final HTML artistLink = UIFactory.createHTMLLikeHyperlink("More by " + preview.getArtist().getScreenname());
				artistLink.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						controller.selectDisplayable(preview.getArtist());
					}
				});
				vp.add(artistLink);
			}
			
			Button b = UIFactory.createButton("Add to basket");
			b.addClickHandler(new ClickHandler(){
				public void onClick(ClickEvent event) {
					controller.addToBasket(preview);
				}
			});
			vp.add(b);
		}
		detailArea.setWidget(0, 0, vp);
		basketBorder.setVisible(true);
	}

	private void zoom(final Design preview){
		final PopupPanel pp = new PopupPanel(true);
		VerticalPanel vp = UIFactory.createVerticalPanel();
		vp.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_RIGHT);

		HorizontalPanel hp = UIFactory.createHorizontalPanel();

		com.google.gwt.user.client.ui.Image addToCart = UIFactory.createImageButton(Images.images.addToCart().createImage());
		addToCart.setTitle("Add this design to the cart...");
		addToCart.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				pp.hide();
				controller.addToBasket(preview);
			}
		});
		hp.add(addToCart);
		
		com.google.gwt.user.client.ui.Image close = UIFactory.createImageButton(Images.images.close().createImage());
		close.setTitle("Close");
		close.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				pp.hide();
			}
		});
		hp.add(close);

		vp.add(hp);


		Image i2 = new Image(preview.getZoomUrl());
		i2.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				pp.hide();
			}
		});
		vp.add(i2);

		pp.clear();
		pp.add(vp);
		pp.center();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void showComments(final Design design, List<DesignComment> comments){
		
		final boolean isModerator = getWizard().getModel().isModerator();
		final boolean isModerationActive = getWizard().getModel().isModerationActive();

		final PopupPanel pp = UIFactory.createPopupPanel(true);
		pp.setAnimationEnabled(true);
		pp.setStyleName(UIFactory.SHOP_CUSTOMER_REVIEWS);
		VerticalPanel vp = UIFactory.createVerticalPanel();
		pp.add(vp);

		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		hp.setWidth("100%");
		hp.setVerticalAlignment(HasVerticalAlignment.ALIGN_BOTTOM);
		HTML l = UIFactory.createHTML("Reviews:&nbsp;&nbsp;&nbsp;&nbsp;");
		l.setStyleName(UIFactory.BIG_BOLD_TEXT);
		hp.add(l);

		com.google.gwt.user.client.ui.Image close = UIFactory.createImageButton(Images.images.close().createImage());
		close.setTitle("Close");
		close.addClickHandler(new ClickHandler() {
			public void onClick(ClickEvent event) {
				pp.hide();
			}
		});
		hp.add(close);
		
		vp.add(hp);
		
		boolean isAbleToAddComments = false;
		if(getWizard().getModel().getSessionInfo() != null){
			if(getWizard().getModel().getSessionInfo().isInRole(Roles.REGISTERED)){
				//if logged in, then add space to add a comment!
				VerticalPanel vp2 = UIFactory.createVerticalPanel();
				l = UIFactory.createHTML("Add a comment:");
				vp2.add(l);

				final HTML remainingLengthLabel = UIFactory.createHTML("Remaining: 4096 characters");
				vp2.add(remainingLengthLabel);

				final TextArea commentArea = UIFactory.createTextArea();
				commentArea.setPixelSize(COMMENTS_WIDTH, 100);
				Handler handler = new Handler(){
					public void onEvent(GwtEvent<? extends EventHandler> event) {
						int length = commentArea.getText().trim().length();
						if(length > 4096){
							commentArea.setText(commentArea.getText().trim().substring(0, 4096));
							length = 4096;
						}
						length = 4096 - length;
						remainingLengthLabel.setText("Remaining: " + length + " characters");
					}
				};
				commentArea.sinkEvents(Event.KEYEVENTS); //for copy/paste TODO 000 get this working!
				commentArea.addBlurHandler(handler);
				commentArea.addChangeHandler(handler);
				commentArea.addKeyPressHandler(handler);
				commentArea.addValueChangeHandler(handler);

				vp2.add(commentArea);
				
				Button add = UIFactory.createButton("Add");
				add.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						String comment = commentArea.getText().trim();
						if(comment.length() > 0){
							controller.addComment(comment, design);
							pp.hide(); 
						}else{
							UIFactory.createWarningDialog("Warning", "Comment may not be empty.").center();
						}
					}
				});
				vp2.add(add);
				
				vp.add(vp2);
				
				isAbleToAddComments = true;
			}
		}
		
		if(!isAbleToAddComments){
			HTML html = UIFactory.createHTMLLikeHyperlink("To add comments, please log in.");
			html.addClickHandler(new ClickHandler() {
				public void onClick(ClickEvent event) {
					Window.Location.assign("/secure/");
				}
			});
			vp.add(html);
		}
		
		ScrollPanel sp = UIFactory.createScrollPanel();
		sp.setAlwaysShowScrollBars(true);
		sp.setPixelSize(COMMENTS_WIDTH, 300);
		
		if(comments == null || comments.isEmpty()){
			l = UIFactory.createHTML("None available.");
			sp.add(l);
		}else{
			VerticalPanel vp2 = UIFactory.createVerticalPanel();
			for(DesignComment dc : comments){
				final DesignComment fdc = dc;
				l = UIFactory.createHTML("<hr>");
				vp2.add(l);

				l = UIFactory.createHTML("Added by " + dc.getCreator().getScreenname() + " at " + ClientFormats.DTF_ddMMMyyyyHHmm.format(dc.getAdded()));
				vp2.add(l);


				//moderators may edit/approve/reject. if moderation is active, then they can approve/reject.
				//if moderation is inactive, they can edit.

				if(isModerator && isModerationActive){
					if(dc.getRejected() != null){
						l = UIFactory.createHTML("Rejected by " + dc.getModerator().getDescription() + " at " + ClientFormats.DTF_ddMMMyyyyHHmm.format(dc.getRejected()));
						l.setStyleName(UIFactory.GENERAL_ERROR);
						vp2.add(l);
					}else if(dc.getApproved() != null){
						l = UIFactory.createHTML("Approved by " + dc.getModerator().getDescription() + " at " + ClientFormats.DTF_ddMMMyyyyHHmm.format(dc.getApproved()));
						l.setStyleName(UIFactory.GENERAL_INFO);
						vp2.add(l);
					}else{
						l = UIFactory.createHTML("Please approved or reject this comment.");
						l.setStyleName(UIFactory.GENERAL_ERROR);
						vp2.add(l);
					}
				}
				
				if(isModerator){
					//display it in a textarea so it can be edited.
					final HTML remainingLengthLabel = UIFactory.createHTML("Remaining: " + (4096 - fdc.getComment().length()) + " characters");
					vp2.add(remainingLengthLabel);

					final TextArea commentArea = UIFactory.createTextArea();
					commentArea.setPixelSize(COMMENTS_WIDTH, 100);
					
					Handler handler = new Handler(){
						public void onEvent(GwtEvent<? extends EventHandler> event) {
							int length = commentArea.getText().trim().length();
							if(length > 4096){
								commentArea.setText(commentArea.getText().trim().substring(0, 4096));
								length = 4096;
							}
							length = 4096 - length;
							remainingLengthLabel.setText("Remaining: " + length + " characters");
						}
					};
					commentArea.sinkEvents(Event.KEYEVENTS); //for copy/paste
					commentArea.addBlurHandler(handler);
					commentArea.addChangeHandler(handler);
					commentArea.addKeyPressHandler(handler);
					commentArea.addValueChangeHandler(handler);

					commentArea.setText(fdc.getComment());
					vp2.add(commentArea);

					HorizontalPanel buttons = UIFactory.createHorizontalPanel();
					vp2.add(buttons);
					Button editApprove = UIFactory.createButton("Edit");
					if(isModerationActive){
						editApprove.setHTML("Approve/edit");
					}
					editApprove.addClickHandler(new ClickHandler() {
						public void onClick(ClickEvent event) {
							String comment = commentArea.getText().trim();
							if(comment.length() > 0){
								controller.approveComment(comment, fdc);
								pp.hide(); 
							}else{
								UIFactory.createWarningDialog("Warning", "Comment may not be empty.").center();
							}
						}
					});
					buttons.add(editApprove);
					
					if(isModerationActive){
						//add a reject button
						Button reject = UIFactory.createButton("Reject");
						reject.addClickHandler(new ClickHandler() {
							public void onClick(ClickEvent event) {
								controller.rejectComment(fdc);
								pp.hide(); 
							}
						});
						buttons.add(reject);
					}
				}else{
					//simply display it as a label
					l = UIFactory.createHTML(dc.getComment());
					vp2.add(l);
				}
			}
			sp.add(vp2);
		}
		
		vp.add(sp);
		pp.center();
	}
	
	/**
	 * checkout button was pressed
	 */
	private void gotoCheckout(){
		if(getWizard().getModel().getBasket().size() == 0){
			UIFactory.createWarningDialog("Empty Basket", "Your basket is empty.<br>Please choose some items first.").center();
		}else{
			getWizard().showNextPage();
		}
	}

	/**
	 * called by servercaller on return from getting the design.
	 * 
	 * this method sets an artist, all of its designs and the viewer!
	 * 
	 * @see #updateShopView(Displayable, List, Integer) for viewing a single artist and all its designs
	 * @see #updateShopView(List) for viewing all artists but not designs
	 * @see #setDesigns(List) which is called by updateShopView when given a single artist
	 */
	public void updateShopView(final Displayable toDisplay,
			List<Design> designsToShow, Integer designToPreSelect) {

		showingArtist = toDisplay != null;
		if(toDisplay == null){
			resultsArea.setWidget(0, 0, UIFactory.createHTML()); //empty
		}else{
			HorizontalPanel hp = UIFactory.createHorizontalPanel();
			hp.setVerticalAlignment(HasVerticalAlignment.ALIGN_TOP);
			Image i = UIFactory.createImage(toDisplay.getUrl());
			i.setTitle(toDisplay.getLongToolTip());
			hp.add(i);
			
			HorizontalPanel hp2 = UIFactory.createHorizontalPanel();
			HTML l = UIFactory.createHTML(toDisplay.getName());
			l.setStyleName(UIFactory.BIG_BOLD_TEXT);
			hp2.add(l);
			if(getWizard().getModel().getSessionInfo().isInRole(Roles.BACKOFFICE)){
				com.google.gwt.user.client.ui.Image edit = Images.images.edit().createImage();
				if(toDisplay.getType().equals(Displayable.ARTIST)){
					edit.setTitle("Edit this artist...");
				}else{
					throw new RuntimeException("code change required - not expecting to display type " + toDisplay.getType());
				}
				edit.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						if(toDisplay.getType().equals(Displayable.ARTIST)){
							getWizard().showArtistAdminPage((Artist)toDisplay);
						}else{
							throw new RuntimeException("code change required - not expecting to display type " + toDisplay.getType());
						}
					}
				});
				hp2.add(edit);
			}
			
			VerticalPanel vp = UIFactory.createVerticalPanel();
			hp.add(vp);
			vp.add(hp2);

			l = UIFactory.createHTML(toDisplay.getDescription());
			vp.add(l);
			resultsArea.setWidget(0, 0, hp);
		}
		
		setDesigns(designsToShow);

		////////////////
		//
		// now handle viewer
		//
		////////////////
		if(designToPreSelect == null){
			designToPreSelect = designsToShow.get(0).getUid();
		}
		
		Design preview = null;
		for(Design d : designsToShow){
			if(d.getUid().equals(designToPreSelect)){
				preview = d;
				break;
			}
		}
		showDesignDetail(preview);
	}
		
	/**
	 * shows a list of artists instead of designs
	 * 
	 * @see #updateShopView(Displayable, List, Integer) for viewing a single artist and all its designs
	 * @see #updateShopView(List) for viewing all artists but not designs
	 * @see #setDesigns(List) which is called by updateShopView when given a single artist
	 */
	public void updateShopView(List<? extends Displayable> toDisplay) {

		resultsArea.setWidget(0, 0, UIFactory.createHTML()); //empty

		//would be nice to use a flowpanel which automatically lays out and such...
		//but that doesnt work. flowpanel can also only handle images when flowing. as soon as
		//you add say a spacer HTML, it just adds a new line, since flowPanels use HTML default layout.
		//so, lets go to the pain of working out how many columns of images we can create and fill a
		//table that way...

		FlexTable table = UIFactory.createFlexTable();
		table.setCellSpacing(IMAGES_FILLER);

		int widthPerImage = ARTIST_WIDTH + IMAGES_FILLER;

		//total available height due to basket and viewer = 700 + viewer image height.
		//so height of scrollpanel is totalHeight - top - bottom - artistImageHeightIfShown
		int height = 
			700
			+ DESIGN_VIEWER_IMAGE_SIZE_WIDTH /* assume square :-( */
			- 210 /* top */
			- 70 /* bottom */
			- (showingArtist ?ARTIST_WIDTH :0/*works better during tests*/)  /* artist image */
			; //TODO 000 works on all monitor sizes?

		int width = Window.getClientWidth() - BASKET_WIDTH - SEARCH_WIDTH - 120/*spacing, padding, etc*/;
		int excess = width % widthPerImage; //show as many as we can, but dont use any more space!
		
		int numCols = (width - excess);
		numCols /= widthPerImage;

		int row = 0;
		int col = 0;
		for(Displayable d : toDisplay){
			final Displayable fd = d;

			ClickHandler ch = new ClickHandler(){
				public void onClick(ClickEvent event) {
					controller.selectDisplayable(fd);
				}
			};
			
			VerticalPanel vp = UIFactory.createVerticalPanel();
			Image img = UIFactory.createImage(d.getUrl());
			img.setStyleName(UIFactory.POINTER_CURSOR);
			img.setTitle(d.getLongToolTip());
			img.addClickHandler(ch);
			
			vp.add(img);
			HorizontalPanel hp = UIFactory.createHorizontalPanel();
			
			////////////////////
			// view details button
			////////////////////
			com.google.gwt.user.client.ui.Image viewDetails = UIFactory.createImageButton(Images.images.viewDetails().createImage());
			viewDetails.setTitle("Click to view details...");
			viewDetails.addClickHandler(ch);
			hp.add(viewDetails);
			
			////////////////////
			// name
			////////////////////
			HTML name = UIFactory.createHTML(d.getName());
			name.setWidth((DESIGN_IMAGE_SIZE_WIDTH - 23) + "px"); //ensure it doesnt screw the page up!
			hp.add(name);
			
			vp.add(hp);
			table.setWidget(row, col, vp);
			col++;
			if(col >= numCols){
				row++;
				col = 0;
			}
		}
		
		ScrollPanel sp = UIFactory.createScrollPanel();
		sp.setPixelSize(width, height);
		sp.add(table);
		resultsArea.setWidget(1, 0, sp);
		
		////////////////
		// now handle viewer
		////////////////
		detailArea.setWidget(0, 0, UIFactory.createHTML());
		basketBorder.setVisible(false);
	}

	/**
	 * sets the artists to display in on the left side of the gui.
	 * if the list is empty, then it says no results were found.
	 * 
	 * @param selectFirstOne if true, and typeToActivate is not set, then shows 
	 * 	the artist. otherwise we do nowt. eg if this is called as a result 
	 * of the first ever showing of the shop, and results are empty coz the 
	 * user clicked a featured design. in such a case, we have already selected the 
	 * artist, design and everything, so theres no need to reload the artist.
	 */
	public void setWhatsOnDisplay(final SearchResults results, ShopResultsType typeToActivate, boolean selectFirstOne) {

		if(results != null){
			StackPanel stp = UIFactory.createStackPanel();
			stp.setPixelSize(SEARCH_RESULTS_WIDTH, SEARCH_RESULTS_HEIGHT);
			int idxDesigns = -1;
			int idxArtists = -1;
			int idx = -1;
			HTML html = null;
			
			//////////////////////////////
			//
			// designs
			//
			//////////////////////////////
			if(results.getDesigns() != null){
				idx++;
				//its null, if we clear the search, since we only show artits then
				ScrollPanel sp = UIFactory.createScrollPanel();
				sp.setPixelSize(SEARCH_WIDTH, SEARCH_RESULTS_HEIGHT);
				VerticalPanel vp = UIFactory.createVerticalPanel();
				sp.add(vp);
				if(results.getDesigns().isEmpty()){
					html = UIFactory.createHTML("None.");
					vp.add(html);
				}else{
					idxDesigns = idx;
					html = UIFactory.createHTMLLikeHyperlink(
							"See all...");
					html.addStyleName(UIFactory.STANDARD_TEXT_BOLD);
					html.addClickHandler(new ClickHandler() {
						public void onClick(ClickEvent event) {
							updateShopView(null, results.getDesigns(), null);
						}
					});
					vp.add(html);
					
					for(Design d : results.getDesigns()){
						final Design fd = d;
						html = UIFactory.createHTMLLikeHyperlink(d.getName());
						html.addClickHandler(new ClickHandler() {
							public void onClick(ClickEvent event) {
								showDesignDetail(fd);
							}
						});
						vp.add(html);
					}
				}
				stp.add(sp, "Designs (" + results.getDesigns().size() + ")");
			}

			//////////////////////////////
			//
			// artists
			//
			//////////////////////////////
			if(results.getArtists() != null){
				idx++;
				ScrollPanel sp = UIFactory.createScrollPanel();
				sp.setPixelSize(SEARCH_WIDTH, SEARCH_RESULTS_HEIGHT);
				VerticalPanel vp = UIFactory.createVerticalPanel();
				sp.add(vp);
				if(results.getArtists().isEmpty()){
					html = UIFactory.createHTML("None");
					vp.add(html);
				}else{
					html = UIFactory.createHTMLLikeHyperlink("See all...");
					html.addStyleName(UIFactory.STANDARD_TEXT_BOLD);
					html.addClickHandler(new ClickHandler() {
						public void onClick(ClickEvent event) {
							updateShopView(results.getArtists());
						}
					});
					vp.add(html);

					idxArtists = idx;
					for(Artist a : results.getArtists()){
						final Artist af = a;
						html = UIFactory.createHTMLLikeHyperlink(a.getScreenname());
						html.addClickHandler(new ClickHandler() {
							public void onClick(ClickEvent event) {
								controller.selectDisplayable(af);
							}
						});
						vp.add(html);
					}
				}
				stp.add(sp, "Artists (" + results.getArtists().size() + ")");
			}
			
			//////////////////////////////
			//
			// featured designs
			//
			//////////////////////////////
			ScrollPanel sp = UIFactory.createScrollPanel();
			sp.setPixelSize(SEARCH_WIDTH, SEARCH_RESULTS_HEIGHT);
			VerticalPanel vp = UIFactory.createVerticalPanel();
			sp.add(vp);
			if(getWizard().getModel().getFeaturedDesigns().isEmpty()){
				html = UIFactory.createHTML("None.");
				vp.add(html);
			}else{
				idxDesigns = idx;
				html = UIFactory.createHTMLLikeHyperlink(
						"See all...");
				html.addStyleName(UIFactory.STANDARD_TEXT_BOLD);
				html.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						updateShopView(null, getWizard().getModel().getFeaturedDesigns(), null);
					}
				});
				vp.add(html);
				
				for(Design d : getWizard().getModel().getFeaturedDesigns()){
					final Design fd = d;
					html = UIFactory.createHTMLLikeHyperlink(d.getName());
					html.addClickHandler(new ClickHandler() {
						public void onClick(ClickEvent event) {
							showDesignDetail(fd);
						}
					});
					vp.add(html);
				}
			}
			stp.add(sp, "Featured Designs");
			
			//////////////////////////////
			//
			// most viewed artists
			//
			//////////////////////////////
			sp = UIFactory.createScrollPanel();
			sp.setPixelSize(SEARCH_WIDTH, SEARCH_RESULTS_HEIGHT);
			vp = UIFactory.createVerticalPanel();
			sp.add(vp);
			if(getWizard().getModel().getCloud().isEmpty()){
				html = UIFactory.createHTML("None");
				vp.add(html);
			}else{
				html = UIFactory.createHTMLLikeHyperlink("See all...");
				html.addStyleName(UIFactory.STANDARD_TEXT_BOLD);
				html.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						updateShopView(getWizard().getModel().getCloud());
					}
				});
				vp.add(html);

				for(Artist a : getWizard().getModel().getCloud()){
					final Artist af = a;
					html = UIFactory.createHTMLLikeHyperlink(a.getName());
					html.addClickHandler(new ClickHandler() {
						public void onClick(ClickEvent event) {
							controller.selectDisplayable(af);
						}
					});
					vp.add(html);
				}
			}
			stp.add(sp, "Most Viewed Artists");
			
			//////////////////////////////
			//
			// newest designs
			//
			//////////////////////////////
			sp = UIFactory.createScrollPanel();
			sp.setPixelSize(SEARCH_WIDTH, SEARCH_RESULTS_HEIGHT);
			vp = UIFactory.createVerticalPanel();
			sp.add(vp);
			if(getWizard().getModel().getNewestDesigns().isEmpty()){
				html = UIFactory.createHTML("None.");
				vp.add(html);
			}else{
				idxDesigns = idx;
				html = UIFactory.createHTMLLikeHyperlink(
						"See all...");
				html.addStyleName(UIFactory.STANDARD_TEXT_BOLD);
				html.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						updateShopView(null, getWizard().getModel().getNewestDesigns(), null);
					}
				});
				vp.add(html);
				
				for(Design d : getWizard().getModel().getNewestDesigns()){
					final Design fd = d;
					html = UIFactory.createHTMLLikeHyperlink(d.getName());
					html.addClickHandler(new ClickHandler() {
						public void onClick(ClickEvent event) {
							showDesignDetail(fd);
						}
					});
					vp.add(html);
				}
			}
			stp.add(sp, "Newest Designs");

			searchResults.setWidget(0, 0, stp);

			//////////////////////////////
			//
			// put it all together...
			//
			//////////////////////////////
			int idxToSelect = 0;
			if(idxDesigns >= 0){
				//we have designs, so open that tab
				idxToSelect = idxDesigns;
			}else if(idxArtists >= 0){
				//no designs, but we have artists, so open that tab
				idxToSelect = idxArtists;
			}
			
			if(typeToActivate != null){
				//override which stack to show!
				if(typeToActivate.equals(ShopResultsType.ALL_ARTISTS)){
					idxToSelect = 1;
				}else if(typeToActivate.equals(ShopResultsType.ALL_DESIGNS)){
					idxToSelect = 0;
				}else if(typeToActivate.equals(ShopResultsType.FEATURED_DESIGNS)){
					idxToSelect = 3;
				}else if(typeToActivate.equals(ShopResultsType.MOST_VIEWED_ARTISTS)){
					idxToSelect = 4;
				}else if(typeToActivate.equals(ShopResultsType.NEWEST_DESIGNS)){
					idxToSelect = 5;
				}else if(typeToActivate.equals(ShopResultsType.DESIGNS_PENDING_MODERATION)){
					idxToSelect = 0;
				}else {
					throw new RuntimeException("code change required - unknown type " + typeToActivate);
				}

				stp.showStack(idxToSelect);

				if(typeToActivate.equals(ShopResultsType.ALL_DESIGNS)){
					updateShopView(null, results.getDesigns(), null);
				}else if(typeToActivate.equals(ShopResultsType.NEWEST_DESIGNS)){
					updateShopView(null, getWizard().getModel().getNewestDesigns(), null);
				}else if(typeToActivate.equals(ShopResultsType.FEATURED_DESIGNS)){
					updateShopView(null, getWizard().getModel().getFeaturedDesigns(), null);
				}else if(typeToActivate.equals(ShopResultsType.MOST_VIEWED_ARTISTS)){
					updateShopView(getWizard().getModel().getCloud());
				}else if(typeToActivate.equals(ShopResultsType.ALL_ARTISTS)){
					updateShopView(results.getArtists());
				}else if(typeToActivate.equals(ShopResultsType.DESIGNS_PENDING_MODERATION)){
					updateShopView(results.getDesigns());
				}
			}else{
				
				stp.showStack(idxToSelect);

				//select the first one by default
				if(selectFirstOne && !results.getArtists().isEmpty()){
					controller.selectDisplayable(results.getArtists().get(0));
				}
			}

		}else{
			searchResults.setWidget(0, 0, UIFactory.createHTML());
		}			
	}

	/**
	 * @see MasterDataListener#notifyMasterDataIsReady()
	 */
	public void notifyMasterDataIsReady() {

		//////////////////////////
		// now handle event
		//////////////////////////
		masterDataHasBeenLoaded = true;
		if(isActive() && isAwaitingMasterData){
			isAwaitingMasterData = false;
			
			updateShopView(getWizard().getModel().getDisplayable(), getWizard().getModel().getDesigns(), getWizard().getModel().getDesignToPreSelect());
		}
	}

	public void performSearch(String search) {
		searchField.setText(search);
		controller.search(searchField.getText().trim());
	}

	/**
	 * call this to do a search for the result type and display them
	 */
	public void viewResults(ShopResultsType resultType) {

		if(resultType != null){
			//the page may be activated when this call returns. if so, we dont want to load artists,
			//which is what the activate method does, if the wizard model is emtpy. so... set some shit
			//in the model, to avoid unncessary loading of artists.
			//its fine to do this, since the calls to the controller below will ensure we load some content!
			WizardModel m = getWizard().getModel();
			m.setDisplayable(new Design());
		}

		if(resultType.equals(ShopResultsType.DESIGNS_PENDING_MODERATION)){
			controller.showDesignsPendingModeration(resultType);
		}else{
			controller.search(null/*wildcard search*/, resultType);
		}
	}

	public void setDesign(Design design) {
		throw new UnsupportedOperationException("this method is only to be called when in the design detail page.");
	}

	public void addedComment() {
		String s = "Your comment has been added.";
		if(getWizard().getModel().isModerationActive()){
			s = "Your comment has been submitted. Once it has been reviewed, it will be visible.";
		}
		UIFactory.createInformationDialog("Thanks", s).center();
	}

	public void approvedComment() {
		String s = "The comment has been edited.";
		if(getWizard().getModel().isModerationActive()){
			s = "The comment has been edited/approved.";
		}
		UIFactory.createInformationDialog("Thanks", s).center();
	}
	
	public void rejectedComment() {
		String s = "The comment has been rejected.";
		UIFactory.createInformationDialog("Thanks", s).center();
	}

}
