/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.to;

import java.io.Serializable;
import java.util.List;

/**
 */
public class Design implements Serializable, Displayable {

	private static final long serialVersionUID = 1L;

	private Integer uid;
	private String name;
	private String description;
	private List<String> tags;
    private boolean inCatalogue;
    private boolean approved;
    private double basePrice;
    private int rating;
	private Artist artist;
	private int numberOfComments;
	private List<DesignComment> comments;
	private boolean imageExists;

    public double getBasePrice() {
		return basePrice;
	}

	public void setBasePrice(double basePrice) {
		this.basePrice = basePrice;
	}

    public int getRating() {
		return rating;
	}

	public void setRating(int rating) {
		this.rating = rating;
	}

	public boolean isApproved() {
		return approved;
	}

	public void setApproved(boolean approved) {
		this.approved = approved;
	}

	public boolean isInCatalogue() {
		return inCatalogue;
	}

	public void setInCatalogue(boolean inCatalogue) {
		this.inCatalogue = inCatalogue;
	}

	/** default constructor */
	public Design() {
	}

	/** minimal constructor */
	public Design(Integer uid, String name, String description) {
		this.uid = uid;
		this.name = name;
		this.description = description;
	}

	// Property accessors
	public Integer getUid() {
		return this.uid;
	}

	public void setUid(Integer uid) {
		this.uid = uid;
	}

	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getDescription() {
		return this.description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public List<String> getTags() {
		return tags;
	}

	public void setTags(List<String> tags) {
		this.tags = tags;
	}

	/**
	 * tests uid equality.
	 */
	@Override
	public boolean equals(Object obj) {
		boolean b = this == obj;
		if(!b){
			if(obj instanceof Design){
				Design that = (Design)obj;
				if(this.uid.equals(that.uid)){
					b = true;
				}
			}
		}
		return b;
	}

	public Artist getArtist() {
		return artist;
	}

	public void setArtist(Artist artist) {
		this.artist = artist;
	}
	
	public String getToolTip() {
		return "Design: " + getName() + " (" + getUid() + ") by " + getArtist().getScreenname();
	}

	public String getLongToolTip() {
		String tt = getName() + " (" + getUid() + ") \r\n";
		tt += getDescription()+ " \r\n"; //TODO remove html from description!
		tt += "By: " + getArtist().getScreenname(); // + " \r\n"; //TODO remove html from description!
		return tt;
	}
	
	/**
	 * @return url for a 100x100 image, or that of the "unknown" image, if none exists.
	 */
	public String getUrl() {
		if(isImageExists()){
			return "/images/designs/design_small_" + getUid() + ".jpg";
		}else{
			return "/images/designs/design_small_unknown.jpg";
		}
	}
	
	/**
	 * @return url for a 200x200 image, or that of the "unknown" image, if none exists.
	 */
	public String getViewerUrl() {
		if(isImageExists()){
			return "/images/designs/design_" + getUid() + ".jpg";
		}else{
			return "/images/designs/design_unknown.jpg";
		}
	}
	
	/**
	 * @return url for a 400x400 image
	 */
	public String getZoomUrl() {
		return "/images/designs/design_large_" + getUid() + ".jpg";
	}
	
	/**
	 * @return url for the secured original
	 */
	public String getHiresUrl() {
		return "/hiresimages/designs/design_large_" + getUid() + ".jpg";
	}

	public String getType() {
		return DESIGN;
	}

	@Override
	public String toString() {
		return "Design " + uid;
	}

	public int getNumberOfComments() {
		return numberOfComments;
	}

	public void setNumberOfComments(int numberOfComments) {
		this.numberOfComments = numberOfComments;
	}

	public List<DesignComment> getComments() {
		return comments;
	}

	public void setComments(List<DesignComment> comments) {
		this.comments = comments;
	}
	
	public boolean isImageExists() {
		return imageExists;
	}

	public void setImageExists(boolean imageExists) {
		this.imageExists = imageExists;
	}
	
}
