/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.util;

import uk.co.maxant.gwtdemo.client.widgets.core.WizardServerCaller;

import com.google.gwt.user.client.rpc.AsyncCallback;

/**
 * generic impl which includes error handling.
 */
public abstract class AsyncCallbackImpl<T> implements AsyncCallback<T> {

	protected WizardServerCaller wsc;
	
	public AsyncCallbackImpl(WizardServerCaller wsc) {
		this.wsc = wsc;
	}
	
	public final void onFailure(Throwable caught) {
		if(onFailurePre(caught)){
			wsc.onFailure(caught);
			onFailurePost(caught);
		}
	}
	
	/**
	 * @return if true, then {@link #onFailure(Throwable)} and {@link #onFailurePost(Throwable)} will also be called.
	 * 	effectively, you return whether or not to continue.
	 */
	protected boolean onFailurePre(Throwable caught) {
		return true;
	}

	protected  void onFailurePost(Throwable caught) {
	}

	public final void onSuccess(T result) {
		wsc.serverReturn();
		onSuccess_(result);
	}
	
	public abstract void onSuccess_(T result);
}
