/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.util;

import uk.co.maxant.gwtdemo.client.ApplicationException;
import uk.co.maxant.gwtdemo.client.generic.widgets.Dialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;

import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.rpc.StatusCodeException;

public final class ErrorHandler {

	private ErrorHandler(){
	}
	
	public static void handleError(Throwable t){
		boolean handled = false;
		if(t instanceof StatusCodeException){
			StatusCodeException sce = (StatusCodeException)t;
			if(sce.getStatusCode() == 401){
				//force login. under normal circumstances, this
				//feature is not used to force a login for 
				//changing the users role. if a user wants to
				//do something which their role does not allow, 
				//the GUI doesnt even give them the option. so
				//this redirect helps hackers find the login 
				//page, and more importantly, if the session 
				//has timed out, it lets the user login again.
				handled = true;
				UIFactory.createErrorDialog(
						"No Authorisation", 
						"Please log in with sufficient " +
						"rights to perform this action.", 
						new Dialog.Callback() {
							public void onClose() {
								Window.Location.assign("/secure");
							}
				}).center();
			}
		}else if(t instanceof ApplicationException){
			ApplicationException ae = (ApplicationException)t;
			UIFactory.createErrorDialog("Error", 
					ae.getUserDetails() + "<br><br>" + 
					ae.getCode() + "/" + ae.getTimestamp()).center();
			handled = true;
		}

		if(!handled){
			UIFactory.createErrorDialog("Error", "Error: " + 
					t.getMessage()).center();
		}
	}
}
