/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.util;

import com.google.gwt.user.client.Command;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.MenuItem;

import uk.co.maxant.gwtdemo.client.Roles;
import uk.co.maxant.gwtdemo.client.generic.widgets.InformationDialog;
import uk.co.maxant.gwtdemo.client.generic.widgets.MenuBar;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.shop.core.ShopController.ShopResultsType;
import uk.co.maxant.gwtdemo.client.to.SessionInfo;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

public class MenuBuilder {

	private MenuBuilder(){}
	
	public static void buildShopMenu(MenuBar root, SessionInfo info, final AppWizard wizard){
//new InformationDialog("", "building menu...").show();
		root.clearItems();

		/////////////////
		// create main menu sub items
		/////////////////
		root.addItem("<div class='menuHome'>Home</div>", true, new Command(){
			public void execute() {
				wizard.showFirstPage();
			}
		});

		/////////////////
		// create main menu sub items
		/////////////////

		root.addItem("<div class='menuShop'>Shop</div>", true, new Command(){
			public void execute() {
				wizard.showShopPage(false);
			}
		});

		/////////////////
		// browse menu
		/////////////////
		MenuBar browse = UIFactory.createMenuBar(true);
		
		MenuItem item = new MenuItem("Artists", new Command(){
			public void execute() {
				wizard.showShopPage(ShopResultsType.ALL_ARTISTS);
			}
		});
		browse.addItem(item);
		
		item = new MenuItem("Designs", new Command(){
			public void execute() {
				wizard.showShopPage(ShopResultsType.ALL_DESIGNS);
			}
		});
		browse.addItem(item);
		
		item = new MenuItem("Featured Designs", new Command(){
			public void execute() {
				wizard.showShopPage(ShopResultsType.FEATURED_DESIGNS);
			}
		});
		browse.addItem(item);
		
		item = new MenuItem("Newest Designs", new Command(){
			public void execute() {
				wizard.showShopPage(ShopResultsType.NEWEST_DESIGNS);
			}
		});
		browse.addItem(item);
		
		root.addItem("<div class='menuBrowse'>Browse</div>", true, browse);
		
		/////////////////
		// BACKOFFICE role
		/////////////////
		if(info.isInRole(Roles.BACKOFFICE)){
			/////////////////
			// create admin menu sub items
			/////////////////
			MenuBar mb = UIFactory.createMenuBar(true);
			mb.addItem("Design Admin", new Command(){
				public void execute() {
					Window.Location.assign("/secure/indexAdmin.jsp?screen=0");
				}
			});
			mb.addItem("Artist Admin", new Command(){
				public void execute() {
					Window.Location.assign("/secure/indexAdmin.jsp?screen=1");
				}
			});
			mb.addItem("Design List", new Command(){
				public void execute() {
					Window.Location.assign("/secure/indexAdmin.jsp?screen=2");
				}
			});
			
			/////////////////
			// create admin menu
			/////////////////
			item = new MenuItem("Admin", mb);
			root.addItem(item);
		}

		/////////////////
		// PICKING role
		/////////////////
		if(info.isInRole(Roles.PICKING)){
			/////////////////
			// create pickers menu sub items
			/////////////////
			MenuBar mb = UIFactory.createMenuBar(true);
			
			mb.addItem("Search Orders", new Command(){
				public void execute() {
					Window.Location.assign("/secure/indexAdmin.jsp?screen=4");
				}
			});
			
			mb.addItem("Orders Pending", new Command(){
				public void execute() {
					Window.Location.assign("/secure/indexAdmin.jsp?screen=3");
				}
			});

			mb.addItem("Problem Orders", new Command(){
				public void execute() {
					Window.Location.assign("/secure/indexAdmin.jsp?screen=6");
				}
			});
			
			mb.addItem("Discarded Orders", new Command(){
				public void execute() {
					Window.Location.assign("/secure/indexAdmin.jsp?screen=7");
				}
			});
			
			mb.addItem("Order Details", new Command(){
				public void execute() {
					Window.Location.assign("/secure/indexAdmin.jsp?screen=5");
				}
			});
			
			/////////////////
			// create pickers menu
			/////////////////
			item = new MenuItem("Picking", mb);
			root.addItem(item);
		}

		if(info.isLoggedIn()){
			//create user menu
			MenuBar userMb = UIFactory.createMenuBar(true);
			
			userMb.addItem("My Orders", new Command(){
				public void execute() {
					wizard.showOrdersCustomerPage();
				}
			});

			userMb.addItem("My Profile", new Command(){
				public void execute() {
					wizard.showProfilePage();
				}
			});

			if(info.isInRole(Roles.ARTIST)){
				userMb.addItem("My Artist", new Command(){
					public void execute() {
						wizard.showArtistProfilePage();
					}
				});

				userMb.addItem("My Designs", new Command(){
					public void execute() {
						wizard.showArtistsDesignPage();
					}
				});
				
				userMb.addItem("My Sales", new Command(){
					public void execute() {
						//TODO 000 create this page
						new InformationDialog("TODO", "This would show the artist their sales and payments..").center();
					}
				});
			}
			
			item = new MenuItem("My Stuff", userMb);
			root.addItem(item);
		}
		
		/////////////////
		// SUPER_ADMIN role
		/////////////////
		if(info.isInRole(Roles.SUPER_ADMIN)){
			item = new MenuItem("Admin Tools", new Command() {
				public void execute() {
					Window.Location.assign("/superAdmin/tools.jsp");
				}
			});
			root.addItem(item);
		}

		/////////////////
		// moderator role
		/////////////////
		if(info.isDesignCommentModerationActive() && info.isInRole(Roles.DESIGN_COMMENTS_MODERATOR)){
			MenuBar mb = UIFactory.createMenuBar(true);
			
			mb.addItem("Show designs needing comment moderation...", new Command(){
				public void execute() {
					wizard.showShopPage(ShopResultsType.DESIGNS_PENDING_MODERATION);
				}
			});
			
			item = new MenuItem("Moderation", mb);
			root.addItem(item);
		}
		
	}

	public static void buildAdminMenu(MenuBar root, SessionInfo info, final AppWizard wizard){

		root.clearItems();

		/////////////////
		// create main admin menu sub items
		/////////////////
		root.addItem("<div class='menuHome'>Home</div>", true, new Command(){
			public void execute() {
				Window.Location.assign("/index.jsp");
			}
		});

		/////////////////
		// create main menu sub items
		/////////////////

		root.addItem("<div class='menuShop'>Shop</div>", true, new Command(){
			public void execute() {
				Window.Location.assign("/index.jsp?screen=1");
			}
		});

		/////////////////
		// BACKOFFICE role
		/////////////////
		if(info.isInRole(Roles.BACKOFFICE)){
			/////////////////
			// create admin menu sub items
			/////////////////
			MenuBar mb = UIFactory.createMenuBar(true);
			mb.addItem("Design Admin", new Command(){
				public void execute() {
					wizard.showDesignAdminPage(null);
				}
			});
			mb.addItem("Artist Admin", new Command(){
				public void execute() {
					wizard.showArtistAdminPage(null);
				}
			});
			mb.addItem("Design List", new Command(){
				public void execute() {
					wizard.showDesignAdminListPage();
				}
			});
			
			/////////////////
			// create admin menu
			/////////////////
			MenuItem item = new MenuItem("Admin", mb);
			root.addItem(item);
		}

		/////////////////
		// PICKING role
		/////////////////
		if(info.isInRole(Roles.PICKING)){
			/////////////////
			// create pickers menu sub items
			/////////////////
			MenuBar mb = UIFactory.createMenuBar(true);
			
			mb.addItem("Search Orders", new Command(){
				public void execute() {
					wizard.showOrdersSearchAdminPage();
				}
			});
			
			mb.addItem("Orders Pending", new Command(){
				public void execute() {
					wizard.showOrdersPendingAdminPage();
				}
			});

			mb.addItem("Problem Orders", new Command(){
				public void execute() {
					wizard.showOrdersProblemsAdminPage();
				}
			});
			
			mb.addItem("Discarded Orders", new Command(){
				public void execute() {
					wizard.showOrdersDiscardedAdminPage();
				}
			});
			
			mb.addItem("Order Details", new Command(){
				public void execute() {
					wizard.showOrderAdminDetailPage(null);
				}
			});
			
			/////////////////
			// create pickers menu
			/////////////////
			MenuItem item = new MenuItem("Picking", mb);
			root.addItem(item);
		}

		if(info.isLoggedIn()){
			//create user menu
			MenuBar userMb = UIFactory.createMenuBar(true);
			
			userMb.addItem("My Orders", new Command(){
				public void execute() {
					Window.Location.assign("/index.jsp?screen=4");
				}
			});

			userMb.addItem("My Profile", new Command(){
				public void execute() {
					Window.Location.assign("/index.jsp?screen=8");
				}
			});

			if(info.isInRole(Roles.ARTIST)){
				userMb.addItem("My Artist", new Command(){
					public void execute() {
						Window.Location.assign("/index.jsp?screen=9");
					}
				});

				userMb.addItem("My Designs", new Command(){
					public void execute() {
						Window.Location.assign("/index.jsp?screen=10");
					}
				});
				
				userMb.addItem("My Sales", new Command(){
					public void execute() {
						//TODO 000 create this page
						new InformationDialog("TODO", "This would show the artist their sales and payments..").center();
					}
				});
			}
			
			MenuItem item = new MenuItem("My Stuff", userMb);
			root.addItem(item);
		}
		
		/////////////////
		// SUPER_ADMIN role
		/////////////////
		if(info.isInRole(Roles.SUPER_ADMIN)){
			MenuItem item = new MenuItem("Admin Tools", new Command() {
				public void execute() {
					Window.Location.assign("/superAdmin/tools.jsp");
				}
			});
			root.addItem(item);
		}

		/////////////////
		// moderator role
		/////////////////
		if(info.isDesignCommentModerationActive() && info.isInRole(Roles.DESIGN_COMMENTS_MODERATOR)){
			MenuBar mb = UIFactory.createMenuBar(true);
			
			mb.addItem("Show designs needing comment moderation...", new Command(){
				public void execute() {
					Window.Location.assign("/index.jsp?screen=0"); //TODO would be nice to also pass a parameter which this page uses in order to start the search for moderation...
				}
			});
			
			MenuItem item = new MenuItem("Moderation", mb);
			root.addItem(item);
		}
		
	}

}
