/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.widgets.core;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import uk.co.maxant.gwtdemo.client.Roles;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.BasketItem;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.Displayable;
import uk.co.maxant.gwtdemo.client.to.SessionInfo;

/**
 * model for all screens
 */
public class WizardModel {

	/**
	 * list of listeners.
	 */
	private Set<ModelListener> listeners = new HashSet<ModelListener>();
	
	/**
	 * shopping cart content.
	 * in this main model, because its used between shop and checkout and orderComplete.
	 */
	private List<BasketItem> basket;
	
	/**
	 * the orderUid created when a customer creates an order
	 * in this main model, because its used between checkout and orderComplete.
	 */
	private Integer orderUid;
	
	/**
	 * the artist on display in the shop
	 * in this main model, because its used between home and shop.
	 */
	private Displayable displayable;

	/**
	 * the design which should be preseleted in the shop view
	 * in this main model, because its used between home and shop.
	 */
	private Integer designToPreSelect;

	/**
	 * the designs that belong to the artist which is currently on display.
	 * in this main model, because its used between home and shop.
	 */
	private List<Design> designs;

	private SessionInfo sessionInfo;

	private Integer optionalUid;

	private List<Artist> cloud;

	private List<Design> featuredDesigns;

	private List<Design> newestDesigns;
	
	/**
	 * notifies listeners.
	 */
	private void fireBasketUpdated(List<BasketItem> basketItems) {
		for(ModelListener listener : listeners){
			listener.updateBasket(basketItems);
		}
	}
	
	public void setBasket(List<BasketItem> basketItems) {
		
		this.basket = basketItems;
		
		fireBasketUpdated(basketItems);
	}

	public List<BasketItem> getBasket() {
		return basket;
	}

	public void setOrderUid(Integer orderUid) {
		this.orderUid = orderUid;
	}

	public Integer getOrderUid() {
		return orderUid;
	}

	/**
	 * sets the artist which the shop is currently displaying
	 * 
	 * @param displayable the artist to show
	 */
	public void setDisplayable(Displayable displayable) {
		this.displayable = displayable;
	}

	public Displayable getDisplayable() {
		return displayable;
	}

	public void setDesigns(List<Design> designs) {
		this.designs = designs;
	}

	public Integer getDesignToPreSelect() {
		return designToPreSelect;
	}

	public void setDesignToPreSelect(Integer designToPreSelect) {
		this.designToPreSelect = designToPreSelect;
	}

	public List<Design> getDesigns() {
		return designs;
	}

	public SessionInfo getSessionInfo() {
		return sessionInfo;
	}

	public void setSessionInfo(SessionInfo sessionInfo) {
		this.sessionInfo = sessionInfo;
	}

	public List<Artist> getAllArtists() {
		if(sessionInfo == null){
			UIFactory.createErrorDialog("Error", "Master data (artists) not loaded. Please contact maxant for information.").center();
		}
		return sessionInfo.getAllArtists();
	}
	
	/**
	 * clears EVERYTHING. called eg when something is done in an admin view, since this models contents
	 * may now be invalid.
	 */
	public void clear(){
		setBasket(new ArrayList<BasketItem>());
		setDesigns(new ArrayList<Design>());
		setDesignToPreSelect(null);
		setDisplayable(null);
	}

	/**
	 * set a UID which a screen can optionally use to load when it is activated for the first time.
	 */
	public void setUid(Integer uid) {
		this.optionalUid = uid;
	}
	
	/**
	 * calling this method sets the UID in this model to be null!
	 * 
	 * @return a UID which a screen can optionally use to load when it is first activated.
	 */
	public Integer getOptionalUid() {
		Integer i = optionalUid;
		optionalUid = null;
		return i;
	}

	public void setFeaturedDesigns(List<Design> designs) {
		featuredDesigns = designs;
	}

	public void setCloud(List<Artist> artists) {
		cloud = artists;
	}

	public List<Artist> getCloud() {
		return cloud;
	}

	public List<Design> getFeaturedDesigns() {
		return featuredDesigns;
	}

	public void setNewestDesigns(List<Design> newestDesigns) {
		this.newestDesigns = newestDesigns;
	}

	public List<Design> getNewestDesigns() {
		return newestDesigns;
	}

	public void addListener(ModelListener listener){
		listeners.add(listener);
	}
	
	/**
	 * @return is design comment moderation turned on in the config?
	 */
	public boolean isModerationActive() {
		boolean isModerationActive = false;
		if(getSessionInfo() != null){
			if(getSessionInfo().isDesignCommentModerationActive()){
				isModerationActive = true;
			}
		}
		return isModerationActive;
	}
	
	/**
	 * @return is the logged in user in the design comment moderator role?
	 */
	public boolean isModerator() {
		boolean isModerator = false;
		if(getSessionInfo() != null){
			if(getSessionInfo().isInRole(Roles.DESIGN_COMMENTS_MODERATOR)){
				isModerator = true;
			}
		}
		return isModerator;
	}

}
