/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.widgets.core;

import java.util.List;

import uk.co.maxant.gwtdemo.client.ApplicationLayerService;
import uk.co.maxant.gwtdemo.client.ApplicationLayerServiceAsync;
import uk.co.maxant.gwtdemo.client.to.BasketItem;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.Displayable;
import uk.co.maxant.gwtdemo.client.util.AsyncCallbackImpl;
import uk.co.maxant.gwtdemo.client.util.ErrorHandler;
import uk.co.maxant.gwtdemo.client.widgets.ui.AppWizard;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.RootPanel;

public abstract class WizardServerCaller {

	protected AppWizard wizard;
	
	/**
	 * incremented for calls to server, decremented on call returns.
	 */
	private static int callCount = 0;

	/**
	 * the waitable to set waiting or not, depending upon callCount
	 */
	private Waitable waitable;
	
	/**
	 * to be called before making a call to the server
	 */
	public void callServer(){
		callCount++;
		if(callCount > 0){
			DOM.setStyleAttribute(RootPanel.get().getElement(), "cursor", "wait"); 
			if(waitable != null) waitable.setWait(true);
		}
	}

	/**
	 * to be called when a server call returns, regardless of whether its an error or not
	 */
	public void serverReturn(){
		callCount--;
		if(callCount <= 0){
			DOM.setStyleAttribute(RootPanel.get().getElement(), "cursor", null); 
			if(waitable != null) waitable.setWait(false);
		}
	}

	private ApplicationLayerServiceAsync serviceClient;

	public WizardServerCaller(AppWizard wizard, Waitable waitable) {
		this.wizard = wizard;
		this.waitable = waitable;
	}
	
	public WizardServerCaller(AppWizard wizard) {
		this.wizard = wizard;
	}
	
	protected synchronized ApplicationLayerServiceAsync getServiceClient(){
		if(serviceClient == null){
			serviceClient = GWT.create(ApplicationLayerService.class);
		}
		return serviceClient;
	}

	/**
	 * gets the designs in detail for the given artist and shows the artist
	 * and preselects the design in the shop. a null for designToPreSelect 
	 * means the first design in the artist gets selected.
	 * 
	 * @param activeShopPage if true, then the shop page will also be activated immediately.
	 * @see ApplicationLayerService#getDesigns(Integer)
	 */
	public void showDisplayableInShop(final Displayable displayable, final Integer designToPreSelect) {
		showDisplayableInShop(displayable, designToPreSelect, false);
	}

	/**
	 * only to be called if you are an artist really. shows artists content including stuff
	 * thats not purchaseable.
	 * 
	 * @see #showDisplayableInShop(Displayable, Integer)
	 */
	public void showDisplayableInShop(final Displayable displayable, final Integer designToPreSelect, boolean getAllContent) {

		AsyncCallback<List<Design>> callback = new AsyncCallbackImpl<List<Design>>(this) {
			
			public void onSuccess_(List<Design> designs) {
				wizard.getModel().setDisplayable(displayable);
				wizard.getModel().setDesigns(designs);
				wizard.getModel().setDesignToPreSelect(designToPreSelect);
				wizard.showShopPage(true);
			}
		};
		callServer();

		if(displayable.getType().equals(Displayable.ARTIST)){
			getServiceClient().getDesignsForArtist(displayable.getUid(), callback);
		}else{
			//ignore. eg this is set by the shop when wanting something in the model to avoid its activation
			//getting all artists.
		}
	}
	
	/**
	 * gets the basket and informs all BasketUpdaters
	 */
	public void getBasket() {
		
		AsyncCallback<List<BasketItem>> callback = new AsyncCallbackImpl<List<BasketItem>>(this) {
			
			public void onSuccess_(List<BasketItem> basketItems) {
				wizard.getModel().setBasket(basketItems);
			}
		};
		callServer();
		getServiceClient().getBasket(callback);
	}

	public void onFailure(Throwable caught){
		serverReturn();
		ErrorHandler.handleError(caught);
	}

}
