/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.widgets.ui;

import com.google.gwt.user.client.Window;

import uk.co.maxant.gwtdemo.client.artist_admin.ui.ArtistAdmin;
import uk.co.maxant.gwtdemo.client.design_admin.ui.DesignAdmin;
import uk.co.maxant.gwtdemo.client.design_admin_list.ui.DesignAdminList;
import uk.co.maxant.gwtdemo.client.order_admin_list.ui.OrdersDiscardedAdminList;
import uk.co.maxant.gwtdemo.client.order_admin_list.ui.OrdersPendingAdminList;
import uk.co.maxant.gwtdemo.client.order_admin_list.ui.OrdersProblemsAdminList;
import uk.co.maxant.gwtdemo.client.order_admin_list.ui.OrdersSearchAdminList;
import uk.co.maxant.gwtdemo.client.order_detail.ui.OrderAdminDetail;
import uk.co.maxant.gwtdemo.client.shop.core.ShopController.ShopResultsType;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.util.MenuBuilder;

/**
 * app specific impl.
 */
public class AdminWizard extends AppWizard {

    protected DesignAdmin designAdmin;

    protected ArtistAdmin artistAdmin;

    protected DesignAdminList designAdminList;

    protected OrdersPendingAdminList ordersPendingAdminList;

    protected OrdersDiscardedAdminList ordersDiscardedAdminList;

    protected OrdersProblemsAdminList ordersProblemsAdminList;

    protected OrdersSearchAdminList ordersSearchAdminList;

    protected OrderAdminDetail orderAdminDetail;

    /**
     * Creates a new Wizard object.
     */
    public AdminWizard() {

        designAdmin = new DesignAdmin(this); // page 0
        addPage(designAdmin);

        artistAdmin = new ArtistAdmin(this); // page 1
        addPage(artistAdmin);

        designAdminList = new DesignAdminList(this); // page 2
        addPage(designAdminList);
        
        ordersPendingAdminList = new OrdersPendingAdminList(this); // page 3
        addPage(ordersPendingAdminList);
        
        ordersSearchAdminList = new OrdersSearchAdminList(this); // page 4
        addPage(ordersSearchAdminList);
        
        orderAdminDetail = new OrderAdminDetail(this); // page 5
        addPage(orderAdminDetail);
        
        ordersProblemsAdminList = new OrdersProblemsAdminList(this); // page 6
        addPage(ordersProblemsAdminList);
        
        ordersDiscardedAdminList = new OrdersDiscardedAdminList(this); // page 7
        addPage(ordersDiscardedAdminList);
    }

	public void showOrdersPendingAdminPage() {
        showPage(ordersPendingAdminList);
	}

	public void showOrdersDiscardedAdminPage() {
		showPage(ordersDiscardedAdminList);
	}
	
	public void showOrdersProblemsAdminPage() {
		showPage(ordersProblemsAdminList);
	}
	
	public void showOrdersSearchAdminPage() {
		showPage(ordersSearchAdminList);
	}
	
	public void showOrdersCustomerPage() {
		Window.Location.assign("/secure/index.jsp?screen=_");
	}
	
	public void showProfilePage() {
		Window.Location.assign("/secure/index.jsp?screen=_");
	}
	
	public void showOrderAdminDetailPage(SalesOrder so) {
		showPage(orderAdminDetail);
		orderAdminDetail.showOrder(so);
	}
	
	public void showOrderCustomerDetailPage(SalesOrder so) {
		throw new UnsupportedOperationException("shouldnt ever be called by the admin wizard");
	}
	
	public void showDesignDetailPage(Design design) {
		Window.Location.assign("/index.jsp?screen=_&designId=" + design.getUid());
	}
	
    /**
     * does what it says on the tin.
     *
     * @param alsoUpdate if true, then the shop view is also updated to display the displayable, designs 
     * 		and preselcted_design from the model.
     */
    public void showShopPage(boolean alsoUpdate) {
		Window.Location.assign("/index.jsp?screen=_");
    }

    /**
     * does what it says on the tin.
     *
     * @param  waitingForServerResponse  if true, wait status is set to true
     */
    public void showShopPage(ShopResultsType results) {
		throw new UnsupportedOperationException("shouldnt ever be called by the admin wizard");
    }
    
    /**
     * does what it says on the tin.
     *
     * @param  d
     */
    public void showDesignAdminPage(Design d) {
        showPage(designAdmin);
        designAdmin.showDesign(d);
    }
    
	public void showArtistProfilePage() {
		Window.Location.assign("/secure/index.jsp?screen=9");
	}

    public void showArtistAdminPage(Artist a) {
        showPage(artistAdmin);
        artistAdmin.showArist(a);
    }

    public void showDesignAdminListPage() {
    	showPage(designAdminList);
    }
    
	public void showArtistsDesignPage() {
		Window.Location.assign("/secure/index.jsp?screen=10");
	}
	
    @Override
    protected void buildMenu(){
        MenuBuilder.buildAdminMenu(menu, model.getSessionInfo(), this);
    }
    
	public void doSearch(String search){
		showShopPage(false);
		//TODO would be nice to pass a parameter so the search is also conducted. although im not sure this method is actaully called from the admin screen!
	}

}
