/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.widgets.ui;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.LoadEvent;
import com.google.gwt.event.dom.client.LoadHandler;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.RootPanel;
import com.google.gwt.user.client.ui.VerticalPanel;

import uk.co.maxant.gwtdemo.client.artist_admin.ui.ArtistAdmin;
import uk.co.maxant.gwtdemo.client.design_admin.ui.DesignAdmin;
import uk.co.maxant.gwtdemo.client.design_admin_list.ui.DesignAdminList;
import uk.co.maxant.gwtdemo.client.generic.widgets.Image;
import uk.co.maxant.gwtdemo.client.generic.widgets.MenuBar;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;
import uk.co.maxant.gwtdemo.client.shop.core.ShopController.ShopResultsType;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.widgets.core.MasterDataListener;
import uk.co.maxant.gwtdemo.client.widgets.core.WizardController;
import uk.co.maxant.gwtdemo.client.widgets.core.WizardModel;


/**
 * app specific impl.
 */
public abstract class AppWizard extends uk.co.maxant.gwtdemo.client.generic.widgets.Wizard implements EntryPoint {

    protected MenuBar menu;

    protected WizardModel model = new WizardModel();
    
    protected List<MasterDataListener> masterDataListeners = new ArrayList<MasterDataListener>();

	private WizardController controller;
    
    /**
     * Creates a new Wizard object.
     */
    public AppWizard() {

    	this.controller = new WizardController(this);
    	
        menu = UIFactory.createMenuBar(false);
        
        //set the width of the menu bar approximately, altho once we load the image, we will set it exactly
		int w = Window.getClientWidth() - /*assumedImageWidth*/ 210 - 30 /*arbitrary width for spacing etc*/;
        menu.setWidth(w + "px");
        
        getSessionInfoAndMasterData();
    }

    /**
     * @return  the model.
     */
    public WizardModel getModel() {
        return model;
    }

	public abstract void showOrdersPendingAdminPage();

	public abstract void showOrdersDiscardedAdminPage();
	
	public abstract void showOrdersProblemsAdminPage();
	
	public abstract void showOrdersSearchAdminPage();
	
	public abstract void showOrdersCustomerPage();
	
	public abstract void showProfilePage();
	
	public abstract void showOrderAdminDetailPage(SalesOrder so);
	
	public abstract void showOrderCustomerDetailPage(SalesOrder so);
	
	public abstract void showDesignDetailPage(Design design);
	
    /**
     * does what it says on the tin.
     *
     * @param alsoUpdate if true, then the shop view is also updated to display the displayable, designs 
     * 		and preselcted_design from the model.
     */
    public abstract void showShopPage(boolean alsoUpdate);

    /**
     * does what it says on the tin.
     *
     * @param  waitingForServerResponse  if true, wait status is set to true
     */
    public abstract void showShopPage(ShopResultsType results);
    
    /**
     * does what it says on the tin.
     *
     * @param  d
     */
    public abstract void showDesignAdminPage(Design d);
    
	public abstract void showArtistProfilePage();

    public abstract void showArtistAdminPage(Artist a);

    public abstract void showDesignAdminListPage();

    public abstract void showArtistsDesignPage();

    /**
     * call server to go get session info and initial data.
     */
    public void getSessionInfoAndMasterData() {
    	controller.getSessionInfoAndMasterData();
	}

    protected abstract void buildMenu();

    /**
     * updates the master data in the model using the given artist. 
     * typically called after admin edits it. saves having to reload
     * all master data.
     */
	public void updateArtistMasterData(Artist artist) {
		boolean found = false;
		for(Artist a : getModel().getAllArtists()){
			if(a.getUid().equals(artist.getUid())){
				found = true;
				a.setDescription(artist.getDescription());
				a.setEmail(artist.getEmail());
				a.setFirstname(artist.getFirstname());
				a.setLastname(artist.getLastname());
				a.setNotes(artist.getNotes());
				a.setScreenname(artist.getScreenname());
				a.setTitle(artist.getTitle());
				a.setRegistrationDate(artist.getRegistrationDate());
				
				break;
			}
		}
		
		if(!found){
			reload();
		}
	}
	
	/**
	 * notifies listeners that master data is ready, as well as builds the menu
	 */
	public void notifyMasterDataIsReady() {
		for(MasterDataListener mdl : masterDataListeners){
			mdl.notifyMasterDataIsReady();
		}

		//now also build the menu
		buildMenu();
	}
	
	public void addMasterDataListener(MasterDataListener listener){
        masterDataListeners.add(listener);
	}

	/**
	 * reloads the page
	 */
	public void reload(){
		String url = Window.Location.getProtocol() + "//";
		url += Window.Location.getHost();
		url += Window.Location.getPath() + "?screen=" + getCurrentPageNumber();
		Window.Location.assign(url);
	}

	/**
	 * causes a search to happen, which activates the shop page to show the results.
	 */
	public abstract void doSearch(String search);

	public void onModuleLoad(){
		GWT.setUncaughtExceptionHandler(new GWT.UncaughtExceptionHandler(){
			public void onUncaughtException(Throwable e) {
				GWT.log("an error occurred", e);
			}
		});

		screenIndex = 0;
		String screen = Window.Location.getParameter("screen");
		if(screen != null){
			try{
				screenIndex = Integer.parseInt(screen);
				
				if(screenIndex >= getPageCount() || screenIndex < 0){
					screenIndex = 0; //stops hackers fucking about.
				}
			}catch(NumberFormatException nfe){
				//oh well, we tried, lets just leave it
				GWT.log("cannot parse screen index from integer " + screen, nfe);
			}
		}

		String uid = Window.Location.getParameter("uid");
		if(uid != null){
			try{
				getModel().setUid(Integer.parseInt(uid));
			}catch(NumberFormatException nfe){
				//oh well, we tried, lets just leave it
				GWT.log("cannot parse uid parameter from integer " + uid, nfe);
			}
		}

		VerticalPanel vp = UIFactory.createVerticalPanel();
		vp.setHorizontalAlignment(HasHorizontalAlignment.ALIGN_LEFT);
		HorizontalPanel hp = UIFactory.createHorizontalPanel();
		final Image i = UIFactory.createImage("/images/logo.jpg");
		i.addLoadHandler(new LoadHandler() {
			public void onLoad(LoadEvent event) {
				int w = Window.getClientWidth() - i.getWidth() - 30 /*arbitrary width for spacing etc*/;
				menu.setWidth(w + "px");
			}
		});
		hp.add(i);
		hp.add(menu);

		vp.add(hp);
		
		vp.add(deck);
		
		RootPanel.get("gwtRoot").add(vp);
		RootPanel.get("startupWait").setVisible(false);
		
		activateCurrentPage(false);
		
		//when loading pages from the off, some need special attention...
		if(getCurrentPage() instanceof DesignAdmin){
			((DesignAdmin)getCurrentPage()).showDesign(null);
		}else if(getCurrentPage() instanceof DesignAdminList){
			//nothing to do for this one
		}else if(getCurrentPage() instanceof ArtistAdmin){
			((ArtistAdmin)getCurrentPage()).showArist(null);
		}
		
		String showDesign = Window.Location.getParameter("designID");
		if(showDesign != null && showDesign.trim().length() > 0){
			try{
				int id = Integer.parseInt(showDesign.trim());
				Design d = new Design();
				d.setUid(id);
				showDesignDetailPage(d);
			}catch(NumberFormatException e){
				//invalid parameter. do nowt...
			}
		}
	}
}
