/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.widgets.ui;

import com.google.gwt.user.client.Window;

import uk.co.maxant.gwtdemo.client.artistprofile.ui.ArtistProfile;
import uk.co.maxant.gwtdemo.client.checkout.ui.Checkout;
import uk.co.maxant.gwtdemo.client.design_artist_list.ui.DesignArtistList;
import uk.co.maxant.gwtdemo.client.home.ui.Home;
import uk.co.maxant.gwtdemo.client.order_admin_list.ui.OrdersCustomerList;
import uk.co.maxant.gwtdemo.client.order_complete.ui.OrderComplete;
import uk.co.maxant.gwtdemo.client.order_detail.ui.OrderCustomerDetail;
import uk.co.maxant.gwtdemo.client.profile.ui.Profile;
import uk.co.maxant.gwtdemo.client.register.ui.Register;
import uk.co.maxant.gwtdemo.client.shop.core.ShopController.ShopResultsType;
import uk.co.maxant.gwtdemo.client.shop.ui.DesignDetail;
import uk.co.maxant.gwtdemo.client.shop.ui.Shop;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.util.MenuBuilder;


/**
 * app specific impl.
 */
public class ShopWizard extends AppWizard {

    protected Home home;

    protected Shop shop;

    protected Checkout checkout;

    protected OrderComplete orderComplete;

    protected OrdersCustomerList ordersCustomerList;
    
    protected OrderCustomerDetail orderCustomerDetail;

    protected DesignDetail designDetail;

    protected Register register;

    protected Profile profile;

    protected ArtistProfile artistProfile;

    protected DesignArtistList designArtistList;
    
    /**
     * Creates a new Wizard object.
     */
    public ShopWizard() {

        home = new Home(this); // page 0
        addPage(home);

        shop = new Shop(this); // page 1
        addPage(shop);
        model.addListener(shop);

        checkout = new Checkout(this); // page 2
        addPage(checkout);
        model.addListener(checkout);

        orderComplete = new OrderComplete(this); // page 3
        addPage(orderComplete);

        ordersCustomerList = new OrdersCustomerList(this); // page 4
        addPage(ordersCustomerList);
        
        orderCustomerDetail = new OrderCustomerDetail(this); // page 5
        addPage(orderCustomerDetail);

        designDetail = new DesignDetail(this); // page 6
        addPage(designDetail);

        register = new Register(this); // page 7
        addPage(register);

        profile = new Profile(this); // page 8
        addPage(profile);
        
        artistProfile = new ArtistProfile(this); // page 9
        addPage(artistProfile);
        
        designArtistList = new DesignArtistList(this); // page 10
        addPage(designArtistList);
    }

	public void showOrdersPendingAdminPage() {
		Window.Location.assign("/secure/indexAdmin.jsp?screen=3");
	}

	public void showOrdersDiscardedAdminPage() {
		Window.Location.assign("/secure/indexAdmin.jsp?screen=7");
	}
	
	public void showOrdersProblemsAdminPage() {
		Window.Location.assign("/secure/indexAdmin.jsp?screen=6");
	}
	
	public void showOrdersSearchAdminPage() {
		Window.Location.assign("/secure/indexAdmin.jsp?screen=4");
	}
	
	public void showOrdersCustomerPage() {
		showPage(ordersCustomerList);
	}
	
	public void showProfilePage() {
		showPage(profile);
	}
	
	public void showOrderAdminDetailPage(SalesOrder so) {
		Window.Location.assign("/secure/indexAdmin.jsp?screen=5");
	}
	
	public void showOrderCustomerDetailPage(SalesOrder so) {
		showPage(orderCustomerDetail);
		orderCustomerDetail.showOrder(so);
	}
	
	public void showDesignDetailPage(Design design) {
		showPage(designDetail);
		designDetail.showDesign(design);
	}
	
    /**
     * does what it says on the tin.
     *
     * @param alsoUpdate if true, then the shop view is also updated to display the displayable, designs 
     * 		and preselcted_design from the model.
     */
    public void showShopPage(boolean alsoUpdate) {
        showPage(shop);
        
        if(alsoUpdate){
        	shop.updateShopView(getModel().getDisplayable(), getModel().getDesigns(), getModel().getDesignToPreSelect());
        }
    }

    /**
     * does what it says on the tin.
     *
     * @param  waitingForServerResponse  if true, wait status is set to true
     */
    public void showShopPage(ShopResultsType results) {
		shop.viewResults(results);
    	showPage(shop);
    }
    
    /**
     * does what it says on the tin.
     *
     * @param  d
     */
    public void showDesignAdminPage(Design d) {
		Window.Location.assign("/secure/indexAdmin.jsp?screen=0");
    }
    
	public void showArtistProfilePage() {
		showPage(artistProfile);
	}

    public void showArtistAdminPage(Artist a) {
		Window.Location.assign("/secure/indexAdmin.jsp?screen=1");
    }

    public void showDesignAdminListPage() {
		Window.Location.assign("/secure/indexAdmin.jsp?screen=2");
    }
    
	public void showArtistsDesignPage() {
		showPage(designArtistList);
	}
	
    @Override
    protected void buildMenu(){
        MenuBuilder.buildShopMenu(menu, model.getSessionInfo(), this);
    }

	/**
	 * causes a search to happen, which activates the shop page to show the results.
	 */
	public void doSearch(String search){
		showShopPage(false);
		shop.performSearch(search);
	}

}
