/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.client.widgets.ui;

import uk.co.maxant.gwtdemo.client.generic.widgets.FlexTable;
import uk.co.maxant.gwtdemo.client.generic.widgets.UIFactory;

import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasAllMouseHandlers;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.dom.client.MouseOutEvent;
import com.google.gwt.event.dom.client.MouseOutHandler;
import com.google.gwt.event.dom.client.MouseOverEvent;
import com.google.gwt.event.dom.client.MouseOverHandler;
import com.google.gwt.user.client.ui.ComplexPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * allows for adding click and rollover handlers to a table.
 * when the table is clicked, it calls {@link #handleClickOnTable(ClickEvent, Object)}
 */
public abstract class WizardPageWithListClickHandling<T> extends WizardPage {

	public WizardPageWithListClickHandling(AppWizard wizard) {
		super(wizard);
	}

	protected final void addHandlers(Widget w, final T t, final FlexTable ft, final int row, final String style) {
		addHandlers(w, t, ft, row, style, null);
	}

	/**
	 * iterates over kids of widget if its a {@link ComplexPanel}. if its a {@link HasClickHandlers}
	 * then a click handler is added which calls the {@link #handleClickOnTable(ClickEvent, Object)}
	 * method. if its a {@link HasAllMouseHandlers}, then rollover handlers are added to highlight
	 * the row and return it to its original style.
	 * 
	 * @param w the widget to add handlers to. 
	 * @param t the object passed into the {@link #handleClickOnTable(ClickEvent, Object)} method.
	 * @param ft a flex table who's row is highlighted.
	 * @param row the row to highlight.
	 * @param style the original style of the row.
	 *  @param extra an additional, optional object which the {@link #handleClickOnTable(ClickEvent, Object, Object)} method might use.
	 */
	protected final void addHandlers(Widget w, final T t, final FlexTable ft, final int row, final String style, final Object extra) {
		if(w instanceof ComplexPanel){
			ComplexPanel cp = (ComplexPanel)w;
			for(int i = 0; i < cp.getWidgetCount(); i++){
				addHandlers(cp.getWidget(i), t, ft, row, style);
			}
		}else{
			if(w instanceof HasClickHandlers){
				HasClickHandlers hch = (HasClickHandlers)w;
				hch.addClickHandler(new ClickHandler() {
					public void onClick(ClickEvent event) {
						handleClickOnTable(event, t, extra);
					}
				});
			}
			if(w instanceof HasAllMouseHandlers){
				HasAllMouseHandlers hamh = (HasAllMouseHandlers)w;
				hamh.addMouseOverHandler(new MouseOverHandler() {
					public void onMouseOver(MouseOverEvent event) {
						ft.getRowFormatter().setStylePrimaryName(row, UIFactory.LIST_ROW_SELECTED);
					}
				});
				hamh.addMouseOutHandler(new MouseOutHandler() {
					public void onMouseOut(MouseOutEvent event) {
						ft.getRowFormatter().setStylePrimaryName(row, style);
					}
				});
			}
		}
	}


	public abstract void handleClickOnTable(ClickEvent event, T t, Object extra);
}
