/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server;

import java.io.IOException;

import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import org.springframework.beans.factory.annotation.Autowired;

import com.google.gwt.user.server.rpc.RemoteServiceServlet;
import com.octo.captcha.service.CaptchaServiceException;

import uk.co.maxant.gwtdemo.client.ApplicationException;
import uk.co.maxant.gwtdemo.client.ApplicationLayerService;
import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.BasketItem;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.DesignComment;
import uk.co.maxant.gwtdemo.client.to.OrderState;
import uk.co.maxant.gwtdemo.client.to.PaypalConfig;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.to.SearchResults;
import uk.co.maxant.gwtdemo.client.to.SessionInfo;
import uk.co.maxant.gwtdemo.client.to.User;
import uk.co.maxant.gwtdemo.server.captcha.CaptchaServiceSingleton;
import uk.co.maxant.gwtdemo.server.util.DependencyResolver;
import uk.co.maxant.gwtdemo.server.util.LogHelper;


/**
 * GWT Service which delegates all work to the orchestration service which lives inside spring, guaranteeing
 * that transactions are started and committed.
 */
public class ApplicationLayerServiceImpl extends RemoteServiceServlet implements ApplicationLayerService {

    private static final long serialVersionUID = 1L;

    private static final Logger log = LogHelper.getLog();

    @Autowired
    private OrchestrationService orchestrationService;

    /**
     * instantiates an instance of the process layer (service).
     */
    public ApplicationLayerServiceImpl() {

        try {
            DependencyResolver.resolve(this);
        } catch (Exception e) {
            String msg = "failed to instantiate app layer";
            log.error(msg, e);
            throw new RuntimeException(msg, e);
        }
    }

    /**
	 * {@inheritDoc}
     */
    public List<Design> getFeaturedDesigns() throws ApplicationException {
        return orchestrationService.getFeaturedDesigns(getThreadLocalRequest());
    }
    
    /**
	 * {@inheritDoc}
     */
    public List<Design> getNewestDesigns() throws ApplicationException {
    	return orchestrationService.getNewestDesigns(getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public List<BasketItem> addToBasket(BasketItem basketItem) throws ApplicationException {
        return orchestrationService.addToBasket(basketItem, getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public List<BasketItem> getBasket() throws ApplicationException {
        return orchestrationService.getBasket(getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public List<BasketItem> emptyBasket() throws ApplicationException {
        return orchestrationService.emptyBasket(getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public Integer createOrder(String email, String name) throws ApplicationException {
        return orchestrationService.createOrder(getThreadLocalRequest(), email, name);
    }

    /**
	 * {@inheritDoc}
     */
    public SessionInfo getSessionInfo() throws ApplicationException {
        return orchestrationService.getSessionInfo(getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public PaypalConfig getPaypalConfig(Integer soUid) throws ApplicationException {
        return orchestrationService.getPaypalConfig(soUid, getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public List<Design> getDesigns(boolean getAllContent) throws ApplicationException {
        return orchestrationService.getDesigns(getAllContent, getThreadLocalRequest());
    }
    
    /**
	 * {@inheritDoc}
     */
    public List<Design> getDesignsForArtist(Integer artistUid) throws ApplicationException {
    	return orchestrationService.getDesignsForArtist(artistUid, getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public List<Design> getDesignsAdmin(String filter) throws ApplicationException {
        return orchestrationService.getDesigns(filter);
    }
    
    /**
	 * {@inheritDoc}
     */
    public List<Design> getDesignsAdminLight() throws ApplicationException {
        return orchestrationService.getAllDesigns(getThreadLocalRequest());
    }
    
    /**
	 * {@inheritDoc}
     */
    public Design getDesign(Integer designID) throws ApplicationException {
    	return orchestrationService.getDesign(designID, getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public List<Artist> getArtistsAdmin(String filter) throws ApplicationException {
        return orchestrationService.getArtists(filter);
    }
    
    /**
	 * {@inheritDoc}
     */
    public SearchResults search(String searchExpression) throws ApplicationException {
        return orchestrationService.search(searchExpression, getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public Integer saveOrUpdateDesign(Design design)
        throws ApplicationException {
        return orchestrationService.saveOrUpdateDesign(design, getThreadLocalRequest());
    }

    /**
	 * {@inheritDoc}
     */
    public Integer saveOrUpdateArtist(Artist artist) throws ApplicationException {
        return orchestrationService.updateArtist(artist);
    }
    
    /**
	 * {@inheritDoc}
     */
	public List<Artist> getArtistViewCloud() throws ApplicationException {
		return orchestrationService.getArtistViewCloud(getThreadLocalRequest());
	}

	/**
	 * {@inheritDoc}
	 */
	public List<SalesOrder> getPendingSalesOrders() throws ApplicationException {
		return orchestrationService.getPendingSalesOrders();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public List<SalesOrder> getDiscardedSalesOrders() throws ApplicationException {
		return orchestrationService.getDiscardedSalesOrders();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public List<SalesOrder> getProblemSalesOrders() throws ApplicationException {
		return orchestrationService.getProblemSalesOrders();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public List<SalesOrder> searchSalesOrders(String search) throws ApplicationException {
		return orchestrationService.searchSalesOrders(search);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public List<SalesOrder> getCustomerSalesOrders() throws ApplicationException {
		return orchestrationService.getCustomerSalesOrders(getThreadLocalRequest());
	}

	/**
	 * {@inheritDoc}
	 */
	public SalesOrder getSalesOrder(String uid) throws ApplicationException {
		return orchestrationService.getSalesOrder(uid, getThreadLocalRequest());
	}

	/**
	 * {@inheritDoc}
	 */
	public Integer registerCustomerOrArtist(User user, String password,
			String captcha, boolean alsoArtist, String artistsDescription)
			throws ApplicationException {

		boolean captchaPassed = true; //default, incase of exception
		try {
			// validate captcha

			String captchaId = getThreadLocalRequest().getSession().getId();
			
			//since the generated images ONLY contain big letters and 
			//numbers @see CaptchaServiceSingleton.MyImageCaptchaEngine
			captcha = captcha.toUpperCase();
			
			captchaPassed = CaptchaServiceSingleton.getInstance().
									validateResponseForID(captchaId, captcha);
		} catch (CaptchaServiceException e) {
			//ok, our problem, not customers...
			// should not happen, may be thrown if the id is not valid
			log.warn("failed to validate captcha", e);
		}

		if (captchaPassed) {
			return orchestrationService.registerCustomerOrArtist(user,
					password, alsoArtist, artistsDescription, getThreadLocalRequest());
		} else {
			throw new ApplicationException("APPSE001", "captcha failed",
					"Your text didn't match that of the image. Please try again.");
		}
	}

	/**
	 * {@inheritDoc}
	 */
	public void rateDesign(Design design, Integer ratingGiven) throws ApplicationException {
		orchestrationService.rateDesign(design.getUid(), getThreadLocalRequest(), ratingGiven);
	}

	/**
	 * {@inheritDoc}
	 */
	public void updateOrderStatus(OrderState requiredState, int soUid, String notes) throws ApplicationException {
		orchestrationService.updateOrderStatus(getThreadLocalRequest(), requiredState, soUid, notes);
	}

	/**
	 * {@inheritDoc}
	 */
	public void updatePaymentStatus(OrderState requiredState, int soUid, String extraNotes, String txnId) throws ApplicationException {
		orchestrationService.updatePaymentStatus(getThreadLocalRequest(), requiredState, soUid, extraNotes, txnId);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void addShopNotes(int soUid, String notes) throws ApplicationException {
		orchestrationService.addShopNotes(getThreadLocalRequest(), soUid, notes);
	}

	/**
	 * {@inheritDoc}
	 */
	public void markOrderAsNotNeedingAttention(int soUid, String notes) throws ApplicationException {
		orchestrationService.markOrderAsNotNeedingAttention(getThreadLocalRequest(), soUid, notes);
	}

	/**
	 * {@inheritDoc}
	 */
	public void markOrderAsNeedingAttention(Integer orderNum, String msg) throws ApplicationException {
		orchestrationService.markOrderAsNeedingAttention(getThreadLocalRequest(), orderNum, msg);
	}

	/**
	 * {@inheritDoc}
	 */
	public List<DesignComment> getAllComments(Integer designUid) throws ApplicationException {
		return orchestrationService.getAllComments(getThreadLocalRequest(), designUid);
	}

	/**
	 * {@inheritDoc}
	 */
	public List<DesignComment> getComments(Integer designUid) throws ApplicationException {
		return orchestrationService.getComments(getThreadLocalRequest(), designUid);
	}

	/**
	 * {@inheritDoc}
	 */
	public Integer addDesignComment(DesignComment dc) throws ApplicationException {
		return orchestrationService.addDesignComment(getThreadLocalRequest(), dc);
	}
	
	/**
	 * {@inheritDoc}
	 */
	public void approveDesignComment(DesignComment dc) throws ApplicationException {
		orchestrationService.approveDesignComment(getThreadLocalRequest(), dc);
	}
	
	/**
	 * {@inheritDoc}
	 */
    public void rejectDesignComment(DesignComment dc) throws ApplicationException {
		orchestrationService.rejectDesignComment(getThreadLocalRequest(), dc);
    }

    /**
     * {@inheritDoc}
     */
    public List<Design> getDesignsPendingModeration() throws ApplicationException {
    	return orchestrationService.getDesignsPendingModeration(getThreadLocalRequest());
    }

    /**
     * {@inheritDoc}
     */
    public Artist getLoggedInArtist() throws ApplicationException{
    	return orchestrationService.getLoggedInArtist(getThreadLocalRequest());
    }
    
    /**
     * {@inheritDoc}
     */
    public Integer updateCustomer(User user, String newPassword) throws ApplicationException{
    	return orchestrationService.updateCustomer(user, newPassword, getThreadLocalRequest());
    }
    
    /**
     * {@inheritDoc}
     */
    public Integer updateArtistProfile(Artist artist) throws ApplicationException {
    	return orchestrationService.updateArtistProfile(artist, getThreadLocalRequest());
    }

	
    /** *******************************************************************************************************
     * override GWT in case necessary...
     * 
     * check for security issues, and handle them appropriately, 
     * by returning a 401 error. the client can then
     * catch this and redirect to the login page.
     *
     * @param  e  the exception to handle
     */
    @Override
    protected void doUnexpectedFailure(Throwable e) {

        if ((e.getCause() != null) && 
        		(e.getCause() instanceof SecurityException)) {

            try {
                getThreadLocalResponse().sendError(
                		HttpServletResponse.SC_UNAUTHORIZED);
            } catch (IOException ioe) {

                // shouldnt ever happen... log and handle 
            	//original exception normally.
                log.warn("failed to send 401 response " +
                		"to a SecurityException", ioe);
                super.doUnexpectedFailure(e);
            }
        } else {
            super.doUnexpectedFailure(e);
        }
    }

}
