/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.Logger;
import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import uk.co.maxant.gwtdemo.client.ApplicationException;
import uk.co.maxant.gwtdemo.server.po.Config;
import uk.co.maxant.gwtdemo.server.util.LogHelper;

/**
 * Provides a cache of all the config parameters from the database.
 */
@Service("configService")
@Transactional(propagation=Propagation.REQUIRED, isolation=Isolation.DEFAULT, rollbackFor=ApplicationException.class)
public class ConfigService {

	//for the following constants, see the descriptions in initialLoadTest.sql
	public static final String paypal_payment_url = "paypal_payment_url";
	public static final String paypal_merchant_email = "paypal_merchant_email";
	public static final String paypal_currency = "paypal_currency";
	public static final String paypalReturnURL = "paypalReturnURL";
	public static final String paypalCancelReturnURL = "paypalCancelReturnURL";
	public static final String paypalIPNURL = "paypalIPNURL";
	public static final String emailsFrom = "emailsFrom";
	public static final String emailOps = "emailOps";
	public static final String emailAdmin = "emailAdmin";
	public static final String emailModerators = "emailModerators";
	public static final String emailOpsOnOrderPlaced = "emailOpsOnOrderPlaced";
	public static final String emailOpsOnSuccessfulPayment = "emailOpsOnSuccessfulPayment";
	public static final String emailOpsOnFailedPayment = "emailOpsOnFailedPayment";
	public static final String emailOpsOnMarkOrderAsNeedingAttention = "emailOpsOnMarkOrderAsNeedingAttention";
	public static final String emailOpsOnOrderPlacement = "emailOpsOnOrderPlacement";
	public static final String registrationForceConfirm = "registrationForceConfirm";
	public static final String emailFooter = "emailFooter";
	public static final String designCommentModerationRequired = "designCommentModerationRequired";
	public static final String designCommentModerationSendEmailToModerator = "designCommentModerationSendEmailToModerator";
	public static final String httpUrl = "httpUrl";
	public static final String httpsUrl = "httpsUrl";
	public static final String siteName = "siteName";

	private static final Logger log = LogHelper.getLog();

	private HibernateTemplate hibernateTemplate;

	@Autowired
	public void setSessionFactory(SessionFactory sessionFactory) {
		this.hibernateTemplate = new HibernateTemplate(sessionFactory);
	}

	protected Map<String, String> params = new HashMap<String, String>();

	protected boolean isLoaded = false;

	public String value(String key) throws ApplicationException {
		if(!isLoaded){
			reload();
		}
		String retVal = params.get(key);
		if(retVal == null){
			throw new RuntimeException("Missing configuration for parameter name " + key);
		}
		return retVal;
	}
	
	public int intValue(String key) throws ApplicationException {
		String temp = value(key);
		if(temp == null){
			throw new RuntimeException("Missing configuration for parameter name " + key);
		}else{
			return new Integer(temp);
		}
	}
	
	public boolean boolValue(String key) throws ApplicationException {
		String temp = value(key);
		if(temp == null){
			throw new RuntimeException("Missing configuration for parameter name " + key);
		}else{
			return new Boolean(temp);
		}
	}
	
	@SuppressWarnings("unchecked")
	public void reload() throws ApplicationException {
		synchronized(params){
			log.info("Reloading the configuration...");
			isLoaded = Boolean.FALSE;

			params = new HashMap<String, String>();

			try{
				//load the config
	            List<Config> configs = hibernateTemplate.find("from Config c");
	            for(Config c : configs){
	            	params.put(c.getName(), c.getVal());
	            }
	            isLoaded = Boolean.TRUE;
			}catch(Exception e){
				log.error("failed to load config from database", e);
				throw new ApplicationException("CFGS001", "Failed to load configuration from database", "An error occurred. Please try again.");
			}
			log.info("Configuration successfully reloaded.");
		}
	}

	/**
	 * @return The map of parameters, mapping keys to values.
	 */
	public Map<String,String> getNames(){
		return params;
	}
	
}
