/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server;

import java.io.File;

import javax.mail.MessagingException;
import javax.mail.internet.MimeMessage;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import uk.co.maxant.gwtdemo.client.ApplicationException;
import uk.co.maxant.gwtdemo.server.util.LogHelper;

/**
 * service for sending email.
 */
@Service("mailService")
@Transactional(propagation=Propagation.REQUIRED, isolation=Isolation.DEFAULT, rollbackFor=ApplicationException.class)
public class MailService {

	private static Logger log = LogHelper.getLog();

	@Autowired
	protected JavaMailSender mailSender;

	@Autowired
	protected ConfigService configService;

	/**
	 * sends an email.
	 * @param to who to send the mail to
	 * @param subject subject line
	 * @param message email body
	 * @throws ApplicationException in case the mail cannot be sent
	 */
	public void sendMail(String to, String subject, String message) throws ApplicationException {
		sendMail(to, subject, message, null, null);
	}

	public void sendMail(String to, String subject, String message, File attachment, String attachmentFilename) throws ApplicationException {
		try{
			String header = 
					"<html>" +
					"	<head>" +
					"		<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\">" +
					"	</head>" +
					"	<body class='standardText'>";
	
			MimeMessage mm = mailSender.createMimeMessage();
			MimeMessageHelper helper = new MimeMessageHelper(mm, true);
			helper.setTo(to);
			String from = configService.value(ConfigService.emailsFrom);
			helper.setFrom(from);
			helper.setSubject(subject);

			if(attachment != null){
				helper.addAttachment((attachmentFilename==null?attachment.getName():attachmentFilename), attachment);
			}
			
			String footer = configService.value(ConfigService.emailFooter);
			footer += 
					"	</body>" +
					"</html>";

			helper.setText(header + message + "<br><br>" + footer, true);
			mailSender.send(mm);
			
		}catch(MessagingException e){
			log.error("failed to send email", e);
			throw new ApplicationException("MASE001", "Failed to send email to " + to + ". See cause.", "An error occurred. Please try again.");
		}
		
	}

}
