/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.captcha;

import java.awt.Color;
import java.awt.Font;

import com.octo.captcha.component.image.backgroundgenerator.BackgroundGenerator;
import com.octo.captcha.component.image.backgroundgenerator.EllipseBackgroundGenerator;
import com.octo.captcha.component.image.backgroundgenerator.FunkyBackgroundGenerator;
import com.octo.captcha.component.image.backgroundgenerator.GradientBackgroundGenerator;
import com.octo.captcha.component.image.backgroundgenerator.UniColorBackgroundGenerator;
import com.octo.captcha.component.image.color.RandomRangeColorGenerator;
import com.octo.captcha.component.image.fontgenerator.FontGenerator;
import com.octo.captcha.component.image.fontgenerator.RandomFontGenerator;
import com.octo.captcha.component.image.textpaster.BaffleRandomTextPaster;
import com.octo.captcha.component.image.textpaster.LineRandomTextPaster;
import com.octo.captcha.component.image.textpaster.NonLinearTextPaster;
import com.octo.captcha.component.image.textpaster.RandomTextPaster;
import com.octo.captcha.component.image.textpaster.TextPaster;
import com.octo.captcha.component.image.wordtoimage.ComposedWordToImage;
import com.octo.captcha.component.image.wordtoimage.WordToImage;
import com.octo.captcha.component.word.wordgenerator.RandomWordGenerator;
import com.octo.captcha.component.word.wordgenerator.WordGenerator;
import com.octo.captcha.engine.image.ListImageCaptchaEngine;
import com.octo.captcha.image.gimpy.GimpyFactory;
import com.octo.captcha.service.captchastore.FastHashMapCaptchaStore;
import com.octo.captcha.service.image.ImageCaptchaService;
import com.octo.captcha.service.image.DefaultManageableImageCaptchaService;

public class CaptchaServiceSingleton {

    private static final ImageCaptchaService instance;
    
    static {
    	
    	instance = new DefaultManageableImageCaptchaService(
    		new FastHashMapCaptchaStore(),
    		new MyImageCaptchaEngine(),
    		180,
    		10000,
    		7500);
    }

    public static ImageCaptchaService getInstance(){
        return instance;
    }




	static class MyImageCaptchaEngine extends ListImageCaptchaEngine {
	    protected void buildInitialFactories() {
	           WordGenerator wgen = new RandomWordGenerator("ABCDEFGHJKMNPQRSTUWXYZ123456789");
	           RandomRangeColorGenerator cgen = new RandomRangeColorGenerator(
	                new int[] {100, 200},
	                new int[] {100, 200},
	                new int[] {100, 200});
	           //TextPaster textPaster = new RandomTextPaster(new Integer(4), new Integer(4), cgen, true);
//	           TextPaster textPaster = new RandomTextPaster(new Integer(4), new Integer(4), new Color[] { new Color(255, 0, 0)});
	           TextPaster textPaster = new NonLinearTextPaster(new Integer(4), new Integer(4), cgen);

	           RandomRangeColorGenerator cgen2 = new RandomRangeColorGenerator(
		                new int[] {0, 100},
		                new int[] {0, 100},
		                new int[] {0, 100});

	           //BackgroundGenerator backgroundGenerator = new FunkyBackgroundGenerator(new Integer(150), new Integer(50));
//	           BackgroundGenerator backgroundGenerator = new GradientBackgroundGenerator(new Integer(150), new Integer(50), new Color(100, 0, 0), new Color(0, 0, 100));
	           BackgroundGenerator backgroundGenerator = new GradientBackgroundGenerator(new Integer(150), new Integer(50), cgen2, cgen2);
//	           BackgroundGenerator backgroundGenerator = new UniColorBackgroundGenerator(new Integer(150), new Integer(50));
	
	           Font[] fontsList = new Font[] {
	               new Font("Arial", 0, 10),
	               new Font("Tahoma", 0, 10),
	               new Font("Verdana", 0, 10),
	            };
	
	            FontGenerator fontGenerator = new RandomFontGenerator(new Integer(20), new Integer(35), fontsList);
	
	            WordToImage wordToImage = new ComposedWordToImage(fontGenerator, backgroundGenerator, textPaster);
	            this.addFactory(new GimpyFactory(wgen, wordToImage));
	    }
	}

}
