/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.util;

import java.lang.annotation.Annotation;
import java.lang.reflect.Field;

import org.springframework.beans.factory.annotation.Autowired;

public class DependencyResolver {

	private DependencyResolver(){}
	
	/**
	 * searches all fields and looks in spring to see if a bean with the 
	 * same name is registered. if so, it sets them, so long as the 
	 * attribute on that field is @Autowired
	 * 
	 * @param objToSetInstancesOn object on which to set the spring beans
	 * @throws IllegalAccessException if setting is not possible
	 * @throws IllegalArgumentException if setting is not possible
	 */
	public static void resolve(Object objToSetInstancesOn) 
					throws IllegalArgumentException, IllegalAccessException{
		
		Field[] fields = objToSetInstancesOn.getClass().getDeclaredFields();

		for(Field f : fields){
			Annotation[] annotations = f.getAnnotations();
			boolean relevant = false;
			for(Annotation a : annotations){
				if(Autowired.class.isAssignableFrom(a.getClass())){
					relevant = true;
					break;
				}
			}
			if(relevant){
				Object o = ServiceLocator.getInstance().getBean(f.getName());
				if(o != null){
					f.setAccessible(true);
					f.set(objToSetInstancesOn, o);
				}
			}
		}
	}
}
