/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.util;

import java.lang.reflect.InvocationTargetException;
import java.math.BigDecimal;
import java.util.ArrayList;

import uk.co.maxant.gwtdemo.client.to.Artist;
import uk.co.maxant.gwtdemo.client.to.Design;
import uk.co.maxant.gwtdemo.client.to.DesignComment;
import uk.co.maxant.gwtdemo.client.to.SalesOrder;
import uk.co.maxant.gwtdemo.client.to.SalesOrderItem;
import uk.co.maxant.gwtdemo.client.to.Staff;
import uk.co.maxant.gwtdemo.client.to.User;
import uk.co.maxant.gwtdemo.server.po.DesignTag;
import uk.co.maxant.gwtdemo.server.po.Party;

/**
 * contains specific mappings from PO to TO
 * 
 * MAPPERS -
 * Generated PO objects (from hibernate) are not available on the client, so we 
 * need to map from TO to PO and the other way too sometimes. 
 * 
 */
public class Mapper {

	public Mapper(){
	}

	public DesignComment mapDesignComment(uk.co.maxant.gwtdemo.server.po.DesignComment designComment) {
		
		try {
			DesignComment dc = (DesignComment) GenericMapper.map(DesignComment.class, designComment);
			
			return dc;
		} catch (InstantiationException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		}
	}
	
	public uk.co.maxant.gwtdemo.server.po.DesignComment mapDesignComment(DesignComment dc) {
		
		try {
			uk.co.maxant.gwtdemo.server.po.DesignComment designComment = (uk.co.maxant.gwtdemo.server.po.DesignComment) GenericMapper.map(uk.co.maxant.gwtdemo.server.po.DesignComment.class, dc);
			
			return designComment;
		} catch (InstantiationException e) {
			throw new RuntimeException(e);
		} catch (IllegalAccessException e) {
			throw new RuntimeException(e);
		} catch (InvocationTargetException e) {
			throw new RuntimeException(e);
		}
	}
	
	public Design mapDesign(uk.co.maxant.gwtdemo.server.po.Design design) {

        try {
            Design d = (Design) GenericMapper.map(Design.class, design);

            d.setInCatalogue(design.isInCatalogue()); // map manually since genericMapper cant handle "is"
                                                      // methods. FIXME fix the generic mapper to handle "is"
                                                      // getters for booleans
            d.setApproved(design.isApproved()); 	// map manually since genericMapper cant handle "is"
										            // methods. FIXME fix the generic mapper to handle "is"
										            // getters for booleans

            if(design.getNumberOfRatings() > 0){
            	d.setRating((int)(design.getSumOfRatings() / design.getNumberOfRatings()));
            }else{
            	d.setRating(2); //average to be fair, since no one has rated it b4.
            }

            if(design.getBasePrice() != null){
            	d.setBasePrice(design.getBasePrice().doubleValue());
            }
            
            // add the tags
            d.setTags(new ArrayList<String>());

            for (DesignTag dt : design.getDesignTags()) {
                d.getTags().add(dt.getTag());
            }

            return d;
        } catch (InstantiationException e) {
            throw new RuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param   design  DOCUMENT ME!
     *
     * @return  DOCUMENT ME!
     */
	public uk.co.maxant.gwtdemo.server.po.Design mapDesign(Design design) {

        try {
            uk.co.maxant.gwtdemo.server.po.Design d = (uk.co.maxant.gwtdemo.server.po.Design) GenericMapper.map(
                    uk.co.maxant.gwtdemo.server.po.Design.class, design);
            d.setInCatalogue(design.isInCatalogue()); // map manually since genericMapper cant handle "is"
										            // methods. FIXME fix the generic mapper to handle "is"
										            // getters for booleans
            d.setApproved(design.isApproved()); // map manually since genericMapper cant handle "is"
										            // methods. FIXME fix the generic mapper to handle "is"
										            // getters for booleans
            
            d.setBasePrice(new BigDecimal(design.getBasePrice()));

            return d;
        } catch (InstantiationException e) {
            throw new RuntimeException(e);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * DOCUMENT ME!
     *
     * @param   artist  DOCUMENT ME!
     *
     * @return  DOCUMENT ME!
     */
	public Artist mapArtist(Party artist) {

        if (artist == null) {
            return null;
        }

        Artist a = new Artist();
        a.setDescription(artist.getArtistDescription());
        a.setDob(artist.getDob());
        a.setEmail(artist.getLogin());
        a.setFirstname(artist.getFirstname());
        a.setJoinedMailingList(artist.isJoinedMailingList());
        a.setJoinedMobileList(artist.isJoinedMobileList());
        a.setLastname(artist.getLastname());
        a.setMobile(artist.getMobile());
        a.setNotes(artist.getArtistNotes());
        a.setPhone(artist.getPhone());
        a.setPreferredLocale(artist.getPreferredLocale());
        a.setRegistrationDate(artist.getRegistrationDate());
        a.setScreenname(artist.getScreenname());
        a.setTitle(artist.getTitle());
        a.setUid(artist.getUid());
        a.setWhereHeardAbout(artist.getWhereHeardAbout());
        a.setArtist(artist.isIsArtist());
        a.setStaff(artist.isIsStaff());
        
        return a;
    }

    /**
     * DOCUMENT ME!
     *
     * @param   artist  DOCUMENT ME!
     *
     * @return  DOCUMENT ME!
     */
	public Party mapArtist(Artist artist) {

        if (artist == null) {
            return null;
        }

        Party u = new Party();
        u.setArtistDescription(artist.getDescription());
        u.setArtistNotes(artist.getNotes());
        u.setDob(artist.getDob());
        u.setFirstname(artist.getFirstname());
        u.setIsArtist(true); //since we are mapping an artist!
        u.setJoinedMailingList(artist.isJoinedMailingList());
        u.setJoinedMobileList(artist.isJoinedMobileList());
        u.setLastname(artist.getLastname());
        u.setLogin(artist.getEmail());
        u.setMobile(artist.getMobile());
        u.setPhone(artist.getPhone());
        u.setPreferredLocale(artist.getPreferredLocale());
        u.setRegistrationDate(artist.getRegistrationDate());
        u.setScreenname(artist.getScreenname());
        u.setTitle(artist.getTitle());
        u.setUid(artist.getUid());
        u.setWhereHeardAbout(artist.getWhereHeardAbout());
        u.setIsArtist(artist.isArtist());
        u.setIsStaff(artist.isStaff());

        return u;
    }

	public uk.co.maxant.gwtdemo.server.po.SalesOrder mapSalesOrder(SalesOrder salesOrder) {
    	
    	try {
    		uk.co.maxant.gwtdemo.server.po.SalesOrder so = (uk.co.maxant.gwtdemo.server.po.SalesOrder) GenericMapper.map(
    				uk.co.maxant.gwtdemo.server.po.SalesOrder.class, salesOrder);
    		
    		return so;
    	} catch (InstantiationException e) {
    		throw new RuntimeException(e);
    	} catch (IllegalAccessException e) {
    		throw new RuntimeException(e);
    	} catch (InvocationTargetException e) {
    		throw new RuntimeException(e);
    	}
    }
    
	public SalesOrder mapSalesOrder(uk.co.maxant.gwtdemo.server.po.SalesOrder so) {
    	
    	try {
    		SalesOrder salesOrder = (SalesOrder) GenericMapper.map(SalesOrder.class, so);

    		salesOrder.setIsGift(so.isIsGift()); // map manually since genericMapper cant handle "is"
									            // methods. FIXME fix the generic mapper to handle "is"
									            // getters for booleans

    		salesOrder.setNeedsAttention(so.isNeedsAttention()); // map manually since genericMapper cant handle "is"
													            // methods. FIXME fix the generic mapper to handle "is"
													            // getters for booleans

    		if(so.getDeliveryTotal() != null){
    			salesOrder.setDeliveryTotal(so.getDeliveryTotal().doubleValue());
    		}
    		if(so.getTaxCost() != null){
    			salesOrder.setTaxCost(so.getTaxCost().doubleValue());
    		}
    		
    		return salesOrder;
    	} catch (InstantiationException e) {
    		throw new RuntimeException(e);
    	} catch (IllegalAccessException e) {
    		throw new RuntimeException(e);
    	} catch (InvocationTargetException e) {
    		throw new RuntimeException(e);
    	}
    }

	public SalesOrder mapSalesOrderFully(uk.co.maxant.gwtdemo.server.po.SalesOrder salesOrder) {
		SalesOrder so = mapSalesOrder(salesOrder);
		so.setCustomer(mapUser(salesOrder.getPartyByCustomerUid()));
		so.setPickedBy(mapStaff(salesOrder.getPartyByPickedBy()));
		so.setSalesOrderItems(new ArrayList<SalesOrderItem>());
		for(uk.co.maxant.gwtdemo.server.po.SalesOrderItem salesOrderItem : salesOrder.getSalesOrderItems()){
			so.getSalesOrderItems().add(mapSalesOrderItemFully(salesOrderItem));
		}
		return so;
	}

	public User mapUser(Party users) {
		User u = new User();
		u.setDob(users.getDob());
		u.setEmail(users.getLogin());
		u.setFirstname(users.getFirstname());
		u.setJoinedMailingList(users.isJoinedMailingList());
		u.setJoinedMobileList(users.isJoinedMobileList());
		u.setLastname(users.getLastname());
		u.setMobile(users.getMobile());
		u.setNotes(users.getCustomerNotes());
		u.setPhone(users.getPhone());
		u.setPreferredLocale(users.getPreferredLocale());
		u.setRegistrationDate(users.getRegistrationDate());
		u.setTitle(users.getTitle());
		u.setScreenname(users.getScreenname());
		u.setUid(users.getUid());
		u.setWhereHeardAbout(users.getWhereHeardAbout());
		return u;
	}
	
	public Party mapUser(User user, boolean isArtist, String artistDescription, 
			String artistNotes, boolean isStaff, String staffNotes, String staffNumber, 
			String customerNotes) {
		
		Party u = new Party();
		u.setDob(user.getDob());
		u.setLogin(user.getEmail());
		u.setFirstname(user.getFirstname());
		u.setJoinedMailingList(user.isJoinedMailingList());
		u.setJoinedMobileList(user.isJoinedMobileList());
		u.setLastname(user.getLastname());
		u.setMobile(user.getMobile());
		u.setPhone(user.getPhone());
		u.setPreferredLocale(user.getPreferredLocale());
		u.setRegistrationDate(user.getRegistrationDate());
        u.setScreenname(user.getScreenname());
		u.setTitle(user.getTitle());
		u.setUid(user.getUid());
		u.setWhereHeardAbout(user.getWhereHeardAbout());

		u.setIsArtist(isArtist);
		u.setArtistDescription(artistDescription);
		u.setArtistNotes(artistNotes);

		u.setIsStaff(isStaff);
		u.setStaffNotes(staffNotes);
		u.setStaffNumber(staffNumber);

		u.setCustomerNotes(customerNotes);
			
		return u;
	}
	
	public Party mapCustomer(User user) {
		
		Party u = new Party();
		u.setDob(user.getDob());
		u.setLogin(user.getEmail());
		u.setFirstname(user.getFirstname());
		u.setJoinedMailingList(user.isJoinedMailingList());
		u.setJoinedMobileList(user.isJoinedMobileList());
		u.setLastname(user.getLastname());
		u.setMobile(user.getMobile());
		u.setPhone(user.getPhone());
		u.setPreferredLocale(user.getPreferredLocale());
		u.setRegistrationDate(user.getRegistrationDate());
		u.setScreenname(user.getScreenname());
		u.setTitle(user.getTitle());
		u.setUid(user.getUid());
		u.setWhereHeardAbout(user.getWhereHeardAbout());
        u.setIsArtist(user.isArtist());
        u.setIsStaff(user.isStaff());

		return u;
	}
	
	public Staff mapStaff(Party users) {
		if(users == null){
			return null;
		}
		Staff s = new Staff();
		s.setDob(users.getDob());
		s.setEmail(users.getLogin());
		s.setFirstname(users.getFirstname());
		s.setLastname(users.getLastname());
		s.setMobile(users.getMobile());
		s.setNotes(users.getCustomerNotes());
		s.setPhone(users.getPhone());
		s.setPreferredLocale(users.getPreferredLocale());
		s.setRegistrationDate(users.getRegistrationDate());
		s.setTitle(users.getTitle());
		s.setUid(users.getUid());
		s.setStaffId(users.getStaffNumber());
        s.setArtist(users.isIsArtist());
        s.setStaff(users.isIsStaff());
		return s;
	}
	
	public SalesOrderItem mapSalesOrderItemFully(uk.co.maxant.gwtdemo.server.po.SalesOrderItem salesOrderItem) {
    	try {
    		SalesOrderItem soi = (SalesOrderItem) GenericMapper.map(SalesOrderItem.class, salesOrderItem);
    		
    		if(salesOrderItem.getAdditionalDeliveryCharge() != null){
    			soi.setAdditionalDeliveryCharge(salesOrderItem.getAdditionalDeliveryCharge().doubleValue());
    		}
    		if(salesOrderItem.getSpecialOfferDiscount() != null){
    			soi.setSpecialOfferDiscount(salesOrderItem.getSpecialOfferDiscount().doubleValue());
    		}
    		if(salesOrderItem.getBasePrice() != null){
    			soi.setBasePrice(salesOrderItem.getBasePrice().doubleValue());
    		}
    		
    		soi.setDesign(mapDesign(salesOrderItem.getDesign()));

    		return soi;
    	} catch (InstantiationException e) {
    		throw new RuntimeException(e);
    	} catch (IllegalAccessException e) {
    		throw new RuntimeException(e);
    	} catch (InvocationTargetException e) {
    		throw new RuntimeException(e);
    	}
	}

}
