/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.util;

import java.util.Date;

import uk.co.maxant.gwtdemo.server.po.Party;
import uk.co.maxant.gwtdemo.server.po.SalesOrder;

public class NotesFormatter extends ServerFormats {

	private NotesFormatter(){}
	
	/**
	 * @param details notes to format
	 * @return a header plus the details
	 */
	private static String formatNotes(String details){
		return "_______________\r\n" + EEE_dd_MMM_yyyy_HH_mm_ss_sss().format(new Date()) + "\r\n\r\n" + details + "\r\n\r\n";
	}

	/**
	 * @param details notes to format
	 * @param user username if a user has appended notes
	 * @return a header plus the details
	 */
	private static String formatNotes(Party staff, String details){
		String user = getStaffDescription(staff);
		return "_______________\r\n" + EEE_dd_MMM_yyyy_HH_mm_ss_sss().format(new Date()) + "\r\n" + user + " added the following notes:\r\n\r\n" + details + "\r\n\r\n";
	}
	
	public static String getStaffDescription(Party staff){
		return staff.getFirstname() + " " + staff.getLastname() + " (" + 
					staff.getStaffNumber() + ", " + staff.getLogin() + ")";
	}
	
	/**
	 * @param newNotes notes to add
	 * @param oldNotes existing notes. can be null.
	 * @return new notes plus old notes. new notes always have a header prepended.
	 */
	private static String appendNotes(String newNotes, String oldNotes){
		return formatNotes(newNotes) + (oldNotes == null?"":oldNotes);
	}
	
	/**
	 * @param newNotes notes to add
	 * @param oldNotes existing notes. can be null.
	 * @return new notes plus old notes. new notes always have a header prepended.
	 */
	private static String appendNotes(Party staff, String newNotes, String oldNotes){
		return formatNotes(staff, newNotes) + (oldNotes == null?"":oldNotes);
	}

	/**
	 * @param newNotes notes to add to the order
	 * @param so the order to add notes to
	 */
	public static void appendNotes(String newNotes, SalesOrder so){
		so.setShopNotes(appendNotes(newNotes, so.getShopNotes()));
	}
	
	/**
	 * @param newNotes notes to add to the order
	 * @param so the order to add notes to
	 */
	public static void appendNotes(Party staff, String newNotes, SalesOrder so){
		so.setShopNotes(appendNotes(staff, newNotes, so.getShopNotes()));
	}
	
	/**
	 * @param newNotes notes to add to the user
	 * @param user the user to add customer notes to
	 */
	public static void appendCustomerNotes(String newNotes, Party user){
		user.setCustomerNotes(appendNotes(newNotes, user.getCustomerNotes()));
	}
	

	
}
