/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.util;

import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * A TO for holding all information about a payment update, for example coming from 
 * PayPal PDT, PayPal IPN or a manual EPOS entry or indeed some other payment gateway 
 * in the future.
 */
public class PaymentDetails implements Serializable {

	private static final long serialVersionUID = -9127955081422756837L;
	
	protected String testIPN;
	protected String notifyVersion;
	protected String verifySign;
	protected String firstName;
	protected String lastName;
	protected String payerBusinessName;
	protected String payerEmail;
	protected String payerID;
	protected String payerStatus;
	protected String residenceCountry;
	protected String business;
	protected String itemName;
	protected String itemNumber;
	protected String quantity;
	protected String receiverEmail;
	protected String receiverID;
	protected String custom;
	protected String invoice;
	protected String memo; //PDT = comment
	protected String authId;
	protected String authExp;
	protected String authAmount;
	protected String authStatus;
	protected String parentTxnId;
	protected String paymentDate;
	protected String paymentStatus; //PDT = st
	protected String paymentType;
	protected String pendingReason;
	protected String reasonCode;
	protected String remainingSettle;
	protected String tax;
	protected String transactionEntity;
	protected String txnId; //PDT = tx
	protected String txnType;
	protected String exchangeRate;
	protected String mcCurrency; //PDT = cc
	protected String mcFee;
	protected String mcGross; //PDT = amt
	protected String mcHandling;
	protected String mcShipping; //PDT = shipping
	protected String paymentFee;
	protected String paymentGross; //PDT = amt - see mcGross
	protected String settleAmount;
	protected String settleCurrency;
	protected String caseId;	//dispute only
	protected String caseType;	//dispute only
	protected String caseCreationDate;	//dispute only
	protected String extras = "PDT/IPN had the following extra values:\r\n";	//added to notes since we dont have these available in the DB. this is our way of handling new data wihch comes from paypal.
	
	protected Integer orderNumber; //taken from the session
	protected String source; //e.g. paypal, other gateway, etc. Could be a staff UID / email / number too.
	protected String instructions; //e.g. the continuation instructions - optional. if present, these are used instead of looking for them
	
	public PaymentDetails(Integer orderNum){
		this.orderNumber = orderNum;
	}
	
	/**
	 * @param parameters A Map of all the parameters to populate in the structure.
	 * @throws IllegalArgumentException If an unexpected parameter name is encountered.
	 */
	public PaymentDetails(Map<String, Object> parameters, Integer orderNum) throws IllegalArgumentException {
		this.orderNumber = orderNum;
		if(parameters.size() == 0) throw new IllegalArgumentException("The request contained no parameters!");
		Set<String> keys = parameters.keySet();
		for(String key : keys){
			Object v = parameters.get(key);
			String value = null;
			if(v instanceof String){
				value = (String)v;
			}else if(v instanceof String[]){
				String[] vals = (String[])v;
				if(vals.length != 1) {
					String values = "";
					for(String val : vals){
						values += val + ", ";
					}
					throw new IllegalArgumentException("Unexpected number of values for parameter name " + key + ". Values were " + values);
				}
				value = vals[0];
			}else{
				throw new IllegalArgumentException("Unexpected parameter type " + v.getClass().toString() + " for parameter name " + key);
			}
			
			if(value == null){
				throw new NullPointerException("Unable to determine String value of parameter name " + key + ". It was " + v.toString());
			}
			
			if(key.equalsIgnoreCase("test_ipn")){
				this.testIPN = value;
			}else if(key.equalsIgnoreCase("notify_version")){
				this.notifyVersion = value;
			}else if(key.equalsIgnoreCase("verify_sign")){
				this.verifySign = value;
			}else if(key.equalsIgnoreCase("first_name")){
				this.firstName = value;
			}else if(key.equalsIgnoreCase("last_name")){
				this.lastName = value;
			}else if(key.equalsIgnoreCase("payer_business_name")){
				this.payerBusinessName = value;
			}else if(key.equalsIgnoreCase("payer_email")){
				this.payerEmail = value;
			}else if(key.equalsIgnoreCase("payer_id")){
				this.payerID = value;
			}else if(key.equalsIgnoreCase("payer_status")){
				this.payerStatus = value;
			}else if(key.equalsIgnoreCase("residence_country")){
				this.residenceCountry = value;
			}else if(key.equalsIgnoreCase("business")){
				this.business = value;
			}else if(key.equalsIgnoreCase("item_name")){
				this.itemName = value;
			}else if(key.equalsIgnoreCase("item_number")){
				this.itemNumber = value;
			}else if(key.equalsIgnoreCase("quantity")){
				this.quantity = value;
			}else if(key.equalsIgnoreCase("receiver_email")){
				this.receiverEmail = value;
			}else if(key.equalsIgnoreCase("receiver_id")){
				this.receiverID = value;
			}else if(key.equalsIgnoreCase("custom")){
				this.custom = value;
			}else if(key.equalsIgnoreCase("invoice")){
				this.invoice = value;
			}else if(key.equalsIgnoreCase("memo")){
				this.memo = value;
			}else if(key.equalsIgnoreCase("tax")){
				this.tax = value;
			}else if(key.equalsIgnoreCase("auth_id")){
				this.authId = value;
			}else if(key.equalsIgnoreCase("auth_exp")){
				this.authExp = value;
			}else if(key.equalsIgnoreCase("auth_amount")){
				this.authAmount = value;
			}else if(key.equalsIgnoreCase("auth_status")){
				this.authStatus = value;
			}else if(key.equalsIgnoreCase("parent_txn_id")){
				this.parentTxnId = value;
			}else if(key.equalsIgnoreCase("payment_date")){
				this.paymentDate = value;
			}else if(key.equalsIgnoreCase("payment_status")){
				this.paymentStatus = value;
			}else if(key.equalsIgnoreCase("payment_type")){
				this.paymentType = value;
			}else if(key.equalsIgnoreCase("pending_reason")){
				this.pendingReason = value;
			}else if(key.equalsIgnoreCase("reason_code")){
				this.reasonCode = value;
			}else if(key.equalsIgnoreCase("remaining_setlle")){
				this.remainingSettle = value;
			}else if(key.equalsIgnoreCase("transaction_entity")){
				this.transactionEntity = value;
			}else if(key.equalsIgnoreCase("txn_id")){
				this.txnId = value;
			}else if(key.equalsIgnoreCase("txn_type")){
				this.txnType = value;
			}else if(key.equalsIgnoreCase("exchange_rate")){
				this.exchangeRate = value;
			}else if(key.equalsIgnoreCase("mc_currency")){
				this.mcCurrency = value;
			}else if(key.equalsIgnoreCase("mc_fee")){
				this.mcFee = value;
			}else if(key.equalsIgnoreCase("mc_gross")){
				this.mcGross = value;
			}else if(key.equalsIgnoreCase("mc_handling")){
				this.mcHandling = value;
			}else if(key.equalsIgnoreCase("mc_shipping")){
				this.mcShipping = value;
			}else if(key.equalsIgnoreCase("shipping")){
				this.mcShipping = value;
			}else if(key.equalsIgnoreCase("payment_fee")){
				this.paymentFee = value;
			}else if(key.equalsIgnoreCase("payment_gross")){
				this.paymentGross = value;
			}else if(key.equalsIgnoreCase("settle_amount")){
				this.settleAmount = value;
			}else if(key.equalsIgnoreCase("settle_currency")){
				this.settleCurrency = value;
			}else if(key.equalsIgnoreCase("case_id")){
				this.caseId = value;
			}else if(key.equalsIgnoreCase("case_type")){
				this.caseType = value;
			}else if(key.equalsIgnoreCase("case_creation_date")){
				this.caseCreationDate = value;
			}else if(key.equalsIgnoreCase("tx")){
				this.txnId = value;
			}else if(key.equalsIgnoreCase("st")){
				this.paymentStatus = value;
			}else if(key.equalsIgnoreCase("amt")){
				this.mcGross = value;
				this.paymentGross = value; //even though we think this field is not required as its legacy
			}else if(key.equalsIgnoreCase("cc")){
				this.mcCurrency = value;
			}else if(key.equalsIgnoreCase("cm")){
				this.memo = value;
			}else{
				this.extras += key + "=" + value + "\r\n";
			}
		}
	}
	
	private static final List<String> fieldsToSkip = new ArrayList<String>();
	private static final char space = ' ';
	private static int longestFieldName;
	static{
		fieldsToSkip.add("longestFieldName");
		fieldsToSkip.add("space");
		fieldsToSkip.add("serialVersionUID");
		fieldsToSkip.add("fieldsToSkip");

		for(Field field : PaymentDetails.class.getDeclaredFields()){
			if(field.getName().length() > longestFieldName) longestFieldName = field.getName().length();
		}
	}
	
public static void main(String[] args){
	System.out.println(new PaymentDetails(1).toString());
}
	
	@Override
	public String toString() {
		StringBuilder sb = new StringBuilder("Payment Details:\r\n");
		try{
			for(Field field : this.getClass().getDeclaredFields()){
				if(!fieldsToSkip.contains(field.getName())){
					Object obj = field.get(this);
					if(obj != null){
						if(obj.toString().length() > 0){ 
							sb.append("\t").append(getSpaces(field.getName())).append(field.getName()).append(" = ").append(obj).append("\r\n");
						}
					}
				}
			}
		}catch(IllegalAccessException e){
			//well, its not going to happen because we are in the class so we have all the rights
			//- if it does, we just return a parial toString()
		}
		return sb.toString();
	}
	
	private String getSpaces(String fieldName){
		int l = longestFieldName - fieldName.length();
		char[] spaces = new char[l];
		for(int i = 0; i < l; i++){
			spaces[i] = space;
		}
		return new String(spaces);
	}

	/* **************************************
	 * 
	 * Generated Accessors
	 */
	
	public String getAuthAmount() {
		return authAmount;
	}

	public void setAuthAmount(String authAmount) {
		this.authAmount = authAmount;
	}

	public String getAuthExp() {
		return authExp;
	}

	public void setAuthExp(String authExp) {
		this.authExp = authExp;
	}

	public String getAuthId() {
		return authId;
	}

	public void setAuthId(String authId) {
		this.authId = authId;
	}

	public String getAuthStatus() {
		return authStatus;
	}

	public void setAuthStatus(String authStatus) {
		this.authStatus = authStatus;
	}

	public String getBusiness() {
		return business;
	}

	public void setBusiness(String business) {
		this.business = business;
	}

	public String getCaseCreationDate() {
		return caseCreationDate;
	}

	public void setCaseCreationDate(String caseCreationDate) {
		this.caseCreationDate = caseCreationDate;
	}

	public String getCaseId() {
		return caseId;
	}

	public void setCaseId(String caseId) {
		this.caseId = caseId;
	}

	public String getCaseType() {
		return caseType;
	}

	public void setCaseType(String caseType) {
		this.caseType = caseType;
	}

	public String getCustom() {
		return custom;
	}

	public void setCustom(String custom) {
		this.custom = custom;
	}

	public String getExchangeRate() {
		return exchangeRate;
	}

	public void setExchangeRate(String exchangeRate) {
		this.exchangeRate = exchangeRate;
	}

	public String getFirstName() {
		return firstName;
	}

	public void setFirstName(String firstName) {
		this.firstName = firstName;
	}

	public String getInvoice() {
		return invoice;
	}

	public void setInvoice(String invoice) {
		this.invoice = invoice;
	}

	public String getItemName() {
		return itemName;
	}

	public void setItemName(String itemName) {
		this.itemName = itemName;
	}

	public String getItemNumber() {
		return itemNumber;
	}

	public void setItemNumber(String itemNumber) {
		this.itemNumber = itemNumber;
	}

	public String getLastName() {
		return lastName;
	}

	public void setLastName(String lastName) {
		this.lastName = lastName;
	}

	public String getMcCurrency() {
		return mcCurrency;
	}

	public void setMcCurrency(String mcCurrency) {
		this.mcCurrency = mcCurrency;
	}

	public String getMcFee() {
		return mcFee;
	}

	public void setMcFee(String mcFee) {
		this.mcFee = mcFee;
	}

	public String getMcGross() {
		return mcGross;
	}

	public void setMcGross(String mcGross) {
		this.mcGross = mcGross;
	}

	public String getMcHandling() {
		return mcHandling;
	}

	public void setMcHandling(String mcHandling) {
		this.mcHandling = mcHandling;
	}

	public String getMcShipping() {
		return mcShipping;
	}

	public void setMcShipping(String mcShipping) {
		this.mcShipping = mcShipping;
	}

	public String getMemo() {
		return memo;
	}

	public void setMemo(String memo) {
		this.memo = memo;
	}

	public String getNotifyVersion() {
		return notifyVersion;
	}

	public void setNotifyVersion(String notifyVersion) {
		this.notifyVersion = notifyVersion;
	}

	public String getParentTxnId() {
		return parentTxnId;
	}

	public void setParentTxnId(String parentTxnId) {
		this.parentTxnId = parentTxnId;
	}

	public String getPayerBusinessName() {
		return payerBusinessName;
	}

	public void setPayerBusinessName(String payerBusinessName) {
		this.payerBusinessName = payerBusinessName;
	}

	public String getPayerEmail() {
		return payerEmail;
	}

	public void setPayerEmail(String payerEmail) {
		this.payerEmail = payerEmail;
	}

	public String getPayerID() {
		return payerID;
	}

	public void setPayerID(String payerID) {
		this.payerID = payerID;
	}

	public String getPayerStatus() {
		return payerStatus;
	}

	public void setPayerStatus(String payerStatus) {
		this.payerStatus = payerStatus;
	}

	public String getPaymentDate() {
		return paymentDate;
	}

	public void setPaymentDate(String paymentDate) {
		this.paymentDate = paymentDate;
	}

	public String getPaymentFee() {
		return paymentFee;
	}

	public void setPaymentFee(String paymentFee) {
		this.paymentFee = paymentFee;
	}

	public String getPaymentGross() {
		return paymentGross;
	}

	public void setPaymentGross(String paymentGross) {
		this.paymentGross = paymentGross;
	}

	public String getPaymentStatus() {
		return paymentStatus;
	}

	public void setPaymentStatus(String paymentStatus) {
		this.paymentStatus = paymentStatus;
	}

	public String getPaymentType() {
		return paymentType;
	}

	public void setPaymentType(String paymentType) {
		this.paymentType = paymentType;
	}

	public String getPendingReason() {
		return pendingReason;
	}

	public void setPendingReason(String pendingReason) {
		this.pendingReason = pendingReason;
	}

	public String getQuantity() {
		return quantity;
	}

	public void setQuantity(String quantity) {
		this.quantity = quantity;
	}

	public String getReasonCode() {
		return reasonCode;
	}

	public void setReasonCode(String reasonCode) {
		this.reasonCode = reasonCode;
	}

	public String getReceiverEmail() {
		return receiverEmail;
	}

	public void setReceiverEmail(String receiverEmail) {
		this.receiverEmail = receiverEmail;
	}

	public String getReceiverID() {
		return receiverID;
	}

	public void setReceiverID(String receiverID) {
		this.receiverID = receiverID;
	}

	public String getRemainingSettle() {
		return remainingSettle;
	}

	public void setRemainingSettle(String remainingSettle) {
		this.remainingSettle = remainingSettle;
	}

	public String getResidenceCountry() {
		return residenceCountry;
	}

	public void setResidenceCountry(String residenceCountry) {
		this.residenceCountry = residenceCountry;
	}

	public String getSettleAmount() {
		return settleAmount;
	}

	public void setSettleAmount(String settleAmount) {
		this.settleAmount = settleAmount;
	}

	public String getSettleCurrency() {
		return settleCurrency;
	}

	public void setSettleCurrency(String settleCurrency) {
		this.settleCurrency = settleCurrency;
	}

	public String getTax() {
		return tax;
	}

	public void setTax(String tax) {
		this.tax = tax;
	}

	public String getTestIPN() {
		return testIPN;
	}

	public void setTestIPN(String testIPN) {
		this.testIPN = testIPN;
	}

	public String getTransactionEntity() {
		return transactionEntity;
	}

	public void setTransactionEntity(String transactionEntity) {
		this.transactionEntity = transactionEntity;
	}

	public String getTxnId() {
		return txnId;
	}

	public void setTxnId(String txnId) {
		this.txnId = txnId;
	}

	public String getTxnType() {
		return txnType;
	}

	public void setTxnType(String txnType) {
		this.txnType = txnType;
	}

	public String getVerifySign() {
		return verifySign;
	}

	public void setVerifySign(String verifySign) {
		this.verifySign = verifySign;
	}

	public Integer getOrderNumber() {
		return orderNumber;
	}

	public void setOrderNumber(Integer orderNumber) {
		this.orderNumber = orderNumber;
	}

	public String getSource() {
		return source;
	}

	public void setSource(String source) {
		this.source = source;
	}

	public String getInstructions() {
		return instructions;
	}

	public void setInstructions(String instructions) {
		this.instructions = instructions;
	}

	public String getExtras() {
		return extras;
	}
	
	public void addExtras(String key, String value){
		this.extras += key + "=" + value + "\r\n";
	}

}
