/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.util;

import java.math.BigDecimal;

import uk.co.maxant.gwtdemo.server.po.SalesOrder;
import uk.co.maxant.gwtdemo.server.po.SalesOrderItem;

/**
 * helper containing methods working on sales orders.
 */
public class SalesOrderHelper {

	private SalesOrderHelper(){}
	
	public static BigDecimal getGrandTotal(SalesOrder so){
		BigDecimal total = getSubTotal(so);
		total = total.add(so.getDeliveryTotal());
		return total;
	}

	/**
	 * @param so the relevant sales order
	 * @return sum of all items in the order
	 */
	public static BigDecimal getSubTotal(SalesOrder so){
		BigDecimal total = BigDecimal.ZERO;
		for(SalesOrderItem soi : so.getSalesOrderItems()){
			BigDecimal discountFactor = new BigDecimal(100.0).subtract(soi.getSpecialOfferDiscount());
			discountFactor = discountFactor.divide(new BigDecimal(100.0));
			BigDecimal t = soi.getBasePrice();
			t = t.multiply(discountFactor);
			t = t.add(soi.getAdditionalDeliveryCharge());
			t = t.multiply(new BigDecimal(soi.getQuantity()));
			total = total.add(t);
		}
		return total;
	}
	
}
