/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.util;

import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import uk.co.maxant.gwtdemo.server.ConfigService;
import uk.co.maxant.gwtdemo.server.DesignService;
import uk.co.maxant.gwtdemo.server.ErrorService;
import uk.co.maxant.gwtdemo.server.MailService;
import uk.co.maxant.gwtdemo.server.OrchestrationService;
import uk.co.maxant.gwtdemo.server.SalesOrderService;
import uk.co.maxant.gwtdemo.server.PartyService;
import uk.co.maxant.gwtdemo.util.ResourceHelper;

public class ServiceLocator {

	private static ServiceLocator instance = null;

	private ApplicationContext context;
	
	private ResourceHelper resourceHelper;
	
	private ServiceLocator(){
		context = new ClassPathXmlApplicationContext("applicationContext.xml");
		resourceHelper = new SpringResourceHelper(context);
	}
	
	public synchronized static ServiceLocator getInstance(){
		if(instance == null){
			instance = new ServiceLocator();
		}
		return instance;
	}

	/**
	 * @param name the name of the bean to find.
	 * @return the bean with the given name.
	 */
	protected Object getBean(String name){
		return context.getBean(name);
	}

	/**
	 * @return a resource helper which can be used to get i18n messages
	 */
	public ResourceHelper getResourceHelper(){
		return resourceHelper;
	}
	
	/**
	 * @return the config service bean.
	 */
	public ConfigService getConfigService(){
		return (ConfigService)getBean("configService");
	}
	
	/**
	 * @return the design service bean.
	 */
	public DesignService getDesignService(){
		return (DesignService)getBean("designService");
	}
	
	/**
	 * @return the sales order service bean.
	 */
	public SalesOrderService getSalesOrderService(){
		return (SalesOrderService)getBean("salesOrderService");
	}
	
	/**
	 * @return the user service bean.
	 */
	public PartyService getPartyService(){
		return (PartyService)getBean("partyService");
	}
	
	/**
	 * @return the mail service bean.
	 */
	public MailService getMailService(){
		return (MailService)getBean("mailService");
	}

	public OrchestrationService getOrchestrationService() {
		return (OrchestrationService)getBean("orchestrationService");
	}

	public ErrorService getErrorService() {
		return (ErrorService)getBean("errorService");
	}
	
}
