/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.web;

import java.io.IOException;
import java.io.UnsupportedEncodingException;

import javax.servlet.http.HttpServletRequest;

import uk.co.maxant.gwtdemo.client.ApplicationException;
import uk.co.maxant.gwtdemo.server.ConfigService;
import uk.co.maxant.gwtdemo.server.PartyService;
import uk.co.maxant.gwtdemo.server.po.Party;
import uk.co.maxant.gwtdemo.server.util.ServiceLocator;

public class ConfirmationHelper {

	private static final String USER_CONFIRMED = "_CONFIRMED_USER";

	private ConfirmationHelper(){}
	
	/**
	 * ensures that the user has confirmed their account. caches the user in the session.
	 * if not confirmed, then redirects to the confirmation page.
	 */
	public static boolean checkConfirmation(HttpServletRequest request) throws UnsupportedEncodingException, ApplicationException, IOException{
		if(request.getUserPrincipal() != null){ //only check if logged in
			Boolean confirmed = (Boolean)request.getSession().getAttribute(USER_CONFIRMED);
			if(confirmed == null || !confirmed.booleanValue()){
				//check confirmation...
				
				//do we need to check?
				ConfigService configService = ServiceLocator.getInstance().getConfigService();
				if(configService.boolValue(ConfigService.registrationForceConfirm)){
					PartyService userService = ServiceLocator.getInstance().getPartyService();
					Party user = userService.getLoggedinUser(request.getUserPrincipal());
					if(user.getRegCode() != null && user.getRegCode().length() != 0){
						return false;
					}else{
						//dont need to check in the future
						request.getSession().setAttribute(USER_CONFIRMED, Boolean.TRUE);
					}
				}else{
					//dont need to check due to config
					request.getSession().setAttribute(USER_CONFIRMED, Boolean.TRUE);
				}
			}else{
				//already confirmed
			}
		}else{
			//not logged in
		}
		return true;
	}
}
