/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.web;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import uk.co.maxant.gwtdemo.client.Roles;
import uk.co.maxant.gwtdemo.server.DesignService;
import uk.co.maxant.gwtdemo.server.po.Design;
import uk.co.maxant.gwtdemo.server.util.ServiceLocator;

public class GetHiResDesignServlet extends HttpServlet {

	private static final long serialVersionUID = 1L;
	
	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		
		doService(request, response);
	}

	@Override
	protected void doPost(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		
		doService(request, response);
	}
	
	public void doService(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		FileInputStream fis = null;
		try{
			//need to still check security... only pickers or admin may view anything. artists may only view own...
			String uid = request.getParameter("uid");
			if(uid == null){
				throw new Exception("uid not found");
			}

			boolean doit = false;
			if(!request.isUserInRole(Roles.PICKING)){
				if(!request.isUserInRole(Roles.BACKOFFICE)){
					//perhaps its a customer?
					if(request.isUserInRole(Roles.ARTIST)){
						//design MUST belong to them, otherwise dont let them view it!
						DesignService ds = ServiceLocator.getInstance().getDesignService();
						Design d = ds.getDesign(Integer.parseInt(uid));
						if(d.getParty().getLogin().equalsIgnoreCase(request.getUserPrincipal().getName())){
							//its their design!
							doit = true;
						}
					}
				}else{
					doit = true;
				}
			}else{
				doit = true;
			}
			if(!doit){
				response.sendError(HttpServletResponse.SC_FORBIDDEN, "No rights to view the high resolution version of design " + uid);
			}else{
				String file = "/hiresimages/designs/design_large_" + uid + ".jpg";
				
				String realPath = request.getSession().getServletContext().getRealPath(file);
				File f = new File(realPath);
				if(f.length() >= Integer.MAX_VALUE) throw new ServletException("File too large for download.");
				fis = new FileInputStream(f);

				//TODO add a proper mime type mapping?
				String mimetype = null;
				if(file.toLowerCase().endsWith(".jpg")){
					mimetype = "image/jpeg";
				}

				String fn = file;
				int idx = fn.lastIndexOf('/');
				if(idx >= 0){
					fn = file.substring(idx + 1);
				}
				
				if(mimetype != null){
					response.setContentType(mimetype);
				}
				response.setContentLength((int)f.length());
				response.setHeader("Content-Disposition","attachment; filename=\"" + fn + "\"");
				response.setHeader("Pragma", "public");
				response.setHeader("Cache-Control","cache");
				response.setHeader("Cache-Control","must-revalidate");	

				//TODO add some buffered streams to this to optimize it!
				int current = -1;
				OutputStream os = response.getOutputStream();
				while((current = fis.read()) != -1){
					os.write(current);
				}
			}
		}catch(Exception e){
			String msg = "Failed to generate the download and put it in the response: " + e.getMessage();
			throw new ServletException(msg, e);
		}finally{
			if(fis != null){
				fis.close();
			}
		}
	}

}
