/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.web;

import javax.servlet.http.HttpSessionEvent;
import javax.servlet.http.HttpSessionListener;

import org.apache.log4j.Logger;

import uk.co.maxant.gwtdemo.client.ApplicationException;
import uk.co.maxant.gwtdemo.server.ErrorService;
import uk.co.maxant.gwtdemo.server.OrchestrationService;
import uk.co.maxant.gwtdemo.server.util.LogHelper;
import uk.co.maxant.gwtdemo.server.util.ServiceLocator;

public class SessionListener implements HttpSessionListener {

	private static Logger log = LogHelper.getLog();

	public void sessionCreated(HttpSessionEvent se) {
		log.debug("started new session " + se.getSession().getId());
	}
	
	public void sessionDestroyed(HttpSessionEvent se) {
		
		//save off potentially saved orders
		try {
			OrchestrationService orchestrationService = ServiceLocator.getInstance().getOrchestrationService();
			orchestrationService.createDiscardedOrder(se.getSession());
		} catch (ApplicationException e) {
			//log the error, since this will otherwise never be seen by a user
			ErrorService errorService = ServiceLocator.getInstance().getErrorService();
			errorService.reportError(se.getSession().getServletContext().getServletContextName(), "failed to create a discarded order", e);
		}
	}
}

/*
TODO 001 security logout - since all server calls go thru a central serverCalling framework, we can track
the last call. we just need a client side timer which runs every few seconds which checks the last call time.
if the session expires, popup a notice. then when the user confirms, if they were logged in, go to the home page anew, so the header is refreshed, showing that they are no longer logged in.
*/
