/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.server.web;

import java.util.Timer;
import java.util.TimerTask;

import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import uk.co.maxant.gwtdemo.client.ApplicationException;
import uk.co.maxant.gwtdemo.server.ErrorService;
import uk.co.maxant.gwtdemo.server.OrchestrationService;
import uk.co.maxant.gwtdemo.server.util.DependencyResolver;
import uk.co.maxant.gwtdemo.server.util.LogHelper;

public class StartupServletContextListener implements ServletContextListener {

	private static final Logger log = LogHelper.getLog();
	
	private static final Long ONE_DAY = 24L * 60 * 60 * 1000;
	
	protected Timer dbCleaner;

	protected String servletContextName;

	private boolean initedServices = false;
	
    @Autowired
    private OrchestrationService orchestrationService;

    @Autowired
    private ErrorService errorService;
    
	public void contextInitialized(ServletContextEvent event) {
		try{
			servletContextName = event.getServletContext().getServletContextName();

			//add a new thread to clean the database
			dbCleaner = new Timer();
			dbCleaner.scheduleAtFixedRate(new TimerTask(){
				public void run() {
					cleanDatabase();
				}
			}, 2000L, ONE_DAY);

			log.info("Application '" + servletContextName + "' has been initialized successfully.");
		}catch(Throwable t){
			String msg = "Application " + servletContextName + " has failed to start up correctly!";
			log.info(msg, t);
			throw new RuntimeException(msg, t);
		}
	}

	public void contextDestroyed(ServletContextEvent event) {
		log.info("Application " + servletContextName + " is shutting down.");
		if(dbCleaner != null) dbCleaner.cancel();
	}


	protected void cleanDatabase() {
		init();
		try{
			//clean out old viewing stats
			orchestrationService.cleanArtistViews();
		}catch(ApplicationException ae){
			errorService.reportError(servletContextName, "Failed to clean old artist views", ae);
		}
	}

	private synchronized void init() {
		if(!initedServices){
	        try {
	            DependencyResolver.resolve(this);
	            initedServices = true;
	        } catch (Exception e) {
	            String msg = "failed to instantiate app layer for startup servlet context";
	            log.error(msg, e);
	            throw new RuntimeException(msg, e);
	        }
		}
	}
}
