/*  
 * Copyright (c) 2009 Ant Kutschera, maxant
 * 
 * This file is part of the maxant GWT Demo.
 * 
 * maxant GWT Demo is free software: you can redistribute it and/or modify
 * it under the terms of the Lesser GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * maxant GWT Demo is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Lesser GNU General Public License for more details.
 * You should have received a copy of the Lesser GNU General Public License
 * along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */

package uk.co.maxant.gwtdemo.tooling;

import java.util.Hashtable;
import java.util.Properties;

import javax.mail.Authenticator;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 * test util. already contains a mail session.
 * 
 * put it into use by setting a sys prop:
 * 		System.setProperty("java.naming.factory.initial", "uk.co.maxant.gwtdemo.tooling.InitialContextFactory");
 */
public class InitialContextFactory implements javax.naming.spi.InitialContextFactory {

	public Context getInitialContext(Hashtable<?, ?> environment)
			throws NamingException {

		InitialContext ic = new InitialContext() {
			
			@Override
			protected Context getDefaultInitCtx() throws NamingException {
				return this;
			}

			public Object addToEnvironment(String propName, Object propVal)
					throws NamingException {
				myProps.put(propName, propVal);
				return null;
			}
			@Override
			public Object lookup(String name) throws NamingException {
				if(myProps.containsKey(name)){
					return myProps.get(name);
				}
				return super.lookup(name);
			}
		};

        Properties mailProps = System.getProperties();
    	mailProps.put("mail.smtp.host", "localhost");
    	mailProps.put("mail.smtp.port", "25");
        //session = Session.getDefaultInstance(mailProps, null);
    	Session session = Session.getDefaultInstance(mailProps, new MailAuthenticator("boss", "boss"));      
		
		ic.addToEnvironment("java:comp/env/mail/Session", session);
		
		return ic;
	}
	
}

/**
 * This class represents an authenticator object required by JavaMail when
 * authentication to the SMTP server is required.
 */
class MailAuthenticator extends Authenticator
{
    protected PasswordAuthentication myPA = null;
    
    public MailAuthenticator(String user, String pwd){
        myPA = new PasswordAuthentication(user, pwd);
    }
    
    public PasswordAuthentication getPasswordAuthentication(){
        return myPA;
    }
}
